Package: factoextra
Type: Package
Title: Extract and Visualize the Results of Multivariate Data Analyses
Version: 2.1.0
Date: 2026-06-26
Authors@R: c(
    person("Alboukadel", "Kassambara", role = c("aut", "cre"), email = "alboukadel.kassambara@gmail.com",
           comment = c(ORCID = "0009-0002-9136-0791")),
    person("Fabian", "Mundt", role = "aut", email = "f.mundt@inventionate.de"),
    person("Laszlo", "Erdey", role = "ctb", email = "erdey.laszlo@econ.unideb.hu",
           comment = c(ORCID = "0000-0002-6781-4303",
                       affiliation = "Faculty of Economics and Business, University of Debrecen, Hungary",
                       contribution = "Modern compatibility fixes, tests, and maintenance updates"))
    )
Description: Provides easy-to-use functions to extract and visualize the output
    of multivariate data analyses, including 'PCA' (Principal Component Analysis),
    'CA' (Correspondence Analysis), 'MCA' (Multiple Correspondence Analysis),
    'FAMD' (Factor Analysis of Mixed Data), 'MFA' (Multiple Factor Analysis),
    and 'HMFA' (Hierarchical Multiple Factor Analysis) from different R packages.
    It also includes support for supplementary qualitative variables in
    'FactoMineR' 'FAMD' and 'MFA' workflows, hardened validation for clustering
    and dimension-reduction helper workflows, backward-compatible phylogenetic
    dendrogram layout support for current 'igraph' APIs, and 'ggplot2'-based
    data visualization.
License: GPL-2
LazyData: true
LazyDataCompression: xz
Encoding: UTF-8
Depends: R (>= 4.1.0), ggplot2 (>= 3.5.2)
Imports: cluster (>= 2.1.8.2), dendextend (>= 1.19.1), FactoMineR (>=
        2.13), ggpubr (>= 0.6.3), grid, rlang (>= 1.1.7), stats,
        ggrepel (>= 0.9.5)
Suggests: ade4, ca, igraph, MASS, knitr, mclust, rmarkdown, testthat
        (>= 3.0.0)
VignetteBuilder: knitr
URL: https://github.com/kassambara/factoextra,
        https://rpkgs.datanovia.com/factoextra/index.html
BugReports: https://github.com/kassambara/factoextra/issues
Collate: 'fviz_add.R' 'eigenvalue.R' 'utilities.R' 'as_factoextra.R'
        'cluster_utilities.R' 'decathlon2.R' 'print.factoextra.R'
        'get_hmfa.R' 'fviz_hmfa.R' 'get_mfa.R' 'fviz_mfa.R'
        'deprecated.R' 'dist.R' 'fviz_dend.R' 'hcut.R' 'get_pca.R'
        'fviz_cluster.R' 'eclust.R' 'facto_summarize.R' 'fviz.R'
        'fviz_ca.R' 'fviz_contrib.R' 'fviz_cos2.R' 'fviz_ellipses.R'
        'fviz_famd.R' 'get_mca.R' 'fviz_mca.R' 'fviz_mclust.R'
        'fviz_nbclust.R' 'fviz_pca.R' 'fviz_silhouette.R' 'get_ca.R'
        'get_clust_tendency.R' 'get_famd.R' 'hkmeans.R' 'housetasks.R'
        'multishapes.R' 'poison.R' 'zzz.R'
RoxygenNote: 7.3.3
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-06-26 13:51:00 UTC; kassambara
Author: Alboukadel Kassambara [aut, cre] (ORCID:
    <https://orcid.org/0009-0002-9136-0791>),
  Fabian Mundt [aut],
  Laszlo Erdey [ctb] (ORCID: <https://orcid.org/0000-0002-6781-4303>,
    affiliation: Faculty of Economics and Business, University of
    Debrecen, Hungary, contribution: Modern compatibility fixes, tests,
    and maintenance updates)
Maintainer: Alboukadel Kassambara <alboukadel.kassambara@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-26 14:40:08 UTC

# Generated by roxygen2: do not edit by hand

S3method(print,factoextra)
S3method(print,hkmeans)
export(as_factoextra_pca)
export(eclust)
export(facto_summarize)
export(factominer_category_map)
export(fviz)
export(fviz_add)
export(fviz_ca)
export(fviz_ca_biplot)
export(fviz_ca_col)
export(fviz_ca_row)
export(fviz_cluster)
export(fviz_contrib)
export(fviz_cos2)
export(fviz_dend)
export(fviz_dist)
export(fviz_eig)
export(fviz_ellipses)
export(fviz_famd)
export(fviz_famd_ind)
export(fviz_famd_var)
export(fviz_gap_stat)
export(fviz_hmfa)
export(fviz_hmfa_group)
export(fviz_hmfa_ind)
export(fviz_hmfa_ind_starplot)
export(fviz_hmfa_quali_biplot)
export(fviz_hmfa_quali_var)
export(fviz_hmfa_quanti_var)
export(fviz_hmfa_var)
export(fviz_mca)
export(fviz_mca_biplot)
export(fviz_mca_ind)
export(fviz_mca_var)
export(fviz_mclust)
export(fviz_mclust_bic)
export(fviz_mfa)
export(fviz_mfa_axes)
export(fviz_mfa_group)
export(fviz_mfa_ind)
export(fviz_mfa_ind_starplot)
export(fviz_mfa_quali_biplot)
export(fviz_mfa_quali_var)
export(fviz_mfa_quanti_var)
export(fviz_mfa_var)
export(fviz_nbclust)
export(fviz_pca)
export(fviz_pca_biplot)
export(fviz_pca_contrib)
export(fviz_pca_ind)
export(fviz_pca_var)
export(fviz_screeplot)
export(fviz_silhouette)
export(get_ca)
export(get_ca_col)
export(get_ca_row)
export(get_clust_tendency)
export(get_dist)
export(get_eig)
export(get_eigenvalue)
export(get_famd)
export(get_famd_ind)
export(get_famd_var)
export(get_hmfa)
export(get_hmfa_group)
export(get_hmfa_ind)
export(get_hmfa_partial)
export(get_hmfa_quali_var)
export(get_hmfa_quanti_var)
export(get_hmfa_var)
export(get_mca)
export(get_mca_ind)
export(get_mca_var)
export(get_mfa)
export(get_mfa_group)
export(get_mfa_ind)
export(get_mfa_partial_axes)
export(get_mfa_quali_var)
export(get_mfa_quanti_var)
export(get_mfa_var)
export(get_pca)
export(get_pca_ind)
export(get_pca_var)
export(hcut)
export(hkmeans)
export(hkmeans_tree)
export(map_factominer_legacy_names)
import(ggplot2)
importFrom(ggrepel,geom_text_repel)
importFrom(grDevices,chull)
importFrom(graphics,plot)
importFrom(rlang,.data)
importFrom(stats,as.dendrogram)
importFrom(stats,as.dist)
importFrom(stats,kmeans)
importFrom(stats,na.omit)
importFrom(stats,qchisq)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,var)
importFrom(utils,data)

[![R-CMD-check](https://github.com/kassambara/factoextra/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/kassambara/factoextra/actions/workflows/R-CMD-check.yaml)
[![CRAN\_Status\_Badge](https://www.r-pkg.org/badges/version/factoextra)](https://cran.r-project.org/package=factoextra)
[![Downloads](https://cranlogs.r-pkg.org/badges/factoextra)](https://cran.r-project.org/package=factoextra)
[![Total
Downloads](https://cranlogs.r-pkg.org/badges/grand-total/factoextra?color=orange)](https://cranlogs.r-pkg.org/badges/grand-total/factoextra)
[![Project Status: Active - The project has reached a stable, usable
state and is being actively
developed.](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)
[![Pending
Pull-Requests](https://githubbadges.herokuapp.com/kassambara/factoextra/pulls.svg?style=flat)](https://github.com/kassambara/factoextra/pulls)
[![Github
Issues](https://githubbadges.herokuapp.com/kassambara/factoextra/issues.svg)](https://github.com/kassambara/factoextra/issues)

# factoextra : Extract and Visualize the Results of Multivariate Data Analyses

[**factoextra**](https://rpkgs.datanovia.com/factoextra/index.html) is
an R package that makes it easy to *extract* and *visualize* the output
of exploratory **multivariate data analyses**, including:

1.  [Principal Component Analysis
    (PCA)](https://www.sthda.com/english/articles/31-principal-component-methods-in-r-practical-guide/112-pca-principal-component-analysis-essentials/),
    which is used to summarize the information contained in continuous
    (i.e., quantitative) multivariate data by reducing the
    dimensionality of the data without losing important information.

2.  [Correspondence Analysis
    (CA)](https://www.sthda.com/english/articles/31-principal-component-methods-in-r-practical-guide/113-ca-correspondence-analysis-in-r-essentials/),
    which is an extension of principal component analysis suited to
    analyzing a large contingency table formed by two *qualitative
    variables* (or categorical data).

3.  [Multiple Correspondence Analysis
    (MCA)](https://www.sthda.com/english/articles/31-principal-component-methods-in-r-practical-guide/114-mca-multiple-correspondence-analysis-in-r-essentials/),
    which is an adaptation of CA to a data table containing more than
    two categorical variables.

4.  [Multiple Factor Analysis
    (MFA)](https://rpkgs.datanovia.com/factoextra/reference/fviz_mfa.html)
    dedicated to datasets where variables are organized into groups
    (qualitative and/or quantitative variables).

5.  [Hierarchical Multiple Factor Analysis
    (HMFA)](https://rpkgs.datanovia.com/factoextra/reference/fviz_hmfa.html):
    An extension of MFA in a situation where the data are organized into
    a hierarchical structure.

6.  [Factor Analysis of Mixed Data
    (FAMD)](https://rpkgs.datanovia.com/factoextra/reference/fviz_famd.html),
    a particular case of the MFA, dedicated to analyze a data set
    containing both quantitative and qualitative variables.

There are a number of R packages implementing principal component
methods. These packages include: *FactoMineR*, *ade4*, *stats*, *ca*,
*MASS* and *ExPosition*.

However, the result is presented differently according to the used
packages. To help in the interpretation and in the visualization of
multivariate analysis - such as cluster analysis and dimensionality
reduction analysis - we developed an easy-to-use R package named
[factoextra](https://rpkgs.datanovia.com/factoextra/index.html).

  - The R package **factoextra** has flexible and easy-to-use methods to
    extract quickly, in a human readable standard data format, the
    analysis results from the different packages mentioned above.  
  - It produces a **ggplot2**-based **elegant data visualization** with
    less typing.
  - It also contains many functions that facilitate clustering analysis
    visualization.

> We’ll use i) the FactoMineR package (Sebastien Le, et al., 2008) to
> compute PCA, (M)CA, FAMD, MFA and HCPC; ii) and the factoextra package
> for extracting and visualizing the results.

The figure below shows methods whose outputs can be visualized using the
factoextra package. The official online documentation is available at:
<https://rpkgs.datanovia.com/factoextra/index.html>.

![factoextra R package](tools/factoextra-r-package.png)

## Why using factoextra?

1.  The *factoextra* R package can handle the results of PCA, CA, MCA,
    MFA, FAMD and HMFA from several packages, for extracting and
    visualizing the most important information contained in your data.

2.  *After PCA, CA, MCA, MFA, FAMD and HMFA, the most important
    row/column elements* can be highlighted using :  

<!-- end list -->

  - their cos2 values corresponding to their quality of representation
    on the factor map
  - their contributions to the definition of the principal dimensions.

<span class="success">If you want to do this, the factoextra package
provides a convenient solution.</span>

3.  *PCA and (M)CA are used sometimes for prediction problems* : one can
    predict the coordinates of new supplementary variables (quantitative
    and qualitative) and supplementary individuals using the information
    provided by the previously performed PCA or (M)CA. This can be done
    easily using the
    [FactoMineR](https://www.sthda.com/english/articles/31-principal-component-methods-in-r-practical-guide/112-pca-principal-component-analysis-essentials/)
    package.

<span class="success">If you want to make predictions with PCA/MCA and
to visualize the position of the supplementary variables/individuals on
the factor map using ggplot2: then factoextra can help you. It’s quick,
write less and do more…</span>

4.  *Several functions from different packages - FactoMineR, ade4,
    ExPosition, stats - are available in R for performing PCA, CA or
    MCA*. However, The components of the output vary from package to
    package.

<span class="success">No matter the package you decided to use,
factoextra can give you a human understandable output.</span>

## Installing FactoMineR

The FactoMineR package can be installed and loaded as follow:

``` r
# Install
install.packages("FactoMineR")

# Load
library("FactoMineR")
```

## Installing and loading factoextra

  - factoextra can be installed from
    [CRAN](https://cran.r-project.org/package=factoextra) as follow:

<!-- end list -->

``` r
install.packages("factoextra")
```

  - Or, install the latest version from
    [Github](https://github.com/kassambara/factoextra)

<!-- end list -->

``` r
if(!require(remotes)) install.packages("remotes")
remotes::install_github("kassambara/factoextra")
```

The current maintenance baseline targets:

  - `R >= 4.1.0`

  - `ggplot2 >= 3.5.2`

  - `ggpubr >= 0.6.3` (CRAN)

  - `FactoMineR >= 2.13`

  - Load factoextra as follow :

<!-- end list -->

``` r
library("factoextra")
```

## Recent maintenance highlights

The current development version includes:

  - helper-level `k = 1` handling for clustering diagnostics such as
    `fviz_nbclust(..., method = "wss")` and hierarchical `eclust()`
    auto-selection
  - silhouette diagnostics now omit the undefined `k = 1` point and keep
    the optimal-k guide aligned with the displayed cluster count
  - stricter validation for scaled clustering data and non-finite
    distance objects
  - clearer MCA `quanti.sup`, `axes`, `ncp`, and `parallel.iter`
    validation paths, including integer-like numeric support for
    `fviz_eig()`
  - refreshed examples and manuals for the updated clustering and
    validation workflows

<!-- end list -->

``` r
data(iris)
iris.scaled <- scale(iris[, -5])
res.pca <- prcomp(iris[, -5], scale = TRUE)

# WSS now computes the k = 1 baseline internally
fviz_nbclust(iris.scaled, hcut, method = "wss", hc_method = "complete")

# Parallel analysis validation is explicit and reproducible
fviz_eig(res.pca, choice = "eigenvalue", parallel = TRUE,
         parallel.iter = 10, parallel.seed = 123)

# FactoMineR MCA quantitative supplementary variables are supported directly
library(FactoMineR)
data(poison)
res.mca <- MCA(poison, quanti.sup = 1:2, graph = FALSE)
get_mca(res.mca, "quanti.sup")

# If installation leaves stale 00LOCK-* directories in your library,
# remove those directories manually before reinstalling.
```

## Main functions in the factoextra package

<span class="warning">See the online documentation
(<https://rpkgs.datanovia.com/factoextra/index.html>) for a complete
list.</span>

### Visualizing dimension reduction analysis outputs

| Functions                         | Description                                                                                                                               |
| --------------------------------- | ----------------------------------------------------------------------------------------------------------------------------------------- |
| *fviz\_eig (or fviz\_eigenvalue)* | Extract and visualize the eigenvalues/variances of dimensions.                                                                            |
| *fviz\_pca*                       | Graph of individuals/variables from the output of *Principal Component Analysis* (PCA).                                                   |
| *fviz\_ca*                        | Graph of column/row variables from the output of *Correspondence Analysis* (CA).                                                          |
| *fviz\_mca*                       | Graph of individuals/variables from the output of *Multiple Correspondence Analysis* (MCA).                                               |
| *fviz\_mfa*                       | Graph of individuals/variables from the output of *Multiple Factor Analysis* (MFA), including supplementary qualitative categories.       |
| *fviz\_famd*                      | Graph of individuals/variables from the output of *Factor Analysis of Mixed Data* (FAMD), including supplementary qualitative categories. |
| *fviz\_hmfa*                      | Graph of individuals/variables from the output of *Hierarchical Multiple Factor Analysis* (HMFA).                                         |
| *fviz\_ellipses*                  | Draw confidence ellipses around the categories.                                                                                           |
| *fviz\_cos2*                      | Visualize the quality of representation of the row/column variable from the results of PCA, CA, MCA functions.                            |
| *fviz\_contrib*                   | Visualize the contributions of row/column elements from the results of PCA, CA, MCA functions.                                            |

### Extracting data from dimension reduction analysis outputs

| Functions          | Description                                                                                                                                                  |
| ------------------ | ------------------------------------------------------------------------------------------------------------------------------------------------------------ |
| *get\_eigenvalue*  | Extract and visualize the eigenvalues/variances of dimensions.                                                                                               |
| *get\_pca*         | Extract all the results (coordinates, squared cosine, contributions) for the active individuals/variables from *Principal Component Analysis* (PCA) outputs. |
| *get\_ca*          | Extract all the results (coordinates, squared cosine, contributions) for the active column/row variables from *Correspondence Analysis* outputs.             |
| *get\_mca*         | Extract results from *Multiple Correspondence Analysis* outputs.                                                                                             |
| *get\_mfa*         | Extract results from *Multiple Factor Analysis* outputs, including supplementary qualitative categories.                                                     |
| *get\_famd*        | Extract results from *Factor Analysis of Mixed Data* outputs, including supplementary qualitative categories.                                                |
| *get\_hmfa*        | Extract results from *Hierarchical Multiple Factor Analysis* outputs.                                                                                        |
| *facto\_summarize* | Subset and summarize the output of factor analyses.                                                                                                          |

### Clustering analysis and visualization

| Functions                                | Description                                                 |
| ---------------------------------------- | ----------------------------------------------------------- |
| *dist*(fviz\_dist, get\_dist)            | Enhanced Distance Matrix Computation and Visualization.     |
| *get\_clust\_tendency*                   | Assessing Clustering Tendency.                              |
| *fviz\_nbclust*(fviz\_gap\_stat)         | Determining and Visualizing the Optimal Number of Clusters. |
| *fviz\_dend*                             | Enhanced Visualization of Dendrogram                        |
| *fviz\_cluster*                          | Visualize Clustering Results                                |
| *fviz\_mclust*                           | Visualize Model-based Clustering Results                    |
| *fviz\_silhouette*                       | Visualize Silhouette Information from Clustering.           |
| *hcut*                                   | Computes Hierarchical Clustering and Cut the Tree           |
| *hkmeans* (hkmeans\_tree, print.hkmeans) | Hierarchical k-means clustering.                            |
| *eclust*                                 | Visual enhancement of clustering analysis                   |

## Dimension reduction and factoextra

As depicted in the figure below, the type of analysis to be performed
depends on the data set formats and structures.

![dimension reduction and
factoextra](tools/multivariate-analysis-factoextra.png)

In this section we start by illustrating classical methods - such as
PCA, CA and MCA - for analyzing a data set containing continuous
variables, contingency table and qualitative variables, respectively.

We continue by discussing advanced methods - such as FAMD, MFA and HMFA
- for analyzing a data set containing a mix of variables (qualitatives &
quantitatives) organized or not into groups.

Finally, we show how to perform hierarchical clustering on principal
components (HCPC), which useful for performing clustering with a data
set containing only qualitative variables or with a mixed data of
qualitative and quantitative variables.

### Principal component analysis

  - Data: *decathlon2* \[in *factoextra* package\]
  - PCA function: *FactoMineR::PCA*()
  - Visualization *factoextra::fviz\_pca*()

<span class="success">Read more about computing and interpreting
principal component analysis at: [**Principal Component Analysis**
(PCA)](https://www.sthda.com/english/articles/31-principal-component-methods-in-r-practical-guide/112-pca-principal-component-analysis-essentials/).</span>

1.  **Loading data**

<!-- end list -->

``` r
library("factoextra")
#> Loading required package: ggplot2
#> Welcome to factoextra!
#> Want to learn more? See two factoextra-related books at https://www.datanovia.com/en/product/practical-guide-to-principal-component-methods-in-r/
data("decathlon2")
df <- decathlon2[1:23, 1:10]
```

2.  **Principal component analysis**

<!-- end list -->

``` r
library("FactoMineR")
res.pca <- PCA(df,  graph = FALSE)
```

3.  **Extract and visualize eigenvalues/variances**:

<!-- end list -->

``` r
# Extract eigenvalues/variances
get_eig(res.pca)
#>        eigenvalue variance.percent cumulative.variance.percent
#> Dim.1   4.1242133        41.242133                    41.24213
#> Dim.2   1.8385309        18.385309                    59.62744
#> Dim.3   1.2391403        12.391403                    72.01885
#> Dim.4   0.8194402         8.194402                    80.21325
#> Dim.5   0.7015528         7.015528                    87.22878
#> Dim.6   0.4228828         4.228828                    91.45760
#> Dim.7   0.3025817         3.025817                    94.48342
#> Dim.8   0.2744700         2.744700                    97.22812
#> Dim.9   0.1552169         1.552169                    98.78029
#> Dim.10  0.1219710         1.219710                   100.00000
# Visualize eigenvalues/variances
fviz_screeplot(res.pca, addlabels = TRUE, ylim = c(0, 50))
```

![](tools/README-pca-eigenvalue-1.png)<!-- -->

4\.**Extract and visualize results for variables**:

``` r
# Extract the results for variables
var <- get_pca_var(res.pca)
var
#> Principal Component Analysis Results for variables
#>  ===================================================
#>   Name       Description                                    
#> 1 "$coord"   "Coordinates for the variables"                
#> 2 "$cor"     "Correlations between variables and dimensions"
#> 3 "$cos2"    "Cos2 for the variables"                       
#> 4 "$contrib" "contributions of the variables"
# Coordinates of variables
head(var$coord)
#>                   Dim.1       Dim.2      Dim.3       Dim.4      Dim.5
#> X100m        -0.8506257 -0.17939806  0.3015564  0.03357320 -0.1944440
#> Long.jump     0.7941806  0.28085695 -0.1905465 -0.11538956  0.2331567
#> Shot.put      0.7339127  0.08540412  0.5175978  0.12846837 -0.2488129
#> High.jump     0.6100840 -0.46521415  0.3300852  0.14455012  0.4027002
#> X400m        -0.7016034  0.29017826  0.2835329  0.43082552  0.1039085
#> X110m.hurdle -0.7641252 -0.02474081  0.4488873 -0.01689589  0.2242200
# Contribution of variables
head(var$contrib)
#>                  Dim.1      Dim.2     Dim.3       Dim.4     Dim.5
#> X100m        17.544293  1.7505098  7.338659  0.13755240  5.389252
#> Long.jump    15.293168  4.2904162  2.930094  1.62485936  7.748815
#> Shot.put     13.060137  0.3967224 21.620432  2.01407269  8.824401
#> High.jump     9.024811 11.7715838  8.792888  2.54987951 23.115504
#> X400m        11.935544  4.5799296  6.487636 22.65090599  1.539012
#> X110m.hurdle 14.157544  0.0332933 16.261261  0.03483735  7.166193
# Graph of variables: default plot
fviz_pca_var(res.pca, col.var = "black")
```

![](tools/README-pca-variables-1.png)<!-- -->

It’s possible to control variable colors using their contributions
(“contrib”) to the principal axes:

``` r
# Control variable colors using their contributions
fviz_pca_var(res.pca, col.var="contrib",
             gradient.cols = c("#00AFBB", "#E7B800", "#FC4E07"),
             repel = TRUE # Avoid text overlapping
             )
```

![](tools/README-pca-variable-colors-by-contributions-1.png)<!-- -->

5.  **Variable contributions to the principal axes**:

<!-- end list -->

``` r
# Contributions of variables to PC1
fviz_contrib(res.pca, choice = "var", axes = 1, top = 10)

# Contributions of variables to PC2
fviz_contrib(res.pca, choice = "var", axes = 2, top = 10)
```

![](tools/README-pca-variable-contributions-1.png)![](tools/README-pca-variable-contributions-2.png)

6.  **Extract and visualize results for individuals**:

<!-- end list -->

``` r
# Extract the results for individuals
ind <- get_pca_ind(res.pca)
ind
#> Principal Component Analysis Results for individuals
#>  ===================================================
#>   Name       Description                       
#> 1 "$coord"   "Coordinates for the individuals" 
#> 2 "$cos2"    "Cos2 for the individuals"        
#> 3 "$contrib" "contributions of the individuals"
# Coordinates of individuals
head(ind$coord)
#>                Dim.1      Dim.2      Dim.3       Dim.4       Dim.5
#> SEBRLE     0.1955047  1.5890567  0.6424912  0.08389652  1.16829387
#> CLAY       0.8078795  2.4748137 -1.3873827  1.29838232 -0.82498206
#> BERNARD   -1.3591340  1.6480950  0.2005584 -1.96409420  0.08419345
#> YURKOV    -0.8889532 -0.4426067  2.5295843  0.71290837  0.40782264
#> ZSIVOCZKY -0.1081216 -2.0688377 -1.3342591 -0.10152796 -0.20145217
#> McMULLEN   0.1212195 -1.0139102 -0.8625170  1.34164291  1.62151286
# Graph of individuals
# 1. Use repel = TRUE to avoid overplotting
# 2. Control automatically the color of individuals using the cos2
    # cos2 = the quality of the individuals on the factor map
    # Use points only
# 3. Use gradient color
fviz_pca_ind(res.pca, col.ind = "cos2", 
             gradient.cols = c("#00AFBB", "#E7B800", "#FC4E07"),
             repel = TRUE # Avoid text overlapping (slow if many points)
             )
```

![](tools/README-principal-component-analysis-data-mining-1.png)<!-- -->

``` r
# Biplot of individuals and variables
fviz_pca_biplot(res.pca, repel = TRUE)
```

![](tools/README-principal-component-analysis-data-mining-2.png)<!-- -->

7.  **Color individuals by groups**:

<!-- end list -->

``` r
# Compute PCA on the iris data set
# The variable Species (index = 5) is removed
# before PCA analysis
iris.pca <- PCA(iris[,-5], graph = FALSE)

# Visualize
# Use habillage to specify groups for coloring
fviz_pca_ind(iris.pca,
             label = "none", # hide individual labels
             habillage = iris$Species, # color by groups
             palette = c("#00AFBB", "#E7B800", "#FC4E07"),
             addEllipses = TRUE # Concentration ellipses
             )
```

![](tools/README-individuals-factor-map-color-by-groups-1.png)<!-- -->

### Correspondence analysis

  - Data: *housetasks* \[in factoextra\]
  - CA function *FactoMineR::CA*()
  - Visualize with *factoextra::fviz\_ca*()

<span class="success">Read more about computing and interpreting
correspondence analysis at: [**Correspondence Analysis**
(CA)](https://www.sthda.com/english/articles/31-principal-component-methods-in-r-practical-guide/113-ca-correspondence-analysis-in-r-essentials/).</span>

  - **Compute CA**:

<!-- end list -->

``` r
 # Loading data
data("housetasks")

 # Computing CA
library("FactoMineR")
res.ca <- CA(housetasks, graph = FALSE)
```

  - **Extract results for row/column variables**:

<!-- end list -->

``` r
# Result for row variables
get_ca_row(res.ca)
  
# Result for column variables
get_ca_col(res.ca)
```

  - **Biplot of rows and columns**

<!-- end list -->

``` r
fviz_ca_biplot(res.ca, repel = TRUE)
```

![](tools/README-correspondence-analysis-biplot-1.png)<!-- -->

To visualize only row points or column points, type this:

``` r
# Graph of row points
fviz_ca_row(res.ca, repel = TRUE)

# Graph of column points
fviz_ca_col(res.ca)

# Visualize row contributions on axes 1
fviz_contrib(res.ca, choice ="row", axes = 1)
     
# Visualize column contributions on axes 1
fviz_contrib(res.ca, choice ="col", axes = 1)
```

### Multiple correspondence analysis

  - Data: **poison** \[in factoextra\]
  - MCA function **FactoMineR::MCA**()
  - Visualization **factoextra::fviz\_mca**()

<span class="success">Read more about computing and interpreting
multiple correspondence analysis at: [**Multiple Correspondence
Analysis**
(MCA)](https://www.sthda.com/english/articles/31-principal-component-methods-in-r-practical-guide/114-mca-multiple-correspondence-analysis-in-r-essentials/).</span>

1.  **Computing MCA**:

<!-- end list -->

``` r
library(FactoMineR)
data(poison)
res.mca <- MCA(poison, quanti.sup = 1:2,
              quali.sup = 3:4, graph=FALSE)
```

2.  **Extract results for variables and individuals**:

<!-- end list -->

``` r
# Extract the results for variable categories
get_mca_var(res.mca)

# Extract the results for individuals
get_mca_ind(res.mca)
```

3.  **Contribution of variables and individuals to the principal axes**:

<!-- end list -->

``` r
# Visualize variable category contributions on axes 1
fviz_contrib(res.mca, choice ="var", axes = 1)

# Visualize individual contributions on axes 1
# select the top 20
fviz_contrib(res.mca, choice ="ind", axes = 1, top = 20)
```

4.  **Graph of individuals**

<!-- end list -->

``` r
# Color by groups
# Add concentration ellipses
# Use repel = TRUE to avoid overplotting
grp <- as.factor(poison[, "Vomiting"])
fviz_mca_ind(res.mca,  habillage = grp,
             addEllipses = TRUE, repel = TRUE)
```

![](tools/README-mca-graph-of-individuals-1.png)<!-- -->

5.  **Graph of variable categories**:

<!-- end list -->

``` r
fviz_mca_var(res.mca, repel = TRUE)
```

![](tools/README-mca-graph-variables-1.png)<!-- -->

6.  **Biplot of individuals and variables**:

<!-- end list -->

``` r
fviz_mca_biplot(res.mca, repel = TRUE)
```

![](tools/README-mca-biplot-1.png)<!-- -->

### Advanced methods

The factoextra R package has also functions that support the
visualization of advanced methods such:

  - Factor Analysis of Mixed Data (FAMD): : [FAMD
    Examples](https://rpkgs.datanovia.com/factoextra/reference/fviz_famd.html)
  - Multiple Factor Analysis (MFA): [MFA
    Examples](https://rpkgs.datanovia.com/factoextra/reference/fviz_mfa.html)
  - Hierarchical Multiple Factor Analysis (HMFA): [HMFA
    Examples](https://rpkgs.datanovia.com/factoextra/reference/fviz_hmfa.html)
  - [Hierachical Clustering on Principal Components
    (HCPC)](https://www.sthda.com/english/articles/31-principal-component-methods-in-r-practical-guide/117-hcpc-hierarchical-clustering-on-principal-components-essentials/)

## Cluster analysis and factoextra

To learn more about cluster analysis, you can refer to the book
available at: [Practical Guide to Cluster Analysis in
R](https://www.datanovia.com/en/product/practical-guide-to-cluster-analysis-in-r/)

<a href = "https://www.datanovia.com/en/product/practical-guide-to-cluster-analysis-in-r/"><img src = "tools/clustering-e1-cover.png" alt = "clustering book cover"/></a>

The main parts of the book include:

  - *distance measures*,
  - *partitioning clustering*,
  - *hierarchical clustering*,
  - *cluster validation methods*, as well as,
  - *advanced clustering methods* such as fuzzy clustering,
    density-based clustering and model-based clustering.

The book presents the basic principles of these tasks and provide many
examples in R. It offers solid guidance in data mining for students and
researchers.

### Partitioning clustering

![Partitioning cluster analysis](tools/partitioning-clustering.png)

``` r
# 1. Loading and preparing data
data("USArrests")
df <- scale(USArrests)

# 2. Compute k-means
set.seed(123)
km.res <- kmeans(scale(USArrests), 4, nstart = 25)

# 3. Visualize
library("factoextra")
fviz_cluster(km.res, data = df,
             palette = c("#00AFBB","#2E9FDF", "#E7B800", "#FC4E07"),
             ggtheme = theme_minimal(),
             main = "Partitioning Clustering Plot"
             )
```

![](tools/README-partitioning-clustering-1.png)<!-- -->

<br/>  

<div class="success">

Read more:

1.  [Cluster analysis in R: All what you should
    know](https://www.datanovia.com/en/blog/types-of-clustering-methods-overview-and-quick-start-r-code/).

2.  [Partitioning cluster
    analysis](https://www.datanovia.com/en/courses/partitional-clustering-in-r-the-essentials/).

</div>

<br/>

### Hierarchical clustering

``` r
library("factoextra")
# Compute hierarchical clustering and cut into 4 clusters
res <- hcut(USArrests, k = 4, stand = TRUE)

# Visualize
fviz_dend(res, rect = TRUE, cex = 0.5,
          k_colors = c("#00AFBB","#2E9FDF", "#E7B800", "#FC4E07"))
```

![](tools/README-hierarchical-clustering-1.png)<!-- -->

``` r
if (requireNamespace("igraph", quietly = TRUE)) {
  # The default compatibility alias "layout.auto" still works, and
  # modern igraph layout names are accepted as well.
  fviz_dend(res, type = "phylogenic", phylo_layout = "layout_nicely",
            show_labels = FALSE)
}
```

![](tools/README-hierarchical-clustering-phylogenic-1.png)<!-- -->

<br/>  

<div class="success">

Read more:

1.  [Cluster analysis in R: All what you should
    know](https://www.datanovia.com/en/blog/types-of-clustering-methods-overview-and-quick-start-r-code/)

2.  [Hierarchical clustering
    essentials](https://www.datanovia.com/en/courses/hierarchical-clustering-in-r-the-essentials/)

</div>

<br/>

### Determine the optimal number of clusters

``` r
# Optimal number of clusters for k-means
library("factoextra")
my_data <- scale(USArrests)
fviz_nbclust(my_data, kmeans, method = "gap_stat")
```

![](tools/README-determine-the-number-of-clusters-gap-statistics-1.png)<!-- -->

## Acknowledgment

Original package authors:

  - [Alboukadel Kassambara](https://github.com/kassambara)
  - [Fabian Mundt](https://github.com/inventionate)

Contributor:

  - Laszlo Erdey (Faculty of Economics and Business, University of
    Debrecen, Hungary)

We sincerely thank all developers for their efforts behind the packages
that **factoextra** depends on, namely,
[ggplot2](https://cran.r-project.org/package=ggplot2) (Hadley Wickham,
Springer-Verlag New York, 2009),
[FactoMineR](https://cran.r-project.org/package=FactoMineR) (Sebastien
Le et al., Journal of Statistical Software, 2008),
[dendextend](https://cran.r-project.org/package=dendextend) (Tal Galili,
Bioinformatics, 2015),
[cluster](https://cran.r-project.org/package=dendextend) (Martin
Maechler et al., 2016) and more …..

## References

  - H. Wickham (2009). ggplot2: Elegant Graphics for Data Analysis.
    Springer-Verlag New York.
  - Maechler, M., Rousseeuw, P., Struyf, A., Hubert, M., Hornik,
    K.(2016). cluster: Cluster Analysis Basics and Extensions. R package
    version 2.0.5.
  - Sebastien Le, Julie Josse, Francois Husson (2008). FactoMineR: An R
    Package for Multivariate Analysis. Journal of Statistical Software,
    25(1), 1-18. 10.18637/jss.v025.i01
  - Tal Galili (2015). dendextend: an R package for visualizing,
    adjusting, and comparing trees of hierarchical clustering.
    Bioinformatics. DOI: 10.1093/bioinformatics/btv428

#' @include print.factoextra.R
NULL
#' Extract the results for individuals/variables - MCA
#' 
#' @description Extract all the results (coordinates, squared cosine and
#'   contributions) for the active individuals/variable categories from
#'   Multiple Correspondence Analysis (MCA) outputs.\cr\cr \itemize{ \item
#'   get_mca(): Extract the results for variables and individuals \item
#'   get_mca_ind(): Extract the results for individuals only \item
#'   get_mca_var(): Extract the results for variables only }
#'
#'   For FactoMineR MCA results, \code{get_mca()} and \code{get_mca_var()} also
#'   support \code{element = "quanti.sup"} for quantitative supplementary
#'   variables and report a clean package-level error when that result is
#'   absent.
#' @param res.mca an object of class MCA [FactoMineR], acm [ade4], expoOutput/epMCA [ExPosition].
#' @param element the element to subset from the output. Possible values are
#'   "var" for variables, "ind" for individuals, "mca.cor" for correlation
#'   between variables and principal dimensions, and "quanti.sup" for
#'   quantitative supplementary variables in FactoMineR MCA results.
#' @return a list of matrices containing the results for the active 
#'   individuals/variable categories including : \item{coord}{coordinates for 
#'   the individuals/variable categories} \item{cos2}{cos2 for the 
#'   individuals/variable categories} \item{contrib}{contributions of the 
#'   individuals/variable categories} \item{inertia}{inertia of the 
#'   individuals/variable categories}
#' @author Alboukadel Kassambara \email{alboukadel.kassambara@@gmail.com}
#' @references \url{https://www.sthda.com/english/}
#' @examples
#' \donttest{
#' # Multiple Correspondence Analysis
#' # ++++++++++++++++++++++++++++++
#' # Install and load FactoMineR to compute MCA
#' # install.packages("FactoMineR")
#' library("FactoMineR")
#' data(poison)
#' res.mca <- MCA(poison, quanti.sup = 1:2, graph = FALSE)
#'  
#'  # Extract the results for variable categories
#'  var <- get_mca_var(res.mca)
#'  print(var)
#'  head(var$coord) # coordinates of variables
#'  head(var$cos2) # cos2 of variables
#'  head(var$contrib) # contributions of variables
#'  
#'  # Extract the results for individuals
#'  ind <- get_mca_ind(res.mca)
#'  print(ind)
#'  head(ind$coord) # coordinates of individuals
#'  head(ind$cos2) # cos2 of individuals
#'  head(ind$contrib) # contributions of individuals
#'  
#'  # You can also use the function get_mca()
#'  get_mca(res.mca, "ind") # Results for individuals
#'  get_mca(res.mca, "var") # Results for variable categories
#'  quanti.sup <- get_mca(res.mca, "quanti.sup")
#'  head(quanti.sup$coord) # coordinates of quantitative supplementary variables
#'  }
#'  
#' @name get_mca
#'   
#' @rdname get_mca
#' @export
get_mca <- function(res.mca, element = c("var", "ind", "mca.cor", "quanti.sup")){
 elmt <- match.arg(element)
 if(elmt == "ind") get_mca_ind(res.mca)
 else get_mca_var(res.mca, elmt)
}


#' @rdname get_mca
#' @export
get_mca_var <- function(res.mca, element = c( "var", "mca.cor", "quanti.sup")){
  choice <- match.arg(element)
  element_desc <- "variables"
  # FactoMineR package
  if(inherits(res.mca, c("MCA"))) {
    if(choice == "quanti.sup" && is.null(res.mca$quanti.sup))
      stop("There are no quantitative supplementary variables in this MCA.")
    vars <- switch(choice,
                   var = res.mca$var,
                   mca.cor = list(coord = res.mca$var$eta2),
                   quanti.sup = res.mca$quanti.sup
    )
    element_desc <- switch(choice,
                           var = "variables",
                           mca.cor = "correlation between variables and dimensions",
                           quanti.sup = "quantitative supplementary variables"
    )
  }
  # ade4 package
  else if(inherits(res.mca, "acm") && inherits(res.mca, "dudi")){
    if (!requireNamespace("ade4", quietly = TRUE)) {
      stop("ade4 package needed for this function to work. Please install it.")
    }
    if(choice == "var"){
    coord <- res.mca$co
    inertia <- ade4::inertia.dudi(res.mca, row.inertia = FALSE, col.inertia = TRUE)
    vv <- as.character(utils::packageVersion("ade4"))
    cc <- utils::compareVersion(vv, "1.7.4") > 0
    if(cc){
      # "v>1.7.4"
      cos2 <- abs(inertia$col.rel/100)[, seq_len(ncol(coord))]
      contrib <- (inertia$col.abs)[, seq_len(ncol(coord))]
    }
    # v<=1.7.4
    else {
      cos2 <- abs(inertia$col.rel/10000)[, seq_len(ncol(coord))]
      contrib <- (inertia$col.abs/100)[, seq_len(ncol(coord))]
    }
    colnames(coord) <- colnames(cos2) <- colnames(contrib) <- paste0("Dim.", seq_len(ncol(coord))) 
    vars <- list(coord = coord, contrib = contrib, cos2 = cos2)
    }
    else{
      vars <- list()
      stop("Don't handle ", choice, " for MCA computed with ade4. Use FactoMineR instead.")
    }
  }
  # ExPosition package
  else if (inherits(res.mca, "expoOutput") && inherits(res.mca$ExPosition.Data, "epMCA")){
    if(choice != "var")  stop("Don't handle ", choice, " for MCA computed with ExPosition.",
                              " Use FactoMineR instead.")
    res <- res.mca$ExPosition.Data
    coord <- res$fj
    inertia <- res$dj*res$W
    cos2 <- res$rj
    contrib <- res$cj*100
    colnames(coord) <- colnames(cos2) <- colnames(contrib) <- paste0("Dim.", seq_len(ncol(coord))) 
    vars <- list(coord = coord, contrib = contrib, cos2 = cos2)
  }
  else stop("An object of class : ", paste(class(res.mca), collapse = ", "), 
            " can't be handled by the function get_mca_var()")
  element_class <-switch(choice,
                         var = "mca_var", mca.cor = "mca_cor", quanti.sup = "quanti.sup")
  class(vars)<-c("factoextra",  "mca", element_class)
  attr(vars, "element") <- element_desc
  return(vars)
}

#' @rdname get_mca
#' @export
get_mca_ind <- function(res.mca){
  # FactoMineR package
  if(inherits(res.mca, c("MCA"))) ind <- res.mca$ind
  # ade4 package
  else if(inherits(res.mca, "acm") && inherits(res.mca, "dudi")){
    if (!requireNamespace("ade4", quietly = TRUE)) {
      stop("ade4 package needed for this function to work. Please install it.")
    }
    coord <- res.mca$li
    inertia <- ade4::inertia.dudi(res.mca, row.inertia = TRUE, col.inertia = FALSE)
    vv <- as.character(utils::packageVersion("ade4"))
    cc <- utils::compareVersion(vv, "1.7.4") > 0
    if(cc){
      # "v>1.7.4"
      cos2 <- abs(inertia$row.rel/100)[, seq_len(ncol(coord))]
      contrib <- (inertia$row.abs)[, seq_len(ncol(coord))]
    }
    # v<=1.7.4
    else {
      cos2 <- abs(inertia$row.rel/10000)[, seq_len(ncol(coord))]
      contrib <- (inertia$row.abs/100)[, seq_len(ncol(coord))]
    }
    colnames(coord) <- colnames(cos2) <- colnames(contrib) <- paste0("Dim.", seq_len(ncol(coord))) 
    ind <- list(coord = coord, contrib = contrib, cos2 = cos2)
  }
  # ExPosition package
  else if (inherits(res.mca, "expoOutput") && inherits(res.mca$ExPosition.Data, "epMCA")){
    res <- res.mca$ExPosition.Data
    coord <- res$fi
    inertia <- res$di*res$M
    cos2 <- res$ri
    contrib <- res$ci*100
    colnames(coord) <- colnames(cos2) <- colnames(contrib) <- paste0("Dim.", seq_len(ncol(coord))) 
    ind <- list(coord = coord, contrib = contrib, cos2 = cos2)
  }
  
  else stop("An object of class : ", paste(class(res.mca), collapse = ", "), 
            " can't be handled by the function get_mca_ind()")
  class(ind)<-c("factoextra", "mca", "mca_ind")
  attr(ind, "element") <- "individuals"
  return(ind)
}

#' @include print.factoextra.R utilities.R
NULL
#' Extract the results for individuals/variables - PCA
#' 
#' @description
#' Extract all the results (coordinates, squared cosine, contributions) for 
#' the active individuals/variables from Principal Component Analysis (PCA) outputs.\cr\cr
#' \itemize{
#' \item get_pca(): Extract the results for variables and individuals
#' \item get_pca_ind(): Extract the results for individuals only
#' \item get_pca_var(): Extract the results for variables only
#' }
#' @param res.pca an object of class PCA [FactoMineR]; 
#' prcomp and princomp [stats]; pca, dudi [adea4]; epPCA [ExPosition].
#' @param element the element to subset from the output. Allowed values are 
#' "var" (for active variables) or "ind" (for active individuals).
#' @param ... not used
#' @return a list of matrices containing all the results for the active individuals/variables including: 
#' \item{coord}{coordinates for the individuals/variables}
#' \item{cos2}{cos2 for the individuals/variables}
#' \item{contrib}{contributions of the individuals/variables}
#' @author Alboukadel Kassambara \email{alboukadel.kassambara@@gmail.com}
#' @references \url{https://www.sthda.com/english/}
#' @examples
#' \donttest{
#' # Principal Component Analysis
#' # +++++++++++++++++++++++++++++
#'  data(iris)
#'  res.pca <- prcomp(iris[, -5],  scale = TRUE)
#'  # Extract the results for individuals
#'  ind <- get_pca_ind(res.pca)
#'  print(ind)
#'  head(ind$coord) # coordinates of individuals
#'  head(ind$cos2) # cos2 of individuals
#'  head(ind$contrib) # contributions of individuals
#'  
#'  # Extract the results for variables
#'  var <- get_pca_var(res.pca)
#'  print(var)
#'  head(var$coord) # coordinates of variables
#'  head(var$cos2) # cos2 of variables
#'  head(var$contrib) # contributions of variables
#'  
#'  # You can also use the function get_pca()
#'  get_pca(res.pca, "ind") # Results for individuals
#'  get_pca(res.pca, "var") # Results for variable categories
#'  }
#' @name get_pca
#' 
#' @rdname get_pca
#' @export 
get_pca <- function(res.pca, element = c("var", "ind")){
  elmt <- match.arg(element)
  if(elmt =="var") get_pca_var(res.pca)
  else if(elmt == "ind") get_pca_ind(res.pca)
}

#' @rdname get_pca
#' @export
get_pca_ind<-function(res.pca, ...){
  
  # FactoMineR package
  if(inherits(res.pca, c('PCA'))) ind <- res.pca$ind

  # user-supplied coordinates wrapped by as_factoextra_pca()
  else if(inherits(res.pca, "factoextra_pca")) ind <- res.pca$ind

  # ade4 package
  else if(inherits(res.pca, "pca") && inherits(res.pca, "dudi")){
    ind.coord <- res.pca$li
    # OPTIMIZED: get the original data using vectorized sweep() instead of apply()
    # sweep() is much faster than t(apply()) for element-wise row operations
    data <- res.pca$tab
    data <- sweep(data, 2, res.pca$norm, "*")
    data <- sweep(data, 2, res.pca$cent, "+")
    ind <- .get_pca_ind_results(ind.coord, data, res.pca$eig,
                                res.pca$cent, res.pca$norm)
  }

  # ade4 between-class / within-class PCA (bca/wca). These dudi objects carry
  # row coordinates ($li), the transformed table ($tab) and row/column weights
  # ($lw/$cw) but not $cent/$norm, so cos2/contrib use the weighted-dudi formula.
  else if(inherits(res.pca, c("between", "within")) && inherits(res.pca, "dudi")){
    ind <- .get_dudi_class_ind_results(res.pca)
  }

  # stats package
  else if(inherits(res.pca, 'princomp')){
    ind.coord <- res.pca$scores
    data <- .prcomp_reconst(res.pca)
    ind <- .get_pca_ind_results(ind.coord, data, res.pca$sdev^2,
                                res.pca$center, res.pca$scale)
    
  }
  else if(inherits(res.pca, 'prcomp')){
    ind.coord <- res.pca$x
    data <- .prcomp_reconst(res.pca)
    ind <- .get_pca_ind_results(ind.coord, data, res.pca$sdev^2,
                                res.pca$center, res.pca$scale)
  }
  # ExPosition package
  else if (inherits(res.pca, "expoOutput") && inherits(res.pca$ExPosition.Data, "epPCA")){
    res <- res.pca$ExPosition.Data
    ind <- list(coord = res$fi,  cos2 = res$ri, contrib = res$ci*100)
  }
  else stop("An object of class : ", paste(class(res.pca), collapse = ", "), 
            " can't be handled by the function get_pca_ind()")
  
  class(ind)<-c("factoextra", "pca_ind")
  
  ind
}


#' @rdname get_pca
#' @export 
get_pca_var<-function(res.pca){
  # FactoMineR package
  if(inherits(res.pca, c('PCA'))) var <- res.pca$var
  # user-supplied coordinates wrapped by as_factoextra_pca()
  else if(inherits(res.pca, "factoextra_pca")){
    if(is.null(res.pca$var))
      stop("This object has no variable coordinates; supply `var.coord` to ",
           "as_factoextra_pca() to use fviz_pca_var()/fviz_pca_biplot().",
           call. = FALSE)
    var <- res.pca$var
  }
  # ade4 package (incl. between-class/within-class PCA: bca/wca). Variable/column
  # coordinates live in $co for all dudi flavours.
  else if(inherits(res.pca, "dudi") &&
          inherits(res.pca, c("pca", "between", "within"))){
    var <- .get_pca_var_results(res.pca$co)
  }
  # stats package
  else if(inherits(res.pca, 'princomp')){
    # OPTIMIZED: Correlation of variables with the principal component
    # Using sweep() instead of t(apply()) - much faster for element-wise operations
    # var.cor[i,j] = loadings[i,j] * sdev[j]
    # unclass() strips the base R "loadings" S3 class so coord/cos2/contrib are
    # returned as plain numeric matrices. Otherwise print.loadings() hides values
    # with |x| < 0.1 (its cutoff) and the result breaks downstream manipulation.
    var.cor <- sweep(unclass(res.pca$loadings), 2, res.pca$sdev, "*")
    var <- .get_pca_var_results(var.cor)
  }
  else if(inherits(res.pca, 'prcomp')){
    # OPTIMIZED: Correlation of variables with the principal component
    # Using sweep() instead of t(apply()) - much faster for element-wise operations
    # var.cor[i,j] = rotation[i,j] * sdev[j]
    var.cor <- sweep(res.pca$rotation, 2, res.pca$sdev, "*")
    var <- .get_pca_var_results(var.cor)
  }
  # ExPosition package
  else if (inherits(res.pca, "expoOutput") && inherits(res.pca$ExPosition.Data, "epPCA")){
    res <- res.pca$ExPosition.Data
    data_matrix <- res$X
    factor_scores <- res$fi
    var.coord <- var.cor <- stats::cor(res$X, res$fi) # cor(t(data_matrix), factor_scores)
    var.coord <- replace(var.coord, is.na(var.coord), 0)
    var <- list(coord = var.coord, cor = var.coord, cos2 = res$rj, contrib = res$cj*100)
  }
  else stop("An object of class : ", paste(class(res.pca), collapse = ", "), 
            " can't be handled by the function get_pca_var()")
  class(var)<-c("factoextra", "pca_var")
  var
}




# Helper functions
#++++++++++++++++++++

# compute all the results for individuals : coord, cor, cos2, contrib
# ind.coord : coordinates of variables on the principal component
# pca.center, pca.scale : numeric vectors corresponding to the pca
# center and scale respectively
# data : the orignal data used during the pca analysis
# eigenvalues : principal component eigenvalues
#
# OPTIMIZATION: Replaced apply() loops with vectorized matrix operations
# - Uses sweep() and rowSums() instead of row-wise apply()
# - Avoids repeated function calls and memory allocations
# - Maintains full econometric precision (identical numerical results)
.get_pca_ind_results <- function(ind.coord, data, eigenvalues, pca.center, pca.scale){

  # Coerce to plain matrices: ade4 dudi objects store $li/$tab as data.frames,
  # and assigning a data.frame into a matrix slice below (ind.cos2[pos, ] <- ...)
  # silently turns the matrix into a list, losing its dimensions. stats prcomp/
  # princomp already pass matrices, so this is a no-op for them.
  ind.coord <- as.matrix(ind.coord)
  data <- as.matrix(data)

  eigenvalues <- eigenvalues[seq_len(ncol(ind.coord))]
  n.ind <- nrow(ind.coord)
  n.dim <- ncol(ind.coord)

  # Handle centering/scaling defaults
  if(isFALSE(pca.center[1])) pca.center <- rep(0, ncol(data))
  if(isFALSE(pca.scale[1])) pca.scale <- rep(1, ncol(data))

  # OPTIMIZED: Compute squared distances using vectorized operations
  # d2[i] = sum(((data[i,] - center) / scale)^2)
  # Using sweep() for center/scale and rowSums() for summation
  data_centered <- sweep(data, 2, pca.center, "-")
  data_scaled <- sweep(data_centered, 2, pca.scale, "/")
  d2 <- rowSums(data_scaled^2)

  # OPTIMIZED: Compute cos2 using vectorized division

  # cos2[i,j] = coord[i,j]^2 / d2[i]
  ind.coord.sq <- ind.coord^2
  ind.cos2 <- matrix(0, nrow = n.ind, ncol = n.dim)
  positive_d2 <- d2 > .Machine$double.eps
  if(any(positive_d2)) {
    ind.cos2[positive_d2, ] <- ind.coord.sq[positive_d2, , drop = FALSE] / d2[positive_d2]
  }

  # OPTIMIZED: Compute contributions using vectorized operations
  # contrib[i,j] = 100 * (1/n) * (coord[i,j]^2 / eigenvalue[j])
  # Use sweep to divide each column by its eigenvalue
  ind.contrib <- sweep(ind.coord.sq, 2, eigenvalues, "/") * (100 / n.ind)

  # Set column and row names
  dim.names <- paste0("Dim.", 1:n.dim)
  colnames(ind.coord) <- colnames(ind.cos2) <- colnames(ind.contrib) <- dim.names

  rnames <- rownames(ind.coord)
  if(is.null(rnames)) rnames <- as.character(1:n.ind)
  rownames(ind.coord) <- rownames(ind.cos2) <- rownames(ind.contrib) <- rnames

  # Return results
  list(coord = ind.coord, cos2 = ind.cos2, contrib = ind.contrib)
}

# compute all the results for variables : coord, cor, cos2, contrib
# var.coord : coordinates of variables on the principal component
#
# OPTIMIZATION: Replaced apply() with colSums() and sweep()
# - colSums() is faster than apply(x, 2, sum)
# - sweep() is faster than t(apply()) for column-wise operations
# - Maintains full econometric precision
.get_pca_var_results <- function(var.coord){

  # Preserve row names from input
  rnames <- rownames(var.coord)

  var.cor <- var.coord # correlation
  var.cos2 <- var.cor^2 # variable qualities

  # OPTIMIZED: variable contributions (in percent)
  # var.cos2*100/total Cos2 of the component
  # Using colSums instead of apply(, 2, sum) - much faster

  comp.cos2 <- colSums(var.cos2)

  # OPTIMIZED: Using sweep instead of t(apply()) for column-wise division
  # contrib[i,j] = var.cos2[i,j] * 100 / comp.cos2[j]
  var.contrib <- sweep(var.cos2, 2, comp.cos2, "/") * 100

  # Set column names
  dim.names <- paste0("Dim.", seq_len(ncol(var.coord)))
  colnames(var.coord) <- colnames(var.cor) <- colnames(var.cos2) <-
    colnames(var.contrib) <- dim.names

  # Preserve row names
  if(!is.null(rnames)) {
    rownames(var.coord) <- rownames(var.cor) <- rownames(var.cos2) <-
      rownames(var.contrib) <- rnames
  }

  # Variable coord, cor, cos2 and contrib
  list(coord = var.coord, cor = var.cor, cos2 = var.cos2, contrib = var.contrib)
}

# Individual results for ade4 between-class / within-class PCA (bca/wca).
# These dudi objects expose row coordinates ($li), the transformed table ($tab)
# and row/column weights ($lw/$cw) but no $cent/$norm, so we use the general
# weighted-dudi definitions:
#   d2[i]        = sum_j cw[j] * tab[i, j]^2      (squared distance, column metric)
#   cos2[i, k]   = li[i, k]^2 / d2[i]
#   contrib[i,k] = 100 * lw[i] * li[i, k]^2 / eig[k]
# For ordinary dudi.pca (uniform lw = 1/n, cw = 1) these reduce exactly to the
# values produced by .get_pca_ind_results(); contrib matches ade4::inertia.dudi.
.get_dudi_class_ind_results <- function(res.pca){
  ind.coord <- as.matrix(res.pca$li)
  tab <- as.matrix(res.pca$tab)
  cw  <- res.pca$cw
  lw  <- res.pca$lw
  n.dim <- ncol(ind.coord)
  eigenvalues <- res.pca$eig[seq_len(n.dim)]

  d2 <- rowSums(sweep(tab^2, 2, cw, "*"))
  coord.sq <- ind.coord^2

  ind.cos2 <- matrix(0, nrow = nrow(ind.coord), ncol = n.dim)
  positive_d2 <- d2 > .Machine$double.eps
  if(any(positive_d2))
    ind.cos2[positive_d2, ] <- coord.sq[positive_d2, , drop = FALSE] / d2[positive_d2]

  ind.contrib <- sweep(coord.sq, 2, eigenvalues, "/")
  ind.contrib <- sweep(ind.contrib, 1, lw, "*") * 100

  dim.names <- paste0("Dim.", seq_len(n.dim))
  colnames(ind.coord) <- colnames(ind.cos2) <- colnames(ind.contrib) <- dim.names
  rnames <- rownames(ind.coord)
  if(is.null(rnames)) rnames <- as.character(seq_len(nrow(ind.coord)))
  rownames(ind.coord) <- rownames(ind.cos2) <- rownames(ind.contrib) <- rnames

  list(coord = ind.coord, cos2 = ind.cos2, contrib = ind.contrib)
}

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  # All chunks are illustrative: they document the extension contract rather
  # than execute backend code, so the vignette builds without any optional
  # backend package (ExPosition, ade4, vegan, ...) installed.
  eval = FALSE
)

## ----quick-start-prcomp-------------------------------------------------------
# library(factoextra)
# 
# # A base-R PCA (any analysis that yields coordinates would do)
# pca <- prcomp(iris[, -5], scale. = TRUE)
# 
# # Wrap the coordinates into a factoextra-ready object
# res <- as_factoextra_pca(
#   ind.coord = pca$x,         # individual scores  (n x k)
#   var.coord = pca$rotation,  # variable loadings  (p x k)
#   eig       = pca$sdev^2     # eigenvalues        (optional)
# )
# 
# # Now the full fviz_pca_* family works on `res`
# fviz_pca_biplot(res, label = "var", habillage = iris$Species,
#                 addEllipses = TRUE)
# fviz_eig(res)
# fviz_contrib(res, choice = "var", axes = 1)

## ----quick-start-mds----------------------------------------------------------
# mds <- stats::cmdscale(dist(scale(mtcars)), k = 3)   # classical MDS
# fviz_pca_ind(as_factoextra_pca(ind.coord = mds), repel = TRUE)

## -----------------------------------------------------------------------------
# # inside .get_facto_class()
# else if (inherits(X, "myPCA"))            # your new class
#   facto_class <- "PCA"

## -----------------------------------------------------------------------------
# else if (inherits(X, "expoOutput")) {
#   if      (inherits(X$ExPosition.Data, "epCA"))  facto_class <- "CA"
#   else if (inherits(X$ExPosition.Data, "epPCA")) facto_class <- "PCA"
#   else if (inherits(X$ExPosition.Data, "epMCA")) facto_class <- "MCA"
# }

## -----------------------------------------------------------------------------
# # inside get_eig(), before the final stop()
# else if (inherits(X, "myPCA"))
#   eig <- X$eigenvalues          # a numeric vector of eigenvalues

## -----------------------------------------------------------------------------
# else if (inherits(X, "expoOutput"))
#   eig <- X$ExPosition.Data$eigs

## -----------------------------------------------------------------------------
# # get_pca_ind(): stats::prcomp template
# else if (inherits(res.pca, "prcomp")) {
#   ind.coord <- res.pca$x
#   data      <- .prcomp_reconst(res.pca)
#   ind <- .get_pca_ind_results(ind.coord, data, res.pca$sdev^2,
#                               res.pca$center, res.pca$scale)
# }
# 
# # get_pca_ind(): ade4 dudi template
# else if (inherits(res.pca, "pca") && inherits(res.pca, "dudi")) {
#   ind.coord <- res.pca$li
#   data <- sweep(res.pca$tab, 2, res.pca$norm, "*")
#   data <- sweep(data,        2, res.pca$cent, "+")
#   ind <- .get_pca_ind_results(ind.coord, data, res.pca$eig,
#                               res.pca$cent, res.pca$norm)
# }

## -----------------------------------------------------------------------------
# # get_pca_var(): stats::prcomp template
# else if (inherits(res.pca, "prcomp")) {
#   var.cor <- sweep(res.pca$rotation, 2, res.pca$sdev, "*")
#   var <- .get_pca_var_results(var.cor)
# }
# 
# # get_pca_var(): ade4 dudi template
# else if (inherits(res.pca, "pca") && inherits(res.pca, "dudi")) {
#   var <- .get_pca_var_results(res.pca$co)
# }

## -----------------------------------------------------------------------------
# # get_ca_row(): build the standardized list directly
# else if (inherits(res.ca, "myCA")) {
#   coord   <- res.ca$row.coord
#   cos2    <- res.ca$row.cos2
#   contrib <- res.ca$row.contrib * 100
#   inertia <- res.ca$row.inertia
#   colnames(coord) <- colnames(cos2) <- colnames(contrib) <-
#     paste0("Dim.", seq_len(ncol(coord)))
#   row <- list(coord = coord, cos2 = cos2, contrib = contrib, inertia = inertia)
# }

## -----------------------------------------------------------------------------
# # inside .get_ca_mass()
# else if (inherits(res.ca, "myCA")) {
#   if      (element == "row") mass <- res.ca$row.mass
#   else if (element == "col") mass <- res.ca$col.mass
# }

## -----------------------------------------------------------------------------
# # R/get_pca.R — ExPosition epPCA, individuals
# else if (inherits(res.pca, "expoOutput") &&
#          inherits(res.pca$ExPosition.Data, "epPCA")) {
#   res <- res.pca$ExPosition.Data
#   ind <- list(coord = res$fi, cos2 = res$ri, contrib = res$ci * 100)
# }

## -----------------------------------------------------------------------------
# library(ExPosition)
# library(factoextra)
# 
# res.pca <- epPCA(iris[, -5], graph = FALSE)
# fviz_pca_ind(res.pca, habillage = iris$Species, addEllipses = TRUE)
# fviz_eig(res.pca)
# fviz_contrib(res.pca, choice = "var")

## -----------------------------------------------------------------------------
# # .get_facto_class(): treat an unconstrained CA component as CA
# else if (inherits(X, c("cca", "rda")))
#   facto_class <- "CA"
# 
# # get_eig(): vegan stores eigenvalues per axis
# else if (inherits(X, c("cca", "rda")))
#   eig <- vegan::eigenvals(X)
# 
# # get_ca_row()/get_ca_col(): use vegan::scores() for sites / species
# else if (inherits(res.ca, c("cca", "rda"))) {
#   sc      <- vegan::scores(res.ca, display = "sites", scaling = "sites")
#   coord   <- as.matrix(sc)
#   colnames(coord) <- paste0("Dim.", seq_len(ncol(coord)))
#   # cos2 / contrib / inertia derived from the ordination as appropriate
#   row <- list(coord = coord, cos2 = cos2, contrib = contrib, inertia = inertia)
# }

## -----------------------------------------------------------------------------
# test_that("CA extractors work with <your package> outputs", {
#   skip_if_not_installed("yourpkg")
#   data("housetasks", package = "factoextra")
#   res <- yourpkg::your_ca(housetasks)
# 
#   cols <- get_ca_col(res)
#   rows <- get_ca_row(res)
# 
#   expect_s3_class(cols, "factoextra")
#   expect_true(all(c("coord", "cos2", "contrib") %in% names(cols)))
#   expect_equal(nrow(cols$coord), ncol(housetasks))
#   expect_equal(nrow(rows$coord), nrow(housetasks))
# })

#' @include utilities.R cluster_utilities.R dist.R fviz_cluster.R fviz_dend.R
NULL
#' Visual enhancement of clustering analysis
#' 
#' @description Provides a convenient workflow for clustering analyses and
#'   ggplot2-based data visualization. When \code{k = NULL}, the gap statistic
#'   selects the number of clusters. Hierarchical backends may validly return
#'   \code{k = 1}; in that case \code{eclust()} returns a one-cluster result
#'   without silhouette information. Read more:
#'  \href{https://www.datanovia.com/en/blog/cluster-analysis-in-r-simplified-and-enhanced/}{Visual enhancement of clustering analysis}.
#' @param x numeric vector, data matrix or data frame. For hierarchical
#'   clustering (\code{FUNcluster} = "hclust", "agnes" or "diana"), a
#'   precomputed dissimilarity matrix (an object of class \code{"dist"}) may be
#'   supplied directly; in that case \code{hc_metric} is ignored and \code{k}
#'   must be specified. This allows custom distances such as Bray-Curtis
#'   (e.g. \code{vegan::vegdist(df, "bray")}).
#' @param FUNcluster a clustering function including "kmeans", "pam", "clara",
#'   "fanny", "hkmeans", "hclust", "agnes" and "diana". Abbreviation is allowed.
#' @param k the number of clusters to be generated. If NULL, the gap statistic
#'   is used to estimate the appropriate number of clusters. For hierarchical
#'   clustering, this automatic selection may return \code{k = 1}. In the case
#'   of kmeans, \code{k} can be either the number of clusters, or a set of
#'   initial (distinct) cluster centers.
#' @param k.max the maximum number of clusters to consider, must be at least 
#'   two.
#' @param stand logical value; default is FALSE. If TRUE, then the data will be
#'   standardized using the function \code{scale()}. Measurements are
#'   standardized for each variable (column), by subtracting the variable's
#'   mean value and dividing by the variable's standard deviation. If scaling
#'   produces \code{NA} values, \code{eclust()} stops with a package-level
#'   error.
#' @param graph logical value. If TRUE, cluster plot is displayed.
#' @param hc_metric character string specifying the metric to be used for 
#'   calculating dissimilarities between observations. Allowed values are those 
#'   accepted by the function dist() [including "euclidean", "manhattan", 
#'   "maximum", "canberra", "binary", "minkowski"] and correlation based 
#'   distance measures ["pearson", "spearman" or "kendall"]. Used only when
#'   FUNcluster is a hierarchical clustering function such as one of "hclust",
#'   "agnes" or "diana". Ignored when \code{x} is already a \code{"dist"} object.
#' @param hc_method the agglomeration method to be used (?hclust): "ward.D", 
#'   "ward.D2", "single", "complete", "average", ...
#' @param gap_maxSE a list containing the parameters (method and SE.factor) for 
#'   determining the location of the maximum of the gap statistic (Read the 
#'   documentation ?cluster::maxSE).
#' @param nboot integer, number of Monte Carlo ("bootstrap") samples. Used only 
#'   for determining the number of clusters using gap statistic.
#' @param verbose logical value. If TRUE, the result of progress is printed.
#' @param seed integer used for seeding the random number generator.
#' @param ... other arguments to be passed to FUNcluster.
#' @return Returns an object of class "eclust" containing the result of the 
#'   standard function used (e.g., kmeans, pam, hclust, agnes, diana, etc.).
#'   
#'   It also includes: \itemize{ \item cluster: the cluster assignment of
#'   observations after cutting the tree \item nbclust: the number of clusters
#'   \item silinfo: the silhouette information of observations, when available
#'   for solutions with at least two clusters, including $widths (silhouette
#'   width values of each observation), $clus.avg.widths (average silhouette
#'   width of each cluster) and $avg.width (average width of all clusters)
#'   \item size: the size of clusters \item data: a matrix containing the
#'   original or the standardized data (if stand = TRUE) } The "eclust" class
#'   has method for fviz_silhouette(), fviz_dend(), fviz_cluster().
#' @seealso \code{\link{fviz_silhouette}}, \code{\link{fviz_dend}}, 
#'   \code{\link{fviz_cluster}}
#' @author Alboukadel Kassambara \email{alboukadel.kassambara@@gmail.com}
#'   
#' @examples 
#' # Load and scale data
#' data("USArrests")
#' df <- scale(USArrests)
#' 
#' # Enhanced k-means clustering
#' # nboot >= 500 is recommended
#' res.km <- eclust(df, "kmeans", nboot = 2)
#' # Silhouette plot
#' fviz_silhouette(res.km)
#' # Optimal number of clusters using gap statistics
#' res.km$nbclust
#' # Print result
#'  res.km
#'  
#' \dontrun{
#' # Enhanced hierarchical clustering
#' res.hc <- eclust(df, "hclust", nboot = 2) # compute hclust
#' fviz_dend(res.hc) # dendrogram
#' if (res.hc$nbclust > 1) fviz_silhouette(res.hc) # silhouette plot
#' }
#'  
#' @name eclust
#' @rdname eclust
#' @export
eclust <- function(x, FUNcluster = c("kmeans", "pam", "clara", "fanny", "hclust", "agnes", "diana", "hkmeans"),
                   k = NULL, k.max = 10, stand = FALSE,
                   graph = TRUE,
                   hc_metric = "euclidean", hc_method = "ward.D2",
                   gap_maxSE = list(method = "firstSEmax", SE.factor = 1), 
                   nboot = 100, verbose = interactive(),
                   seed = 123,  ...)
  {
  has_seed <- exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)
  if(has_seed) old_seed <- get(".Random.seed", envir = .GlobalEnv)
  on.exit({
    if(!has_seed && exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)){
      rm(".Random.seed", envir = .GlobalEnv)
    } else if(has_seed){
      assign(".Random.seed", old_seed, envir = .GlobalEnv)
    }
  }, add = TRUE)
  set.seed(seed)
  data <- x
  # A precomputed distance matrix (class "dist") may be passed as x for
  # hierarchical clustering (mirrors hcut()); hc_metric is then ignored (#182).
  is_dist <- inherits(x, "dist")
  if(stand) {
    if(is_dist)
      stop("'stand = TRUE' is not supported when 'x' is a distance matrix (class 'dist').")
    x <- scale(x)
    if(anyNA(x))
      stop("Scaling produced NA values. Check for constant columns or non-finite values.")
  }
  # Define the type of clustering
  FUNcluster <- match.arg(FUNcluster)
  if(is_dist && FUNcluster %in% c("kmeans", "pam", "clara", "fanny", "hkmeans"))
    stop("A distance matrix (class 'dist') is supported only for hierarchical clustering ",
         "(FUNcluster = 'hclust', 'agnes', or 'diana'). For '", FUNcluster,
         "', supply the raw data, or use hcut() for precomputed distances.")
  fun_clust <- switch(FUNcluster,
                 kmeans = stats::kmeans,
                 pam = cluster::pam,
                 clara = cluster::clara,
                 fanny = cluster::fanny,
                 hkmeans = hkmeans,
                 diana = hcut,
                 agnes = hcut,
                 hclust = hcut
                 )
  
  if(!inherits(data, c("matrix", "data.frame")) ) graph = FALSE
  else if(ncol(data)< 2) graph = FALSE
      
  gap_stat <- NULL
  auto_k <- is.null(k)
  # Partitioning clustering
  # ++++++++++++++++++++++++++++++
  clust <- list()
  if(FUNcluster %in% c("kmeans", "pam", "clara", "fanny", "hkmeans")){
    # Number of cluster
    if(is.null(k)) {
      gap <- .gap_stat(x, fun_clust, k.max = k.max, nboot = nboot,
                       gap_maxSE = gap_maxSE, verbose = verbose, ...)
      k <- gap$k
      gap_stat <- gap$stat
    }
    clust <- fun_clust(x, k, ...)
    
    if(inherits(k, c("matrix", "data.frame"))) k <- nrow(k) # cluster centers are provided as k
    
    # Plot
    if(graph) {
      clust$clust_plot <- fviz_cluster(clust, x)
      print(clust$clust_plot + labs(title = paste0(toupper(FUNcluster), " Clustering")))
    }
    if(k > 1) clust$silinfo <-.get_silinfo(clust$cluster, stats::dist(x))
  }
  
  
  # Hierarchical clustering
  # ++++++++++++++++++++++++++++++++
  else if(FUNcluster %in% c("hclust", "agnes", "diana")){

    # Use a precomputed distance directly; otherwise compute it from the data.
    res.dist <- if(is_dist) x else get_dist(x, method = hc_metric)
    if(auto_k && is_dist)
      stop("When 'x' is a distance matrix, the number of clusters 'k' must be specified: ",
           "the gap statistic requires the original data and cannot be computed from a 'dist'.")
    # Number of cluster
    if(auto_k) {
      gap <- .gap_stat(x, fun_clust, k.max = k.max, nboot = nboot,
                       gap_maxSE = gap_maxSE, verbose = verbose, diss = res.dist)
      k <- gap$k
      gap_stat <- gap$stat
    }
    if(auto_k && k == 1) {
      res.hc <- .single_cluster_hcut(res.dist, hc_func = FUNcluster, hc_method = hc_method)
    } else {
      res.hc <- hcut(res.dist, k, hc_func = FUNcluster, hc_method = hc_method )
    }
    clust <- res.hc
    if(graph) {
      if(k > 1) fviz_dend(clust, k)
      else fviz_dend(clust)
    }
  }
  
  clust$nbclust <- k
  clust$data <- x
  clust$gap_stat <- gap_stat
  class(clust) <- c(class(clust), "eclust")
  clust
}


# Compute the hierarchical tree for the requested backend
.compute_hc_tree <- function(diss, hc_func, hc_method){
  if(hc_func == "hclust") stats::hclust(diss, method = hc_method)
  else if(hc_func == "agnes") {
    if(hc_method %in% c("ward.D", "ward.D2")) hc_method <- "ward"
    cluster::agnes(diss, method = hc_method)
  }
  else if(hc_func == "diana") cluster::diana(diss)
  else stop("Don't support the function ", hc_func)
}

# Build a one-cluster hierarchical result without relaxing direct hcut validation.
.single_cluster_hcut <- function(diss, hc_func, hc_method){
  n_obs <- attr(diss, "Size")
  if(is.null(n_obs) || !is.numeric(n_obs))
    stop("Unable to determine number of observations from distance data")

  hc <- .compute_hc_tree(diss, hc_func = hc_func, hc_method = hc_method)
  hc$cluster <- rep(1L, n_obs)
  cluster_labels <- attr(diss, "Labels")
  if(!is.null(cluster_labels) && length(cluster_labels) == n_obs)
    names(hc$cluster) <- cluster_labels
  hc$nbclust <- 1L
  hc$size <- as.vector(table(hc$cluster))
  hc$data <- diss
  class(hc) <- c(class(hc), "hcut")
  hc
}


# Compute gap stat and get k
.gap_stat <- function(x, fun_clust, k.max = 10, nboot = 100,
                   gap_maxSE = list(method = "firstmax", SE.factor = 1),
                   verbose = interactive(), ...)
  {
  gap_stat <- cluster::clusGap(x, fun_clust, K.max = k.max,  B = nboot, 
                               verbose = verbose, ...)
  gap <- gap_stat$Tab[, "gap"]
  se <- gap_stat$Tab[, "SE.sim"]
  k <- .maxSE(gap, se, method = gap_maxSE$method, SE.factor = gap_maxSE$SE.factor)
  list(stat = gap_stat, k = k)
}

test_that("Gap statistic plot helper returns ggplot", {
  set.seed(123)
  x <- scale(iris[, 1:4])
  gap <- cluster::clusGap(x, FUN = stats::kmeans, K.max = 4, B = 5)

  p <- fviz_gap_stat(gap)
  expect_s3_class(p, "ggplot")
})

test_that("Hopkins tendency returns statistic and optional plot", {
  old_opt <- getOption("factoextra.warn_hopkins", TRUE)
  on.exit(options(factoextra.warn_hopkins = old_opt), add = TRUE)
  options(factoextra.warn_hopkins = FALSE)

  res <- get_clust_tendency(iris[, 1:4], n = 10, graph = FALSE)
  expect_type(res$hopkins_stat, "double")
  expect_false(is.null(res$hopkins_stat))
  expect_null(res$plot)
})

test_that("eclust() accepts a precomputed dist for hierarchical clustering (#182)", {
  set.seed(1)
  df <- matrix(rpois(20 * 6, 5), nrow = 20)
  rownames(df) <- paste0("s", seq_len(nrow(df)))

  # Bray-Curtis dissimilarity, computed locally (no extra dependency)
  braycurtis <- function(m) {
    n <- nrow(m)
    d <- matrix(0, n, n)
    for (i in seq_len(n)) for (j in seq_len(n)) {
      d[i, j] <- sum(abs(m[i, ] - m[j, ])) / sum(m[i, ] + m[j, ])
    }
    stats::as.dist(d)
  }
  bc <- braycurtis(df)

  res <- eclust(bc, "hclust", k = 3, graph = FALSE)
  expect_s3_class(res, "eclust")
  # The supplied distance is honored, not recomputed as Euclidean
  expect_equal(res$height, stats::hclust(bc, method = "ward.D2")$height)

  # Auto-k cannot run on a bare dist (gap statistic needs the original data)
  expect_error(eclust(bc, "hclust", graph = FALSE), "number of clusters 'k'")
  # Partitioning methods that need raw data reject a dist clearly
  expect_error(eclust(bc, "kmeans", k = 3, graph = FALSE), "hierarchical clustering")
  # Scaling a dissimilarity is rejected
  expect_error(eclust(bc, "hclust", k = 3, stand = TRUE, graph = FALSE), "stand = TRUE")
})

test_that("eclust() hierarchical clustering on raw data is unchanged (#182 no-regression)", {
  res <- eclust(iris[, 1:4], "hclust", hc_metric = "manhattan", k = 3, graph = FALSE)
  ref <- stats::hclust(get_dist(iris[, 1:4], method = "manhattan"), method = "ward.D2")
  expect_equal(res$height, ref$height)
})

test_that("eclust() supports hkmeans as a FUNcluster (#78)", {
  df <- scale(iris[, 1:4])

  set.seed(1)
  res <- eclust(df, "hkmeans", k = 3, graph = FALSE)
  expect_s3_class(res, "eclust")
  expect_s3_class(res, "hkmeans")
  expect_length(res$cluster, nrow(df))
  expect_false(is.null(res$silinfo))
  # delegates to hkmeans() unchanged
  set.seed(1)
  expect_equal(unname(res$cluster), unname(hkmeans(df, 3)$cluster))

  # a distance matrix is rejected for hkmeans (needs raw data), like kmeans
  expect_error(eclust(get_dist(df), "hkmeans", k = 3, graph = FALSE),
               "hierarchical clustering")
})

#' @include utilities.R
NULL
#' Enhanced Distance Matrix Computation and Visualization
#' @description Clustering methods classify data samples into groups of similar 
#'   objects. This process requires some methods for measuring the distance or 
#'   the (dis)similarity between the observations. Read more: 
#'   \href{https://www.datanovia.com/en/lessons/clustering-distance-measures/}{STHDA
#'    website - clarifying distance measures.}. \itemize{ \item get_dist():
#'   Computes a distance matrix between the rows of a data matrix. Compared to
#'   the standard \code{\link[stats]{dist}}() function, it supports
#'   correlation-based distance measures including "pearson", "kendall" and
#'   "spearman" methods. \item fviz_dist(): Visualizes a distance matrix }
#'
#'   When \code{stand = TRUE}, scaling that produces non-finite values is
#'   rejected. \code{fviz_dist()} also validates that supplied distance objects
#'   contain only finite values before plotting.
#' @param x a numeric matrix or a data frame.
#' @param method the distance measure to be used. This must be one of 
#'   "euclidean", "maximum", "manhattan", "canberra", "binary", "minkowski", 
#'   "pearson", "spearman" or "kendall".
#' @param stand logical value; default is FALSE. If TRUE, then the data will be
#'   standardized using the function \code{scale()}. Measurements are
#'   standardized for each variable (column), by subtracting the variable's
#'   mean value and dividing by the variable's standard deviation. If scaling
#'   produces \code{NA} values, \code{get_dist()} stops with a package-level
#'   error.
#' @param ... other arguments to be passed to the function dist() when using get_dist().
#' @return \itemize{ \item get_dist(): returns an object of class "dist". \item 
#'   fviz_dist(): returns a ggplot2 }
#' @seealso \code{\link[stats]{dist}}
#' @author Alboukadel Kassambara \email{alboukadel.kassambara@@gmail.com}
#' @examples
#' data(USArrests)
#' res.dist <- get_dist(USArrests, stand = TRUE, method = "pearson")
#' 
#' fviz_dist(res.dist, 
#'    gradient = list(low = "#00AFBB", mid = "white", high = "#FC4E07"))
#' @name dist
#' @rdname dist
#' @export
get_dist <- function(x, method = "euclidean",  stand = FALSE, ...){
  
  if(stand) {
    x <- scale(x)
    if(anyNA(x))
      stop("Scaling produced NA values. Check for constant columns or non-finite values.")
  }
  if(method %in% c("pearson", "spearman", "kendall")){
    res.cor <- stats::cor(t(x),  method = method)
    res.dist <- stats::as.dist(1 - res.cor, ...)
  }
  else res.dist <- stats::dist(x, method = method, ...)
  
  res.dist
}

.check_dist_finite <- function(dist.obj){
  if(anyNA(dist.obj) || any(!is.finite(unclass(dist.obj))))
    stop("The distance matrix contains NA/NaN/Inf values. Check for constant columns or non-finite values.")
}


#' @param dist.obj an object of class "dist" as generated by the function
#'   \code{dist()} or \code{get_dist()}, containing only finite dissimilarity
#'   values.
#' @param order logical value. if TRUE the ordered dissimilarity image (ODI) is shown.
#' @param show_labels logical value. If TRUE, the labels are displayed.
#' @param lab_size the size of labels.
#' @param gradient a list containing three elements specifying the colors for low, mid and high values in 
#'  the ordered dissimilarity image. The element "mid" can take the value of NULL.
#' @rdname dist
#' @export
fviz_dist<- function(dist.obj, order = TRUE, show_labels = TRUE, lab_size = NULL,        
                     gradient = list(low = "red", mid = "white", high = "blue")
                     )
{
  
  if(!inherits(dist.obj, "dist"))
    stop("An object of class dist is required.")
  .check_dist_finite(dist.obj)
  
  if(order){
  res.hc <- stats::hclust(dist.obj, method = "ward.D2")
  dist.obj <- as.matrix(dist.obj)[res.hc$order, res.hc$order]
  }
  else dist.obj <- as.matrix(dist.obj)
  
  rownames(dist.obj ) <- colnames(dist.obj ) <- paste0(rownames(dist.obj), "-")

  # Convert distance matrix to long format (replacing reshape2::melt)
  # This creates a data frame with Var1, Var2, value columns
  d <- data.frame(
    Var1 = rep(rownames(dist.obj), ncol(dist.obj)),
    Var2 = rep(colnames(dist.obj), each = nrow(dist.obj)),
    value = as.vector(dist.obj)
  )
  # FIX: ggplot2 3.0.0+ deprecation - aes_string() replaced with aes() + .data pronoun
  # See: https://github.com/kassambara/factoextra/issues/190
  p <- ggplot(d, aes(x = .data[["Var1"]], y = .data[["Var2"]]))+
    ggplot2::geom_tile(aes(fill = .data[["value"]])) 
  if(is.null(gradient$mid)) p <- p + ggplot2::scale_fill_gradient(low=gradient$low, high=gradient$high)
  else p <- p + ggplot2::scale_fill_gradient2(midpoint=mean(dist.obj), low=gradient$low, 
                                                    mid=gradient$mid, high=gradient$high, space = "Lab")
  if(show_labels) p <- p + theme(axis.title.x = element_blank(), 
                                       axis.title.y = element_blank(),
                                       axis.text.x = element_text(angle = 45, hjust = 1, size = lab_size),
                                       axis.text.y = element_text(size = lab_size))
  else p <- p + 
    theme(axis.text = element_blank(), axis.ticks = element_blank(),
          axis.title.x = element_blank(),
          axis.title.y = element_blank())
  
  return(p)
}

test_that("hcut validates k and scaled inputs", {
  x <- iris[, 1:4]
  expect_error(hcut(x, k = 1), "k must be a single integer >= 2")
  expect_error(
    hcut(x, k = nrow(x) + 1),
    "elements of 'k' must be between 1 and"
  )
  expect_error(
    hcut(stats::dist(x), k = nrow(x) + 1, isdiss = TRUE),
    "elements of 'k' must be between 1 and"
  )

  x_const <- data.frame(a = 1:10, b = rep(1, 10))
  expect_error(
    hcut(x_const, k = 2, stand = TRUE),
    "Scaling produced NA values"
  )
})

test_that("hcut requires dist input when isdiss is TRUE", {
  x <- iris[, 1:4]
  bad_dist <- stats::as.dist(matrix(c(
    0, NA, 1,
    NA, 0, 2,
    1, 2, 0
  ), nrow = 3, byrow = TRUE))

  expect_error(
    hcut(x, k = 2, isdiss = TRUE),
    "must be an object of class dist"
  )
  expect_error(
    hcut(bad_dist, k = 2, isdiss = TRUE),
    "distance matrix contains NA/NaN/Inf values"
  )
})

test_that("hkmeans validates inputs and k bounds", {
  x <- iris[, 1:4]
  expect_error(hkmeans(x, k = 1), "k must be a single integer >= 2")
  expect_error(
    hkmeans(x, k = nrow(x) + 1),
    "elements of 'k' must be between 1 and"
  )
  expect_error(hkmeans(1:10, k = 2), "x must be a matrix or data.frame")
})

test_that("eclust validates scaled inputs consistently across backends", {
  x_const <- data.frame(a = 1:10, b = rep(1, 10))

  expect_error(
    eclust(x_const, FUNcluster = "hclust", k = 2, stand = TRUE, graph = FALSE),
    "Scaling produced NA values"
  )
  expect_error(
    eclust(x_const, FUNcluster = "kmeans", k = 2, stand = TRUE, graph = FALSE),
    "Scaling produced NA values"
  )
})

test_that("get_clust_tendency validates numeric data and n", {
  bad_x <- as.matrix(data.frame(a = letters[1:5], b = letters[1:5]))
  expect_error(get_clust_tendency(bad_x, n = 2, graph = FALSE), "numeric")
  expect_error(get_clust_tendency(iris[, 1:4], n = 0, graph = FALSE), "positive integer")
  expect_error(get_clust_tendency(iris[, 1:4], n = nrow(iris), graph = FALSE), "no larger")

  x_na <- matrix(c(1, 2, NA, NA), ncol = 2)
  expect_error(get_clust_tendency(x_na, n = 1, graph = FALSE), "at least two complete rows")
})

test_that("get_dist validates scaled inputs and fviz_dist validates dist values", {
  x_const <- data.frame(a = 1:10, b = rep(1, 10))
  bad_dist <- stats::as.dist(matrix(c(
    0, NA, 1,
    NA, 0, 2,
    1, 2, 0
  ), nrow = 3, byrow = TRUE))

  expect_error(
    get_dist(x_const, method = "pearson", stand = TRUE),
    "Scaling produced NA values"
  )
  expect_error(
    fviz_dist(bad_dist),
    "distance matrix contains NA/NaN/Inf values"
  )
})

test_that("fviz_cluster validates scaled plotting data", {
  degenerate_2d <- data.frame(a = 1:10, b = rep(1, 10))
  degenerate_3d <- data.frame(a = 1:10, b = rep(1, 10), c = rep(2, 10))
  cluster_obj_2d <- list(data = degenerate_2d, cluster = rep(1:2, each = 5))
  cluster_obj_3d <- list(data = degenerate_3d, cluster = rep(1:2, each = 5))

  expect_error(
    fviz_cluster(cluster_obj_2d, stand = TRUE),
    "Scaling produced NA values"
  )
  expect_error(
    fviz_cluster(cluster_obj_3d, stand = TRUE),
    "Scaling produced NA values"
  )
})

test_that("MCA quanti.sup workflows error cleanly when absent", {
  skip_if_not_installed("FactoMineR")

  data(poison, package = "FactoMineR")
  res.mca <- FactoMineR::MCA(poison[, 5:15], graph = FALSE)

  expect_error(
    get_mca_var(res.mca, "quanti.sup"),
    "There are no quantitative supplementary variables in this MCA"
  )
  expect_error(
    facto_summarize(res.mca, "quanti.sup", axes = 1:2),
    "There are no quantitative supplementary variables in this MCA"
  )
  expect_error(
    fviz(res.mca, "quanti.sup"),
    "There are no quantitative supplementary variables in this MCA"
  )
})

test_that("axes validation rejects invalid axis indices", {
  res.pca <- stats::prcomp(iris[, 1:4], scale. = TRUE)

  expect_error(
    facto_summarize(res.pca, "var", axes = c(1, NA)),
    "The value of the argument axes is incorrect"
  )
  expect_error(
    facto_summarize(res.pca, "var", axes = 0),
    "The value of the argument axes is incorrect"
  )
  expect_error(
    facto_summarize(res.pca, "var", axes = c(1, 1.5)),
    "The value of the argument axes is incorrect"
  )
  expect_error(
    fviz_cos2(res.pca, choice = "var", axes = c(1, NA)),
    "The value of the argument axes is incorrect"
  )
  expect_error(
    fviz_contrib(res.pca, choice = "var", axes = c(1, 0)),
    "The value of the argument axes is incorrect"
  )
  expect_error(
    fviz_ellipses(res.pca, habillage = iris$Species, axes = c(1, 1.5)),
    "The value of the argument axes is incorrect"
  )
})

test_that("fviz_eig validates ncp", {
  res.pca <- stats::prcomp(iris[, 1:4], scale. = TRUE)

  expect_error(
    fviz_eig(res.pca, ncp = NA),
    "ncp must be a single positive integer value in"
  )
  expect_error(
    fviz_eig(res.pca, ncp = -1),
    "ncp must be a single positive integer value in"
  )
  expect_error(
    fviz_eig(res.pca, ncp = 1.5),
    "ncp must be a single positive integer value in"
  )
})

test_that("fviz_eig accepts integerish doubles for validated arguments", {
  res.pca <- stats::prcomp(iris[, 1:4], scale. = TRUE)

  expect_s3_class(fviz_eig(res.pca, ncp = 2 + 1e-15), "ggplot")
  expect_s3_class(
    fviz_eig(
      res.pca, choice = "eigenvalue", parallel = TRUE,
      ncp = 2 + 1e-15, parallel.iter = 2 + 1e-15, parallel.seed = 1 + 1e-15
    ),
    "ggplot"
  )
})

#' @include utilities.R
NULL

#' Build a factoextra-ready object from pre-computed coordinates
#'
#' @description
#' \code{as_factoextra_pca()} wraps pre-computed individual (and, optionally,
#' variable) coordinates into an object that the \code{\link{fviz_pca}} family
#' (\code{fviz_pca_ind()}, \code{fviz_pca_var()}, \code{fviz_pca_biplot()}),
#' \code{\link{fviz_eig}}, \code{\link{fviz_contrib}} and \code{\link{fviz_cos2}}
#' can plot directly.
#'
#' It lets you apply factoextra's visualizations to the output of \emph{any}
#' dimension-reduction method - for example \code{stats::cmdscale()},
#' \code{ape::pcoa()}, UMAP/t-SNE embeddings, \code{vegan::rda()}/\code{cca()},
#' or a custom analysis - without having to write a dedicated backend. You bring
#' the coordinates; factoextra draws the biplot, scree plot, contributions and
#' cos2.
#'
#' @param ind.coord individual (observation) coordinates: a numeric matrix or
#'   data frame with one column per dimension (the "scores"). Required.
#' @param var.coord optional variable coordinates / loadings: a numeric matrix or
#'   data frame with one column per dimension. Supplying it enables
#'   \code{fviz_pca_var()} and \code{fviz_pca_biplot()}.
#' @param eig optional numeric vector of eigenvalues (length \eqn{\ge} number of
#'   dimensions). Used by the scree plot and to label the axes with the percentage
#'   of explained variance. When \code{NULL} (default) it is set to the variance
#'   of each coordinate column (the natural definition of a PCA eigenvalue).
#' @param ind.cos2,ind.contrib,var.cos2,var.contrib,var.cor optional pre-computed
#'   quality (\code{cos2}), contribution and (variable) correlation matrices, with
#'   the same dimensions as the corresponding coordinates. When omitted they are
#'   derived from the coordinates (see \strong{Details}).
#' @param scale.unit logical. If \code{TRUE}, the variable coordinates are treated
#'   as correlations and the correlation circle is drawn by \code{fviz_pca_var()}.
#'   Default is \code{FALSE}.
#'
#' @details
#' When \code{cos2}/\code{contrib} are not supplied they are computed from the
#' coordinates:
#' \itemize{
#'   \item \code{contrib = 100 * coord^2 / colSums(coord^2)} - the exact
#'     contribution of each element to each dimension.
#'   \item \code{cos2 = coord^2 / rowSums(coord^2)} - the quality of
#'     representation \emph{within the supplied dimensions}. This equals the true
#'     cos2 only when all components are provided; with a truncated set of
#'     dimensions it is the quality restricted to that sub-space. Pass
#'     \code{ind.cos2}/\code{var.cos2} explicitly if you have the exact values.
#' }
#'
#' @return An object of class \code{c("factoextra_pca", "list")} holding the
#'   standardized \code{ind} (and \code{var}) results and eigenvalues, ready for
#'   the \code{fviz_pca_*()} functions.
#'
#' @author Alboukadel Kassambara \email{alboukadel.kassambara@@gmail.com}
#'
#' @examples
#' # 1. Bring your own coordinates: classical MDS (cmdscale) -> factoextra
#' d <- dist(scale(mtcars))
#' mds <- cmdscale(d, k = 3)
#' obj <- as_factoextra_pca(ind.coord = mds)
#' fviz_pca_ind(obj, repel = TRUE)
#' fviz_eig(obj)
#'
#' # 2. Round-trip a prcomp result through the constructor (biplot)
#' pca <- prcomp(iris[, -5], scale. = TRUE)
#' obj2 <- as_factoextra_pca(
#'   ind.coord = pca$x,
#'   var.coord = pca$rotation,
#'   eig       = pca$sdev^2
#' )
#' fviz_pca_biplot(obj2, label = "var", col.ind = "steelblue")
#'
#' @rdname as_factoextra
#' @export
as_factoextra_pca <- function(ind.coord, var.coord = NULL, eig = NULL,
                              ind.cos2 = NULL, ind.contrib = NULL,
                              var.cos2 = NULL, var.contrib = NULL, var.cor = NULL,
                              scale.unit = FALSE){

  if(missing(ind.coord) || is.null(ind.coord))
    stop("`ind.coord` is required.", call. = FALSE)

  ind.coord <- .fe_as_dim_matrix(ind.coord, "ind.coord")
  k <- ncol(ind.coord)

  # Eigenvalues: default to the variance of each coordinate column.
  if(is.null(eig)) eig <- apply(ind.coord, 2, stats::var)
  if(!is.numeric(eig) || length(eig) < k)
    stop("`eig` must be a numeric vector with at least ", k,
         " value(s) (one per dimension).", call. = FALSE)

  ind <- list(
    coord   = ind.coord,
    cos2    = if(is.null(ind.cos2))    .fe_cos2(ind.coord)    else .fe_check_like(ind.cos2,    ind.coord, "ind.cos2"),
    contrib = if(is.null(ind.contrib)) .fe_contrib(ind.coord) else .fe_check_like(ind.contrib, ind.coord, "ind.contrib")
  )

  var <- NULL
  if(!is.null(var.coord)){
    var.coord <- .fe_as_dim_matrix(var.coord, "var.coord")
    if(ncol(var.coord) != k)
      stop("`var.coord` must have the same number of dimensions as `ind.coord` (",
           k, ").", call. = FALSE)
    var <- list(
      coord   = var.coord,
      cor     = if(is.null(var.cor))     var.coord               else .fe_check_like(var.cor,     var.coord, "var.cor"),
      cos2    = if(is.null(var.cos2))    .fe_cos2(var.coord)     else .fe_check_like(var.cos2,    var.coord, "var.cos2"),
      contrib = if(is.null(var.contrib)) .fe_contrib(var.coord) else .fe_check_like(var.contrib, var.coord, "var.contrib")
    )
  }

  structure(
    list(ind = ind, var = var, eig.values = as.numeric(eig),
         scale.unit = isTRUE(scale.unit)),
    class = c("factoextra_pca", "list")
  )
}

# ---- internal helpers -------------------------------------------------------

# Coerce coordinates to a numeric matrix with Dim.1..k colnames and row names.
.fe_as_dim_matrix <- function(x, argname){
  x <- as.matrix(x)
  if(!is.numeric(x))
    stop("`", argname, "` must be numeric.", call. = FALSE)
  if(ncol(x) < 1)
    stop("`", argname, "` must have at least one column (dimension).", call. = FALSE)
  colnames(x) <- paste0("Dim.", seq_len(ncol(x)))
  if(is.null(rownames(x))) rownames(x) <- as.character(seq_len(nrow(x)))
  x
}

# Validate that a user-supplied cos2/contrib/cor matrix matches the coordinates.
.fe_check_like <- function(x, coord, argname){
  x <- as.matrix(x)
  if(!is.numeric(x))
    stop("`", argname, "` must be numeric.", call. = FALSE)
  if(!all(dim(x) == dim(coord)))
    stop("`", argname, "` must have the same dimensions as its coordinates (",
         nrow(coord), " x ", ncol(coord), ").", call. = FALSE)
  colnames(x) <- colnames(coord)
  rownames(x) <- rownames(coord)
  x
}

# Per-dimension contribution (in percent): exact from coordinates.
.fe_contrib <- function(coord){
  ss <- colSums(coord^2)
  ss[ss == 0] <- 1   # avoid 0/0 for an all-zero dimension
  contrib <- sweep(coord^2, 2, ss, "/") * 100
  dimnames(contrib) <- dimnames(coord)
  contrib
}

# Per-row quality of representation within the supplied dimensions.
.fe_cos2 <- function(coord){
  d2 <- rowSums(coord^2)
  d2[d2 == 0] <- 1
  cos2 <- sweep(coord^2, 1, d2, "/")
  dimnames(cos2) <- dimnames(coord)
  cos2
}

#' @include print.factoextra.R
NULL
#' Extract the results for individuals/variables/group/partial axes - MFA
#' 
#' @description
#' Extract all the results (coordinates, squared cosine and contributions) 
#' for the active individuals/quantitative variables/qualitative variable categories/groups/partial axes from Multiple Factor Analysis (MFA) outputs.\cr\cr
#' \itemize{
#' \item get_mfa(): Extract the results for variables and individuals
#' \item get_mfa_ind(): Extract the results for individuals only
#' \item get_mfa_var(): Extract the results for variables (quantitatives, qualitatives and groups)
#' \item get_mfa_partial_axes(): Extract the results for partial axes only
#' }   
#'     
#' 
#' @param res.mfa an object of class MFA [FactoMineR].
#' @param element the element to subset from the output. Possible values are
#'   "ind", "quanti.var", "quali.var", "quali.sup", "group" or
#'   "partial.axes".
#' @return a list of matrices containing the results for the active 
#' individuals/quantitative variable categories/qualitative variable categories/groups/partial axes including : 
#' \item{coord}{coordinates for the individuals/quantitative variable categories/qualitative variable categories/groups/partial axes}
#' \item{cos2}{cos2 for the individuals/quantitative variable categories/qualitative variable categories/groups/partial axes}
#' \item{contrib}{contributions of the individuals/quantitative variable categories/qualitative variable categories/groups/partial axes}
#' \item{inertia}{inertia of the individuals/quantitative variable categories/qualitative variable categories/groups/partial axes}
#' @author Alboukadel Kassambara \email{alboukadel.kassambara@@gmail.com}
#' @author Fabian Mundt \email{f.mundt@@inventionate.de}
#' @examples
#' # Multiple Factor Analysis
#' # ++++++++++++++++++++++++
#' # Install and load FactoMineR to compute MFA
#' # install.packages("FactoMineR")
#' library("FactoMineR")
#' data(poison)
#' res.mfa <- MFA(poison, group=c(2,2,5,6), type=c("s","n","n","n"),
#' name.group=c("desc","desc2","symptom","eat"), num.group.sup=1:2,
#' graph = FALSE)
#'  
#'  # Extract the results for qualitative variable categories
#'  var <- get_mfa_var(res.mfa, "quali.var")
#'  print(var)
#'  head(var$coord) # coordinates of qualitative variables
#'  head(var$cos2) # cos2 of qualitative variables
#'  head(var$contrib) # contributions of qualitative variables
#'  
#'  # Extract the results for individuals
#'  ind <- get_mfa_ind(res.mfa)
#'  print(ind)
#'  head(ind$coord) # coordinates of individuals
#'  head(ind$cos2) # cos2 of individuals
#'  head(ind$contrib) # contributions of individuals
#'  
#'  # You can also use the function get_mfa()
#'  get_mfa(res.mfa, "ind") # Results for individuals
#'  get_mfa(res.mfa, "quali.var") # Results for qualitative variable categories
#'  get_mfa(res.mfa, "quali.sup") # Results for supplementary qualitative variable categories
#'  
#' @name get_mfa
#' 
#' @rdname get_mfa
#' @export 
get_mfa <- function(res.mfa, element = c("ind", "quanti.var", "quali.var", "quali.sup", "group", "partial.axes")){
  elmt <- match.arg(element)
  if(elmt == "ind") get_mfa_ind(res.mfa)
  else if(elmt %in% c("quanti.var", "quali.var", "quali.sup", "group"))
    get_mfa_var(res.mfa, elmt)
  else if(elmt == "partial.axes") get_mfa_partial_axes(res.mfa)
}

#' @rdname get_mfa
#' @export
get_mfa_ind <- function(res.mfa){
  # FactoMineR package
  if(inherits(res.mfa, c("MFA"))) ind <- res.mfa$ind
  # @todo ade4 Support muss noch eingebaut werden!
  else stop("An object of class : ", paste(class(res.mfa), collapse = ", "), 
            " can't be handled by the function get_mfa_ind()")
  class(ind)<-c("factoextra", "mfa", "mfa_ind")
  attr(ind, "element") <- "individuals"
  return(ind)
}

#' @rdname get_mfa
#' @export
get_mfa_var <- function(res.mfa, element = c("quanti.var", "quali.var", "quali.sup", "group")){
  
  choice <- match.arg(element)
  if(!inherits(res.mfa, "MFA"))
    stop("An object of class : ", paste(class(res.mfa), collapse = ", "), " can't be handled.")
  
  if(choice == "quanti.var" && is.null(res.mfa$quanti.var)) 
    stop("There are no quantitative variables in this MFA.")
  else if(choice == "quali.var" && is.null(res.mfa$quali.var)) 
    stop("There are no qualitative variables in this MFA.")
  else if(choice == "quali.sup" && is.null(.get_factominer_quali_sup(res.mfa)))
    stop("There are no supplementary qualitative variables in this MFA.")
    
  
    vars <- switch(choice,
                   quanti.var = res.mfa$quanti.var,
                   quali.var = res.mfa$quali.var,
                   quali.sup = .get_factominer_quali_sup(res.mfa),
                   group = res.mfa$group
    )
    element_desc <- switch(choice,
                           quanti.var = "quantitative variables",
                           quali.var = "qualitative variable categories",
                           quali.sup = "supplementary qualitative variable categories",
                           group = "variable groups"
    )
  
  class(vars)<-c("factoextra",  "mfa", paste0("mfa_", gsub(".", "_", choice, fixed = TRUE)))
  attr(vars, "element") <- element_desc
  return(vars)
}

#' @rdname get_mfa
#' @export
get_mfa_partial_axes <- function(res.mfa){
  # FactoMineR package
  if(inherits(res.mfa, c("MFA"))) partial_axes <- res.mfa$partial.axes
  else stop("An object of class : ", paste(class(res.mfa), collapse = ", "), 
            " can't be handled by the function get_mfa_partial_axes()")
  class(partial_axes)<-c("factoextra", "mfa", "mfa_partial_axes")
  attr(partial_axes, "element") <- "partial axes"
  return(partial_axes)
}

.factoextra_state <- new.env(parent = emptyenv())

.onLoad <- function(libname, pkgname) {
  .factoextra_state$hopkins_warned <- FALSE
}

.onAttach <- function(libname, pkgname) {
  packageStartupMessage("Welcome to factoextra!")
  packageStartupMessage("Want to learn more? See two factoextra-related books at https://www.datanovia.com/en/product/practical-guide-to-principal-component-methods-in-r/")
}

#' Clean up stale package lock files
#'
#' @description Removes stale lock directories that can prevent package installation.
#'
#' Lock files are created during package installation and should be automatically
#' removed when installation completes. If installation is interrupted (e.g., by
#' closing R or a crash), these lock files may remain and block future installations.
#'
#' @param package Character string specifying which package lock to remove.
#'   Default is "factoextra". Use "all" to remove all lock files.
#' @param lib Library path to check. Default is the first library in .libPaths().
#' @param ask Logical. If TRUE (default), asks for confirmation before removing.
#'
#' @return Invisibly returns TRUE if files were removed, FALSE otherwise.
#' @keywords internal
#'
#' @examples
#' \dontrun{
#' # Remove factoextra lock file
#' clean_lock_files()
#'
#' # Remove all lock files
#' clean_lock_files("all")
#'
#' # Remove without confirmation
#' clean_lock_files(ask = FALSE)
#' }
clean_lock_files <- function(package = "factoextra", lib = .libPaths()[1], ask = TRUE) {

  if (package == "all") {
    # Find all lock directories
    lock_dirs <- list.dirs(lib, recursive = FALSE, full.names = TRUE)
    lock_dirs <- lock_dirs[grepl("^00LOCK-", basename(lock_dirs))]
  } else {
    lock_dirs <- file.path(lib, paste0("00LOCK-", package))
    lock_dirs <- lock_dirs[dir.exists(lock_dirs)]
  }

  if (length(lock_dirs) == 0) {
    message("No lock files found.")
    return(invisible(FALSE))
  }

  message("Found lock directories:")
  for (d in lock_dirs) {
    message("  ", d)
  }

  if (ask) {
    response <- readline("Remove these directories? (y/n): ")
    if (!tolower(response) %in% c("y", "yes")) {
      message("Aborted.")
      return(invisible(FALSE))
    }
  }

  # Remove lock directories
  success <- TRUE
  for (d in lock_dirs) {
    tryCatch({
      unlink(d, recursive = TRUE, force = TRUE)
      if (dir.exists(d)) {
        # unlink doesn't always error on failure, check if still exists
        warning("Failed to remove ", d, ": directory still exists. ",
                "You may need to remove it manually or check permissions.")
        success <- FALSE
      } else {
        message("Removed: ", d)
      }
    }, error = function(e) {
      warning("Failed to remove ", d, ": ", e$message,
              "\nOn Linux/macOS, you may need: sudo rm -rf '", d, "'")
      success <<- FALSE
    })
  }

  if (success) {
    message("Lock files cleaned up successfully.")
    message("You can now reinstall the package.")
  }

  invisible(success)
}

#' Athletes' performance in decathlon
#'
#' @description
#' Athletes' performance during two sporting meetings
#' @name decathlon2
#' @docType data
#' @usage data("decathlon2")
#' @format
#' A data frame with 27 observations on the following 13 variables.
#' \describe{
#' \item{\code{X100m}}{a numeric vector}
#' \item{\code{Long.jump}}{a numeric vector}
#' \item{\code{Shot.put}}{a numeric vector}
#' \item{\code{High.jump}}{a numeric vector}
#' \item{\code{X400m}}{a numeric vector}
#' \item{\code{X110m.hurdle}}{a numeric vector}
#' \item{\code{Discus}}{a numeric vector}
#' \item{\code{Pole.vault}}{a numeric vector}
#' \item{\code{Javeline}}{a numeric vector}
#' \item{\code{X1500m}}{a numeric vector}
#' \item{\code{Rank}}{a numeric vector corresponding to the rank}
#' \item{\code{Points}}{a numeric vector specifying the point obtained}
#' \item{\code{Competition}}{a factor with levels \code{Decastar} \code{OlympicG}}
#' }
#' 
#' @source
#' This data is a subset of decathlon data in FactoMineR package.
#' @examples
#' \donttest{
#' data(decathlon2)
#' decathlon.active <- decathlon2[1:23, 1:10]
#' res.pca <- prcomp(decathlon.active, scale = TRUE)
#' fviz_pca_biplot(res.pca)
#' }
#' 
NULL

#' @include print.factoextra.R
NULL
#' Extract the results for individuals/variables/group/partial axes - HMFA
#' 
#' @description Extract all the results (coordinates, squared cosine and
#' contributions) for the active individuals/quantitative variables/qualitative
#' variable categories/groups/partial axes from Hierarchical Multiple Factor
#' Analysis (HMFA) outputs.\cr\cr \itemize{ \item get_hmfa(): Extract the
#' results for variables and individuals \item get_hmfa_ind(): Extract the
#' results for individuals only \item get_mfa_var(): Extract the results for
#' variables (quantitatives, qualitatives and groups) 
#' \item get_hmfa_partial(): Extract the results for partial.node. }
#' 
#' @param res.hmfa an object of class HMFA [FactoMineR].
#' @param element the element to subset from the output. Possible values are
#'   "ind", "quanti.var", "quali.var", "group" or "partial.node".
#' @return a list of matrices containing the results for the active 
#'   individuals, variables, groups and partial nodes, including : 
#'   \item{coord}{coordinates} \item{cos2}{cos2} 
#'   \item{contrib}{contributions}
#' @author Alboukadel Kassambara \email{alboukadel.kassambara@@gmail.com}
#' @author Fabian Mundt \email{f.mundt@@inventionate.de}
#' @examples
#' # Multiple Factor Analysis
#' # ++++++++++++++++++++++++
#' # Install and load FactoMineR to compute MFA
#' # install.packages("FactoMineR")
#' library("FactoMineR")
#' data(wine)
#' hierar <- list(c(2,5,3,10,9,2), c(4,2))
#' res.hmfa <- HMFA(wine, H = hierar, type=c("n",rep("s",5)), graph = FALSE)
#'  
#'  # Extract the results for qualitative variable categories
#'  var <- get_hmfa_var(res.hmfa, "quali.var")
#'  print(var)
#'  head(var$coord) # coordinates of qualitative variables
#'  head(var$cos2) # cos2 of qualitative variables
#'  head(var$contrib) # contributions of qualitative variables
#'  
#'  # Extract the results for individuals
#'  ind <- get_hmfa_ind(res.hmfa)
#'  print(ind)
#'  head(ind$coord) # coordinates of individuals
#'  head(ind$cos2) # cos2 of individuals
#'  head(ind$contrib) # contributions of individuals
#'  
#'  # You can also use the function get_hmfa()
#'  get_hmfa(res.hmfa, "ind") # Results for individuals
#'  get_hmfa(res.hmfa, "quali.var") # Results for qualitative variable categories
#'  
#' @name get_hmfa
#'   
#' @rdname get_hmfa
#' @export
get_hmfa <- function(res.hmfa, element = c("ind", "quanti.var", "quali.var", "group", "partial.node")){
  elmt <- match.arg(element)
  if(elmt == "ind") get_hmfa_ind(res.hmfa)
  else if(elmt %in% c("quanti.var", "quali.var", "group"))
    get_hmfa_var(res.hmfa, elmt)
  else if(elmt == "partial.node") get_hmfa_partial(res.hmfa)
}

#' @rdname get_hmfa
#' @export
get_hmfa_ind <- function(res.hmfa){
  # FactoMineR package
  if(inherits(res.hmfa, c("HMFA"))) ind <- res.hmfa$ind
  # @todo ade4 Support muss noch eingebaut werden!
  else stop("An object of class : ", paste(class(res.hmfa), collapse = ", "), 
            " can't be handled by the function get_hmfa_ind()")
  class(ind)<-c("factoextra", "hmfa", "hmfa_ind")
  attr(ind, "element") <- "individuals"
  return(ind)
}

#' @rdname get_hmfa
#' @export
get_hmfa_var <- function(res.hmfa, element = c( "quanti.var", "quali.var", "group")){
  
  choice <- match.arg(element)
  if(!inherits(res.hmfa, "HMFA"))
    stop("An object of class : ", paste(class(res.hmfa), collapse = ", "), " can't be handled.")
  
  if(choice == "quanti.var" && is.null(res.hmfa$quanti.var)) 
    stop("There are no quantitative variables in this HMFA.")
  else if(choice == "quali.var" && is.null(res.hmfa$quali.var)) 
    stop("There are no qualitative variables in this HMFA.")
  
  
  vars <- switch(choice,
                 quanti.var = res.hmfa$quanti.var,
                 quali.var = res.hmfa$quali.var,
                 group = list(coord = res.hmfa$group$coord[[1]], canonical = res.hmfa$group$canonical)
  )
  element_desc <- switch(choice,
                         quanti.var = "quantitative variables",
                         quali.var = "qualitative variable categories",
                         group = "variable groups"
  )
  
  class(vars)<-c("factoextra",  "hmfa", paste0("hmfa_", gsub(".", "_", choice, fixed = TRUE)))
  attr(vars, "element") <- element_desc
  return(vars)
}

#' @rdname get_hmfa
#' @export
get_hmfa_partial <- function(res.hmfa){
  # FactoMineR package
  # Group calculation is only for first layer valid (see Pages 2015)
  if(inherits(res.hmfa, c("HMFA"))) partial <- res.hmfa$partial
  else stop("An object of class : ", paste(class(res.hmfa), collapse = ", "), 
            " can't be handled by the function get_hmfa_partial()")
  class(partial)<-c("list",  "hmfa_partial")
  attr(partial, "element") <- "partial node"
  return(partial)
}

#'Poison
#'
#'@description This data is a result from a survey carried out on children of
#'  primary school who suffered from food poisoning. They were asked about their
#'  symptoms and about what they ate.
#'@name poison
#'@docType data
#'@usage data("poison")
#'@format A data frame with 55 rows and 15 columns. 
#'@source
#'This data is from FactoMineR package.
#'
#' @examples
#' \donttest{
#' library(FactoMineR)
#' data(poison)
#' res.mca <- MCA(poison, quanti.sup = 1:2, quali.sup = c(3,4), 
#'    graph = FALSE)
#' fviz_mca_biplot(res.mca, repel = TRUE)+
#' theme_minimal()
#' 
#' }
#' 
NULL

#' Print method for an object of class factoextra
#' 
#' @description
#' Print method for an object of class factoextra
#' @param x an object of class factoextra
#' @param ... further arguments to be passed to print method
#' @author Alboukadel Kassambara \email{alboukadel.kassambara@@gmail.com}
#' @examples
#'  data(iris)
#'  res.pca <- prcomp(iris[, -5],  scale = TRUE)
#'  ind <- get_pca_ind(res.pca, data = iris[, -5])
#'  print(ind)
#'  
#' @export
print.factoextra<-function(x, ...){
  if(!inherits(x, "factoextra"))
    stop("Can't handle data of class ", paste(class(x), collapse = ", "))
  
  .factoextra_descriptions <- function(nms){
    desc_map <- c(
      coord = "Coordinates",
      cor = "Correlations between variables and dimensions",
      cos2 = "Cos2, quality of representation",
      contrib = "Contributions",
      inertia = "Inertia",
      correlation = "Correlation between groups and principal dimensions",
      canonical = "canonical correlation coefficient",
      `coord.partiel` = "Partial coordinates",
      `coord.partial` = "Partial coordinates",
      `within.inertia` = "Within inertia",
      `within.partial.inertia` = "Within partial inertia",
      `v.test` = "V-test"
    )
    desc <- unname(desc_map[nms])
    missing <- is.na(desc)
    if(any(missing)){
      desc[missing] <- tools::toTitleCase(gsub("[.]", " ", nms[missing]))
    }
    desc
  }
  
  if(inherits(x, "pca_ind")){
    cat("Principal Component Analysis Results for individuals\n",
        "===================================================\n")
    res <- array(data="", dim=c(3,2), dimnames=list(1:3, c("Name", "Description")))
    res[1, ] <- c("$coord", "Coordinates for the individuals")
    res[2, ] <- c("$cos2", "Cos2 for the individuals")
    res[3, ] <- c("$contrib", "contributions of the individuals")
    print(res[1:3,], ...)
  }
  
  else if(inherits(x, "pca_var")){
    cat("Principal Component Analysis Results for variables\n",
        "===================================================\n")
    res <- array(data="", dim=c(4,2), dimnames=list(1:4, c("Name", "Description")))
    res[1, ] <- c("$coord", "Coordinates for the variables")
    res[2, ] <- c("$cor", "Correlations between variables and dimensions")
    res[3, ] <- c("$cos2", "Cos2 for the variables")
    res[4, ] <- c("$contrib", "contributions of the variables")
    print(res[1:4,])
  }
  
  else if(inherits(x, "ca_row")){
    cat("Correspondence Analysis - Results for rows\n",
        "===================================================\n")
    res <- array(data="", dim=c(4,2), dimnames=list(1:4, c("Name", "Description")))
    res[1, ] <- c("$coord", "Coordinates for the rows")
    res[2, ] <- c("$cos2", "Cos2 for the rows")
    res[3, ] <- c("$contrib", "contributions of the rows")
    res[4, ] <- c("$inertia", "Inertia of the rows")
    print(res[1:4,])
  }
  
  else if(inherits(x, "ca_col")){
    cat("Correspondence Analysis - Results for columns\n",
        "===================================================\n")
    res <- array(data="", dim=c(4,2), dimnames=list(1:4, c("Name", "Description")))
    res[1, ] <- c("$coord", "Coordinates for the columns")
    res[2, ] <- c("$cos2", "Cos2 for the columns")
    res[3, ] <- c("$contrib", "contributions of the columns")
    res[4, ] <- c("$inertia", "Inertia of the columns")
    print(res[1:4,])
  }
  else if(inherits(x, "mca_ind")){
    cat("Multiple Correspondence Analysis Results for individuals\n",
        "===================================================\n")
    res <- array(data="", dim=c(3,2), dimnames=list(1:3, c("Name", "Description")))
    res[1, ] <- c("$coord", "Coordinates for the individuals")
    res[2, ] <- c("$cos2", "Cos2 for the individuals")
    res[3, ] <- c("$contrib", "contributions of the individuals")
    print(res[1:3,], ...)
  }
  else if(inherits(x, "mca_var")){
    cat("Multiple Correspondence Analysis Results for variables\n",
        "===================================================\n")
    res <- array(data="", dim=c(3,2), dimnames=list(1:3, c("Name", "Description")))
    res[1, ] <- c("$coord", "Coordinates for categories")
    res[2, ] <- c("$cos2", "Cos2 for categories")
    res[3, ] <- c("$contrib", "contributions of categories")
    print(res[1:3,])
  }
  else if(inherits(x, "famd")){
    element <- attr(x, "element") # description
    cat("FAMD results for", element, "\n",
        "===================================================\n")
    nms <- names(x)
    nrows <- length(nms)
    res <- array(data="", dim = c(nrows, 2), dimnames = list(seq_len(nrows), c("Name", "Description")))
    res[, 1] <- paste0("$", nms)
    res[, 2] <- .factoextra_descriptions(nms)
    print(res[seq_len(nrows),], ...)
  }
  else if(inherits(x, "mfa")){
    # Element
    element <- attr(x, "element") # description
    cat("Multiple Factor Analysis results for", element, "\n",
        "===================================================\n")
    nms <- names(x)
    nrows <- length(nms)
    res <- array(data="", dim = c(nrows, 2), dimnames=list(seq_len(nrows), c("Name", "Description")))
    res[, 1] <- paste0("$", nms)
    res[, 2] <- .factoextra_descriptions(nms)
    print(res[seq_len(nrows),], ...)
  }
  else if(inherits(x, c("hmfa_ind", "hmfa_quali_var", "hmfa_quanti_var"))){
    element <- attr(x, "element") # description
    cat("Hierarchical Multiple Factor Analysis results for", element, "\n",
        "===================================================\n")
    res <- array(data="", dim=c(3,2), dimnames=list(1:3, c("Name", "Description")))
    res[1, ] <- c("$coord", "Coordinates")
    res[2, ] <- c("$cos2", "Cos2, quality of representation")
    res[3, ] <- c("$contrib", "Contributions")
    print(res[1:3,], ...)
  }
  # canonical correlation coefficients added
  else if(inherits(x, "hmfa_group")){
    cat("Hierarchical Multiple Factor Analysis Results for groups\n",
        "===================================================\n")
    nms <- names(x)
    nrows <- length(nms)
    res <- array(data = "", dim = c(nrows, 2), dimnames = list(seq_len(nrows), c("Name", "Description")))
    res[, 1] <- paste0("$", nms)
    res[, 2] <- .factoextra_descriptions(nms)
    print(res[seq_len(nrows),], ...)
  }
  
}

#' @include cluster_utilities.R dist.R
NULL
#' Computes Hierarchical Clustering and Cut the Tree
#' 
#' @description
#' Computes hierarchical clustering (hclust, agnes, diana) and cuts the tree
#' into \code{k} clusters. It also accepts correlation-based distance measures
#' such as "pearson", "spearman" and "kendall". Direct calls require
#' \code{k >= 2}; helper-level one-cluster handling is implemented in callers
#' such as \code{eclust()} and \code{fviz_nbclust()}.
#' @param x a numeric matrix, numeric data frame or a dissimilarity matrix.
#' @param k a single integer specifying the number of clusters to be generated.
#'   Must be at least 2 and smaller than the number of observations.
#' @param hc_method the agglomeration method to be used (?hclust) for hclust() and agnes(): 
#' "ward.D", "ward.D2", "single", "complete", "average", ...
#' @param hc_metric character string specifying the metric to be used for calculating 
#' dissimilarities between observations. Allowed values are those accepted by the function dist() [including "euclidean", "manhattan", "maximum",
#'  "canberra", "binary", "minkowski"] and correlation based distance measures ["pearson", "spearman" or "kendall"].
#' @param stand logical value; default is FALSE. If TRUE, then the data will be standardized using the function scale(). 
#' Measurements are standardized for each variable (column), by subtracting the
#' variable's mean value and dividing by the variable's standard deviation. If
#' scaling produces \code{NA} values, \code{hcut()} stops with a package-level
#' error.
#' @param isdiss logical value specifying whether \code{x} is already a
#'   dissimilarity matrix. If TRUE, \code{x} must inherit from class
#'   \code{"dist"} and contain only finite values.
#' @param hc_func the hierarchical clustering function to be used. Default value is "hclust". Possible values 
#' is one of "hclust", "agnes", "diana". Abbreviation is allowed.
#' @param graph logical value. If TRUE, the dendrogram is displayed.
#' @param ... not used.
#' @return an object of class "hcut" containing the result 
#' of the standard function used (read the documentation of hclust, agnes, diana). 
#'       
#' It includes also:
#' \itemize{
#' \item cluster: the cluster assignment of observations after cutting the tree
#' \item nbclust: the number of clusters
#' \item silinfo: the silhouette information of observations (available when
#'   \code{k > 1})
#' \item size: the size of clusters
#' \item data: a matrix containing the original or  the standardized data (if stand = TRUE)
#' }
#' 
#' @seealso \code{\link{fviz_dend}}, \code{\link{hkmeans}}, \code{\link{eclust}}
#' @examples 
#' \donttest{
#' data(USArrests)
#' 
#' # Compute hierarchical clustering and cut into 4 clusters
#' res <- hcut(USArrests, k = 4, stand = TRUE)
#' 
#' # Cluster assignments of observations
#' res$cluster
#' # Size of clusters
#' res$size
#' 
#' # Visualize the dendrogram
#' fviz_dend(res, rect = TRUE)
#' 
#' # Visualize the silhouette
#' fviz_silhouette(res)
#' 
#' # Visualize clusters as scatter plots
#' fviz_cluster(res)
#' }
#' 
#' @export
hcut <- function(x, k = 2, isdiss = inherits(x, "dist"), 
                 hc_func = c("hclust", "agnes", "diana"),
                 hc_method = "ward.D2", hc_metric = "euclidean",
                 stand = FALSE, graph = FALSE, ...){
  
  if(!inherits(x, c("matrix", "data.frame", "dist")))
    stop("The data must be of class matrix, data.frame, or dist")
  if(!is.numeric(k) || length(k) != 1L || is.na(k) || k %% 1 != 0 || k < 2)
    stop("k must be a single integer >= 2")
  # NOTE: do NOT pre-empt k > n_obs with a custom stop() here. stats::cutree()
  # below already errors ("elements of 'k' must be between 1 and N") for k > n,
  # and the reverse dependency chooseGCM pins that exact native message. A custom
  # early error broke chooseGCM on CRAN (factoextra 2.0.0/2.1.0). Keep the native
  # error. See tests/testthat/test-regressions.R (chooseGCM revdep guard).
  if(isdiss && !inherits(x, "dist"))
    stop("When isdiss = TRUE, x must be an object of class dist")
  if(isdiss)
    .check_dist_finite(x)
  if(stand && !inherits(x, "dist")) {
    x <- scale(x)
    if(anyNA(x))
      stop("Scaling produced NA values. Check for constant columns or non-finite values.")
  }
  data <- x
  
  hc_func <- match.arg(hc_func)
  hc_func <- hc_func[1]
  
  if(!isdiss) x <- get_dist(x, method = hc_metric)

  if(hc_func == "hclust") hc <- stats::hclust(x, method = hc_method)
  else if(hc_func == "agnes") {
    if(hc_method %in%c("ward.D", "ward.D2")) hc_method = "ward"
    hc <- cluster::agnes(x, method = hc_method)
  }
  else if(hc_func == "diana") hc <- cluster::diana(x)
  else stop("Don't support the function ", hc_func)
  hc.cut <- stats::cutree(hc, k = k)
  hc$cluster = hc.cut
  hc$nbclust <- k
  if(k > 1)  hc$silinfo <- .get_silinfo(hc.cut, x)
  hc$size <- as.vector(table(hc.cut))
  hc$data <- data
  class(hc) <- c(class(hc), "hcut")
  if(graph) fviz_dend(hc)
  hc
}

#' Enhanced Visualization of Dendrogram
#' 
#' @description Draws easily beautiful dendrograms using either R base plot or 
#'   ggplot2. Provides also an option for drawing a circular dendrogram and 
#'   phylogenic trees.
#' @param x an object of class dendrogram, hclust, agnes, diana, hcut, 
#'   hkmeans or HCPC (FactoMineR).
#' @param k the number of groups for cutting the tree.
#' @param h a numeric value. Cut the dendrogram by cutting at height h. (k 
#'   overrides h)
#' @param k_colors,palette a vector containing colors to be used for the groups.
#'   It should contains k number of colors. Allowed values include also "grey" 
#'   for grey color palettes; brewer palettes e.g. "RdBu", "Blues", ...;  and 
#'   scientific journal palettes from ggsci R package, e.g.: "npg", "aaas", 
#'   "lancet", "jco", "ucscgb", "uchicago", "simpsons" and "rickandmorty".
#' @param show_labels a logical value. If TRUE, leaf labels are shown. Default 
#'   value is TRUE.
#' @param color_labels_by_k logical value. If TRUE, labels are colored
#'   automatically by group when k != NULL.
#' @param match_coord_colors logical value. Default is FALSE, where dendrogram
#'   colors follow the left-to-right leaf order. If TRUE, cluster colors are
#'   remapped to cluster-label order so they match \code{\link{fviz_cluster}()}
#'   and \code{\link{fviz_silhouette}()} for the same clustering.
#' @param label_cols a vector containing the colors for labels.
#' @param labels_font font face for the leaf labels of "rectangle"/"circular"
#'   dendrograms. One of "plain" (default), "bold", "italic" or "bold.italic".
#'   Default "plain" leaves labels unchanged.
#' @param labels_track_height a positive numeric value for adjusting the room for the 
#'   labels. Used only when type = "rectangle".
#' @param repel logical value. Use repel = TRUE to avoid label overplotting when
#'   type = "phylogenic".
#' @param lwd a numeric value specifying dendrogram branch and rectangle line
#'   width.
#' @param type type of plot. Allowed values are one of "rectangle", "triangle", 
#'   "circular", "phylogenic".
#' @param phylo_layout the layout to be used for phylogenic trees. Default value
#'   is "layout.auto", which is kept as a compatibility alias for
#'   \code{"layout_nicely"}. Allowed values include:
#'   \code{\link[igraph]{layout.auto}}, \code{\link[igraph]{layout_nicely}},
#'   \code{\link[igraph]{layout_with_drl}}, \code{\link[igraph]{layout_as_tree}},
#'   \code{\link[igraph]{layout.gem}}, \code{\link[igraph]{layout_with_gem}},
#'   \code{\link[igraph]{layout.mds}}, \code{\link[igraph]{layout_with_mds}} and
#'   \code{\link[igraph]{layout_with_lgl}}.
#' @param rect logical value specifying whether to add a rectangle around 
#'   groups. Used only when k != NULL.
#' @param rect_border,rect_lty border color and line type for rectangles.
#' @param rect_fill a logical value. If TRUE, fill the rectangle.
#' @param lower_rect a value of how low should the lower part of the rectangle 
#'   around clusters. Ignored when rect = FALSE.
#' @param horiz a logical value. If TRUE, an horizontal dendrogram is drawn.
#' @param cex size of labels
#' @param main,xlab,ylab main and axis titles
#' @param sub Plot subtitle. Default is NULL (no subtitle). Set to a character
#'   string to display a subtitle below the title, e.g.
#'   \code{sub = paste0("Method: ", "ward.D2")}.
#' @param ggtheme function, ggplot2 theme name. Default value is 
#'   theme_classic(). Allowed values include ggplot2 official themes: 
#'   theme_gray(), theme_bw(), theme_minimal(), theme_classic(), theme_void(), 
#'   ....
#' @param ... other arguments to be passed to the function plot.dendrogram()
#' @return an object of class fviz_dend which is a ggplot with the attributes 
#'   "dendrogram" accessible using attr(x, "dendrogram"), where x is the result 
#'   of fviz_dend().
#' @examples 
#' \donttest{
#' # Load and scale the data
#' data(USArrests)
#' df <- scale(USArrests)
#' 
#' # Hierarchical clustering
#' res.hc <- hclust(dist(df))
#' 
#' # Default plot
#' fviz_dend(res.hc)
#'
#' # Increase branch and rectangle line widths
#' fviz_dend(res.hc, lwd = 2)
#' 
#' # Cut the tree
#' fviz_dend(res.hc, cex = 0.5, k = 4, color_labels_by_k = TRUE)
#' 
#' # Don't color labels, add rectangles
#' fviz_dend(res.hc, cex = 0.5, k = 4, 
#'  color_labels_by_k = FALSE, rect = TRUE)
#'  
#' # Change the color of tree using black color for all groups
#' # Change rectangle border colors
#' fviz_dend(res.hc, rect = TRUE, k_colors ="black",
#' rect_border = 2:5, rect_lty = 1)
#' 
#' # Customized color for groups
#' fviz_dend(res.hc, k = 4, 
#'  k_colors = c("#1B9E77", "#D95F02", "#7570B3", "#E7298A"))
#'  
#'  
#'  # Color labels using k-means clusters
#'  km.clust <- kmeans(df, 4)$cluster
#'  fviz_dend(res.hc, k = 4, 
#'    k_colors = c("blue", "green3", "red", "black"),
#'    label_cols =  km.clust[res.hc$order], cex = 0.6)
#' 
#'  # Phylogenic tree layouts support both compatibility aliases and
#'  # current igraph layout names
#'  if (requireNamespace("igraph", quietly = TRUE)) {
#'    fviz_dend(res.hc, type = "phylogenic", phylo_layout = "layout_nicely",
#'              show_labels = FALSE)
#'  }
#' 
#' }
#' @export
fviz_dend <- function(x, k = NULL, h = NULL, k_colors = NULL, palette = NULL,  show_labels = TRUE, color_labels_by_k = TRUE,
                      match_coord_colors = FALSE,
                      label_cols = NULL, labels_font = "plain", labels_track_height = NULL, repel = FALSE, lwd = 0.7,
                      type = c("rectangle",  "circular", "phylogenic"),
                      phylo_layout = "layout.auto",
                      rect = FALSE, rect_border = "gray", rect_lty = 2, rect_fill = FALSE, lower_rect,
                      horiz = FALSE, cex = 0.8, main = "Cluster Dendrogram", xlab = "", ylab = "Height", 
                      sub = NULL, ggtheme = theme_classic(),  ...)
{
  
  if(missing(k_colors) && !is.null(palette)) {
    k_colors <- palette
    palette <- NULL
  }
  if(!color_labels_by_k && is.null(label_cols)) label_cols <- "black"
  type <- match.arg(type)
  circular <- type == "circular"
  phylogenic <- type == "phylogenic"
  rectangle <- type == "rectangle"
  
  if(inherits(x, "HCPC")){
    # Honor an explicit k; default to the number of HCPC clusters when k is
    # NULL. The previous code always overwrote k, so a user-supplied k (e.g.
    # fviz_dend(hcpc, k = 5)) was silently ignored and colored at the HCPC
    # cluster count instead. (#81)
    if(is.null(k)) k <- length(unique(x$data.clust$clust))
    x <- x$call$t$tree #hclust
  }
    
  if(inherits(x, "hcut")){
    k <- x$nbclust
    dend <- as.dendrogram(x)
    method <- x$method
  }
  else if(inherits(x, "hkmeans")){
    k <- length(unique(x$cluster))
    dend <- as.dendrogram(x$hclust)
    method <- x$hclust$method
  }
  else if(inherits(x, c("hclust", "agnes", "diana"))) {
    dend <- as.dendrogram(x)
    method <- x$method
  }
  else if(inherits(x, "dendrogram")) {
    dend <- x
    method <- ""
  }
  else stop("Can't handle an object of class ", paste(class(x), collapse =", ") )
  if(is.null(method)) method <- ""
  else if(is.na(method)) method <- ""
  # `sub` defaults to NULL (no subtitle), preserving the previous appearance.
  # When set, it is rendered via labs(subtitle=) below (#54).
  
  if(!is.null(dendextend::labels_cex(dend))) cex <- dendextend::labels_cex(dend)
  dend <- dendextend::set(dend, "labels_cex", cex) 
  dend <- dendextend::set(dend, "branches_lwd", lwd) 
  
  k <- .get_k(dend, k, h)
  if(!is.null(k)) {
    if(.is_color_palette(k_colors)) k_colors <- ggpubr::get_palette(k_colors, k = k)
    else if(is.null(k_colors)) k_colors <- ggpubr::get_palette("default", k = k)
    # By default colours follow the left-to-right leaf order (dendextend). With
    # match_coord_colors = TRUE, remap them to cluster-label order so they match
    # fviz_cluster()/fviz_silhouette() for the same clustering (#103).
    branch_colors <- k_colors
    if(match_coord_colors){
      ord <- .coord_color_order(dend, k)
      if(!is.null(ord)) branch_colors <- k_colors[ord]
    }
    dend <- dendextend::set(dend, what = "branches_k_color", k = k, value = branch_colors)
    if(color_labels_by_k) dend <- dendextend::set(dend, "labels_col",  k = k, value = branch_colors)
  }
  
  if(!is.null(label_cols)){
    dend <- dendextend::set(dend, "labels_col", label_cols) 
  }
  
  leaflab <- ifelse(show_labels, "perpendicular", "none")
  
  if(xlab =="") xlab <- NULL
  if(ylab=="") ylab <- NULL
  
  max_height <- max(dendextend::get_branches_heights(dend))
  if(missing(labels_track_height))
    labels_track_height <- max_height/8
  if(max_height < 1) offset_labels <- -max_height/100
  else offset_labels <- -0.1
  
  if(rectangle || circular){
    p <- .ggplot_dend(dend, type = "rectangle", offset_labels = offset_labels, nodes = FALSE,
                      ggtheme = ggtheme, horiz = horiz, circular = circular, palette = palette,
                      labels = show_labels, label_cols = label_cols, labels_font = labels_font,
                      labels_track_height = labels_track_height, ...)
    if(!circular) p <- p + labs(title = main, subtitle = sub, x = xlab, y = ylab)
  }
  else if(phylogenic){
    p <- .phylogenic_tree(dend, labels = show_labels, label_cols = label_cols,
                                 palette = palette, repel = repel,
                                 ggtheme = ggtheme, phylo_layout = phylo_layout, ...)
  }
  # Add rectangle around clusters
  if(circular || phylogenic || is.null(k)) rect <- FALSE
  if(rect_fill && missing(rect_lty)) rect_lty = "blank"
  if(missing(lower_rect)) {
    # The absolute -0.5 offset overwhelms short trees (e.g. correlation/gower
    # distances with max height < 1), pushing the rectangles far below the
    # labels. For those, scale the offset to the tree height instead; taller
    # trees (>= 1) keep the previous default unchanged (#55).
    lower_rect <- if(max_height < 1) -(labels_track_height + max_height/8)
                  else -(labels_track_height + 0.5)
  }
  if(rect){
    p <- p + .rect_dendrogram(dend, k = k, palette = rect_border, rect_fill = rect_fill,
                              rect_lty = rect_lty, linewidth = lwd,
                              lower_rect = lower_rect, match_coord_colors = match_coord_colors)
  }
  
  # Keep the leaf-label text layer out of the legend (no stray "a" glyph),
  # mirroring the scatter-plot cleanup in .fviz_finish() (#14).
  p <- .hide_text_legend(p)

  attr(p, "dendrogram") <- dend
  structure(p, class = c(class(p), "fviz_dend"))
  return(p)
}


# require igraph
.phylogenic_tree <- function(dend, labels = TRUE, label_cols = NULL,
                              palette = NULL, repel = FALSE,
                              ggtheme = theme_classic(),
                             phylo_layout = "layout.auto", ...){
  
  if (!requireNamespace("igraph", quietly = TRUE)) {
    stop("igraph package needed for phylogenic tree. Please install it using install.packages('igraph').")
  }
  
  
  allowed_layouts <- c("layout.auto", "layout_nicely", "layout_with_drl", "layout_as_tree",
                      "layout.gem", "layout_with_gem", "layout.mds", "layout_with_mds", "layout_with_lgl")
  
  if(!(phylo_layout %in% allowed_layouts)) stop( phylo_layout, " is not supported as layout. ", "Allowed phylogenic layout are: ",
                                                paste( allowed_layouts, collapse = ", "))
  
  layout_func <- switch(phylo_layout,
                        layout.auto = igraph::layout_nicely,
                        layout_nicely = igraph::layout_nicely,
                        layout_with_drl = igraph::layout_with_drl,
                        layout_as_tree = igraph::layout_as_tree,
                        layout.gem = igraph::layout_with_gem,
                        layout_with_gem = igraph::layout_with_gem,
                        layout.mds = igraph::layout_with_mds,
                        layout_with_mds = igraph::layout_with_mds,
                        layout_with_lgl = igraph::layout_with_lgl
                        )
  
  # Convert to 'phylo' object
  hc <- stats::as.hclust(dend)
  phylo_tree <- .as.phylo(hc)
  graph_edges <- phylo_tree$edge
  
  # get graph from edge list
  graph_net <- igraph::graph_from_edgelist(graph_edges)
  
  # extract layout (x-y coords)
  graph_layout <- .with_preserved_seed(123, layout_func(graph_net))
  # number of observations
  nobs <- length(hc$labels)
  
  # draw tree branches
  data.segments <- data.frame(
    x = graph_layout[graph_edges[,1],1], 
    y = graph_layout[graph_edges[,1],2],
    xend = graph_layout[graph_edges[,2],1],
    yend = graph_layout[graph_edges[,2],2]
  )
  
  data.labels <- data.frame(
    x = graph_layout[1:nobs,1], 
    y = graph_layout[1:nobs,2],
    label = phylo_tree$tip.label
  )
  data.labels <- data.labels[order(as.vector(data.labels$label)), ]
  
  # Support for dendextend
  gdend <- dendextend::as.ggdend(dend)
  gdat <- dendextend::prepare.ggdend(gdend)
  gdat$labels <- gdat$labels[order(as.vector(gdat$labels$label)), ]
  data.labels <- cbind(data.labels, gdat$labels[, c("col", "cex")])
  if(!is.null(dendextend::labels_cex(dend))) font.label <- round(dendextend::labels_cex(dend)[1]*12)
  else font.label <- 12
  
  # FIX: ggplot2 3.0.0+ deprecation - aes_string() replaced with aes() + .data pronoun
  # See: https://github.com/kassambara/factoextra/issues/190
  p <- ggplot() + geom_segment(data = data.segments,
                    aes(x = .data[["x"]], y = .data[["y"]], xend = .data[["xend"]], yend = .data[["yend"]]),
                    lineend = "square")
  if(is.null(label_cols)) label_cols <- "col"
  if(!labels) labels <- NULL
  else labels <- "label"
  
  p <- ggpubr::ggscatter(data.labels, "x", "y", label = labels,
                         color = label_cols,
                         ggp = p, repel = repel, font.label = font.label, ...)
  
  if(is.null(palette)) p <- p + scale_colour_identity()
  
  p <- ggpubr::ggpar(p, ggtheme = ggtheme, palette = palette, ...) 
  
  p <- p + theme(axis.title.x = element_blank(), axis.title.y = element_blank(),
                 axis.text= element_blank(), 
                 axis.line = element_blank(), axis.ticks = element_blank(),
                 legend.position = "none")
  p
}


# Helper functions
#%%%%%%%%%%%%%%%%%%%%

# Plot dendrogram using ggplot
# .ggplot_dend derrived from dendextend::ggplot.ggdend
# data: a ggdend class object.
.ggplot_dend <- function (dend, segments = TRUE, labels = TRUE, nodes = TRUE, 
                        horiz = FALSE, ggtheme = theme_classic(), 
                        offset_labels = 0, circular = FALSE, type = "rectangle",
                        palette = NULL, label_cols = NULL, labels_font = "plain", labels_track_height = 1,
                        ...) {
  
  gdend <- dendextend::as.ggdend(dend, type = type)
  #angle <- ifelse(horiz, 0, 90)
  #hjust <- ifelse(horiz, 0, 1)
  gdend$labels$angle <- ifelse(horiz, 0, 90)
  gdend$labels$hjust <- ifelse(horiz, 0, 1)
  gdend$labels$vjust <- 0.5
  if(circular){
    # If circular, change the angle and hjust so that the labels rotate
    if(circular) {
      pms <- .get_label_params(gdend$labels)
      gdend$labels$angle <- pms$angle
      gdend$labels$hjust <- pms$hjust
    }
  }
  
  data <- dendextend::prepare.ggdend(gdend)
  # To avoid overlaping of labels at coord_polar start
  if(circular) {
    n_rows <- nrow(data$labels)
    data$labels$x[1] <- 0.7
    data$labels$vjust[1] <- 1.7
  }

  
  p <- ggplot()
  if (segments) {
    # FIX: ggplot2 3.0.0+ deprecation - aes_string() replaced with aes() + .data pronoun
    # FIX: ggplot2 3.4.0+ deprecation - size replaced with linewidth for line geoms
    # See: https://github.com/kassambara/factoextra/issues/190, #191
    p <- p + geom_segment(data = data$segments,
      aes(x = .data[["x"]], y = .data[["y"]], xend = .data[["xend"]], yend = .data[["yend"]],
                 colour = .data[["col"]], linetype = .data[["lty"]], linewidth = .data[["lwd"]]), lineend = "square") +
      # FIX: ggplot2 3.3.4+ deprecation - use "none" instead of FALSE for guides()
      # See: https://github.com/kassambara/factoextra/issues/179
      guides(linetype = "none", col = "none", linewidth = "none") + #scale_colour_identity() +
      scale_linewidth_identity() + scale_linetype_identity()
    if(is.null(palette)) p <- p + scale_colour_identity()
  }
  if (nodes) {
    # FIX: ggplot2 3.0.0+ deprecation - aes_string() replaced with aes() + .data pronoun
    # See: https://github.com/kassambara/factoextra/issues/190
    p <- p + geom_point(data = data$nodes,
      aes(x = .data[["x"]], y = .data[["y"]], colour = .data[["col"]], shape = .data[["pch"]], size = .data[["cex"]])) +
      # FIX: ggplot2 3.3.4+ deprecation - use "none" instead of FALSE for guides()
      # See: https://github.com/kassambara/factoextra/issues/179
      guides(shape = "none", col = "none", size = "none") +
      scale_shape_identity()
  }
  if (labels) {
    data$labels$cex <- 5 * data$labels$cex
    data$labels$y <- data$labels$y + offset_labels
    if(is.null(label_cols)) label_cols <- "col"
    p <- p + ggpubr::geom_exec(geom_text, data = data$labels,
                               x = "x", y = "y", label = "label", color = label_cols, size = "cex",
                                angle = "angle", hjust = "hjust", vjust = "vjust")
    # Apply a single font face to all leaf labels (e.g. "italic") as a fixed
    # layer parameter. Only when non-default, so the "plain" path is untouched (#121).
    if(!identical(labels_font, "plain"))
      p$layers[[length(p$layers)]]$aes_params$fontface <- labels_font
  }
  p <- ggpubr::ggpar(p, ggtheme = ggtheme, palette = palette, ...) + theme(axis.line = element_blank())
  if (horiz && !circular) {
    p <- p + coord_flip() + scale_y_reverse()+
      theme(axis.text.y = element_blank(), axis.ticks.y = element_blank(),
            axis.text.x = element_text())
  }
  else p <- p + theme(axis.text.x = element_blank(), axis.ticks.x = element_blank())
  
  if(circular){
    p <- p + theme(plot.margin = margin(0, 0, 0, 0),
                  axis.title.x = element_blank(), axis.title.y = element_blank(),
                  axis.text= element_blank(), 
                  axis.line = element_blank(), axis.ticks = element_blank())+
                  ylim(max(dendextend::get_branches_heights(dend)), -1)+
                  coord_polar(theta = 'x', direction = 1)
    
  }
  else{
    p <- p + expand_limits(y=-labels_track_height)
  }
  
  p
}

# Function used for circular dendrogram
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# Create the angle and hjust vectors so that the labels 
# rotation switches from 6 o'clock to 12 o'clock to improve readability. 
.get_label_params <- function(labeldf) {        
  nn <- length(labeldf$y)
  halfn <- floor(nn/2)
  firsthalf <- rev(90 + seq(0,360, length.out = nn))
  secondhalf <- rev(-90 + seq(0,360, length.out = nn))
  angle <- numeric(nn)
  angle[1:halfn] <- firsthalf[1:halfn]
  angle[(halfn+1):nn] <- secondhalf[(halfn+1):nn]
  
  hjust <- numeric(nn)
  hjust[1:halfn] <- 0
  hjust[(halfn+1):nn] <- 1
  
  return(list(angle = angle, hjust = hjust))
}


# Convert 'hclust' to 'phylo' object
# used for phylogenic tree
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# from ape::as.phylo.hclust
# x hclust
.as.phylo <- function (x, ...) 
{
  N <- dim(x$merge)[1]
  edge <- matrix(0L, 2 * N, 2)
  edge.length <- numeric(2 * N)
  node <- integer(N)
  node[N] <- N + 2L
  cur.nod <- N + 3L
  j <- 1L
  for (i in N:1) {
    edge[j:(j + 1), 1] <- node[i]
    for (l in 1:2) {
      k <- j + l - 1L
      y <- x$merge[i, l]
      if (y > 0) {
        edge[k, 2] <- node[y] <- cur.nod
        cur.nod <- cur.nod + 1L
        edge.length[k] <- x$height[i] - x$height[y]
      }
      else {
        edge[k, 2] <- -y
        edge.length[k] <- x$height[i]
      }
    }
    j <- j + 2L
  }
  if (is.null(x$labels)) 
    x$labels <- as.character(1:(N + 1))
  obj <- list(edge = edge, edge.length = edge.length/2, tip.label = x$labels, 
              Nnode = N)
  class(obj) <- "phylo"
  obj
  #ape::reorder.phylo(obj)
}

# Get k value if h specified
# Make also some checking
# dend a dendrogram object
# h: tree height
.get_k <- function(dend, k = NULL, h = NULL){
  
  if (!dendextend::is.dendrogram(dend)) stop("x is not a dendrogram object.")
  if (length(h) > 1L || length(k) > 1L) 
    stop("'k' and 'h' must be a scalar(i.e.: of length 1)")
  tree_heights <- dendextend::heights_per_k.dendrogram(dend)[-1]
  tree_order <- stats::order.dendrogram(dend)
  if (!is.null(h)) {
    if (!is.null(k)) 
      stop("specify exactly one of 'k' and 'h'")
    ss_ks <- tree_heights < h
    k <- min(as.numeric(names(ss_ks))[ss_ks])
    k <- max(k, 2)
  }
  k
}


# Left-to-right cluster-label order of a dendrogram cut into k groups.
# dendextend colours branches in leaf (left-to-right) order, whereas
# fviz_cluster()/fviz_silhouette() colour by cluster label. Remapping a length-k
# colour vector by this order makes the colours match across the three plots
# (#103). Returns NULL when the cut is not a clean permutation of seq_len(k).
.coord_color_order <- function(dend, k){
  # dendextend colours the g-th cluster in LEAF (left-to-right) order with
  # value[g]. fviz_cluster()/fviz_silhouette() colour by the DATA-order cluster
  # label (cutree's labelling). So value[g] should be k_colors[ data-label of the
  # g-th leaf cluster ]. That permutation is the data-order labels read in leaf
  # order: unique(cutree(.., as_data = TRUE)[order.dendrogram(dend)]).
  cl  <- tryCatch(dendextend::cutree(dend, k = k, order_clusters_as_data = TRUE),
                  error = function(e) NULL)
  ord <- tryCatch(stats::order.dendrogram(dend), error = function(e) NULL)
  if(is.null(cl) || is.null(ord) || length(cl) != length(ord)) return(NULL)
  lr <- unique(cl[ord])
  lr <- lr[!is.na(lr)]
  if(length(lr) == k && setequal(lr, seq_len(k))) as.integer(lr) else NULL
}

# Add rectangle to a dendrogram
# lower_rect: a (scalar) value of how low should the lower part of the rect be.
.rect_dendrogram <- function (dend, k = NULL,  h = NULL,
                             k_colors = NULL, palette = NULL, rect_fill = FALSE, rect_lty = 2,
                             lower_rect=-1.5, match_coord_colors = FALSE,
          ...)
{

  if(missing(k_colors) && !is.null(palette)) k_colors <- palette
  # value (should be between 0 to 1): proportion of the height 
  # our rect will be between the height needed for k and k+1 clustering.
  prop_k_height <-  0.5
  
  if (!dendextend::is.dendrogram(dend)) 
    stop("x is not a dendrogram object.")
  
  k <- .get_k(dend, k, h)
  tree_heights <- dendextend::heights_per_k.dendrogram(dend)[-1]
  tree_order <- stats::order.dendrogram(dend)
  
  if (is.null(k)) stop("specify k")
  if (k < 2) {
      stop(gettextf("k must be between 2 and %d", length(tree_heights)), 
              domain = NA)
  }

  cluster <- dendextend::cutree(dend, k = k)
  clustab <- table(cluster)[unique(cluster[tree_order])]
  m <- c(0, cumsum(clustab))
  which <- 1L:k
  
  # Collapse height lookups to a single value: tied merge heights can make
  # `names(tree_heights) == k` match more than one entry, which would make
  # ytop (and thus the rectangle data frame) longer than k rows and break the
  # geom_rect aesthetics. Well-behaved dendrograms match exactly one (#154, #168).
  k_height <- tree_heights[names(tree_heights) == k]
  k_height <- if (length(k_height) == 0) 0 else max(k_height)
  next_k_height <- tree_heights[names(tree_heights) == k + 1]
  if (length(next_k_height) == 0) {
    next_k_height <- 0
    prop_k_height <- 1
  } else next_k_height <- max(next_k_height)

  xleft <- ybottom <- xright <- ytop <- list()
  for (n in seq_along(which)) {
    xleft[[n]] = m[which[n]] + 0.66
    ybottom[[n]] = lower_rect
    xright[[n]] = m[which[n] + 1] + 0.33
    ytop[[n]] <- k_height * prop_k_height + next_k_height * (1 - prop_k_height)
  }
  
  df <- data.frame(xmin = unlist(xleft), ymin = unlist(ybottom), xmax = unlist(xright), ymax = unlist(ytop))
  
  color <- k_colors
  # FIX: R 4.2.0+ deprecation - condition has length > 1 warning
  # Use length() == 1 check before scalar comparison
  # See: https://github.com/kassambara/factoextra/issues/163, #180
  if(length(color) == 1 && color == "cluster") color <- "default"
  if(.is_color_palette(color)) color <- ggpubr::get_palette(color, k = k)
  else if(length(color) > 1 && length(color) != k){
    # recycle/trim a multi-colour vector to exactly k (one per rectangle)
    color <- rep_len(color, k)
  }
  # Align rectangle colours to cluster-label order (matches branch/label colours
  # and fviz_cluster()/fviz_silhouette()) when requested (#103).
  if(match_coord_colors && length(color) == k){
    ord <- .coord_color_order(dend, k)
    if(!is.null(ord)) color <- color[ord]
  }
  if(rect_fill){
    fill <- color
    alpha <- 0.2
  }
  else {
    fill <- "transparent"
    alpha <- 0
  }
  df$color <- color
  
  df$cluster <- as.factor(paste0("c", 1:k))
  ggpubr::geom_exec(geom_rect, data = df, 
                    xmin = "xmin", ymin = "ymin", xmax = "xmax", ymax = "ymax",
                    fill = fill, color = color, linetype = rect_lty, alpha = alpha,  ...)
}

#' @include utilities.R get_pca.R eigenvalue.R
NULL
#' Subset and summarize the output of factor analyses
#' 
#' @description Subset and summarize the results of Principal Component
#'   Analysis (PCA), Correspondence Analysis (CA), Multiple Correspondence
#'   Analysis (MCA), Factor Analysis of Mixed Data (FAMD), Multiple Factor
#'   Analysis (MFA) and Hierarchical Multiple Factor Analysis (HMFA) functions
#'   from several packages. Axis indices are validated before extraction, and
#'   MCA quantitative supplementary summaries inherit the package-level error
#'   raised when that result is unavailable.
#' @param X an object of class PCA, CA, MCA, FAMD, MFA and HMFA [FactoMineR]; prcomp
#'   and princomp [stats]; dudi, pca, coa and acm [ade4]; ca [ca package]; expoOutput [ExPosition].
#' @param element the element to subset from the output. Possible values are
#'   "row" or "col" for CA; "var", "ind", "mca.cor" or "quanti.sup" for MCA;
#'   "var" or "ind" for PCA; and 'quanti.var', 'quali.var', 'quali.sup',
#'   'group' or 'ind' for FAMD, MFA and HMFA.
#' @param result the result to be extracted for the element. Possible values are
#'   the combination of c("cos2", "contrib", "coord")
#' @param group.names a vector containing the name of the groups (by default, 
#'   NULL and the group are named group.1, group.2 and so on).
#' @param node.level a single number indicating the HMFA node level.
#' @param axes a numeric vector specifying the axes of interest. Values must be
#'   positive integer indices within the available dimensions. Default values
#'   are 1:2 for axes 1 and 2.
#' @param select a selection of variables. Allowed values are NULL or a list 
#'   containing the arguments name, cos2 or contrib. Default is list(name = 
#'   NULL, cos2 = NULL, contrib = NULL): \itemize{ \item name: is a character 
#'   vector containing variable names to be selected \item cos2: if cos2 is in 
#'   [0, 1], ex: 0.6, then variables with a cos2 > 0.6 are selected. if cos2 > 
#'   1, ex: 5, then the top 5 variables with the highest cos2 are selected \item
#'   contrib: if contrib > 1, ex: 5,  then the top 5 variables with the highest 
#'   cos2 are selected. }
#' @return A data frame containing the (total) coord, cos2 and the contribution 
#'   for the axes.
#' @details If length(axes) > 1, then the columns contrib and cos2 correspond to
#'   the total contributions and total cos2 of the axes. In this case, the 
#'   column coord is calculated as x^2 + y^2 + ...+; x, y, ... are the 
#'   coordinates of the points on the specified axes.
#' @author Alboukadel Kassambara \email{alboukadel.kassambara@@gmail.com}
#' @references \url{https://www.sthda.com/english/}
#' @examples
#' # Principal component analysis
#' # +++++++++++++++++++++++++++++
#' data(decathlon2)
#' decathlon2.active <- decathlon2[1:23, 1:10]
#' res.pca <- prcomp(decathlon2.active,  scale = TRUE)
#' 
#' # Summarize variables on axes 1:2
#' facto_summarize(res.pca, "var", axes = 1:2)[,-1]
#' # Select the top 5 contributing variables
#' facto_summarize(res.pca, "var", axes = 1:2,
#'            select = list(contrib = 5))[,-1]
#' # Select variables with cos2 >= 0.6
#' facto_summarize(res.pca, "var", axes = 1:2,
#'            select = list(cos2 = 0.6))[,-1]
#' # Select by names
#' facto_summarize(res.pca, "var", axes = 1:2,
#'      select = list(name = c("X100m", "Discus", "Javeline")))[,-1]
#'            
#' # Summarize individuals on axes 1:2
#' facto_summarize(res.pca, "ind", axes = 1:2)[,-1]
#' 
#' # Correspondence Analysis
#' # ++++++++++++++++++++++++++
#' # Install and load FactoMineR to compute CA
#' # install.packages("FactoMineR")
#' library("FactoMineR")
#' data("housetasks")
#' res.ca <- CA(housetasks, graph = FALSE)
#' # Summarize row variables on axes 1:2
#' facto_summarize(res.ca, "row", axes = 1:2)[,-1]
#' # Summarize column variables on axes 1:2
#' facto_summarize(res.ca, "col", axes = 1:2)[,-1]
#' 
#' # Multiple Correspondence Analysis
#' # +++++++++++++++++++++++++++++++++
#' library(FactoMineR)
#' data(poison)
#' res.mca <- MCA(poison, quanti.sup = 1:2, 
#'               quali.sup = 3:4, graph=FALSE)
#' # Summarize variables on axes 1:2
#' res <- facto_summarize(res.mca, "var", axes = 1:2)
#' head(res)
#' # Summarize individuals on axes 1:2
#' res <- facto_summarize(res.mca, "ind", axes = 1:2)
#' head(res)
#' # Summarize quantitative supplementary variables on axes 1:2
#' res <- facto_summarize(res.mca, "quanti.sup", axes = 1:2)
#' head(res)
#' 
#' # Multiple factor Analysis
#' # +++++++++++++++++++++++++++++++++
#' library(FactoMineR)
#' data(poison)
#' res.mfa <- MFA(poison, group=c(2,2,5,6), type=c("s","n","n","n"),
#'                name.group=c("desc","desc2","symptom","eat"),
#'                num.group.sup=1:2, graph=FALSE)
#' # Summarize categorical variables on axes 1:2
#' res <- facto_summarize(res.mfa, "quali.var", axes = 1:2)
#' head(res)
#' # Summarize individuals on axes 1:2
#' res <- facto_summarize(res.mfa, "ind", axes = 1:2)
#' head(res)
#' @export
facto_summarize <- function(X, element, node.level = 1, group.names, 
                            result = c("coord", "cos2", "contrib"),
                            axes=1:2, select = NULL)
                            
  { 
  # check element
  allowed_elmts <- c("row", "col", "var", "ind", "quanti.var", "quali.var",
                     "quali.sup",
                     "mca.cor", "quanti.sup",  "group", "partial.axes", "partial.node")
  if(!element %in% allowed_elmts) stop("Can't handle element = '", element, "'") 
  if(element %in% c("mca.cor", "quanti.sup")) {
    if(!inherits(X, "MCA")) stop("element = 'mca_cor' is supported only for FactoMineR::MCA().")
    result <- NULL
  }
  if(element == "quali.sup") {
    result <- intersect(result, c("coord", "cos2"))
  }
  
  # Check and get the classe of X
  facto_class <- .get_facto_class(X)
  # Extract the element
  element <- element[1]
  elmt <- switch(facto_class,
                 CA = get_ca(X, element),
                 PCA = get_pca(X, element),
                 MCA = get_mca(X, element),
                 FAMD = get_famd(X, element),
                 MFA = get_mfa(X, element),
                 HMFA = get_hmfa(X, element)
                 )
  
  # Check axes
  if(inherits(elmt, "hmfa_partial"))
    ndim <- ncol(elmt[[1]])
  else
    ndim <- ncol(elmt$coord)
  axes <- .validate_axis_indices(axes, ndim = ndim)
  
  # Summarize the result
  res = NULL
  # Fix for PR #143: respect axes parameter for mca.cor and quanti.sup
  if(element %in% c("mca.cor", "quanti.sup")) res <- elmt$coord[, axes, drop = FALSE]
  
  # 1.Extract the coordinates x, y and coord
  # OPTIMIZATION: Replaced apply(dd^2, 1, sum) with rowSums(dd^2)
  # rowSums() is significantly faster than apply() for row summation
  if("coord" %in% result){
    dd <- data.frame(elmt$coord[, axes, drop=FALSE])
    coord <- rowSums(dd^2) # x^2 + y2 + ... (vectorized)
    res = cbind(dd, coord = coord)
  }

  # 2. Extract the cos2
  # OPTIMIZATION: Replaced apply() with rowSums() for multi-axis case
  if("cos2" %in% result){
    cos2 <- elmt$cos2[, axes, drop = FALSE]
    if(length(axes) > 1) {
      cos2 <- rowSums(cos2, na.rm=TRUE)
    } else {
      # Single axis - convert to vector to ensure column name is 'cos2'
      cos2 <- as.vector(cos2)
    }
    res <- cbind(res, cos2 = cos2)
  }

  # 3. Extract the contribution
  # OPTIMIZATION: Replaced t(apply()) with sweep() and rowSums()
  # sweep() for element-wise multiplication, rowSums() for summation
  if("contrib" %in% result){
    contrib <- elmt$contrib[, axes, drop = FALSE]
    if(length(axes) > 1) {
      eig <- get_eigenvalue(X)[axes,1]
      # OPTIMIZED: Adjust variable contributions by the Dimension eigenvalues
      # Using sweep() instead of t(apply()) - much faster
      contrib <- sweep(contrib, 2, eig, "*")
      contrib <- rowSums(contrib) / sum(eig)
    } else {
      # Single axis - convert to vector to ensure column name is 'contrib'
      contrib <- as.vector(contrib)
    }
    res <- cbind(res, contrib = contrib)
  }
  
  # 4.Extract the coordinates x, y and coord partial - MFA
  # OPTIMIZATION: Replaced lapply/do.call with vectorized string operations
  # and apply() with rowSums()
  if("coord.partial" %in% result){
    dd <- data.frame(elmt$coord.partiel[, axes, drop=FALSE])
    rnames <- rownames(dd)
    group.names <- .mfa_group_names(X)
    groupnames <- .split_partial_names(rnames, group.names)
    # OPTIMIZED: Use rowSums instead of apply
    coord.partial <- rowSums(dd^2)
    res.partial <- data.frame(groupnames, dd, coord.partial)
  }
  
  # 5. Extract the coordinates x, y and coord partial - HMFA
  # OPTIMIZATION: Pre-allocate vectors and use rowSums instead of apply
  if("coord.node.partial" %in% result){
    # Select hierarchical node
    node <- as.data.frame(elmt[[node.level]])
    n_rows <- nrow(node)
    n_groups <- length(group.names)
    n_axes <- length(axes)

    name <- rep(rownames(node), n_groups)

    # OPTIMIZED: Pre-allocate matrix instead of growing with cbind
    dd <- matrix(0, nrow = n_rows * n_groups, ncol = n_axes)
    dim.group <- character(n_rows * n_groups)

    # Fill the matrix
    row_idx <- 1
    for(j in seq_len(n_groups)) {
      for(i in seq_along(axes)) {
        dim.name <- paste0("Dim", axes[i], ".", j)
        dd[row_idx:(row_idx + n_rows - 1), i] <- node[, dim.name]
      }
      dim.group[row_idx:(row_idx + n_rows - 1)] <- group.names[j]
      row_idx <- row_idx + n_rows
    }

    colnames(dd) <- paste0("Dim", axes)
    dd <- as.data.frame(dd)

    # OPTIMIZED: Use rowSums instead of apply
    coord.partial <- rowSums(dd^2)
    res.partial <- data.frame(group.name = dim.group, name, dd, coord.partial)
  }
  
  if("coord.node.partial" %in% result) 
    res <- res.partial
  else {
    name <- rownames(elmt$coord)
    if(is.null(name)) name <- as.character(seq_len(nrow(elmt$coord)))
    name <- as.character(name)
    # Disambiguate duplicated category names (e.g. FAMD/MFA qualitative
    # variables sharing factor-level names) so they can be used as row names
    # (#184, #140). No-op when names are already unique.
    name <- .disambiguate_category_names(X, name, element, facto_class)
    res <- cbind.data.frame(name = name, res)
    rownames(res) <- name
    if(!is.null(select) && !is.null(select$name) && .factominer_needs_category_map(facto_class, element)){
      select$name <- map_factominer_legacy_names(X, select$name, element = element)
    }
    if(!is.null(select)) res <- .select(res, select)
    if("coord.partial" %in% result){
    res = list(res = res, res.partial = res.partial)
    }
  }
  
  res
}

#' A dataset containing clusters of multiple shapes
#' 
#' @description Data containing clusters of any shapes. Useful for comparing
#' density-based clustering (DBSCAN) and standard partitioning methods such as
#' k-means clustering.
#' @name multishapes
#' @docType data
#' @usage data("multishapes")
#' @format A data frame with 1100 observations on the following 3 variables. 
#'  \describe{
#' \item{\code{x}}{a numeric vector containing the x coordinates of observations}
#' \item{\code{y}}{a numeric vector containing the y coordinates of observations}
#' \item{\code{shape}}{a numeric vector corresponding to the cluster number of each observation.}
#' }
#' @details 
#' The dataset contains 5 clusters and some outliers/noises.
#' 
#' @examples
#' \donttest{
#'data(multishapes)
#'plot(multishapes[,1], multishapes[, 2],
#'     col = multishapes[, 3], pch = 19, cex = 0.8)
#' }
#' 
NULL

#' @include get_pca.R fviz.R
 NULL
#' Visualize Principal Component Analysis
#' 
#' 
#' @description Principal component analysis (PCA) reduces the dimensionality of
#'   multivariate data, to two or three that can be visualized graphically with 
#'   minimal loss of information. fviz_pca() provides ggplot2-based elegant 
#'   visualization of PCA outputs from: i) prcomp and princomp [in built-in R 
#'   stats], ii) PCA [in FactoMineR], iii) dudi.pca [in ade4] and epPCA 
#'   [ExPosition]. Read more: 
#'   \href{https://www.sthda.com/english/articles/31-principal-component-methods-in-r-practical-guide/112-pca-principal-component-analysis-essentials/}{Principal
#'    Component Analysis}
#'   
#'   \itemize{ \item fviz_pca_ind(): Graph of individuals \item fviz_pca_var():
#'   Graph of variables \item fviz_pca_biplot(): Biplot of individuals and
#'   variables \item fviz_pca(): An alias of fviz_pca_biplot() }
#'   
#'   Note that, \code{fviz_pca_xxx()} functions are wrapper around the core
#'   function \code{\link{fviz}()}, which is also a wrapper around the
#'   function \code{\link[ggpubr]{ggscatter}()} [in ggpubr]. Therefore, further arguments, to be
#'   passed to the function \code{\link{fviz}()} and \code{\link[ggpubr]{ggscatter}()}, can be specified in
#'   \code{\link{fviz_pca_ind}()} and \code{\link{fviz_pca_var}()}.
#'   
#' @param X an object of class PCA [FactoMineR]; prcomp and princomp [stats]; 
#'   dudi and pca [ade4]; expOutput/epPCA [ExPosition].
#' @param axes a numeric vector of length 2 specifying the dimensions to be 
#'   plotted.
#' @param geom a text specifying the geometry to be used for the graph. Allowed 
#'   values are the combination of \code{c("point", "arrow", "text")}. Use 
#'   \code{"point"} (to show only points); \code{"text"} to show only labels; 
#'   \code{c("point", "text")} or \code{c("arrow", "text")} to show arrows and 
#'   texts. Using \code{c("arrow", "text")} is sensible only for the graph of 
#'   variables.
#' @param geom.ind,geom.var as \code{geom} but for individuals and variables, 
#'   respectively. Default is geom.ind = c("point", "text), geom.var = 
#'   c("arrow", "text").
#' @param label a text specifying the elements to be labelled. Default value is 
#'   "all". Allowed values are "none" or the combination of c("ind", "ind.sup", 
#'   "quali", "var", "quanti.sup"). "ind" can be used to label only active 
#'   individuals. "ind.sup" is for supplementary individuals. "quali" is for 
#'   supplementary qualitative variables. "var" is for active variables. 
#'   "quanti.sup" is for quantitative supplementary variables.
#' @param invisible a text specifying the elements to be hidden on the plot. 
#'   Default value is "none". Allowed values are the combination of c("ind", 
#'   "ind.sup", "quali", "var", "quanti.sup").
#' @param title the title of the graph
#' @param habillage an optional factor variable for coloring the observations by
#'   groups. Default value is "none". If X is a PCA object from FactoMineR 
#'   package, habillage can also specify the supplementary qualitative variable 
#'   (by its index or name) to be used for coloring individuals by groups (see 
#'   ?PCA in FactoMineR).
#' @param addEllipses logical value. If TRUE, draws ellipses around the 
#'   individuals when habillage != "none".
#' @param col.ind,col.var color for individuals and variables, respectively. Can
#'   be a continuous variable or a factor variable. Possible values include also
#'   : "cos2", "contrib", "coord", "x" or "y". In this case, the colors for 
#'   individuals/variables are automatically controlled by their qualities of 
#'   representation ("cos2"), contributions ("contrib"), coordinates (x^2+y^2, 
#'   "coord"), x values ("x") or y values ("y"). To use automatic coloring (by 
#'   cos2, contrib, ....), make sure that habillage ="none".
#' @param fill.ind,fill.var same as col.ind and col.var but for the fill color.
#' @param shape.ind optional factor variable to map the point \strong{shape} of
#'   individuals, independently of their color. This allows colouring
#'   individuals by one grouping variable (\code{col.ind}/\code{habillage}) and
#'   shaping them by another (e.g. \code{fviz_pca_ind(res, col.ind = group1,
#'   shape.ind = group2)}). Default \code{NULL} keeps the previous behaviour
#'   (shape follows the colour grouping). Use \code{+ labs(shape = , colour = )}
#'   to rename the legends.
#' @param col.ind.sup color for supplementary individuals
#' @param alpha.ind,alpha.var controls the transparency of individual and 
#'   variable colors, respectively. The value can variate from 0 (total 
#'   transparency) to 1 (no transparency). Default value is 1. Possible values 
#'   include also : "cos2", "contrib", "coord", "x" or "y". In this case, the 
#'   transparency for the individual/variable colors are automatically 
#'   controlled by their qualities ("cos2"), contributions ("contrib"), 
#'   coordinates (x^2+y^2, "coord"), x values("x") or y values("y"). To use 
#'   this, make sure that habillage ="none".
#' @param col.quanti.sup a color for the quantitative supplementary variables.
#' @param select.ind,select.var a selection of individuals/variables to be
#'   drawn. Allowed values are NULL or a list containing the arguments name,
#'   cos2 or contrib: \itemize{ \item name: is a character vector containing
#'   individuals/variables to be drawn \item cos2: if cos2 is in [0, 1], ex:
#'   0.6, then individuals/variables with a cos2 > 0.6 are drawn. if cos2 > 1,
#'   ex: 5, then the top 5 individuals/variables with the highest cos2 are
#'   drawn. \item contrib: if contrib > 1, ex: 5,  then the top 5
#'   individuals/variables with the highest contrib are drawn }
#' @param biplot.type type of biplot scaling for fviz_pca_biplot(). Options are:
#'   \itemize{
#'     \item "auto" (default): Uses range-based rescaling for visualization
#'     \item "form": Form-oriented scaling (Gabriel-style). Prioritizes
#'       readability of individual relationships.
#'     \item "covariance": Covariance-oriented scaling (Gabriel-style).
#'       Prioritizes readability of variable relationships.
#'   }
#'   Note: "form" and "covariance" scaling requires prcomp or princomp objects.
#' @inheritParams ggpubr::ggpar
#' @inheritParams fviz
#' @param ... Additional arguments. \itemize{ \item in fviz_pca_ind() and 
#'   fviz_pca_var(): Additional arguments are passed to the functions fviz() and
#'   ggpubr::ggpar(). \item in fviz_pca_biplot() and fviz_pca(): Additional 
#'   arguments are passed to fviz_pca_ind() and fviz_pca_var().}
#'   
#'   
#' @return a ggplot
#' @author Alboukadel Kassambara \email{alboukadel.kassambara@@gmail.com}
#' @seealso \code{\link{fviz_ca}}, \code{\link{fviz_mca}}
#' @examples
#' \donttest{
#' # Principal component analysis
#' # ++++++++++++++++++++++++++++++
#' data(iris)
#' res.pca <- prcomp(iris[, -5],  scale = TRUE)
#' 
#' # Graph of individuals
#' # +++++++++++++++++++++
#' 
#' # Default plot
#' # Use repel = TRUE to avoid overplotting (slow if many points)
#' fviz_pca_ind(res.pca, col.ind = "#00AFBB",
#'    repel = TRUE)
#' 
#'  
#' # 1. Control automatically the color of individuals 
#'    # using the "cos2" or the contributions "contrib"
#'    # cos2 = the quality of the individuals on the factor map
#' # 2. To keep only point or text use geom = "point" or geom = "text".
#' # 3. Change themes using ggtheme: http://www.sthda.com/english/wiki/ggplot2-themes
#' 
#' fviz_pca_ind(res.pca, col.ind="cos2", geom = "point",
#'    gradient.cols = c("white", "#2E9FDF", "#FC4E07" ))
#' 
#' # Color individuals by groups, add concentration ellipses
#' # Change group colors using RColorBrewer color palettes
#' # Read more: http://www.sthda.com/english/wiki/ggplot2-colors
#' # Remove labels: label = "none".
#' fviz_pca_ind(res.pca, label="none", habillage=iris$Species,
#'      addEllipses=TRUE, ellipse.level=0.95, palette = "Dark2")
#'              
#'      
#' # Change group colors manually
#' # Read more: http://www.sthda.com/english/wiki/ggplot2-colors
#' fviz_pca_ind(res.pca, label="none", habillage=iris$Species,
#'      addEllipses=TRUE, ellipse.level=0.95,
#'      palette = c("#999999", "#E69F00", "#56B4E9"))
#'       
#' # Select and visualize some individuals (ind) with select.ind argument.
#'  # - ind with cos2 >= 0.96: select.ind = list(cos2 = 0.96)
#'  # - Top 20 ind according to the cos2: select.ind = list(cos2 = 20)
#'  # - Top 20 contributing individuals: select.ind = list(contrib = 20)
#'  # - Select ind by names: select.ind = list(name = c("23", "42", "119") )
#'  
#'  # Example: Select the top 40 according to the cos2
#' fviz_pca_ind(res.pca, select.ind = list(cos2 = 40))
#' 
#'  
#' # Graph of variables
#' # ++++++++++++++++++++++++++++
#'   
#' # Default plot
#' fviz_pca_var(res.pca, col.var = "steelblue")
#'  
#' # Control variable colors using their contributions
#' fviz_pca_var(res.pca, col.var = "contrib", 
#'    gradient.cols = c("white", "blue", "red"),
#'    ggtheme = theme_minimal())
#'  
#'     
#' # Biplot of individuals and variables
#' # ++++++++++++++++++++++++++
#' # Keep only the labels for variables
#' # Change the color by groups, add ellipses
#' fviz_pca_biplot(res.pca, label = "var", habillage=iris$Species,
#'                addEllipses=TRUE, ellipse.level=0.95,
#'                ggtheme = theme_minimal())
#'
#' # Biplot scaling modes:
#' # Form biplot - focus on individual distances
#' fviz_pca_biplot(res.pca, biplot.type = "form",
#'                label = "var", habillage = iris$Species)
#' # Covariance biplot - focus on variable correlations
#' fviz_pca_biplot(res.pca, biplot.type = "covariance",
#'                label = "var", habillage = iris$Species)
#'
#'  }
#'
#' @rdname fviz_pca
#' @export
fviz_pca <- function(X, ...){
  fviz_pca_biplot(X, ...)
}


#' @rdname fviz_pca 
#' @export 
fviz_pca_ind <- function(X,  axes = c(1,2), geom = c("point", "text"),
                         geom.ind = geom, repel = FALSE,
                         habillage="none", palette = NULL, addEllipses=FALSE,
                         col.ind = "black", fill.ind = "white", col.ind.sup = "blue", alpha.ind =1,
                         shape.ind = NULL,
                         select.ind = list(name = NULL, cos2 = NULL, contrib = NULL),
                         ...)
{

  .args <- list(X = X, element = "ind", axes = axes, geom = geom.ind,
                habillage = habillage, palette = palette, addEllipses = addEllipses,
                color = col.ind, fill = fill.ind, alpha = alpha.ind, col.row.sup = col.ind.sup,
                select = select.ind, repel = repel, ...)
  # shape.ind = a factor maps point shape to a (possibly different) grouping than
  # col.ind, e.g. colour by one variable and shape by another. NULL (default)
  # keeps the previous behavior (shape follows habillage/colour). (#36, #51)
  if(!is.null(shape.ind)) .args$pointshape <- shape.ind
  do.call(fviz, .args)

}


#' @rdname fviz_pca
#' @export 
fviz_pca_var <- function(X, axes=c(1,2), geom = c("arrow", "text"), 
                         geom.var = geom,
                         repel = FALSE, col.var="black", fill.var = "white", alpha.var=1, 
                         col.quanti.sup="blue", col.circle ="grey70", 
                         select.var = list(name = NULL, cos2 = NULL, contrib = NULL),
                          ...)
{
  fviz (X, element = "var", axes = axes, geom = geom.var,
                color = col.var, fill = fill.var, alpha = alpha.var,  select = select.var,
                repel = repel, col.col.sup = col.quanti.sup, 
                col.circle = col.circle,...)
  
}



#' @rdname fviz_pca
#' @export
fviz_pca_biplot <- function(X,  axes = c(1,2), geom = c("point", "text"),
                            geom.ind = geom, geom.var = c("arrow", "text"),
                            col.ind = "black", fill.ind = "white",
                            col.var = "steelblue", fill.var = "white", gradient.cols = NULL,
                            label = "all", invisible="none", repel = FALSE,
                            habillage = "none", palette = NULL, addEllipses=FALSE,
                            shape.ind = NULL,
                            title = "PCA - Biplot",
                            biplot.type = c("auto", "form", "covariance"), ...)
{
  biplot.type <- match.arg(biplot.type)
  
  # Check if individuals or variables are colored by variables
  is.individuals.colored.by.variable <- .is_grouping_var(fill.ind) | .is_grouping_var(col.ind)
  is.variables.colored.by.variable <-  .is_continuous_var(col.var) | .is_grouping_var(col.var)
  # If coloring variables are continuous, then gradient coloring should be applied
  is.gradient.color <- .is_continuous_var(col.ind) | .is_continuous_var(col.var) 
  is.gradient.fill <- .is_continuous_var(fill.ind) | .is_continuous_var(fill.var)
  # If coloring variables are qualitative, then discrete coloring should be applied
  is.discrete.color <- .is_grouping_var(col.ind) | .is_grouping_var(habillage) | .is_grouping_var(col.var)
  is.discrete.fill <- .is_grouping_var(fill.ind) | .is_grouping_var(fill.var) |
                       .is_grouping_var(habillage) | (.is_grouping_var(col.ind) & addEllipses)
  
  
  # Data frame to be used for plotting
  var <- facto_summarize(X, element = "var", 
                         result = c("coord", "contrib", "cos2"), axes = axes)
  colnames(var)[2:3] <-  c("x", "y")
  
  pca.ind <- get_pca_ind(X)
  ind <- data.frame(pca.ind$coord[, axes, drop=FALSE])
  colnames(ind)<- c("x", "y")
  
  # Rescale variable coordinates based on biplot.type

  # Heuristic Gabriel-style scaling choices for improved readability.
  if(biplot.type == "form" || biplot.type == "covariance") {
    # Get eigenvalues (singular values squared for scaling)
    if(inherits(X, "prcomp")) {
      sdev <- X$sdev[axes]
    } else if(inherits(X, "princomp")) {
      sdev <- X$sdev[axes]
    } else {
      # Fall back to auto for other object types
      warning("biplot.type 'form' or 'covariance' requires prcomp/princomp objects. Using 'auto'.")
      biplot.type <- "auto"
    }

    if(biplot.type == "form") {
      # Form biplot (scale=0): preserves distances between individuals
      # Variables are scaled down, individuals keep their scores
      r <- 1 / max(abs(var[, c("x", "y")])) * 0.8 * max(abs(ind[, c("x", "y")]))
    } else if(biplot.type == "covariance") {
      # Covariance biplot (scale=1): preserves correlations between variables
      # Variables scaled by sqrt(n) * sdev, individuals scaled down
      n_obs <- nrow(ind)
      # Scale factor to make variable vectors meaningful while keeping individuals visible
      r <- sqrt(n_obs) * 0.15
    }
  }

  if(biplot.type == "auto") {
    # Default range-based rescaling for visualization
    # Fix for PR #129: correct parenthesis placement for ratio calculation
    ratio_x <- (max(ind[, "x"]) - min(ind[, "x"])) / (max(var[, "x"]) - min(var[, "x"]))
    ratio_y <- (max(ind[, "y"]) - min(ind[, "y"])) / (max(var[, "y"]) - min(var[, "y"]))
    valid_ratios <- c(ratio_x, ratio_y)
    valid_ratios <- valid_ratios[is.finite(valid_ratios) & valid_ratios > 0]
    r <- if(length(valid_ratios) > 0) min(valid_ratios) else 1
  }
  
  # When fill.ind = grouping variable & col.var = continuous variable,
  # we should inactivate ellipse border and ind.point border colors,
  # otherwise --> error: Discrete value supplied to continuous scale
  # Reason: individuals are in discrete color and variable in gradient colors, 
  # and we can't change the color (https://github.com/kassambara/factoextra/issues/42)
  ellipse.border.remove  <- FALSE
  if(is.individuals.colored.by.variable && is.variables.colored.by.variable)
    ellipse.border.remove <- TRUE

  
  
  # Individuals
  p <- fviz_pca_ind(X,  axes = axes, geom = geom.ind, repel = repel,
                    col.ind = col.ind, fill.ind = fill.ind, shape.ind = shape.ind,
                    label = label, invisible=invisible, habillage = habillage,
                    addEllipses = addEllipses, # palette = palette,
                    ellipse.border.remove = ellipse.border.remove,
                    ...)
  # Add variables
  p <- fviz_pca_var(X, axes = axes, geom =  geom.var, repel = repel,
                    col.var = col.var, fill.var = fill.var,
                    label = label, invisible = invisible,
                    scale.= r*0.7, ggp = p,  ...)
  
  if(!is.null(gradient.cols)){
    if(is.gradient.color) p <- p + ggpubr::gradient_color(gradient.cols)
    if(is.gradient.fill) p <- p + ggpubr::gradient_fill(gradient.cols)
  }
  
  if(!is.null(palette)){
    if(is.discrete.color) p <- p + ggpubr::color_palette(palette)
    if(is.discrete.fill) p <- p + ggpubr::fill_palette(palette)
     
  }
  
  p+labs(title=title)
}

test_that("PCA/MCA visualization helpers return ggplot objects", {
  skip_if_not_installed("FactoMineR")

  pca <- FactoMineR::PCA(iris[, 1:4], graph = FALSE)
  p_ind <- fviz_pca_ind(pca)
  p_var <- fviz_pca_var(pca)

  expect_s3_class(p_ind, "ggplot")
  expect_s3_class(p_var, "ggplot")

  set.seed(123)
  mca_df <- data.frame(
    a = factor(sample(letters[1:3], 40, TRUE)),
    b = factor(sample(LETTERS[1:3], 40, TRUE)),
    c = factor(sample(c("x", "y"), 40, TRUE)),
    sup_q = factor(sample(c("q1", "q2"), 40, TRUE)),
    sup_n = rnorm(40)
  )
  mca <- FactoMineR::MCA(mca_df, quali.sup = 4, quanti.sup = 5, graph = FALSE)
  p_mca <- fviz_mca_var(mca, choice = "mca.cor")
  expect_s3_class(p_mca, "ggplot")
})

test_that("cluster visual helpers work on hcut/hkmeans outputs", {
  x <- scale(USArrests)

  hc <- hcut(x, k = 4)
  km <- hkmeans(x, k = 4)

  p1 <- fviz_cluster(hc)
  p2 <- fviz_cluster(km)
  p3 <- fviz_silhouette(hc, print.summary = FALSE)

  expect_s3_class(p1, "ggplot")
  expect_s3_class(p2, "ggplot")
  expect_s3_class(p3, "ggplot")
})

test_that("fviz_dend supports rectangular dendrogram layout", {
  hc <- hclust(dist(iris[, 1:4]))
  p <- fviz_dend(hc, k = 3, rect = TRUE, show_labels = FALSE)
  expect_s3_class(p, "ggplot")
})

test_that("fviz_dend applies lwd to branch segments without adding a linewidth guide", {
  hc <- hclust(dist(scale(USArrests)))

  p_default <- fviz_dend(hc)
  p_custom <- fviz_dend(hc, lwd = 2)

  default_layers <- ggplot2::ggplot_build(p_default)
  custom_layers <- ggplot2::ggplot_build(p_custom)
  legend_labels <- function(plot) {
    gt <- ggplot2::ggplotGrob(plot)
    idx <- which(gt$layout$name == "guide-box-right")
    if(length(idx) == 0) return(character(0))
    legend <- gt$grobs[[idx[1]]]$grobs[[1]]
    extract_labels <- function(grob) {
      out <- character()
      if(inherits(grob, "text")) out <- c(out, grob$label)
      if(!is.null(grob$children)) {
        for(child in grob$children) out <- c(out, extract_labels(child))
      }
      if(!is.null(grob$grobs)) {
        for(child in grob$grobs) out <- c(out, extract_labels(child))
      }
      out
    }
    unique(Filter(nzchar, extract_labels(legend)))
  }

  expect_equal(unique(default_layers$data[[1]]$linewidth), 0.7)
  expect_equal(unique(custom_layers$data[[1]]$linewidth), 2)
  expect_false("lwd" %in% legend_labels(p_custom))
})

test_that("fviz_pca_biplot supports form and covariance scaling modes", {
  res.pca <- stats::prcomp(iris[, 1:4], scale. = TRUE)
  p_form <- fviz_pca_biplot(res.pca, biplot.type = "form")
  p_cov <- fviz_pca_biplot(res.pca, biplot.type = "covariance")
  expect_s3_class(p_form, "ggplot")
  expect_s3_class(p_cov, "ggplot")
})

test_that("fviz_eig parallel analysis is reproducible with parallel.seed", {
  res.pca <- stats::prcomp(iris[, 1:4], scale. = TRUE)

  extract_parallel_threshold <- function(p) {
    layers <- ggplot2::ggplot_build(p)$data
    idx <- which(vapply(layers, function(df) {
      "shape" %in% names(df) && all(df$shape == 4)
    }, logical(1)))
    if(length(idx) == 0) return(numeric(0))
    layers[[idx[1]]]$y
  }

  p1 <- fviz_eig(
    res.pca, choice = "eigenvalue", parallel = TRUE,
    parallel.iter = 20, parallel.seed = 42
  )
  p2 <- fviz_eig(
    res.pca, choice = "eigenvalue", parallel = TRUE,
    parallel.iter = 20, parallel.seed = 42
  )
  p3 <- fviz_eig(
    res.pca, choice = "eigenvalue", parallel = TRUE,
    parallel.iter = 20, parallel.seed = 43
  )

  th1 <- extract_parallel_threshold(p1)
  th2 <- extract_parallel_threshold(p2)
  th3 <- extract_parallel_threshold(p3)
  expect_true(length(th1) > 0)
  expect_equal(th1, th2)
  expect_false(isTRUE(all.equal(th1, th3)))
})

test_that("fviz_famd_var supports supplementary qualitative categories", {
  skip_if_not_installed("FactoMineR")

  set.seed(321)
  famd_df <- data.frame(
    cat_a = factor(sample(c("a", "b", "c"), 60, TRUE)),
    cat_b = factor(sample(c("x", "y"), 60, TRUE)),
    num_a = rnorm(60),
    num_b = rnorm(60),
    sup_q = factor(sample(c("north", "south"), 60, TRUE))
  )
  res.famd <- FactoMineR::FAMD(famd_df, sup.var = 5, graph = FALSE)

  p_sup <- fviz_famd_var(res.famd, choice = "quali.sup", repel = TRUE, col.var = "cos2")
  p_overlay <- fviz_famd_var(res.famd, choice = "quali.var", invisible = "var", repel = TRUE)

  expect_s3_class(p_sup, "ggplot")
  expect_s3_class(p_overlay, "ggplot")
  expect_gt(length(p_overlay$layers), 1)
})

test_that("fviz_famd_ind can show supplementary qualitative categories without active ones", {
  skip_if_not_installed("FactoMineR")

  set.seed(4242)
  famd_df <- data.frame(
    cat_a = factor(sample(c("a", "b", "c"), 60, TRUE)),
    num_a = rnorm(60),
    num_b = rnorm(60),
    sup_q = factor(sample(c("north", "south"), 60, TRUE))
  )
  res.famd <- FactoMineR::FAMD(famd_df, sup.var = 4, graph = FALSE)

  p <- fviz_famd_ind(res.famd, invisible = "quali.var", repel = TRUE)
  layer_rows <- vapply(ggplot2::ggplot_build(p)$data, nrow, integer(1))

  expect_s3_class(p, "ggplot")
  expect_true(any(layer_rows == nlevels(famd_df$sup_q)))
})

test_that("fviz_famd_ind overlays active and supplementary qualitative categories together", {
  skip_if_not_installed("FactoMineR")

  set.seed(4243)
  famd_df <- data.frame(
    cat_a = factor(sample(c("a", "b", "c"), 60, TRUE)),
    num_a = rnorm(60),
    num_b = rnorm(60),
    sup_q = factor(sample(c("north", "south"), 60, TRUE))
  )
  res.famd <- FactoMineR::FAMD(famd_df, sup.var = 4, graph = FALSE)

  p <- fviz_famd_ind(res.famd, repel = TRUE)
  layer_rows <- vapply(ggplot2::ggplot_build(p)$data, nrow, integer(1))

  expect_s3_class(p, "ggplot")
  expect_true(any(layer_rows == nrow(res.famd$quali.var$coord)))
  expect_true(any(layer_rows == nlevels(famd_df$sup_q)))
})

test_that("fviz_famd_ind respects multi-element invisible vectors", {
  skip_if_not_installed("FactoMineR")

  set.seed(4343)
  famd_df <- data.frame(
    cat_a = factor(sample(c("a", "b", "c"), 60, TRUE)),
    num_a = rnorm(60),
    num_b = rnorm(60),
    sup_q = factor(sample(c("north", "south"), 60, TRUE))
  )
  res.famd <- FactoMineR::FAMD(famd_df, sup.var = 4, graph = FALSE)

  p <- fviz_famd_ind(res.famd, invisible = c("quali.var", "ind"), repel = TRUE)
  layer_rows <- vapply(ggplot2::ggplot_build(p)$data, nrow, integer(1))

  expect_s3_class(p, "ggplot")
  expect_true(any(layer_rows == nlevels(famd_df$sup_q)))
  expect_false(any(layer_rows == nrow(famd_df)))
})

test_that("fviz_mfa_var supports supplementary qualitative categories", {
  skip_if_not_installed("FactoMineR")

  data(poison, package = "FactoMineR")
  res.mfa <- FactoMineR::MFA(
    poison,
    group = c(2, 2, 5, 6),
    type = c("s", "n", "n", "n"),
    name.group = c("desc", "desc2", "symptom", "eat"),
    num.group.sup = 1:2,
    graph = FALSE
  )

  p_sup <- fviz_mfa_var(res.mfa, choice = "quali.sup", repel = TRUE, col.var = "cos2")
  p_overlay <- fviz_mfa_var(res.mfa, choice = "quali.var", invisible = "quali.var", repel = TRUE)
  p_biplot <- fviz_mfa_quali_biplot(res.mfa, repel = TRUE)
  p_biplot_default <- fviz_mfa_quali_biplot(res.mfa)
  p_biplot_hidden <- fviz_mfa_quali_biplot(res.mfa, repel = TRUE, invisible = "ind")
  p_biplot_choice <- fviz_mfa_quali_biplot(res.mfa, choice = "group")
  p_biplot_ggp <- fviz_mfa_quali_biplot(res.mfa, ggp = ggplot2::ggplot())

  overlay_rows <- vapply(ggplot2::ggplot_build(p_overlay)$data, nrow, integer(1))
  biplot_rows <- vapply(ggplot2::ggplot_build(p_biplot)$data, nrow, integer(1))
  biplot_default_rows <- vapply(ggplot2::ggplot_build(p_biplot_default)$data, nrow, integer(1))
  biplot_choice_rows <- vapply(ggplot2::ggplot_build(p_biplot_choice)$data, nrow, integer(1))
  biplot_ggp_rows <- vapply(ggplot2::ggplot_build(p_biplot_ggp)$data, nrow, integer(1))
  sup_rows <- vapply(ggplot2::ggplot_build(p_sup)$data, nrow, integer(1))
  sup_n <- nrow(res.mfa$quali.var.sup$coord)

  expect_s3_class(p_sup, "ggplot")
  expect_s3_class(p_overlay, "ggplot")
  expect_s3_class(p_biplot, "ggplot")
  expect_s3_class(p_biplot_default, "ggplot")
  expect_s3_class(p_biplot_hidden, "ggplot")
  expect_s3_class(p_biplot_choice, "ggplot")
  expect_s3_class(p_biplot_ggp, "ggplot")
  expect_true(any(overlay_rows == sup_n))
  expect_equal(sum(biplot_rows == sup_n), sum(sup_rows == sup_n))
  expect_equal(sum(biplot_default_rows == sup_n), sum(sup_rows == sup_n))
  expect_equal(sum(biplot_choice_rows == sup_n), sum(sup_rows == sup_n))
  expect_equal(sum(biplot_ggp_rows == sup_n), sum(sup_rows == sup_n))
})

test_that("fviz_mfa_ind respects multi-element invisible vectors for partial plots", {
  skip_if_not_installed("FactoMineR")

  data(poison, package = "FactoMineR")
  res.mfa <- FactoMineR::MFA(
    poison,
    group = c(2, 2, 5, 6),
    type = c("s", "n", "n", "n"),
    name.group = c("desc", "desc2", "symptom", "eat"),
    num.group.sup = 1:2,
    graph = FALSE
  )

  p <- fviz_mfa_ind(res.mfa, partial = "all", invisible = c("quali.var", "ind"), repel = TRUE)
  layer_rows <- vapply(ggplot2::ggplot_build(p)$data, nrow, integer(1))
  partial_n <- nrow(facto_summarize(
    res.mfa, element = "ind",
    result = c("coord", "contrib", "cos2", "coord.partial"),
    axes = 1:2
  )$res.partial)

  expect_s3_class(p, "ggplot")
  expect_false(any(layer_rows == partial_n))
})

test_that("fviz_hmfa_ind respects multi-element invisible vectors for partial plots", {
  skip_if_not_installed("FactoMineR")

  data(wine, package = "FactoMineR")
  hierar <- list(c(2, 5, 3, 10, 9, 2), c(4, 2))
  res.hmfa <- FactoMineR::HMFA(wine, H = hierar, type = c("n", rep("s", 5)), graph = FALSE)

  p <- fviz_hmfa_ind(res.hmfa, partial = "all", invisible = c("quali.var", "ind"), repel = TRUE)
  layer_rows <- vapply(ggplot2::ggplot_build(p)$data, nrow, integer(1))
  partial_n <- nrow(facto_summarize(
    res.hmfa, element = "partial.node", node.level = 1,
    group.names = rownames(res.hmfa$group$coord[[1]]),
    result = c("coord.node.partial"), axes = 1:2
  ))

  expect_s3_class(p, "ggplot")
  expect_false(any(layer_rows == partial_n))
})

test_that("quali.sup plots reject contrib-based selection cleanly", {
  skip_if_not_installed("FactoMineR")

  set.seed(5151)
  famd_df <- data.frame(
    cat_a = factor(sample(c("a", "b", "c"), 60, TRUE)),
    cat_b = factor(sample(c("x", "y"), 60, TRUE)),
    num_a = rnorm(60),
    num_b = rnorm(60),
    sup_q = factor(sample(c("north", "south"), 60, TRUE))
  )
  res.famd <- FactoMineR::FAMD(famd_df, sup.var = 5, graph = FALSE)

  data(poison, package = "FactoMineR")
  res.mfa <- FactoMineR::MFA(
    poison,
    group = c(2, 2, 5, 6),
    type = c("s", "n", "n", "n"),
    name.group = c("desc", "desc2", "symptom", "eat"),
    num.group.sup = 1:2,
    graph = FALSE
  )

  expect_error(
    fviz_famd_var(res.famd, choice = "quali.sup", select.var = list(contrib = 1)),
    "Contributions are not available"
  )
  expect_error(
    fviz_mfa_var(res.mfa, choice = "quali.sup", select.var = list(contrib = 1)),
    "Contributions are not available"
  )

  res.famd.no.sup <- FactoMineR::FAMD(famd_df[, 1:4], graph = FALSE)
  expect_error(
    fviz_famd_var(res.famd.no.sup, choice = "quali.sup"),
    "There are no supplementary qualitative variables"
  )

  res.mfa.no.sup <- FactoMineR::MFA(
    poison,
    group = c(2, 2, 5, 6),
    type = c("s", "n", "n", "n"),
    name.group = c("desc", "desc2", "symptom", "eat"),
    graph = FALSE
  )
  expect_error(
    fviz_mfa_var(res.mfa.no.sup, choice = "quali.sup"),
    "There are no supplementary qualitative variables"
  )
})

#' @include utilities.R
NULL
#'Visualize Correspondence Analysis
#'
#'@description Correspondence analysis (CA) is an extension of Principal 
#'  Component Analysis (PCA) suited to analyze frequencies formed by two 
#'  categorical variables. fviz_ca() provides ggplot2-based elegant 
#'  visualization of CA outputs from the R functions: CA [in FactoMineR], ca [in
#'  ca], coa [in ade4], correspondence [in MASS] and expOutput/epCA [in
#'  ExPosition]. Read more: 
#'  \href{https://www.sthda.com/english/articles/31-principal-component-methods-in-r-practical-guide/113-ca-correspondence-analysis-in-r-essentials/}{Correspondence
#'   Analysis}
#'  
#'  \itemize{ \item fviz_ca_row(): Graph of row variables \item fviz_ca_col():
#'  Graph of column variables \item fviz_ca_biplot(): Biplot of row and column
#'  variables \item fviz_ca(): An alias of fviz_ca_biplot() }
#'  
#'@param X an object of class CA [FactoMineR], ca [ca], coa [ade4]; 
#'  correspondence [MASS] and expOutput/epCA [ExPosition].
#'@param axes a numeric vector of length 2 specifying the dimensions to be 
#'  plotted.
#'@param shape.row,shape.col the point shapes to be used for row/column 
#'  variables. Default values are 19 for rows and 17 for columns.
#'@param geom a character specifying the geometry to be used for the graph. 
#'  Allowed values are the combination of c("point", "arrow", "text"). Use 
#'  "point" (to show only points); "text" to show only labels; c("point", 
#'  "text") or c("arrow", "text") to show both types.
#' @param geom.row,geom.col as \code{geom} but for row and column elements,
#'   respectively. Default is geom.row = c("point", "text), geom.col =
#'   c("point", "text").
#'@param label a character vector specifying the elements to be labelled. 
#'  Default value is "all". Allowed values are "none" or the combination of 
#'  c("row", "row.sup", "col", "col.sup"). Use "col" to label only active column
#'  variables; "col.sup" to label only supplementary columns; etc
#'@param invisible a character value specifying the elements to be hidden on the
#'  plot. Default value is "none". Allowed values are the combination of 
#'  c("row", "row.sup","col", "col.sup").
#'@param title the title of the graph
#'@param col.col,col.row color for column/row points. The default values are 
#'  "red" and "blue", respectively. Can be a continuous variable or a factor
#'  variable. Allowed values include also : "cos2", "contrib", "coord", "x" or
#'  "y". In this case, the colors for row/column variables are automatically
#'  controlled by their qualities ("cos2"), contributions ("contrib"),
#'  coordinates (x^2 + y^2, "coord"), x values("x") or y values("y")
#'@param alpha.col,alpha.row controls the transparency of colors. The value can 
#'  variate from 0 (total transparency) to 1 (no transparency). Default value is
#'  1. Allowed values include also : "cos2", "contrib", "coord", "x" or "y" as 
#'  for the arguments col.col and col.row.
#'@param col.col.sup,col.row.sup colors for the supplementary column and row 
#'  points, respectively.
#'@param repel a boolean, whether to use ggrepel to avoid overplotting text
#'  labels or not. The old \code{jitter} argument is kept for backward
#'  compatibility and is converted to \code{repel = TRUE} with a deprecation warning.
#'@param select.col,select.row a selection of columns/rows to be drawn. Allowed 
#'  values are NULL or a list containing the arguments name, cos2 or contrib: 
#'  \itemize{ \item name is a character vector containing column/row names to be
#'  drawn \item cos2 if cos2 is in [0, 1], ex: 0.6, then columns/rows with a 
#'  cos2 > 0.6 are drawn. if cos2 > 1, ex: 5, then the top 5 columns/rows with 
#'  the highest cos2 are drawn. \item contrib if contrib > 1, ex: 5,  then the 
#'  top 5 columns/rows with the highest contrib are drawn }
#'@param map character string specifying the map type. Allowed options include: 
#'  "symmetric", "rowprincipal", "colprincipal", "symbiplot", "rowgab", 
#'  "colgab", "rowgreen" and "colgreen". See details
#'@param arrows Vector of two logicals specifying if the plot should contain 
#'  points (FALSE, default) or arrows (TRUE). First value sets the rows and the 
#'  second value sets the columns.
#'@param ... Additional arguments. \itemize{ \item in fviz_ca_row() and 
#'  fviz_ca_col(): Additional arguments are passed to the functions fviz() and
#'  ggpubr::ggpar(). \item in fviz_ca_biplot() and fviz_ca(): Additional 
#'  arguments are passed to fviz_ca_row() and fviz_ca_col().}
#'@details The default plot of (M)CA is a "symmetric" plot in which both rows
#'  and columns are in principal coordinates. In this situation, it's not
#'  possible to interpret the distance between row points and column points. To
#'  overcome this problem, the simplest way is to make an asymmetric plot. This
#'  means that, the column profiles must be presented in row space or
#'  vice-versa. The allowed options for the argument map are: \itemize{ \item
#'  "rowprincipal" or "colprincipal": asymmetric plots with either rows in
#'  principal coordinates and columns in standard coordinates, or vice versa.
#'  These plots preserve row metric or column metric respectively. \item
#'  "symbiplot": Both rows and columns are scaled to have variances equal to the
#'  singular values (square roots of eigenvalues), which gives a symmetric
#'  biplot but does not preserve row or column metrics. \item "rowgab" or
#'  "colgab": Asymmetric maps, proposed by Gabriel & Odoroff (1990), with rows
#'  (respectively, columns) in principal coordinates and columns (respectively,
#'  rows) in standard coordinates multiplied by the mass of the corresponding
#'  point. \item "rowgreen" or "colgreen": The so-called contribution biplots
#'  showing visually the most contributing points (Greenacre 2006b). These are
#'  similar to "rowgab" and "colgab" except that the points in standard
#'  coordinates are multiplied by the square root of the corresponding masses,
#'  giving reconstructions of the standardized residuals. }
#'@return a ggplot
#'@author Alboukadel Kassambara \email{alboukadel.kassambara@@gmail.com}
#'@seealso \code{\link{get_ca}}, \code{\link{fviz_pca}}, \code{\link{fviz_mca}}
#'@references \url{https://www.sthda.com/english/}
#' @examples
#' # Correspondence Analysis
#' # ++++++++++++++++++++++++++++++
#' # Install and load FactoMineR to compute CA
#' # install.packages("FactoMineR")
#' 
#' library("FactoMineR")
#' data(housetasks)
#' head(housetasks)
#' res.ca <- CA(housetasks, graph=FALSE)
#' 
#' # Biplot of rows and columns
#' # ++++++++++++++++++++++++++
#' # Symetric Biplot of rows and columns
#' fviz_ca_biplot(res.ca)
#' 
#' # Asymetric biplot, use arrows for columns
#' fviz_ca_biplot(res.ca, map ="rowprincipal",
#'  arrow = c(FALSE, TRUE))
#'  
#' # Keep only the labels for row points
#' fviz_ca_biplot(res.ca, label ="row")
#' 
#' # Keep only labels for column points
#' fviz_ca_biplot(res.ca, label ="col")
#' 
#'        
#' # Select the top 7 contributing rows
#' # And the top 3 columns
#' fviz_ca_biplot(res.ca,  
#'                select.row = list(contrib = 7),
#'                select.col = list(contrib = 3))
#' 
#' # Graph of row variables
#' # +++++++++++++++++++++
#'    
#' # Control automatically the color of row points
#'    # using the "cos2" or the contributions "contrib"
#'    # cos2 = the quality of the rows on the factor map
#'    # Change gradient color
#'    # Use repel = TRUE to avoid overplotting (slow if many points)
#' fviz_ca_row(res.ca, col.row = "cos2",
#'    gradient.cols = c("#00AFBB", "#E7B800", "#FC4E07"),
#'    repel = TRUE)
#' 
#' # You can also control the transparency 
#' # of the color by the "cos2" or "contrib"
#' fviz_ca_row(res.ca, alpha.row="contrib") 
#'       
#' # Select and visualize some rows with select.row argument.
#'  # - Rows with cos2 >= 0.5: select.row = list(cos2 = 0.5)
#'  # - Top 7 rows according to the cos2: select.row = list(cos2 = 7)
#'  # - Top 7 contributing rows: select.row = list(contrib = 7)
#'  # - Select rows by names: select.row = list(name = c("Breakfeast", "Repairs", "Holidays"))
#'  
#'  # Example: Select the top 7 contributing rows
#' fviz_ca_row(res.ca, select.row = list(contrib = 7))
#' 
#'  
#' # Graph of column points
#' # ++++++++++++++++++++++++++++
#' 
#'  
#' # Control colors using their contributions
#' fviz_ca_col(res.ca, col.col = "contrib",
#'    gradient.cols = c("#00AFBB", "#E7B800", "#FC4E07"))
#'        
#' # Select columns with select.col argument
#'    # You can select by contrib, cos2 and name 
#'    # as previously described for ind
#' # Select the top 3 contributing columns
#' fviz_ca_col(res.ca, select.col = list(contrib = 3))
#'     
#'     
#'  
#'@name fviz_ca
#'  
#'@rdname fviz_ca
#'@export
fviz_ca_row <-function(X,  axes = c(1,2), geom = c("point", "text"), geom.row = geom,
                       shape.row = 19, col.row ="blue", alpha.row = 1,
                       col.row.sup="darkblue",  
                       select.row = list(name = NULL, cos2 = NULL, contrib = NULL),
                       map ="symmetric", repel = FALSE,
                       ...)
{
  
  p <- fviz (X, element = "row", axes = axes, geom = geom.row,
                      color = col.row, alpha = alpha.row,
                      pointshape = shape.row, select = select.row, 
                      map = map, repel = repel, 
                      col.row.sup = col.row.sup, shape.sup = shape.row,   ...)
  p
  
}

#' @rdname fviz_ca
#' @export 
fviz_ca_col <-function(X,  axes = c(1,2), shape.col = 17, 
                       geom=c("point", "text"), geom.col = geom,
                       col.col ="red", col.col.sup="darkred",  alpha.col = 1,
                       select.col = list(name = NULL, cos2 = NULL, contrib = NULL),
                       map ="symmetric", repel = FALSE,
                       ...)
{
  
  fviz (X, element = "col", axes = axes, geom = geom.col,
        color = col.col, alpha = alpha.col,
        pointshape = shape.col, select = select.col, 
        map = map, repel = repel, 
        col.col.sup = col.col.sup, shape.sup = shape.col, ...)
  
  
}




#' @rdname fviz_ca
#' @export 
fviz_ca_biplot <-function(X,  axes = c(1,2), geom=c("point", "text"), 
                          geom.row = geom, geom.col = geom,
                          label = "all", invisible="none", arrows = c(FALSE, FALSE),
                          repel = FALSE, title = "CA - Biplot",
                       ...)
{
  # Rows
  geom2 <- geom.row
  if(arrows[1]==TRUE) geom2 <- setdiff(unique(c(geom2, "arrow")), "point")
  p <- fviz_ca_row(X,  axes = axes, geom=geom2, repel = repel,
        label = label, invisible = invisible, ...)
  # Add columns
  geom2 <- geom.col
  if(arrows[2]==TRUE) geom2 <- setdiff(unique(c(geom2, "arrow")), "point")
  p <- fviz_ca_col(X,  axes = axes, geom=geom2, repel = repel,
                   label = label, invisible = invisible, 
                   ggp = p, ...)
  
  p + labs(title=title)
  
}

#' @rdname fviz_ca
#' @export
fviz_ca <- function(X, ...){
  fviz_ca_biplot(X, ...)
}

#' Plot Model-Based Clustering Results using ggplot2
#' @description Plots the classification, the uncertainty and the BIC values returned by the Mclust() function.
#' @inheritParams ggpubr::ggpar
#' @inheritParams fviz_cluster
#' @param object an object of class Mclust
#' @param what choose from one of the following three options: "classification" (default), "uncertainty" and "BIC".
#' @param shape point shape. To change point shape by model names use shape = "model".
#' @param color point and line color.
#' @param ... other arguments to be passed to the functions \link{fviz_cluster} and \link[ggpubr]{ggpar}.
#' 
#' @examples
#' 
#' if(requireNamespace("mclust", quietly = TRUE)){
#'
#' # Compute model-based-clustering
#' library("mclust")
#' data("diabetes")
#' mc <- Mclust(diabetes[, -1])
#' 
#' # Visualize BIC values
#' fviz_mclust_bic(mc)
#' 
#' # Visualize classification
#' fviz_mclust(mc, "classification", geom = "point")
#' }
#' 
#' @return A ggplot2 object.
#' @describeIn fviz_mclust Plots classification and uncertainty.
#' @export
fviz_mclust <- function(object, 
                        what = c("classification", "uncertainty", "BIC"),
                        ellipse.type = "norm", ellipse.level = 0.4, 
                        ggtheme = theme_classic(), ...)
{
  uncertainty <- cluster <- NULL
  what <- match.arg(what)
  if(what == "classification")
    p <- fviz_cluster(object, ellipse.type = ellipse.type, ellipse.level =ellipse.level,
                      ggtheme = theme_classic(), ...)+
    labs(subtitle = "Classification")
  if(what == "uncertainty")
    p <- fviz_cluster(object, ellipse.type = ellipse.type, ellipse.level =ellipse.level,
                      ggtheme = theme_classic(), geom = "none", ...)+
    geom_point(aes(size = uncertainty, color = cluster))+
    scale_size(range =c(0, 2))+
    labs(subtitle = "Uncertainty")+
    # FIX: ggplot2 3.3.4+ deprecation - use "none" instead of FALSE for guides()
    # See: https://github.com/kassambara/factoextra/issues/179
    guides(size = "none")
  else if(what == "BIC") p <- fviz_mclust_bic(object, ggtheme = theme_classic(),  ...)
  
  return(p)
}


#' @describeIn fviz_mclust Plots the BIC values.
#' @param model.names one or more model names corresponding to models fit in object. The default
#'   is to plot the BIC for all of the models fit.
#' @export
fviz_mclust_bic <- function(object, model.names = NULL, shape = 19, color = "model",
                            palette = NULL, legend = NULL,
                            main = "Model selection", xlab = "Number of components", ylab = "BIC", ...){
  # Avoid R CMD check NOTE for non-standard evaluation
  cluster <- NULL

  if(!inherits(object, "Mclust"))
    stop("An object of class Mclust is required.")
  
  best_model <- object$modelName
  number_of_cluster <- object$G
  x <- object$BIC
  
  # Convert to a data frame
  n <- ncol(x)
  dnx <- dimnames(x)
  x <- matrix(as.vector(x), ncol = n)
  dimnames(x) <- dnx
  x <- as.data.frame(x)
  if (is.null(model.names)) 
    model.names <- dimnames(x)[[2]]
  x <- x[, model.names, drop = FALSE]
  # Add number of clusters
  x <- cbind.data.frame(cluster = rownames(x), x)
  # Convert wide to long format using base R (replaces tidyr::pivot_longer)
  value_cols <- setdiff(colnames(x), "cluster")
  x <- stats::reshape(x, direction = "long",
                      varying = value_cols,
                      v.names = "BIC",
                      timevar = "model",
                      times = value_cols,
                      idvar = "cluster")
  rownames(x) <- NULL
  x <- x[!is.na(x$BIC), , drop = FALSE]
  x$model <- factor(x$model, levels = dnx[[2]])
  x$cluster <- factor(x$cluster, levels = unique(x$cluster))
 
  if(.is_color_palette(palette)) palette <- ggpubr::get_palette(palette, k = length(model.names))
  ggline.opts <- list(data = x, x ="cluster", y = "BIC", group = "model",
                      color = color, shape = shape, palette = palette,
                      main = main, xlab = xlab, ylab = ylab,...)
  # The x-axis is a discrete factor, so the optimal-cluster line must use the
  # position of `number_of_cluster` among the cluster levels, not its numeric
  # value (those differ when the cluster counts don't start at 1, e.g. a
  # restricted G range or a noise/outlier model). For the standard G = 1:k case
  # the position equals the value, so this is unchanged. (#116)
  opt_x <- match(as.character(number_of_cluster), levels(x$cluster))
  p <- do.call(ggpubr::ggline, ggline.opts)+
    labs(subtitle = paste0("Best model: ", best_model,
                           " | Optimal clusters: n = ",  number_of_cluster))+
    theme(legend.title = element_blank())
  if(!is.na(opt_x))
    p <- p + geom_vline(xintercept = opt_x, linetype = 2, color = "red")

  
  if(missing(legend)) p + theme(legend.position = c(0.7, 0.2), legend.direction = "horizontal",
                                legend.key.height = unit(0.5, "line"))+
    guides(color = guide_legend(nrow=5,byrow=TRUE))
  else p + theme(legend.position = legend)
}

# Ge the silhouette information
# cluster: the cluster assignment of observation
# diss the dissimilarity matrix
.get_silinfo <- function(cluster, diss){
  
  k <- length(unique(cluster))
  silinfo <- list()
  if(k > 1) {
    sil.obj <- cluster::silhouette(cluster, diss)
    widths <- as.data.frame(sil.obj[, 1:3])
    rownames(widths) <- colnames(as.matrix(diss))
    widths <- widths[order(widths$cluster, -widths$sil_width), ]
    widths$cluster <- as.factor(widths$cluster)
    
    # summary
    silinfo$widths <- widths
    silinfo$clus.avg.widths <- as.vector(tapply(widths$sil_width, widths$cluster, mean))
    silinfo$avg.width <- mean(widths$sil_width)
  }
  
  silinfo
}

#' @include print.factoextra.R fviz_mfa.R fviz_hmfa.R
NULL
#' Deprecated Functions
#'
#' @description Deprecated functions. Will be removed in a future version.
#'
#' \itemize{
#' \item get_mfa_quanti_var(). Deprecated. Use get_mfa_var(res.mfa, "quanti.var") instead.
#' \item get_mfa_quali_var().  Deprecated. Use get_mfa_var(res.mfa, "quali.var") instead.
#' \item get_mfa_group(). Deprecated. Use get_mfa_var(res.mfa, "group") instead.
#'
#' \item fviz_mfa_ind_starplot(): Star graph of individuals (draws partial points).
#' Deprecated. Use fviz_mfa_ind(res.mfa, partial = "all") instead.
#' \item fviz_mfa_quanti_var(): Graph of quantitative variables. Deprecated. Use fviz_mfa(X, "quanti.var") instead.
#' \item fviz_mfa_quali_var(): Graph of qualitative variables. Deprecated. Use fviz_mfa(X, "quali.var") instead.
#'
#'  \item
#' get_hmfa_quanti_var(). Deprecated. Use get_hmfa_var(res.hmfa, "quanti.var")
#' instead. \item get_hmfa_quali_var().  Deprecated. Use get_hmfa_var(res.hmfa,
#' "quali.var") instead. \item get_hmfa_group(). Deprecated. Use
#' get_hmfa_var(res.hmfa, "group") instead.
#'
#' \item fviz_hmfa_ind_starplot(): Graph of partial individuals. Deprecated. Use fviz_hmfa_ind(X, partial = "all") instead.
#' \item fviz_hmfa_quanti_var(): Graph of quantitative variables. Deprecated. Use fviz_hmfa_var(X, "quanti.var") instead.
#' \item fviz_hmfa_quali_var(): Graph of qualitative variables. Deprecated. Use fviz_hmfa_var(X, "quali.var") instead.
#' \item fviz_hmfa_group(): Graph of the groups representation. Deprecated. Use fviz_hmfa_var(X, "group") instead.
#'
#' }
#'
#' @param res.hmfa an object of class HMFA [FactoMineR].
#' @param res.mfa an object of class MFA [FactoMineR].
#' @param X an object of class MFA or HMFA [FactoMineR].
#' @param ... Other arguments.
#' @author Alboukadel Kassambara \email{alboukadel.kassambara@@gmail.com}
#' @name deprecated
NULL

# Internal helper function for deprecation warnings
# Consolidates duplicated warning+dispatch pattern
# Not exported or documented
.deprecate_warn <- function(old_fn, new_fn) {
  .Deprecated(new_fn, package = "factoextra", old = old_fn)
}

# ============================================================================
# MFA deprecated functions
# ============================================================================

#' @rdname deprecated
#' @export
get_mfa_quanti_var <- function(res.mfa) {
  .deprecate_warn("get_mfa_quanti_var", "get_mfa_var(res.mfa, 'quanti.var')")
  get_mfa_var(res.mfa, "quanti.var")
}

#' @rdname deprecated
#' @export
get_mfa_quali_var <- function(res.mfa) {
  .deprecate_warn("get_mfa_quali_var", "get_mfa_var(res.mfa, 'quali.var')")
  get_mfa_var(res.mfa, "quali.var")
}

#' @rdname deprecated
#' @export
get_mfa_group <- function(res.mfa) {
  .deprecate_warn("get_mfa_group", "get_mfa_var(res.mfa, 'group')")
  get_mfa_var(res.mfa, "group")
}

#' @rdname deprecated
#' @export
fviz_mfa_ind_starplot <- function(X, ...) {
  .deprecate_warn("fviz_mfa_ind_starplot", "fviz_mfa_ind(X, partial = 'all')")
  fviz_mfa_ind(X, partial = "all", ...)
}

#' @rdname deprecated
#' @export
fviz_mfa_group <- function(X, ...) {
  .deprecate_warn("fviz_mfa_group", "fviz_mfa_var(X, 'group')")
  fviz_mfa_var(X, choice = "group", ...)
}

#' @rdname deprecated
#' @export
fviz_mfa_quanti_var <- function(X, ...) {
  .deprecate_warn("fviz_mfa_quanti_var", "fviz_mfa_var(X, 'quanti.var')")
  fviz_mfa_var(X, choice = "quanti.var", ...)
}

#' @rdname deprecated
#' @export
fviz_mfa_quali_var <- function(X, ...) {
  .deprecate_warn("fviz_mfa_quali_var", "fviz_mfa_var(X, 'quali.var')")
  fviz_mfa_var(X, choice = "quali.var", ...)
}

# ============================================================================
# HMFA deprecated functions
# ============================================================================

#' @rdname deprecated
#' @export
get_hmfa_quanti_var <- function(res.hmfa) {
  .deprecate_warn("get_hmfa_quanti_var", "get_hmfa_var(res.hmfa, 'quanti.var')")
  get_hmfa_var(res.hmfa, "quanti.var")
}

#' @rdname deprecated
#' @export
get_hmfa_quali_var <- function(res.hmfa) {
  .deprecate_warn("get_hmfa_quali_var", "get_hmfa_var(res.hmfa, 'quali.var')")
  get_hmfa_var(res.hmfa, "quali.var")
}

#' @rdname deprecated
#' @export
get_hmfa_group <- function(res.hmfa) {
  .deprecate_warn("get_hmfa_group", "get_hmfa_var(res.hmfa, 'group')")
  get_hmfa_var(res.hmfa, "group")
}

#' @rdname deprecated
#' @export
fviz_hmfa_quanti_var <- function(X, ...) {
  .deprecate_warn("fviz_hmfa_quanti_var", "fviz_hmfa_var(X, 'quanti.var')")
  fviz_hmfa_var(X, "quanti.var", ...)
}

#' @rdname deprecated
#' @export
fviz_hmfa_quali_var <- function(X, ...) {
  .deprecate_warn("fviz_hmfa_quali_var", "fviz_hmfa_var(X, 'quali.var')")
  fviz_hmfa_var(X, "quali.var", ...)
}

#' @rdname deprecated
#' @export
fviz_hmfa_ind_starplot <- function(X, ...) {
  .deprecate_warn("fviz_hmfa_ind_starplot", "fviz_hmfa_ind(X, partial = 'all')")
  fviz_hmfa_ind(X, partial = "all", ...)
}

#' @rdname deprecated
#' @export
fviz_hmfa_group <- function(X, ...) {
  .deprecate_warn("fviz_hmfa_group", "fviz_hmfa_var(X, 'group')")
  fviz_hmfa_var(X, choice = "group", ...)
}

#' @include eigenvalue.R fviz_add.R
NULL
#' @import ggplot2
#' @importFrom rlang .data
#' @importFrom ggrepel geom_text_repel
#' @importFrom grDevices chull
#' @importFrom graphics plot
#' @importFrom utils data
#' @importFrom stats as.dendrogram
#' @importFrom stats as.dist
#' @importFrom stats kmeans
#' @importFrom stats na.omit
#' @importFrom stats qchisq
#' @importFrom stats rnorm
#' @importFrom stats runif
#' @importFrom stats var
#'

# Helper function to check if a palette name is a predefined color palette
# Replaces ggpubr:::.is_col_palette to avoid using unexported functions
# Known palette names from RColorBrewer, ggsci, and ggpubr defaults
.is_color_palette <- function(pal) {
 if (is.null(pal)) return(FALSE)
 # RColorBrewer palettes
 brewer_pals <- c("Blues", "BuGn", "BuPu", "GnBu", "Greens", "Greys", "Oranges",
                  "OrRd", "PuBu", "PuBuGn", "PuRd", "Purples", "RdPu", "Reds",
                  "YlGn", "YlGnBu", "YlOrBr", "YlOrRd", "BrBG", "PiYG", "PRGn",
                  "PuOr", "RdBu", "RdGy", "RdYlBu", "RdYlGn", "Spectral",
                  "Accent", "Dark2", "Paired", "Pastel1", "Pastel2", "Set1",
                  "Set2", "Set3")
 # ggsci palettes
 ggsci_pals <- c("npg", "aaas", "nejm", "lancet", "jama", "jco", "ucscgb",
                 "d3", "locuszoom", "igv", "uchicago", "startrek", "tron",
                 "futurama", "rickandmorty", "simpsons")
 # ggpubr built-in
 other_pals <- c("default", "hue", "grey_pal", "gray_pal")
 all_pals <- c(brewer_pals, ggsci_pals, other_pals)
 return(length(pal) == 1 && pal[1] %in% all_pals)
}

.class_label <- function(x){
  paste(class(x), collapse = ", ")
}

.extract_nested_element <- function(x, element){
  parts <- strsplit(element, "$", fixed = TRUE)[[1]]
  out <- x
  for(part in parts){
    if(is.null(out)) return(NULL)
    out <- out[[part]]
  }
  out
}

.get_factominer_quali_sup <- function(X){
  if(!is.null(X$quali.var.sup)) return(X$quali.var.sup)
  if(!is.null(X$quali.sup)) return(X$quali.sup)
  NULL
}

.with_preserved_seed <- function(seed, expr){
  has_seed <- exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)
  if(has_seed) old_seed <- get(".Random.seed", envir = .GlobalEnv)
  on.exit({
    if(!has_seed && exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)){
      rm(".Random.seed", envir = .GlobalEnv)
    } else if(has_seed){
      assign(".Random.seed", old_seed, envir = .GlobalEnv)
    }
  }, add = TRUE)
  set.seed(seed)
  force(expr)
}

.coerce_integerish <- function(value, arg, lower = 1L, upper = .Machine$integer.max,
                               value_label = "single positive integer value"){
  tol <- sqrt(.Machine$double.eps)
  if(!is.numeric(value) || length(value) != 1L || is.na(value) || !is.finite(value))
    stop(arg, " must be a ", value_label, " in [", lower, ", ", upper, "].")

  rounded <- round(value)
  if(abs(value - rounded) > tol || rounded < lower || rounded > upper)
    stop(arg, " must be a ", value_label, " in [", lower, ", ", upper, "].")

  as.integer(rounded)
}

# Check and get the class of the output of a factor analysis
# ++++++++++++++++++++++++++++
# X: an output of factor analysis (PCA, CA, MCA, MFA) 
# from different packages (FactoMineR, ade4, ....) 
.get_facto_class <- function(X){
  
  if(inherits(X, c('PCA', 'princomp', 'prcomp')))
    facto_class ="PCA"
  # User-supplied coordinates wrapped by as_factoextra_pca().
  else if(inherits(X, "factoextra_pca"))
    facto_class ="PCA"
  else if(inherits(X, "pca") && inherits(X, "dudi"))
    facto_class ="PCA"
  # ade4 between-class / within-class PCA (bca/wca): treated as a PCA. These
  # objects are class c("between"/"within", "dudi") and drop the "pca" class.
  else if(inherits(X, c("between", "within")) && inherits(X, "dudi"))
    facto_class ="PCA"
  else if(inherits(X, c("CA", "ca", "coa", "correspondence"))) facto_class="CA"
  else if(inherits(X, c("MCA", "acm"))) facto_class = "MCA"
  else if(inherits(X, c("MFA","mfa"))) facto_class = "MFA"
  else if(inherits(X, c("HMFA"))) facto_class = "HMFA"
  else if(inherits(X, c("FAMD"))) facto_class = "FAMD"
  else if (inherits(X, "expoOutput")){
   if (inherits(X$ExPosition.Data,'epCA')) facto_class="CA"
   else if (inherits(X$ExPosition.Data,'epPCA')) facto_class="PCA"
   else if (inherits(X$ExPosition.Data,'epMCA')) facto_class="MCA"
  }
  else stop("An object of class : ", .class_label(X), 
            " can't be handled by factoextra")   
}

# Get the result for supplementary points
#++++++++++++++++++++++++++
## X: an output of factor analysis (PCA, CA, MCA, MFA, HMFA) 
## element possible values are "row.sup", "col.sup" (CA);
# quanti, ind.sup (PCA); quali.sup (MCA); quanti.var.sup, quali.var.sup (MFA)
## result the result tobe extracted for the element. Possible values are
#  the combination of c("cos2", "coord")
## axes a numeric vector specifying the axes of interest. Default values are 1:2
##  for axes 1 and 2
## select: a selection of variables. See the function .select()
.get_supp <- function(X, element = NULL, axes = 1:2, 
                      result = c("coord", "cos2"), select = NULL){
  
  if(inherits(X, "MFA") && element == "group")
    elmt <- .get_mfa_group_sup(X)
  else if(element == "quali.sup")
    elmt <- .get_factominer_quali_sup(X)
  else if(inherits(X, c("CA", "PCA", "MCA", "MFA", "HMFA", "FAMD"))) {
    elmt <- .extract_nested_element(X, element)
  }
  else if(inherits(X, "ca")){
    if(element == "col.sup") elmt <- .get_ca_col_sup(X)
    else if(element == "row.sup") elmt <- .get_ca_row_sup(X)
  }
  else stop("An object of class : ", .class_label(X), 
            " can't be handled by the function .get_supp()")
  
  # summarize the result
  res = NULL
  if(!is.null(elmt)){
    if(inherits(X, "MCA")){
      if(element %in% c("quanti.sup", "quali.sup$eta2")) result <- "coord"
      if(element == "quanti.sup") elmt$coord <- elmt$coord^2
      else if(element == "quali.sup$eta2") elmt<-list(coord = elmt)
    }
    # check axes
    if(max(axes) > ncol(elmt$coord))
      stop("The value of the argument axes is incorrect. ",
           "The number of axes in the data is: ", ncol(elmt$coord), 
           ". Please try again with axes between 1 - ", ncol(elmt$coord))
    
    # 1.Extract the coordinates x, y and coord
    if("coord" %in% result){
      dd <- data.frame(elmt$coord[, axes, drop=FALSE])
      coord <- rowSums(dd^2, na.rm = TRUE) # x^2 + y2 + ...
      res <- cbind(dd, coord = coord)
    }
    
    # 2. Extract the cos2
    if("cos2" %in% result){
      cos2 <- data.frame(elmt$cos2[, axes, drop=FALSE])
      if(length(axes) > 1) cos2 <- rowSums(cos2, na.rm = TRUE)
      res <- cbind(res, cos2 = cos2)
    }
    
    res <- cbind.data.frame(name =rownames(elmt$coord), res)
    
    # selection of variables
    if(!is.null(select)){
      if(!is.null(select$contrib)) res <- NULL # supp points don't have contrib
      else res <- .select(res, select, check = FALSE)
    }
  }
  if(!is.null(res)){
    if(nrow(res) == 0) res <- NULL
  }
  res
}


# Add supplementary elements to a plot
# p a ggplot
# X an object from FactoMiner
# element: "ind.sup", "quanti"
# axes: axes of interest
# scale.: numeric, the data is multiplied by scale. before ploting
# select: a selection of individuals/variables to be drawn.
# ca_map: applied to (m)ca only, see fviz_ca map argument
.add_supp <- function(p, X, element, axes, select, scale.=1, 
                       ca_map = NULL,
                      ...){
  for(el in element){
    dd <- .get_supp(X, element = el, axes = axes, select = select)
    
    if(!is.null(dd)){
      colnames(dd)[2:3] <-  c("x", "y")
      if(!is.null(ca_map)) dd <- .scale_ca(dd, res.ca = X,  element = el, 
                           type = ca_map, axes = axes)
      p <- fviz_add(p, df = dd[, 2:3, drop = FALSE]*scale., ...)
    }
  }
  p
}

# get supplementary columns from ca output (ca package)
#
# OPTIMIZATION: Replaced t(apply()) with sweep() for column-wise multiplication
# sweep() is much faster for element-wise row/column operations
.get_ca_col_sup <- function(res.ca){
  # supplementary points
  index <- res.ca$colsup
  cols <- NULL
  if(length(index) > 0){
    # OPTIMIZED: principal coord = standard coord X sqrt(eig)
    # Using sweep() instead of t(apply()) - multiply each column by sv
    coord <- sweep(res.ca$colcoord, 2, res.ca$sv, "*")
    # OPTIMIZED: cos2 = coord^2 / dist^2 (divide each row by its distance squared)
    cos2 <- coord^2 / res.ca$coldist^2
    cols <- list(coord = coord[index, , drop = FALSE],
                cos2 = cos2[index, , drop = FALSE])
  }
  cols
}

# get supplementary rows from ca output (ca package)
#
# OPTIMIZATION: Replaced t(apply()) with sweep() for column-wise multiplication
.get_ca_row_sup <- function(res.ca){
  rows <- NULL
  # supplementary points
  index <- res.ca$rowsup
  if(length(index) > 0){
    # OPTIMIZED: principal coord = standard coord X sqrt(eig)
    coord <- sweep(res.ca$rowcoord, 2, res.ca$sv, "*")
    # OPTIMIZED: cos2 = coord^2 / dist^2
    cos2 <- coord^2 / res.ca$rowdist^2
    rows <- list(coord = coord[index, , drop = FALSE],
                                     cos2 = cos2[index, , drop = FALSE])
  }
  rows
}

# get supplementary groups from MFA
.get_mfa_group_sup <- function(res.mfa){
  res <- NULL
  # supplementary points
  coord <- res.mfa$group$coord.sup
  if(!is.null(coord)){
    res <- list(coord = coord, cos2 = res.mfa$group$cos2.sup) 
  }
 res
}

# Scale row coordinates depending on the map type
#++++++++++++++++++++++
## row.res, data: an output of facto_summarize containing the
# name, x, y, coord,... of the row variables
## res.ca an object of class CA, MCA
## type: a character string specifying the map type. 
# Allowed values include "symmetric", "rowprincipal", "colprincipal", "rowgab", "colgab", "rowgreen", "colgreen".
## element: possible values are "row", "col", "row.sup", "col.sup",
# "ind", "var", "ind.sup", "quali.sup"
.scale_ca <- function(data, res.ca, element="row", type="symmetric", axes = 1:2)
{
  res <- NULL
  if(!is.null(data)){
  if(element %in% c("row", "ind")) res <- .scale_ca_row(data, res.ca, type, axes)
  else if(element %in% c("col", "var")) res <- .scale_ca_col(data, res.ca, type, axes)
  else if(element %in% c( "row.sup", "ind.sup")) res <- .scale_ca_rowsupp(data, res.ca, type, axes)
  else if(element %in% c("col.sup", "quali.sup")) res <- .scale_ca_colsupp(data, res.ca, type, axes)
  } 
  res
}

.scale_ca_row <- function(row.res, res.ca, type ="symmetric", axes = 1:2){
  data <- row.res
  res <- data
  
  eig <- get_eigenvalue(res.ca)[axes,1]
  sv <- sqrt(eig)
  mass <- .get_ca_mass(res.ca, "row")[as.vector(data$name)]
  
  # rows/columns in principal coordinates
  if(type %in% c("symmetric", "rowprincipal", "rowgab", "rowgreen")) res <- data
  else{
    # get standard coordinates 
    x <- res$x/sqrt(eig[1])
    y <- res$y/sqrt(eig[2])
    
    # standard coordinate
    if(type %in% c("colprincipal")){
      res$x <- x
      res$y <- y
      res$coord <- res$x^2 + res$y^2
    }
    # standard coordinate X mass
    else if(type %in% c("colgab")){
      res$x <- x*mass
      res$y <- y*mass
      res$coord <- res$x^2 + res$y^2
    }
    
    # standard coordinate X sqrt(mass)
    else if(type %in% c("colgreen")){
      res$x <- x*sqrt(mass)
      res$y <- y*sqrt(mass)
      res$coord <- res$x^2 + res$y^2
    }
    else  if(type %in% c("symbiplot")){
      res$x <- x*sqrt(sv[1])
      res$y <- y*sqrt(sv[2])
      res$coord <- res$x^2 + res$y^2
    }
  }
  
  return(res)
}

# Scale ca column
.scale_ca_col <- function(col.res, res.ca, type ="symmetric", axes = 1:2){
  data <- col.res
  res <- data
  
  eig <- get_eigenvalue(res.ca)[axes,1]
  sv <- sqrt(eig)
  mass <- .get_ca_mass(res.ca, "col")[as.vector(data$name)]
  
  # rows/columns in principal coordinates
  if(type %in% c("symmetric", "colprincipal", "colgab", "colgreen")) res <- data
  else{
    # get standard coordinates 
    x <- res$x/sqrt(eig[1])
    y <- res$y/sqrt(eig[2])
    
    # standard coordinate
    if(type %in% c("rowprincipal")){
      res$x <- x
      res$y <- y
      res$coord <- res$x^2 + res$y^2
    }
    # standard coordinate X mass
    else if(type %in% c("rowgab")){
      res$x <- x*mass
      res$y <- y*mass
      res$coord <- res$x^2 + res$y^2
    }
    
    # standard coordinate X sqrt(mass)
    else if(type %in% c("rowgreen")){
      res$x <- x*sqrt(mass)
      res$y <- y*sqrt(mass)
      res$coord <- res$x^2 + res$y^2
    }
    else  if(type %in% c("symbiplot")){
      res$x <- x*sqrt(sv[1])
      res$y <- y*sqrt(sv[2])
      res$coord <- res$x^2 + res$y^2
    }
  }
  
  return(res)
}

# Scale ca row sup
.scale_ca_rowsupp <- function(rowsup.res, res.ca, type ="symmetric", axes = 1:2){
  data <- rowsup.res
  res <- data
  
  eig <- get_eigenvalue(res.ca)[axes,1]
  sv <- sqrt(eig)
  
  # rows/columns in principal coordinates
  if(type %in% c("symmetric", "rowprincipal", "rowgab", "rowgreen")) res <- data
  else{
    # get standard coordinates 
    x <- res$x/sqrt(eig[1])
    y <- res$y/sqrt(eig[2])
    
    # standard coordinate
    if(type %in% c("colprincipal")){
      res$x <- x
      res$y <- y
      res$coord <- res$x^2 + res$y^2
    }
    # standard coordinate X mass
    else if(type %in% c("colgab")) res <- NULL
    
    # standard coordinate X sqrt(mass)
    else if(type %in% c("colgreen")) res <- NULL
    
    else  if(type %in% c("symbiplot")){
      res$x <- x*sqrt(sv[1])
      res$y <- y*sqrt(sv[2])
      res$coord <- res$x^2 + res$y^2
    }
  }
  
  return(res)
}


.scale_ca_colsupp <- function(colsup.res, res.ca, type ="symmetric", axes = 1:2){
  data <- colsup.res
  res <- data
  
  eig <- get_eigenvalue(res.ca)[axes,1]
  sv <- sqrt(eig)
  
  # rows/columns in principal coordinates
  if(type %in% c("symmetric", "colprincipal", "colgab", "colgreen")) res <- data
  else{
    # get standard coordinates 
    x <- res$x/sqrt(eig[1])
    y <- res$y/sqrt(eig[2])
    
    # standard coordinate
    if(type %in% c("rowprincipal")){
      res$x <- x
      res$y <- y
      res$coord <- res$x^2 + res$y^2
    }
    # standard coordinate X mass
    else if(type %in% c("rowgab")) res <- NULL
    # standard coordinate X sqrt(mass)
    else if(type %in% c("rowgreen")) res <- NULL
    else  if(type %in% c("symbiplot")){
      res$x <- x*sqrt(sv[1])
      res$y <- y*sqrt(sv[2])
      res$coord <- res$x^2 + res$y^2
    }
  }
  
  return(res)
}


# get the mass of an element
#++++++++++++++++++++++++++++
# res.ca : CA object
# element: possible values are "row" or "col"
.get_ca_mass <- function(res.ca, element){
  if(inherits(res.ca, "ca")){
    if(element == "row"){
    mass <- res.ca$rowmass
    names(mass) <- res.ca$rownames
    }
    else if(element == "col"){
      mass <- res.ca$colmass
      names(mass) <- res.ca$colnames
    }
  }
  # FactoMiner
  else if(inherits(res.ca, c("CA", "MCA"))){
    if(element %in% c("row", "ind")) mass <- res.ca$call$marge.row
    else if(element %in% c("col", "var")) mass <- res.ca$call$marge.col
    
  }
  # Ade4
  else if(inherits(res.ca, c("coa", "acm"))){
    if(element =="row") mass <- res.ca$lw
    else if(element == "col") mass <- res.ca$cw
  }
  # Mass package
  else if(inherits(res.ca, "correspondence")){
    row.sum <- rowSums(res.ca$Freq)
    col.sum <- colSums(res.ca$Freq)
    n <- sum(res.ca$Freq)
    if(element =="row") mass <- row.sum/n
    else if(element =="col") mass <- col.sum/n
  }
  # ExPosition
  else if (inherits(res.ca, "expoOutput")){
    if(element =="row") mass <- res.ca$ExPosition.Data$M
    else if(element =="col") mass <- res.ca$ExPosition.Data$W
  }
  else stop("An object of class : ", .class_label(res.ca), 
            " can't be handled") 
  return(mass)
}


# Reconstruct data: get the original data
# res.pca is an object of class prcomp or princomp
.prcomp_reconst <- function(res.pca){
  if(inherits(res.pca, "prcomp")){
    centered <- is.numeric(res.pca$center)
    scaled <- is.numeric(res.pca$scale)
    
    if(centered && scaled) 
      t(t(res.pca$x %*% t(res.pca$rotation)) * res.pca$scale + res.pca$center)
    else if(centered)
      t(t(res.pca$x %*% t(res.pca$rotation)) + res.pca$center)
    else if(scaled)
      t(t(res.pca$x %*% t(res.pca$rotation)) * res.pca$scale)
    else t(t(res.pca$x %*% t(res.pca$rotation)))
  }
  
  else if(inherits(res.pca, "princomp")){ 
    if(!is.null(res.pca$scores))
      t(t(res.pca$scores %*% t(res.pca$loadings)) * res.pca$scale + res.pca$center)
    else stop("The object res.pca doesn't have the element scores. ", 
              "Please use the function princomp() with the argument ",
              "scores = TRUE.")
  }
  
  else stop("Can't handle an object of class ", .class_label(res.pca))
}



# Build plot title for contribution, cos2 plots
# used in fviz_cos2, fviz_contrib, etc
#+++++++++++++++++++++++
# element: possible values are row, col, var, ind
# varname: the name of the variable name to be plotted
  # possible values are "Cos2", "Contribution"
# axes: a numeric vector specifying the axes of interest
# example: .build_title("row", "Cos2", 1:2)
.build_title <- function(element, varname,  axes){
  if(varname =="cos2") varname = "Cos2"
  else if(varname =="contrib") varname = "Contribution"
  
  if(element=="row") 
    title <- paste0(varname, " of rows to Dim-", paste(axes, collapse="-"))
  else if(element=="col")
    title <- paste0(varname, " of columns to Dim-", paste(axes, collapse="-"))
  else if(element=="var")
    title <- paste0(varname, " of variables to Dim-", paste(axes, collapse="-"))
  else if(element=="ind")
    title <- paste0(varname, " of individuals to Dim-", paste(axes, collapse="-"))
  else if(element=="quanti.var")
    title <- paste0(varname, " of quantitive variables to Dim-", paste(axes, collapse="-"))
  else if(element=="quali.var")
    title <- paste0(varname, " of qualitative variables to Dim-", paste(axes, collapse="-"))
  else if(element=="group")
    title <- paste0(varname, " of groups to Dim-", paste(axes, collapse="-"))
  else if(element=="partial.axes")
    title <- paste0(varname, " of partial axes to Dim-", paste(axes, collapse="-"))
  
  return(title)
}

# Select rows according to a filter
# ++++++++++++++++++++
# d a data frame containing row names, coordinates, cos2, contrib, etc
# filter: a filter to be applied. Allowed values are NULL or a list containing the arguments either
#  name, cos2 or contrib
# - name: is a character vector containing row names of interest
# - cos2: if cos2 is in [0, 1], ex: 0.6, then rows with a cos2 > 0.6 are extracted.
#   if cos2 > 1, ex: 5, then the top 5 rows with the highest cos2 are extracted
# - contrib: if contrib > 1, ex: 5,  then the top 5 rows with the highest cos2 are extracted
# check: if TRUE, check the data after filtering
.select <- function(d, filter = NULL, check= TRUE){
  
  if(!is.null(filter)){
    
    # Filter by name
    if(!is.null(filter$name)){
      name <- filter$name
      common <- intersect(name, d$name)
      diff <- setdiff(name, d$name)
      d <- d[common, , drop = FALSE]
    }
    
    # Filter by cos2
    if(!is.null(filter$cos2) && nrow(d) >= 1){
      # case 1 cos2 is in [0, 1]
      # rows with cos2 > value are selected
      if(0 <= filter$cos2 && filter$cos2 <= 1){
        d <- d[which(d$cos2 >= filter$cos2), , drop = FALSE]
        if(check && nrow(d)==0)
          stop("There are no observations with cos2 >=", filter$cos2, 
               ". Please, change the value of cos2 and try again.")
      }
      # case 2 - cos2 > 1 : the top rows are selected 
      else if(filter$cos2 > 1){
        cos2 <- round(filter$cos2)
        d <- d[order(d$cos2, decreasing = TRUE), , drop = FALSE]
        d <- d[seq_len(min(filter$cos2, nrow(d))),, drop = FALSE]
      }
    }
    
    # Filter by contrib: the top rows are selected 
    if(!is.null(filter$contrib) && nrow(d) >= 1){
      contrib <- round(filter$contrib)
      if(contrib < 1) stop("The value of the argument contrib >", 1)
      d <- d[order(d$contrib, decreasing = TRUE), , drop = FALSE]
      d <- d[seq_len(min(contrib, nrow(d))), , drop = FALSE]
    }
    
  }
  
  return (d)
}


# Add supplementary points

# Make a ggplot2 bar plot
#+++++++++++++++++++++++++
# x: a numeric vector
# title, xlab, ylab : labels for the graph
# fill: a fill color for the bar plot
# color: an outline color for the bar plot
# sort.value: a string specifying whether x should be sorted or not. 
# Allowed values are "none" (no sorting), "asc" (for ascending) or "desc" (for descending)
# top a numeric value specifing the top  elements to be shown
.ggbarplot <- function(x,  title ="", xlab ="", ylab="",
                       fill="steelblue", color = "steelblue",  
                       sort.value = c("desc", "asc", "none"), top = Inf ){
  
 
  # top elements
  if(top != Inf && top < length(x))
    x <- sort(x, decreasing=TRUE)[1:top]
  # sorting
  if(sort.value[1]=="desc") x <- sort(x, decreasing = TRUE)
  else if(sort.value[1]=="asc") x <- sort(x, decreasing = FALSE)
  # bar names
  if(is.null(names(x))) names(x) <- seq_along(x)
  
  #data frame for ggplot2
  d <- cbind.data.frame(name = factor(names(x), levels = names(x)), val = x)
  
  # plot
  # FIX: ggplot2 3.0.0+ deprecation - aes_string() replaced with aes() + .data pronoun
  # aes_string() was soft-deprecated in ggplot2 3.0.0
  # See: https://github.com/kassambara/factoextra/issues/190
  p <- ggplot(d, aes(x = .data[["name"]], y = .data[["val"]])) +
    geom_bar(stat="identity", fill=fill, color = color) +
    labs(title = title,  x =xlab, y = ylab)+
    theme(axis.text.x = element_text(angle=45), 
                   axis.title.x = element_blank())
  
  return(p)
}



# Points jitter, to reduce overploting
# data : a result from facto_summarize containing the x and y coordinates of points
# jitter: a vector of length 3 containing the width and the height of jitter and the 
# element to be jittered ("label", "point", "both")
.jitter <- function(data, jitter = list(what = "label", width = NULL, height = NULL)){
  
  if(!is.null(data)){
    if(!is.null(jitter$width)){
      width <- abs(jitter$width)
      xjit <- .with_preserved_seed(1234, runif(nrow(data), min = -width, max = width))
      data$x <- data$x + ((xjit + rev(xjit))/2)
    }
    
    if(!is.null(jitter$height)){
      height <- abs(jitter$height)
      yjit <- .with_preserved_seed(12345, runif(nrow(data), min = -height, max = height))
      data$y <- data$y + ((yjit + rev(yjit))/2)
    }
    
  }
  
  return(data)
}



# Finih a plot
#++++++++++++++++++++++++++++++
# p a ggplot2
# X an object of class PCA, MCA or CA
# axes the plotted axes
.fviz_finish <- function(p, X, axes = 1:2, linetype = "dashed", xlab = NULL, ylab=NULL, ...){
  
  cc <- .get_facto_class(X)
  title <- paste0(cc, " factor map")
  
  eig <- get_eigenvalue(X)[,2]
  if(is.null(xlab)) xlab = paste0("Dim", axes[1], " (", round(eig[axes[1]],1), "%)") 
  if(is.null(ylab)) ylab = paste0("Dim", axes[2], " (", round(eig[axes[2]], 1),"%)")
  
  p <- p +
    geom_hline(yintercept = 0, color = "black", linetype=linetype) +
    geom_vline(xintercept = 0, color = "black", linetype=linetype) +
    labs(title = title, x = xlab, y = ylab)

  p <- .hide_text_legend(p)

  return(p)
}

# Stop text/label layers from injecting a stray glyph (e.g. "a") into the
# colour/fill legend (#14). Labels are still drawn on the plot; they are only
# removed from the legend keys. The colour/shape guides come from the point
# layers, so this never drops a legend that should exist.
.hide_text_legend <- function(p){
  text_geoms <- c("GeomText", "GeomTextRepel", "GeomLabel", "GeomLabelRepel")
  for(i in seq_along(p$layers)){
    if(inherits(p$layers[[i]]$geom, text_geoms))
      p$layers[[i]]$show.legend <- FALSE
  }
  p
}

# Check the element to be labelled
#+++++++++++++++++
## label: a character vector specifying the elements to be labelled
# possible values are "all", "none" or the combination of 
# c("row", "row.sup", "col", "col.sup", "ind", "ind.sup", "quali", "var", "quanti.sup")
## Returns a list 
# Warn when a label/invisible vector contains tokens that are not recognized, so
# a typo such as label = "id" (instead of "ind") no longer silently draws nothing
# but tells the user which values are valid. Only warns for genuinely unknown
# tokens; recognized values keep their exact previous behavior. (#165)
.warn_unknown_elements <- function(values, valid, arg){
  if(is.null(values)) return(invisible(NULL))
  unknown <- setdiff(as.character(values), valid)
  if(length(unknown))
    warning("Unknown ", arg, " value(s): ",
            paste0('"', unknown, '"', collapse = ", "),
            ". Valid values are \"all\", \"none\" or any of: ",
            paste0('"', setdiff(valid, c("all", "none")), '"', collapse = ", "),
            ".", call. = FALSE)
  invisible(NULL)
}

.label <- function(label){
  lab  <- list()
  element <- c("var", "quanti.sup", "quali.sup", "quanti.sup","quanti", # var - PCA, MCA, MFA
               "quanti.var.sup", "quanti.var", "quali.var", # MFA
               "ind", "ind.sup", "quali", "mca.cor",  # ind - PCA, MCA, MFA
               "group", "group.sup", # group - MFA, HMFA
               "partial.axes", # partial.axes - MFA
               "row", "row.sup", # row - ca
               "col", "col.sup" # col - ca
               )
  .warn_unknown_elements(label, c("all", "none", element), "label")
  for(el in element){
    if(label[1] == "all" || el %in% label) lab[[el]] <- TRUE
    else lab[[el]] <- FALSE
  }
  lab
}


# Check the element to be hidden
#+++++++++++++++++
## invisible: a character vector specifying the elements to be hidden
# possible values are "all", "none" or the combination of 
# c("row", "row.sup", "col", "col.sup", "ind", "ind.sup", "quali", "quali.sup",  "var", "quanti.sup")
## Returns a list 
.hide <- function(invisible){
  hide  <- list()
  element <- c("var", "quanti.sup", "quali.sup", "quanti.sup","quanti", # var - PCA, MCA, MFA
               "quanti.var.sup", "quanti.var",  "quali.var", # MFA
               "ind", "ind.sup", "quali", "mca.cor",  # ind - PCA, MCA, MFA
               "group", "group.sup", # group - MFA, HMFA
               "partial.axes", # partial.axes - MFA
               "row", "row.sup", # row - ca
               "col", "col.sup" # col - ca
  )
  .warn_unknown_elements(invisible, c("all", "none", element), "invisible")
  for(el in element){
    if(el %in% invisible) hide[[el]] <- TRUE
    else hide[[el]] <- FALSE
  }
  hide
}

# Generate a data containing a cluster of any shapes
# For comparison between dbscan and k-means
.generate_multishapes <- function(){
  .with_preserved_seed(1234, {
    # First circle (big)
    x <- matrix(rnorm(800, sd = 2), ncol = 2)
    y <- x/sqrt(rowSums(x^2))
    y[,1] <- y[,1] + rnorm(400, 0, 0.1)
    y[,2] <- y[,2] + rnorm(400, 0, 0.1)
    x1 <- y[, 1]
    y1 <- y[, 2]
    # Second circle (small)
    x2 <- x1/2.5
    y2 <- y1/2.5
    shape <- rep(c(1,2), each = 400)
    # Line 1
    x3 <- runif(100, min = -1.5, 0)
    y3 <- rnorm(100, -2, 0.1)
    shape <- c(shape, rep(3, 100))
    # Line 2
    x4 <- runif(100, min = -1.5, 0)
    y4 <- rnorm(100, -3, 0.1)
    shape <- c(shape, rep(4, 100))
    # compact points
    x6 <- rnorm(50, 1, 0.1)
    y6 <- rnorm(50, -2.5, 0.1)
    shape <- c(shape, rep(5, 50))
    # noises/outliers
    x5 <- runif(50, min = -1.5, 1.5)
    y5 <- rnorm(50, -1, 1)
    shape <- c(shape, rep(6, 50))
    data.frame(x = c(x1, x2, x3, x4, x5, x6), y = c(y1, y2, y3, y4, y5, y6), shape = shape)
  })
}


# Deprecated argument
.facto_dep <- function(arg, replace, return_val){
    warning("argument ", arg, " is deprecated; please use ", replace, " instead.", 
            call. = FALSE)
  return_val
}

# Check axis values
.validate_axis_indices <- function(axes, ndim = NULL){
  if(!is.numeric(axes) || length(axes) == 0L || anyNA(axes) || any(!is.finite(axes)) ||
     any(axes %% 1 != 0) || any(axes < 1))
    stop("The value of the argument axes is incorrect. axes should contain positive integers.")

  axes <- as.integer(axes)
  if(!is.null(ndim) && max(axes) > ndim)
    stop("The value of the argument axes is incorrect. ",
         "The number of axes in the data is: ", ndim,
         ". Please try again with axes between 1 - ", ndim)

  axes
}

# Check axis lengths
.check_axes <- function(axes, .length){
  axes <- .validate_axis_indices(axes)
  if(length(axes) != .length) stop("axes should be of length ", .length)
}

# Add individual groups column
# x an object of class PCA, MCA, ...
# ind: individuals data generated by facto_summarize()
# grp: group column index or factor
.add_ind_groups <- function(X, ind, grp){
  if(inherits(X, c("PCA", "MCA", "MFA", "FAMD")) && length(grp) > 1){
    if(is.numeric(grp) || is.character(grp)){
      data <- X$call$X
      # Only treat grp as column selectors when it isn't a per-row grouping vector.
      if(length(grp) != nrow(ind)){
        if(is.numeric(grp)){
          if(!all(grp %in% seq_len(ncol(data)))){
            stop("Some grouping variable indices are out of range for the original data.")
          }
        } else {
          if(!all(grp %in% colnames(data))){
            stop("Some grouping variable names are not present in the original data.")
          }
        }
        grp <- as.data.frame(data[rownames(ind), grp, drop = FALSE])
      }
    }
    #if(!is.null(X$call$ind.sup)) grp <- grp[-X$call$ind.sup, , drop = FALSE]
  }
  habillage <- grp
  # Data frame containing multiple grouping variables
  if(inherits(grp, c("matrix", "data.frame"))){
    if(nrow(ind) != nrow(grp)) stop("The length of grouping variables ",
                                    "should be the same as the number of individuals.")
    ind <- cbind.data.frame(ind, grp)
    ind[, colnames(grp)] <- apply(ind[, colnames(grp), drop = FALSE], 2, as.character)
    # Convert wide to long format using base R (replaces tidyr::pivot_longer)
    grp_cols <- colnames(grp)
    id_cols <- setdiff(colnames(ind), grp_cols)
    ind <- stats::reshape(ind, direction = "long",
                          varying = grp_cols,
                          v.names = "Groups",
                          timevar = "facet_vars",
                          times = grp_cols,
                          idvar = id_cols)
    rownames(ind) <- NULL
    ind$facet_vars <- as.factor(ind$facet_vars)
    ind$Groups <- as.factor(ind$Groups)
    name.quali <- "Groups"
  }
  else{
    # X is from FactoMineR outputs
    if(inherits(X, c("PCA", "MCA", "MFA", "FAMD")) && length(habillage) == 1){
      data <- X$call$X
      if (is.numeric(habillage)) name.quali <- colnames(data)[habillage]
      else name.quali <- habillage 
      ind <- cbind.data.frame(data[rownames(ind),name.quali], ind)
      colnames(ind)[1]<-name.quali
      if(!inherits(ind[, 1], "factor")) ind[, 1]<-as.factor(ind[,1])
    }
    else{
      if(nrow(ind)!=length(habillage))
        stop("The number of active individuals is different ",
             "from the length of the factor habillage. Please, remove the supplementary ",
             "individuals in the variable habillage.")
      name.quali <- "Groups"
      ind <- cbind.data.frame(Groups = habillage, ind)
      if(!inherits(ind[, 1], "factor")) ind[, 1] <- as.factor(ind[,1])
    }
  }
  
  list(ind = ind, name.quali = name.quali, is_multiple_habillage = is.data.frame(grp))
}


# MFA helpers
.mfa_group_names <- function(X){
  if(!inherits(X, "MFA")) return(NULL)
  group.names <- NULL
  if(!is.null(X$group$Lg)){
    group.names <- rownames(X$group$Lg)
    if(!is.null(group.names) && length(group.names) > 0){
      if(length(group.names) == length(X$call$group) + 1)
        group.names <- group.names[-length(group.names)]
      else if(length(group.names) > length(X$call$group))
        group.names <- group.names[seq_along(X$call$group)]
    }
  }
  if(is.null(group.names) || length(group.names) == 0){
    if(!is.null(X$call$name.group)) group.names <- X$call$name.group
  }
  if(is.null(group.names) || length(group.names) == 0){
    group.names <- paste0("Gr", seq_along(X$call$group))
  }
  group.names
}

.mfa_group_map <- function(X){
  if(!inherits(X, "MFA")) return(NULL)
  data <- X$call$X
  if(is.null(data)) return(NULL)
  data <- as.data.frame(data)
  group.sizes <- X$call$group
  group.types <- X$call$type
  if(is.null(group.sizes) || is.null(group.types)) return(NULL)

  group.names <- .mfa_group_names(X)
  vars <- colnames(data)
  if(is.null(vars)) vars <- paste0("V", seq_len(ncol(data)))

  group.id <- rep(seq_along(group.sizes), group.sizes)
  if(length(group.id) != length(vars)){
    n <- min(length(group.id), length(vars))
    group.id <- group.id[seq_len(n)]
    vars <- vars[seq_len(n)]
  }

  is.group.sup <- rep(FALSE, length(group.sizes))
  if(!is.null(X$call$num.group.sup)) is.group.sup[X$call$num.group.sup] <- TRUE
  var.is.numeric <- vapply(data[, seq_along(vars), drop = FALSE], is.numeric, logical(1))

  var.type <- character(length(vars))
  for(i in seq_along(vars)){
    g <- group.id[i]
    gtype <- group.types[g]
    if(gtype %in% c("c", "s")) var.type[i] <- "quanti"
    else if(gtype == "n") var.type[i] <- "quali"
    else if(gtype == "m") var.type[i] <- ifelse(var.is.numeric[i], "quanti", "quali")
    else var.type[i] <- NA_character_
  }

  data.frame(
    var = vars,
    group = group.names[group.id],
    group.id = group.id,
    group.type = group.types[group.id],
    var.type = var.type,
    is.group.sup = is.group.sup[group.id],
    stringsAsFactors = FALSE
  )
}

# MFA: get quantitative variables groups
# For plotting
.get_quanti_var_groups <- function(X){
  map <- .mfa_group_map(X)
  if(is.null(map)) return(NULL)
  quanti <- map[map$var.type == "quanti" & !map$is.group.sup, , drop = FALSE]
  if(nrow(quanti) == 0) return(NULL)
  if(!is.null(X$quanti.var$coord)){
    qnames <- rownames(X$quanti.var$coord)
    idx <- match(qnames, quanti$var)
    if(any(is.na(idx))) return(quanti$group)
    return(quanti$group[idx])
  }
  quanti$group
}

# Get qualitative supplementary variables
# Each variable is repeated xtimes = levels(variables)
# Used to color variable categories by variables
.get_quali_var_sup_names <- function(X){
  map <- .mfa_group_map(X)
  if(is.null(map)) return(NULL)
  sup <- map[map$is.group.sup & map$var.type == "quali", , drop = FALSE]
  if(nrow(sup) == 0) return(NULL)
  data <- X$call$X
  res <- NULL
  for(v in sup$var){
    res <- c(res, rep(v, length(levels(as.factor(data[, v])))))
  }
  res
}

.factominer_needs_category_map <- function(facto.class, element){
  if(!(facto.class %in% c("MCA", "MFA", "FAMD", "HMFA"))) return(FALSE)
  if(element == "var") return(facto.class == "MCA")
  element %in% c("quali.var", "quali.sup")
}

.split_partial_names <- function(names, group.names = NULL){
  if(length(names) == 0) return(data.frame(name = character(0), group.name = character(0)))
  name.part <- rep(NA_character_, length(names))
  group.part <- rep(NA_character_, length(names))

  if(!is.null(group.names) && length(group.names) > 0){
    escape_regex <- function(x){
      if(is.na(x)) return(NA_character_)
      specials <- c("\\", ".", "^", "$", "|", "(", ")", "[", "]", "{", "}", "*", "+", "?")
      for(s in specials){
        x <- gsub(s, paste0("\\\\", s), x, fixed = TRUE)
      }
      x
    }
    escaped <- vapply(group.names, escape_regex, character(1))
    pattern <- paste0("\\.(", paste(escaped, collapse = "|"), ")$")
    matched <- grepl(pattern, names)
    if(any(matched)){
      group.part[matched] <- sub(pattern, "\\1", names[matched])
      name.part[matched] <- sub(pattern, "", names[matched])
    }
  } else {
    matched <- rep(FALSE, length(names))
  }

  fallback <- !matched
  if(any(fallback)){
    split.pos <- regexpr(".", names[fallback], fixed = TRUE)
    name.part[fallback] <- ifelse(split.pos > 0, substr(names[fallback], 1, split.pos - 1), names[fallback])
    group.part[fallback] <- ifelse(split.pos > 0, substr(names[fallback], split.pos + 1, nchar(names[fallback])), NA_character_)
  }

  data.frame(name = name.part, group.name = group.part)
}

#' Map FactoMineR category labels to legacy naming patterns
#'
#' @param X a FactoMineR object (MCA, MFA, FAMD, HMFA).
#' @param element element to map. Use "var" for MCA categories or "quali.var"
#'   for MFA/FAMD/HMFA qualitative categories. "quali.sup" maps supplementary
#'   qualitative categories when available.
#' @return A data.frame with current labels, variable names, levels, and legacy
#'   naming patterns.
#' @examples
#' \donttest{
#' if (requireNamespace("FactoMineR", quietly = TRUE)) {
#'   data(poison)
#'   res.mca <- FactoMineR::MCA(poison, quanti.sup = 1:2, quali.sup = 3:4, graph = FALSE)
#'   head(factominer_category_map(res.mca, element = "var"))
#' }
#' }
#' @export
factominer_category_map <- function(X, element = c("quali.var", "quali.sup", "var")){
  element <- match.arg(element)
  facto.class <- .get_facto_class(X)
  if(!facto.class %in% c("MCA", "MFA", "FAMD", "HMFA"))
    stop("factominer_category_map() supports MCA, MFA, FAMD, and HMFA objects only.")

  elmt <- NULL
  if(element == "var"){
    if(!inherits(X, "MCA")) stop("element = 'var' is only supported for MCA outputs.")
    elmt <- X$var
  } else if(element == "quali.var"){
    if(!is.null(X$quali.var)) elmt <- X$quali.var
    else if(inherits(X, "MCA")) elmt <- X$var
  } else if(element == "quali.sup"){
    elmt <- .get_factominer_quali_sup(X)
  }

  if(is.null(elmt) || is.null(elmt$coord)){
    warning("No qualitative categories found for element = '", element, "'.")
    return(data.frame(current = character(0), variable = character(0), level = character(0),
                      legacy_dot = character(0), legacy_underscore = character(0),
                      legacy_equals = character(0), legacy_colon = character(0),
                      legacy_level = character(0), stringsAsFactors = FALSE))
  }

  current.names <- rownames(elmt$coord)
  if(is.null(current.names)) current.names <- character(0)

  data <- X$call$X
  if(is.null(data)){
    warning("Original data not found in X$call$X; cannot build category map.")
    return(data.frame(current = current.names, variable = NA_character_, level = NA_character_,
                      legacy_dot = NA_character_, legacy_underscore = NA_character_,
                      legacy_equals = NA_character_, legacy_colon = NA_character_,
                      legacy_level = NA_character_, stringsAsFactors = FALSE))
  }
  data <- as.data.frame(data)
  is.quali <- which(!vapply(data, is.numeric, logical(1)))
  if(length(is.quali) == 0){
    return(data.frame(current = current.names, variable = NA_character_, level = NA_character_,
                      legacy_dot = NA_character_, legacy_underscore = NA_character_,
                      legacy_equals = NA_character_, legacy_colon = NA_character_,
                      legacy_level = NA_character_, stringsAsFactors = FALSE))
  }

  data.quali <- data[, is.quali, drop = FALSE]
  data.quali <- droplevels(as.data.frame(lapply(data.quali, as.factor)))
  level.orig <- lapply(data.quali, levels)

  niveau <- unlist(level.orig)
  if(sum(duplicated(niveau)) > 0){
    for(j in seq_along(data.quali)){
      if (sum(niveau %in% levels(data.quali[[j]])) != nlevels(data.quali[[j]])) {
        levels(data.quali[[j]]) <- paste(names(data.quali)[j], levels(data.quali[[j]]), sep = "_")
      }
    }
  }

  variable <- rep(names(data.quali), vapply(data.quali, nlevels, integer(1)))
  listModa <- unlist(lapply(data.quali, levels))
  wlistModa <- which(listModa %in% c("y", "n", "Y", "N"))
  if(length(wlistModa) > 0){
    listModa[wlistModa] <- paste(variable[wlistModa], listModa[wlistModa], sep = ".")
  }
  level <- unlist(level.orig)

  map.all <- data.frame(current = listModa, variable = variable, level = level, stringsAsFactors = FALSE)
  map <- data.frame(current = current.names, variable = NA_character_, level = NA_character_, stringsAsFactors = FALSE)
  if(length(current.names) > 0){
    idx <- match(current.names, map.all$current)
    matched <- !is.na(idx)
    if(any(matched)){
      map[matched, c("variable", "level")] <- map.all[idx[matched], c("variable", "level")]
    }
  }

  map$legacy_dot <- ifelse(is.na(map$variable), NA_character_, paste(map$variable, map$level, sep = "."))
  map$legacy_underscore <- ifelse(is.na(map$variable), NA_character_, paste(map$variable, map$level, sep = "_"))
  map$legacy_equals <- ifelse(is.na(map$variable), NA_character_, paste(map$variable, map$level, sep = "="))
  map$legacy_colon <- ifelse(is.na(map$variable), NA_character_, paste(map$variable, map$level, sep = ":"))
  map$legacy_level <- map$level
  map
}

# Disambiguate duplicated category names (e.g. FAMD/MFA qualitative variables
# that share factor-level names such as "Low"/"High"). FactoMineR returns the
# raw level names as row names, which are not necessarily unique across
# variables; setting them as data.frame row names then errors with
# "duplicate 'row.names' are not allowed" (see issues #184, #140).
#
# When duplicates exist, colliding categories are relabelled as
# "variable_level" (only the levels actually involved in a collision are
# prefixed; unique levels keep their plain label). If the original data or the
# variable mapping is unavailable, falls back to make.unique() so the call
# never crashes. When there are no duplicates this is a no-op and the names are
# returned unchanged.
.disambiguate_category_names <- function(X, name, element, facto_class){
  if(length(name) == 0 || !any(duplicated(name))) return(name)
  if(!is.null(facto_class) && facto_class %in% c("FAMD", "MFA", "MCA", "HMFA") &&
     element %in% c("quali.var", "quali.sup")){
    data <- tryCatch(as.data.frame(X$call$X), error = function(e) NULL)
    if(!is.null(data)){
      is.quali <- which(!vapply(data, is.numeric, logical(1)))
      if(length(is.quali) > 0){
        data.quali <- droplevels(as.data.frame(lapply(data[, is.quali, drop = FALSE], as.factor)))
        level.pos <- unlist(lapply(data.quali, levels), use.names = FALSE)
        var.pos   <- rep(names(data.quali), vapply(data.quali, nlevels, integer(1)))
        # Only trust the positional variable mapping when it lines up exactly
        # with the supplied names (same length and order).
        if(length(level.pos) == length(name) && all(level.pos == name)){
          dup.level <- level.pos %in% level.pos[duplicated(level.pos)]
          new <- ifelse(dup.level, paste(var.pos, level.pos, sep = "_"), level.pos)
          if(!any(duplicated(new))) return(new)
        }
      }
    }
  }
  make.unique(name)
}

#' Map legacy FactoMineR category names to current labels
#'
#' @param X a FactoMineR object (MCA, MFA, FAMD, HMFA).
#' @param names character vector of category labels.
#' @param element element to map. Use "var" for MCA categories or "quali.var"
#'   for MFA/FAMD/HMFA qualitative categories. "quali.sup" maps supplementary
#'   qualitative categories when available.
#' @param quiet if TRUE, suppress warnings.
#' @return Character vector of mapped labels.
#' @examples
#' \donttest{
#' if (requireNamespace("FactoMineR", quietly = TRUE)) {
#'   data(poison)
#'   res.mca <- FactoMineR::MCA(poison, quanti.sup = 1:2, quali.sup = 3:4, graph = FALSE)
#'   map <- factominer_category_map(res.mca, element = "var")
#'   map_factominer_legacy_names(res.mca, map$legacy_underscore[1:3], element = "var", quiet = TRUE)
#' }
#' }
#' @export
map_factominer_legacy_names <- function(X, names, element = c("quali.var", "quali.sup", "var"), quiet = FALSE){
  element <- match.arg(element)
  names <- as.character(names)
  map <- factominer_category_map(X, element = element)
  if(nrow(map) == 0) return(names)

  candidate.cols <- c("current", "legacy_dot", "legacy_underscore", "legacy_equals", "legacy_colon", "legacy_level")
  candidate.tbl <- do.call(rbind, lapply(candidate.cols, function(col){
    data.frame(candidate = map[[col]], current = map$current, stringsAsFactors = FALSE)
  }))
  candidate.tbl <- candidate.tbl[!is.na(candidate.tbl$candidate) & nzchar(candidate.tbl$candidate), , drop = FALSE]
  if(nrow(candidate.tbl) == 0) return(names)

  counts <- table(candidate.tbl$candidate)
  ambiguous <- names(counts[counts > 1])
  lookup <- candidate.tbl[!candidate.tbl$candidate %in% ambiguous, , drop = FALSE]
  lookup <- stats::setNames(lookup$current, lookup$candidate)

  res <- names
  mapped <- logical(length(names))
  unmatched <- character(0)

  for(i in seq_along(names)){
    nm <- names[i]
    if(is.na(nm) || !nzchar(nm)) next
    if(nm %in% map$current) next
    if(nm %in% names(lookup)){
      res[i] <- lookup[[nm]]
      mapped[i] <- TRUE
    } else {
      unmatched <- c(unmatched, nm)
    }
  }

  if(!quiet){
    if(any(mapped)){
      warning(
        "Mapped legacy FactoMineR category labels to current names. ",
        "Use factominer_category_map() to inspect mappings."
      )
    }
    if(length(unmatched) > 0){
      unmatched <- unique(unmatched)
      warning(
        "Some category labels were not found in the current FactoMineR output: ",
        paste(utils::head(unmatched, 8), collapse = ", "),
        if(length(unmatched) > 8) " ..." else "",
        ". Use factominer_category_map() and map_factominer_legacy_names() to map legacy labels."
      )
    }
  }

  res
}


# Principal component methods with FactoMineR
f_pca <- function(X, graph = FALSE){
  FactoMineR::PCA(X, graph = FALSE)
}

test_that("PCA extractors return expected structures", {
  res <- stats::prcomp(iris[, 1:4], scale. = TRUE)

  ind <- get_pca_ind(res)
  var <- get_pca_var(res)

  expect_s3_class(ind, "factoextra")
  expect_s3_class(var, "factoextra")
  expect_true(all(c("coord", "cos2", "contrib") %in% names(ind)))
  expect_true(all(c("coord", "cor", "cos2", "contrib") %in% names(var)))
  expect_true(all(startsWith(colnames(ind$coord), "Dim.")))
  expect_true(all(startsWith(colnames(var$coord), "Dim.")))
})

test_that("CA extractors work with FactoMineR outputs", {
  skip_if_not_installed("FactoMineR")

  data("housetasks", package = "factoextra")
  res.ca <- FactoMineR::CA(housetasks, graph = FALSE)

  cols <- get_ca_col(res.ca)
  rows <- get_ca_row(res.ca)

  expect_s3_class(cols, "factoextra")
  expect_s3_class(rows, "factoextra")
  expect_true(all(c("coord", "cos2", "contrib") %in% names(cols)))
  expect_true(all(c("coord", "cos2", "contrib") %in% names(rows)))
})

test_that("facto_summarize respects axes for MCA supplementary elements", {
  skip_if_not_installed("FactoMineR")

  set.seed(123)
  mca_df <- data.frame(
    a = factor(sample(letters[1:3], 80, TRUE)),
    b = factor(sample(LETTERS[1:4], 80, TRUE)),
    c = factor(sample(c("x", "y", "z"), 80, TRUE)),
    sup_q = factor(sample(c("q1", "q2"), 80, TRUE)),
    sup_n = rnorm(80)
  )
  mca <- FactoMineR::MCA(mca_df, quali.sup = 4, quanti.sup = 5, graph = FALSE)

  mca12 <- facto_summarize(mca, element = "mca.cor", axes = 1:2)
  mca23 <- facto_summarize(mca, element = "mca.cor", axes = 2:3)
  expect_equal(ncol(mca12), 3)
  expect_equal(ncol(mca23), 3)
  expect_false(isTRUE(all.equal(mca12[, 2:3], mca23[, 2:3])))

  q12 <- facto_summarize(mca, element = "quanti.sup", axes = 1:2)
  q23 <- facto_summarize(mca, element = "quanti.sup", axes = 2:3)
  expect_equal(ncol(q12), 3)
  expect_equal(ncol(q23), 3)
  expect_false(isTRUE(all.equal(q12[, 2:3], q23[, 2:3])))

  ind12 <- facto_summarize(mca, element = "ind", axes = 1:2)
  ind23 <- facto_summarize(mca, element = "ind", axes = 2:3)
  expect_equal(ncol(ind12), 6)
  expect_equal(ncol(ind23), 6)
  expect_identical(colnames(ind12)[2:3], c("Dim.1", "Dim.2"))
  expect_identical(colnames(ind23)[2:3], c("Dim.2", "Dim.3"))
})

test_that("FactoMineR category mapping helpers map legacy labels", {
  skip_if_not_installed("FactoMineR")

  set.seed(456)
  mca_df <- data.frame(
    color = factor(sample(c("red", "blue", "green"), 100, TRUE)),
    shape = factor(sample(c("round", "square"), 100, TRUE)),
    size = factor(sample(c("small", "medium", "large"), 100, TRUE))
  )
  mca <- FactoMineR::MCA(mca_df, graph = FALSE)
  map <- factominer_category_map(mca, element = "var")
  expect_s3_class(map, "data.frame")
  expect_true(all(c("current", "legacy_underscore") %in% colnames(map)))

  candidates <- map$legacy_underscore
  candidates <- candidates[!is.na(candidates) & nzchar(candidates)]
  uniq <- candidates[!(duplicated(candidates) | duplicated(candidates, fromLast = TRUE))]
  skip_if(length(uniq) < 1)

  mapped <- map_factominer_legacy_names(mca, uniq[1], element = "var", quiet = TRUE)
  expect_true(mapped %in% map$current)

  expect_warning(
    map_factominer_legacy_names(mca, uniq[1], element = "var", quiet = FALSE),
    "Mapped legacy FactoMineR category labels"
  )
  expect_warning(
    map_factominer_legacy_names(mca, "unknown_level", element = "var", quiet = FALSE),
    "not found in the current FactoMineR output"
  )
  expect_identical(
    map_factominer_legacy_names(mca, "unknown_level", element = "var", quiet = TRUE),
    "unknown_level"
  )
})

test_that("FAMD supplementary qualitative variables can be extracted and summarized", {
  skip_if_not_installed("FactoMineR")

  set.seed(789)
  famd_df <- data.frame(
    cat_a = factor(sample(c("a", "b", "c"), 80, TRUE)),
    cat_b = factor(sample(c("x", "y"), 80, TRUE)),
    num_a = rnorm(80),
    num_b = rnorm(80),
    sup_q = factor(sample(c("north", "south", "west"), 80, TRUE))
  )
  res.famd <- FactoMineR::FAMD(famd_df, sup.var = 5, graph = FALSE)

  quali.sup <- get_famd_var(res.famd, "quali.sup")
  expect_s3_class(quali.sup, "factoextra")
  expect_true(all(c("coord", "cos2") %in% names(quali.sup)))
  expect_gt(nrow(quali.sup$coord), 0)
  expect_no_error(print(quali.sup))

  sup.sum <- facto_summarize(res.famd, element = "quali.sup", axes = 1:2)
  expect_true(all(c("name", "Dim.1", "Dim.2", "coord", "cos2") %in% colnames(sup.sum)))
  expect_false("contrib" %in% colnames(sup.sum))

  res.famd.no.sup <- FactoMineR::FAMD(famd_df[, 1:4], graph = FALSE)
  expect_error(
    get_famd_var(res.famd.no.sup, "quali.sup"),
    "There are no supplementary qualitative variables"
  )
})

test_that("MFA supplementary qualitative variables can be extracted and mapped", {
  skip_if_not_installed("FactoMineR")

  data(poison, package = "FactoMineR")
  res.mfa <- FactoMineR::MFA(
    poison,
    group = c(2, 2, 5, 6),
    type = c("s", "n", "n", "n"),
    name.group = c("desc", "desc2", "symptom", "eat"),
    num.group.sup = 1:2,
    graph = FALSE
  )

  quali.sup <- get_mfa_var(res.mfa, "quali.sup")
  expect_s3_class(quali.sup, "factoextra")
  expect_true(all(c("coord", "cos2") %in% names(quali.sup)))
  expect_identical(
    rownames(quali.sup$coord),
    rownames(res.mfa$quali.var.sup$coord)
  )

  sup.sum <- facto_summarize(res.mfa, element = "quali.sup", axes = 1:2)
  expect_true(all(c("name", "Dim.1", "Dim.2", "coord", "cos2") %in% colnames(sup.sum)))
  expect_false("contrib" %in% colnames(sup.sum))

  map <- factominer_category_map(res.mfa, element = "quali.sup")
  expect_s3_class(map, "data.frame")
  expect_identical(map$current, rownames(res.mfa$quali.var.sup$coord))

  expect_no_error(print(quali.sup))

  res.mfa.no.sup <- FactoMineR::MFA(
    poison,
    group = c(2, 2, 5, 6),
    type = c("s", "n", "n", "n"),
    name.group = c("desc", "desc2", "symptom", "eat"),
    graph = FALSE
  )
  expect_error(
    get_mfa_var(res.mfa.no.sup, "quali.sup"),
    "There are no supplementary qualitative variables"
  )
})

test_that(".add_ind_groups handles multi-column habillage reshape path", {
  skip_if_not_installed("FactoMineR")

  pca <- FactoMineR::PCA(iris[, 1:4], graph = FALSE)
  ind <- facto_summarize(pca, element = "ind", axes = 1:2)
  grp <- data.frame(
    species = iris$Species,
    width_band = ifelse(
      iris$Sepal.Width > stats::median(iris$Sepal.Width),
      "high", "low"
    ),
    stringsAsFactors = FALSE
  )

  res <- factoextra:::.add_ind_groups(pca, ind, grp)
  expect_true(is.list(res))
  expect_true(res$is_multiple_habillage)
  expect_identical(res$name.quali, "Groups")
  expect_true(all(c("facet_vars", "Groups") %in% colnames(res$ind)))
  expect_equal(nrow(res$ind), nrow(ind) * ncol(grp))
  expect_setequal(levels(res$ind$facet_vars), colnames(grp))
})

test_that("get_clust_tendency restores RNG state when seed is provided", {
  old_opt <- getOption("factoextra.warn_hopkins", TRUE)
  on.exit(options(factoextra.warn_hopkins = old_opt), add = TRUE)
  options(factoextra.warn_hopkins = FALSE)

  set.seed(2026)
  seed_before <- get(".Random.seed", envir = .GlobalEnv)
  get_clust_tendency(iris[, 1:4], n = 10, graph = FALSE, seed = 123)
  seed_after <- get(".Random.seed", envir = .GlobalEnv)
  expect_identical(seed_after, seed_before)
})

test_that("get_clust_tendency Hopkins value matches regression baseline", {
  old_opt <- getOption("factoextra.warn_hopkins", TRUE)
  on.exit(options(factoextra.warn_hopkins = old_opt), add = TRUE)
  options(factoextra.warn_hopkins = FALSE)

  res <- get_clust_tendency(iris[, 1:4], n = 10, graph = FALSE, seed = 123)
  # BLAS matrix operations (tcrossprod/outer) yield slightly different

  # floating-point results across platforms; use a wider tolerance.
  expect_equal(res$hopkins_stat, 0.989362891844348, tolerance = 0.01)
})

test_that("get_clust_tendency low-memory fallback matches vectorized computation", {
  old_warn <- getOption("factoextra.warn_hopkins", TRUE)
  old_cells <- getOption("factoextra.hopkins.max_matrix_cells", 2e7)
  on.exit(
    options(
      factoextra.warn_hopkins = old_warn,
      factoextra.hopkins.max_matrix_cells = old_cells
    ),
    add = TRUE
  )
  options(factoextra.warn_hopkins = FALSE)

  options(factoextra.hopkins.max_matrix_cells = 1)
  res_loop <- get_clust_tendency(iris[, 1:4], n = 10, graph = FALSE, seed = 123)

  options(factoextra.hopkins.max_matrix_cells = 1e9)
  res_vec <- get_clust_tendency(iris[, 1:4], n = 10, graph = FALSE, seed = 123)

  # Different BLAS accumulation order in chunked vs full-matrix tcrossprod
  # can shift nearest-neighbor selection, producing different Hopkins values.
  expect_equal(res_loop$hopkins_stat, res_vec$hopkins_stat, tolerance = 0.01)
})

test_that("get_clust_tendency emits correction warning only once per session", {
  old_opt <- getOption("factoextra.warn_hopkins", TRUE)
  on.exit(options(factoextra.warn_hopkins = old_opt), add = TRUE)
  options(factoextra.warn_hopkins = TRUE)

  state <- getFromNamespace(".factoextra_state", "factoextra")
  state$hopkins_warned <- FALSE
  expect_warning(
    get_clust_tendency(iris[, 1:4], n = 10, graph = FALSE, seed = 123),
    "corrected formula"
  )
  expect_no_warning(get_clust_tendency(iris[, 1:4], n = 10, graph = FALSE, seed = 123))
})

test_that("fviz_nbclust gap_stat handles maxSE in dots and forwards clusGap args", {
  set.seed(123)
  x <- scale(iris[, 1:4])
  p <- fviz_nbclust(
    x, FUNcluster = stats::kmeans, method = "gap_stat",
    k.max = 3, nboot = 3, verbose = FALSE,
    nstart = 5,
    maxSE = list(method = "firstmax", SE.factor = 1)
  )
  expect_s3_class(p, "ggplot")
})

test_that("fviz_nbclust wss path returns ggplot and forwards FUNcluster args", {
  set.seed(123)
  x <- scale(iris[, 1:4])
  p <- fviz_nbclust(x, FUNcluster = stats::kmeans, method = "wss", k.max = 4, nstart = 5)
  expect_s3_class(p, "ggplot")
  expect_equal(nrow(p$data), 4)
  expect_false(anyNA(p$data$y))
})

test_that("fviz_nbclust wss handles clustering helpers that reject k = 1", {
  x <- scale(iris[, 1:4])

  p_hcut <- fviz_nbclust(
    x, FUNcluster = hcut, method = "wss", k.max = 4,
    hc_method = "complete"
  )
  expect_s3_class(p_hcut, "ggplot")
  expect_equal(nrow(p_hcut$data), 4)
  expect_false(anyNA(p_hcut$data$y))

  p_hkmeans <- fviz_nbclust(x, FUNcluster = hkmeans, method = "wss", k.max = 4)
  expect_s3_class(p_hkmeans, "ggplot")
  expect_equal(nrow(p_hkmeans$data), 4)
  expect_false(anyNA(p_hkmeans$data$y))
})

test_that("fviz_nbclust gap_stat continues to support hcut", {
  set.seed(123)
  x <- scale(iris[, 1:4])
  p <- fviz_nbclust(
    x, FUNcluster = hcut, method = "gap_stat",
    k.max = 3, nboot = 2, verbose = FALSE,
    hc_method = "complete"
  )
  expect_s3_class(p, "ggplot")
})

test_that("fviz_nbclust handles matrix Best.nc and preserves numeric cluster order", {
  best_nc <- rbind(
    Number_clusters = c(2, 10, 3, 10),
    Value_Index = c(1, 1, 1, 1)
  )
  p <- fviz_nbclust(list(Best.nc = best_nc), print.summary = FALSE)
  expect_s3_class(p, "ggplot")
  expect_identical(levels(p$data$Number_clusters), c("2", "3", "10"))
})

test_that("fviz_nbclust silhouette handles k >= n without error", {
  x <- matrix(seq_len(20), ncol = 4)
  toy_cluster <- function(x, k, ...) {
    n <- nrow(as.matrix(x))
    if (k >= n) cl <- seq_len(n) else cl <- rep(seq_len(k), length.out = n)
    list(cluster = cl)
  }
  p <- fviz_nbclust(x, FUNcluster = toy_cluster, method = "silhouette", k.max = 5)
  expect_s3_class(p, "ggplot")
  expect_true(anyNA(p$data$y))
})

test_that("fviz_nbclust silhouette omits the undefined k = 1 point", {
  x <- rbind(
    matrix(rnorm(20, mean = 0, sd = 0.1), ncol = 2),
    matrix(rnorm(20, mean = 5, sd = 0.1), ncol = 2)
  )
  toy_cluster <- function(x, k, ...) {
    n <- nrow(as.matrix(x))
    list(cluster = rep(seq_len(k), length.out = n))
  }

  p <- fviz_nbclust(x, FUNcluster = toy_cluster, method = "silhouette", k.max = 3)
  expect_s3_class(p, "ggplot")
  expect_identical(as.integer(as.character(p$data$clusters)), 2:3)

  built <- ggplot2::ggplot_build(p)
  vline_data <- built$data[[length(built$data)]]
  expect_equal(as.numeric(vline_data$xintercept), 1)
})

test_that(".is_color returns a logical vector type-stably", {
  res_empty <- factoextra:::.is_color(character(0))
  expect_type(res_empty, "logical")
  expect_length(res_empty, 0)

  res <- factoextra:::.is_color(c("red", "not-a-color"))
  expect_type(res, "logical")
  expect_identical(unname(res), c(TRUE, FALSE))
})

test_that(".is_color_palette recognizes known palette names", {
  expect_true(factoextra:::.is_color_palette("Blues"))
  expect_true(factoextra:::.is_color_palette("jco"))
  expect_false(factoextra:::.is_color_palette("not-a-palette"))
  expect_false(factoextra:::.is_color_palette(NULL))
})

test_that("eclust restores RNG state after execution", {
  set.seed(2027)
  seed_before <- get(".Random.seed", envir = .GlobalEnv)
  eclust(scale(USArrests), FUNcluster = "kmeans", k = 2, graph = FALSE)
  seed_after <- get(".Random.seed", envir = .GlobalEnv)
  expect_identical(seed_after, seed_before)
})

test_that("eclust hierarchical auto-k handles a one-cluster gap selection", {
  x <- scale(iris[, 1:4])
  fake_gap <- structure(
    list(Tab = cbind(gap = c(0.5, 0.4, 0.3), SE.sim = c(0.05, 0.05, 0.05))),
    class = "clusGap"
  )
  testthat::local_mocked_bindings(
    .gap_stat = function(...) list(stat = fake_gap, k = 1L),
    .env = environment(eclust)
  )

  for(fun in c("hclust", "agnes", "diana")) {
    res <- eclust(
      x, FUNcluster = fun, k = NULL, k.max = 3, nboot = 2,
      verbose = FALSE, graph = FALSE, hc_method = "complete", seed = 1
    )
    expect_s3_class(res, "eclust")
    expect_identical(res$nbclust, 1L)
    expect_true(all(res$cluster == 1L))
    expect_identical(names(res$cluster), rownames(x))
    expect_identical(res$size, nrow(x))
  }
})

test_that("fviz_silhouette errors cleanly for one-cluster hierarchical results", {
  x <- scale(iris[, 1:4])
  fake_gap <- structure(
    list(Tab = cbind(gap = c(0.5, 0.4, 0.3), SE.sim = c(0.05, 0.05, 0.05))),
    class = "clusGap"
  )
  testthat::local_mocked_bindings(
    .gap_stat = function(...) list(stat = fake_gap, k = 1L),
    .env = environment(eclust)
  )

  res <- eclust(
    x, FUNcluster = "hclust", k = NULL, k.max = 3, nboot = 2,
    verbose = FALSE, graph = FALSE, hc_method = "complete", seed = 1
  )
  expect_error(
    fviz_silhouette(res),
    "Silhouette information is unavailable"
  )
})

test_that("internal jitter and multishape helpers preserve RNG state", {
  set.seed(3030)
  seed_before <- get(".Random.seed", envir = .GlobalEnv)
  out <- factoextra:::.jitter(
    data.frame(x = 1:5, y = 1:5),
    jitter = list(width = 0.2, height = 0.3)
  )
  seed_after <- get(".Random.seed", envir = .GlobalEnv)
  expect_identical(seed_after, seed_before)
  expect_equal(dim(out), c(5, 2))

  set.seed(3031)
  seed_before <- get(".Random.seed", envir = .GlobalEnv)
  generated <- factoextra:::.generate_multishapes()
  seed_after <- get(".Random.seed", envir = .GlobalEnv)
  expect_identical(seed_after, seed_before)
  expect_true(nrow(generated) > 0)
})

test_that(".with_preserved_seed evaluates correctly through wrapper frames", {
  wrapper <- function(seed, expr) {
    inner <- function(code) factoextra:::.with_preserved_seed(seed, code)
    inner(expr)
  }

  set.seed(9090)
  seed_before <- get(".Random.seed", envir = .GlobalEnv)

  set.seed(123)
  expected <- stats::runif(5)
  assign(".Random.seed", seed_before, envir = .GlobalEnv)

  out <- wrapper(123, stats::runif(5))
  seed_after <- get(".Random.seed", envir = .GlobalEnv)
  expect_equal(out, expected)
  expect_identical(seed_after, seed_before)
  expect_length(out, 5)
})

test_that("phylogenic dendrogram layout does not leak RNG state", {
  skip_if_not_installed("igraph")
  set.seed(4040)
  seed_before <- get(".Random.seed", envir = .GlobalEnv)
  hc <- hclust(dist(iris[, 1:4]))
  p <- expect_no_warning(
    fviz_dend(hc, k = 3, type = "phylogenic", show_labels = FALSE)
  )
  seed_after <- get(".Random.seed", envir = .GlobalEnv)
  expect_identical(seed_after, seed_before)
  expect_s3_class(p, "ggplot")
})

test_that("phylogenic dendrogram accepts layout_nicely compatibility alias", {
  skip_if_not_installed("igraph")
  set.seed(5050)
  seed_before <- get(".Random.seed", envir = .GlobalEnv)
  hc <- hclust(dist(iris[, 1:4]))
  p <- expect_no_warning(
    fviz_dend(hc, k = 3, type = "phylogenic", phylo_layout = "layout_nicely", show_labels = FALSE)
  )
  seed_after <- get(".Random.seed", envir = .GlobalEnv)
  expect_identical(seed_after, seed_before)
  expect_s3_class(p, "ggplot")
})

test_that("phylogenic dendrogram keeps legacy layout.gem warning-free", {
  skip_if_not_installed("igraph")
  set.seed(6060)
  seed_before <- get(".Random.seed", envir = .GlobalEnv)
  hc <- hclust(dist(iris[, 1:4]))
  p <- expect_no_warning(
    fviz_dend(hc, k = 3, type = "phylogenic", phylo_layout = "layout.gem", show_labels = FALSE)
  )
  seed_after <- get(".Random.seed", envir = .GlobalEnv)
  expect_identical(seed_after, seed_before)
  expect_s3_class(p, "ggplot")
})

test_that("phylogenic dendrogram accepts layout_with_gem alias", {
  skip_if_not_installed("igraph")
  set.seed(7070)
  seed_before <- get(".Random.seed", envir = .GlobalEnv)
  hc <- hclust(dist(iris[, 1:4]))
  p <- expect_no_warning(
    fviz_dend(hc, k = 3, type = "phylogenic", phylo_layout = "layout_with_gem", show_labels = FALSE)
  )
  seed_after <- get(".Random.seed", envir = .GlobalEnv)
  expect_identical(seed_after, seed_before)
  expect_s3_class(p, "ggplot")
})

test_that("phylogenic dendrogram keeps legacy layout.mds warning-free", {
  skip_if_not_installed("igraph")
  set.seed(8080)
  seed_before <- get(".Random.seed", envir = .GlobalEnv)
  hc <- hclust(dist(iris[, 1:4]))
  p <- expect_no_warning(
    fviz_dend(hc, k = 3, type = "phylogenic", phylo_layout = "layout.mds", show_labels = FALSE)
  )
  seed_after <- get(".Random.seed", envir = .GlobalEnv)
  expect_identical(seed_after, seed_before)
  expect_s3_class(p, "ggplot")
})

test_that("phylogenic dendrogram accepts layout_with_mds alias", {
  skip_if_not_installed("igraph")
  set.seed(9090)
  seed_before <- get(".Random.seed", envir = .GlobalEnv)
  hc <- hclust(dist(iris[, 1:4]))
  p <- expect_no_warning(
    fviz_dend(hc, k = 3, type = "phylogenic", phylo_layout = "layout_with_mds", show_labels = FALSE)
  )
  seed_after <- get(".Random.seed", envir = .GlobalEnv)
  expect_identical(seed_after, seed_before)
  expect_s3_class(p, "ggplot")
})

test_that("hmfa group printing lists only available result components", {
  skip_if_not_installed("FactoMineR")

  data(wine, package = "FactoMineR")
  hierar <- list(c(2, 5, 3, 10, 9, 2), c(4, 2))
  res.hmfa <- FactoMineR::HMFA(wine, H = hierar, type = c("n", rep("s", 5)), graph = FALSE)
  grp <- get_hmfa_var(res.hmfa, "group")

  out <- capture.output(print(grp))
  expect_true(any(grepl("\\$coord", out)))
  expect_true(any(grepl("\\$canonical", out)))
  expect_false(any(grepl('""\\s+""', out)))
})

test_that("get_pca_ind returns finite cos2 for zero-distance rows", {
  x <- data.frame(a = c(-1, 0, 1), b = c(-2, 0, 2))
  res <- stats::prcomp(x, center = TRUE, scale. = FALSE)

  ind <- get_pca_ind(res)

  expect_true(all(is.finite(ind$cos2)))
  expect_equal(unname(ind$cos2[2, ]), c(0, 0))
})

test_that("fviz text labels do not leak a glyph into the legend (#14)", {
  res <- stats::prcomp(mtcars[, 1:7], scale. = TRUE)
  p <- fviz_pca_ind(res, habillage = factor(mtcars$carb))

  text_geoms <- c("GeomText", "GeomTextRepel", "GeomLabel", "GeomLabelRepel")
  text_layers <- Filter(function(L) inherits(L$geom, text_geoms), p$layers)

  expect_gt(length(text_layers), 0)
  expect_true(all(vapply(text_layers, function(L) isFALSE(L$show.legend), logical(1))))
})

test_that("FAMD plots handle qualitative variables with shared factor levels (#184, #140)", {
  skip_if_not_installed("FactoMineR")

  set.seed(1)
  n <- 60
  df <- data.frame(
    x1 = rnorm(n), x2 = rnorm(n), x3 = rnorm(n),
    f1 = factor(sample(c("Low", "Medium", "High"), n, replace = TRUE)),
    f2 = factor(sample(c("Low", "High"), n, replace = TRUE)) # shares Low/High with f1
  )
  res <- FactoMineR::FAMD(df, graph = FALSE)

  # Previously errored with "duplicate 'row.names' are not allowed"
  expect_s3_class(fviz_contrib(res, choice = "quali.var"), "ggplot")
  expect_s3_class(fviz_contrib(res, choice = "quanti.var"), "ggplot")
  expect_s3_class(fviz_famd_ind(res), "ggplot")
  expect_s3_class(fviz_famd_var(res, "quali.var"), "ggplot")

  # Colliding categories are disambiguated as variable_level; unique ones stay plain
  fc <- facto_summarize(res, element = "quali.var", result = "contrib", axes = 1)
  expect_false(any(duplicated(fc$name)))
  expect_true(all(c("f1_Low", "f1_High", "f2_Low", "f2_High") %in% fc$name))
  expect_true("Medium" %in% fc$name) # Medium only in f1 -> not prefixed
})

test_that("FAMD without shared factor levels keeps plain category labels (#184)", {
  skip_if_not_installed("FactoMineR")

  data(wine, package = "FactoMineR")
  res <- FactoMineR::FAMD(wine, graph = FALSE)
  fc <- facto_summarize(res, element = "quali.var", result = "contrib", axes = 1)

  expect_true(all(c("Saumur", "Env1") %in% fc$name))
  expect_false(any(grepl("_", fc$name))) # no disambiguation when names are unique
})

test_that("fviz_pca_biplot auto scaling keeps plot coordinates finite", {
  x <- data.frame(a = c(1, 1, 1), b = c(-1, 0, 1), c = c(-2, 0, 2))
  res <- stats::prcomp(x, center = TRUE, scale. = FALSE)

  p <- fviz_pca_biplot(res, axes = c(2, 3), biplot.type = "auto")
  built <- ggplot2::ggplot_build(p)
  coords <- lapply(
    built$data,
    function(layer) layer[, intersect(c("x", "y", "xend", "yend"), names(layer)), drop = FALSE]
  )

  expect_false(any(vapply(coords, function(layer) any(!is.finite(as.matrix(layer))), logical(1))))
})

test_that("legacy fviz_cluster arguments emit deprecation warnings", {
  x <- scale(USArrests)
  km <- stats::kmeans(x, centers = 3, nstart = 5)

  expect_warning(
    fviz_cluster(km, data = x, jitter = list(width = 0.1)),
    "deprecated"
  )
  expect_warning(
    fviz_cluster(km, data = x, frame = TRUE),
    "deprecated"
  )
  expect_warning(
    fviz_cluster(km, data = x, frame.type = "norm"),
    "deprecated"
  )
  expect_warning(
    fviz_cluster(km, data = x, frame.level = 0.8),
    "deprecated"
  )
  expect_warning(
    fviz_cluster(km, data = x, frame.alpha = 0.3),
    "deprecated"
  )
  expect_warning(
    fviz_cluster(km, data = x, title = "Legacy title"),
    "deprecated"
  )
  expect_warning({
    p_jitter_true <- fviz_cluster(km, data = x, jitter = TRUE)
    expect_s3_class(p_jitter_true, "ggplot")
  }, "deprecated")
})

test_that("fviz_eig validates parallel.seed range and integer-ness", {
  res.pca <- stats::prcomp(iris[, 1:4], scale. = TRUE)

  expect_error(
    fviz_eig(
      res.pca, choice = "eigenvalue", parallel = TRUE,
      parallel.iter = 2, parallel.seed = 1e10
    ),
    "single integer value in"
  )
  expect_error(
    fviz_eig(
      res.pca, choice = "eigenvalue", parallel = TRUE,
      parallel.iter = 2, parallel.seed = -1
    ),
    "single integer value in"
  )
  expect_error(
    fviz_eig(
      res.pca, choice = "eigenvalue", parallel = TRUE,
      parallel.iter = 2, parallel.seed = 1.5
    ),
    "single integer value in"
  )
})

test_that("fviz_eig validates parallel.iter for parallel analysis", {
  res.pca <- stats::prcomp(iris[, 1:4], scale. = TRUE)

  expect_error(
    fviz_eig(
      res.pca, choice = "eigenvalue", parallel = TRUE,
      parallel.iter = NA, parallel.seed = 1
    ),
    "parallel.iter must be a single positive integer value in"
  )
  expect_error(
    fviz_eig(
      res.pca, choice = "eigenvalue", parallel = TRUE,
      parallel.iter = -1, parallel.seed = 1
    ),
    "parallel.iter must be a single positive integer value in"
  )
  expect_error(
    fviz_eig(
      res.pca, choice = "eigenvalue", parallel = TRUE,
      parallel.iter = 1.5, parallel.seed = 1
    ),
    "parallel.iter must be a single positive integer value in"
  )
})

test_that("arrow.linetype controls variable-arrow linetype; default unchanged (#73)", {
  skip_if_not_installed("FactoMineR")
  res <- FactoMineR::PCA(decathlon2[1:23, 1:10], graph = FALSE)

  arrow_lt <- function(p) {
    seg <- Filter(function(l) inherits(l$geom, "GeomSegment") &&
                    !is.null(l$geom_params$arrow), p$layers)
    if (!length(seg)) return(NA_character_)
    lt <- seg[[1]]$aes_params$linetype
    if (is.null(lt)) "solid" else as.character(lt)
  }

  expect_equal(arrow_lt(fviz_pca_var(res)), "solid")                       # default
  expect_equal(arrow_lt(fviz_pca_var(res, arrow.linetype = "dashed")), "dashed")
  expect_equal(arrow_lt(fviz_pca_biplot(res, arrow.linetype = "dotted")), "dotted")
})

test_that("fviz_dend renders sub only when set; default has no subtitle (#54)", {
  hc <- hclust(dist(USArrests[1:15, ]), method = "ward.D2")

  # default: no subtitle (unchanged behavior)
  expect_null(fviz_dend(hc)$labels$subtitle)
  # explicit subtitle is rendered
  expect_equal(fviz_dend(hc, sub = "Method: ward.D2")$labels$subtitle, "Method: ward.D2")
  expect_s3_class(fviz_dend(hc, k = 3), "ggplot")
})

test_that("fviz_nbclust accepts a precomputed dist for diss-capable methods (#90)", {
  skip_if_not_installed("cluster")
  df <- scale(USArrests)
  d  <- get_dist(df, method = "euclidean")

  # diss-capable methods cluster on the dissimilarity
  expect_s3_class(fviz_nbclust(d, cluster::pam, method = "silhouette"), "ggplot")
  expect_s3_class(fviz_nbclust(d, hcut, method = "wss"), "ggplot")

  # non-diss methods are rejected with a clear error (no silent mis-clustering)
  expect_error(fviz_nbclust(d, stats::kmeans, method = "wss"), "distance matrix")
  expect_error(fviz_nbclust(d, cluster::clara, method = "silhouette"), "distance matrix")
  # gap_stat needs raw data
  expect_error(fviz_nbclust(d, cluster::pam, method = "gap_stat"), "gap_stat")

  # NO-REGRESSION: data input still works for all methods
  expect_s3_class(fviz_nbclust(df, stats::kmeans, method = "silhouette"), "ggplot")
  expect_s3_class(fviz_nbclust(df, cluster::pam, method = "silhouette", diss = d), "ggplot")
})

test_that("fviz_dend keeps leaf labels out of the legend (#14 sibling)", {
  hc <- hclust(dist(scale(USArrests[1:20, ])), method = "ward.D2")
  text_geoms <- c("GeomText", "GeomTextRepel", "GeomLabel", "GeomLabelRepel")
  for (p in list(fviz_dend(hc), fviz_dend(hc, k = 3, cex = 0.5))) {
    txt <- Filter(function(L) inherits(L$geom, text_geoms), p$layers)
    expect_true(all(vapply(txt, function(L) isFALSE(L$show.legend), logical(1))))
  }
})

test_that("fviz_dend rectangles match k even with tied heights (#154, #168)", {
  nrects <- function(p) {
    i <- which(vapply(p$layers, function(l) inherits(l$geom, "GeomRect"), logical(1)))[1]
    nrow(ggplot2::ggplot_build(p)$data[[i]])
  }

  # seed 234: binary/ward.D2 dendrogram whose heights_per_k has duplicate names
  # (k = 2 matches >1 height) -> used to make the rect data frame longer than k,
  # erroring with "Aesthetics must be either length 1 or the same as the data".
  set.seed(234)
  m <- matrix(sample(0:1, 16 * 4, replace = TRUE), ncol = 4)
  rownames(m) <- paste0("o", seq_len(nrow(m)))
  hc <- hclust(dist(m), method = "ward.D2")

  expect_equal(nrects(fviz_dend(hc, k = 2, rect = TRUE, rect_fill = TRUE,
                                rect_border = c("red", "blue"))), 2)
  expect_equal(nrects(fviz_dend(hc, k = 4, rect = TRUE,
                                rect_border = ggpubr::get_palette("jco", 4))), 4)
  # more colours than rectangles are recycled/trimmed, not errored
  expect_equal(nrects(fviz_dend(hc, k = 3, rect = TRUE, rect_border = rainbow(6))), 3)

  # NO-REGRESSION: well-behaved dendrogram still yields exactly k rectangles
  hc2 <- hclust(dist(scale(USArrests)), method = "ward.D2")
  for (k in 2:5) expect_equal(nrects(fviz_dend(hc2, k = k, rect = TRUE)), k)
})

test_that("fviz_dend lower_rect scales for short trees; tall trees unchanged (#55)", {
  rect_ymin <- function(p) {
    i <- which(vapply(p$layers, function(l) inherits(l$geom, "GeomRect"), logical(1)))[1]
    min(ggplot2::ggplot_build(p)$data[[i]]$ymin)
  }

  # short tree (max height < 1): default lower_rect is now height-relative
  set.seed(7)
  m <- matrix(rnorm(18 * 3, sd = 0.04), ncol = 3); rownames(m) <- paste0("s", 1:18)
  hc <- hclust(dist(m), "ward.D2"); mh <- max(hc$height)
  expect_lt(mh, 1)
  expect_equal(rect_ymin(fviz_dend(hc, k = 3, rect = TRUE)), -mh / 4)

  # tall tree (max height >= 1): previous default preserved (no regression)
  hc2 <- hclust(dist(scale(USArrests)), "ward.D2"); mh2 <- max(hc2$height)
  expect_equal(rect_ymin(fviz_dend(hc2, k = 4, rect = TRUE)), -(mh2 / 8 + 0.5))
})

test_that("fviz_dend match_coord_colors aligns colours to cluster labels (#103)", {
  skip_if_not_installed("dendextend")
  hc <- hclust(dist(scale(USArrests)), "ward.D2"); k <- 4
  pal <- ggpubr::get_palette("default", k)

  label_colors <- function(match) {
    dd <- attr(fviz_dend(hc, k = k, match_coord_colors = match), "dendrogram")
    gd <- dendextend::cutree(dd, k = k, order_clusters_as_data = TRUE)
    lc <- dendextend::get_leaves_branches_col(dd)
    as.vector(tapply(lc, gd[order.dendrogram(dd)], function(x) x[1])[as.character(seq_len(k))])
  }

  # opt-in: cluster label i -> palette[i], matching fviz_cluster()/fviz_silhouette()
  expect_equal(label_colors(TRUE), as.vector(pal))
  # this dataset is a known mismatch case, so the default differs from palette order
  expect_false(identical(label_colors(FALSE), as.vector(pal)))

  # NO-REGRESSION: default (FALSE) is byte-identical to omitting the argument
  expect_identical(
    dendextend::get_leaves_branches_col(attr(fviz_dend(hc, k = k), "dendrogram")),
    dendextend::get_leaves_branches_col(attr(fviz_dend(hc, k = k, match_coord_colors = FALSE), "dendrogram"))
  )
})

test_that("fviz_dend labels_font sets leaf-label font face; default unchanged (#121)", {
  hc <- hclust(dist(scale(USArrests[1:20, ])), "ward.D2")
  text_layer <- function(p) {
    i <- which(vapply(p$layers, function(l) inherits(l$geom, "GeomText"), logical(1)))[1]
    p$layers[[i]]
  }

  # opt-in font face applies to the leaf-label layer
  expect_equal(text_layer(fviz_dend(hc, k = 3, labels_font = "italic"))$aes_params$fontface, "italic")
  expect_equal(text_layer(fviz_dend(hc, k = 3, labels_font = "bold.italic"))$aes_params$fontface, "bold.italic")

  # NO-REGRESSION: default (and explicit "plain") leave the layer untouched
  expect_null(text_layer(fviz_dend(hc, k = 3))$aes_params$fontface)
  expect_null(text_layer(fviz_dend(hc, k = 3, labels_font = "plain"))$aes_params$fontface)

  expect_s3_class(fviz_dend(hc, k = 3, rect = TRUE, labels_font = "italic"), "ggplot")
})

test_that("alpha.var/alpha.ind fade text labels too; default unchanged (#130)", {
  res <- prcomp(iris[1:20, -5], scale. = TRUE)
  text_alpha <- function(p) {
    L <- Filter(function(l) inherits(l$geom, c("GeomText", "GeomTextRepel")), p$layers)
    vapply(L, function(l) { a <- l$aes_params$alpha; if (is.null(a)) NA_real_ else a }, numeric(1))
  }

  # labels now fade together with arrows/points
  expect_true(all(text_alpha(fviz_pca_var(res, alpha.var = 0.3)) == 0.3))
  expect_true(all(text_alpha(fviz_pca_ind(res, alpha.ind = 0.3)) == 0.3))

  # NO-REGRESSION: default (alpha = 1) keeps labels opaque
  expect_true(all(text_alpha(fviz_pca_var(res)) == 1))
  expect_true(all(text_alpha(fviz_pca_ind(res)) == 1))
  # metric-mapped alpha does not fade labels to a constant
  expect_true(all(text_alpha(fviz_pca_var(res, alpha.var = "contrib")) == 1))

  # biplot: alpha.var fades variable labels but leaves individual labels opaque
  ta <- text_alpha(fviz_pca_biplot(res, alpha.var = 0.3, label = "all"))
  expect_true(0.3 %in% ta && 1 %in% ta)
})

test_that("fviz_mclust_bic optimal-cluster line uses factor position (#116)", {
  skip_if_not_installed("mclust")
  # mclust::Mclust() calls mclustBIC() by name, which requires mclust to be
  # ATTACHED (not just namespaced). Attach it (mclust is in Suggests) and detach
  # on exit to keep the search path clean.
  suppressPackageStartupMessages(library(mclust))
  on.exit(detach("package:mclust", unload = FALSE), add = TRUE)
  vline_x <- function(p) {
    L <- Filter(function(l) inherits(l$geom, "GeomVline"), p$layers)
    if (!length(L)) return(NA_real_)
    L[[1]]$data$xintercept
  }

  # restricted G range: cluster levels start at 3, so the line must sit at the
  # *position* of the optimal G, landing on the correctly-labelled tick.
  set.seed(1)
  m_restr <- Mclust(iris[, -5], G = 3:9, verbose = FALSE)
  p_restr <- fviz_mclust_bic(m_restr)
  lv <- levels(p_restr$data$cluster)
  expect_equal(vline_x(p_restr), match(as.character(m_restr$G), lv))
  expect_equal(lv[vline_x(p_restr)], as.character(m_restr$G))

  # NO-REGRESSION: standard G = 1:9 -> levels start at 1, position == G
  set.seed(1)
  m_std <- Mclust(iris[, -5], verbose = FALSE)
  expect_equal(vline_x(fviz_mclust_bic(m_std)), m_std$G)
})

test_that("fviz_pca add.circle forces/suppresses the correlation circle (#88)", {
  has_circle <- function(p) {
    any(vapply(p$layers, function(l) inherits(l$geom, c("GeomPath", "GeomCircle")),
               logical(1)))
  }
  sc <- prcomp(iris[, -5], scale. = TRUE)
  un <- prcomp(iris[, -5], scale. = FALSE)

  # NO-REGRESSION: default (NULL) keeps the auto behavior
  expect_true(has_circle(fviz_pca_var(sc)))   # scaled -> circle
  expect_false(has_circle(fviz_pca_var(un)))  # unscaled -> no circle

  # opt-in: force on a manually-scaled (scale = FALSE) fit, or suppress
  expect_true(has_circle(fviz_pca_var(un, add.circle = TRUE)))
  expect_false(has_circle(fviz_pca_var(sc, add.circle = FALSE)))
  expect_true(has_circle(fviz_pca_biplot(un, add.circle = TRUE)))
})

test_that("fviz_cluster plots diss-based pam/fanny when data is supplied (#128)", {
  skip_if_not_installed("cluster")
  df <- scale(USArrests)
  d  <- get_dist(df, method = "euclidean")

  pr <- cluster::pam(d, k = 4, diss = TRUE)
  p  <- fviz_cluster(pr, data = df, geom = "point")
  expect_s3_class(p, "ggplot")
  # cluster assignments come from the (dissimilarity-based) object, aligned per
  # observation by name (not recomputed from data=)
  pt_cluster <- function(p) {
    setNames(as.integer(as.character(p$data$cluster)), rownames(p$data))
  }
  expect_equal(pt_cluster(p)[names(pr$clustering)], pr$clustering)

  fa <- cluster::fanny(d, k = 3, diss = TRUE)
  expect_s3_class(fviz_cluster(fa, data = df, geom = "point"), "ggplot")

  # data= in a DIFFERENT row order than the dissimilarity must still colour each
  # point with its own cluster (aligned by row name), not silently mis-mapped
  dfs <- df[sample(nrow(df)), ]
  ps  <- fviz_cluster(pr, data = dfs, geom = "point")
  expect_equal(pt_cluster(ps)[names(pr$clustering)], pr$clustering)

  # data= for different observations (row names don't match) -> clear error
  expect_error(fviz_cluster(pr, data = scale(mtcars[, 1:4])), "do not match")

  # no data available -> clear, actionable error (not the cryptic NULL one)
  expect_error(fviz_cluster(pr), "dissimilarity matrix")

  # NO-REGRESSION: pam fitted on raw data (object$data present) is unchanged
  expect_s3_class(fviz_cluster(cluster::pam(df, k = 4), geom = "point"), "ggplot")
})

test_that("fviz colour/fill length-mismatch gives a clear message; valid usage works (#139)", {
  res <- prcomp(iris[, -5], scale. = TRUE)   # 150 individuals, 4 variables

  # colouring VARIABLES by an observation-level group (length 150 != 4 variables)
  expect_error(fviz_pca_var(res, col.var = iris$Species),
               "number of elements being plotted")
  expect_error(fviz_pca_ind(res, fill.ind = c("a", "b")),
               "number of elements being plotted")

  # valid usages still work (no behavior change)
  expect_s3_class(fviz_pca_var(res, col.var = "contrib"), "ggplot")
  expect_s3_class(fviz_pca_var(res, col.var = factor(c("a", "a", "b", "b"))), "ggplot")
  expect_s3_class(fviz_pca_ind(res, col.ind = iris$Species), "ggplot")
})

test_that("fviz_cluster keeps point labels out of the legend (#14 sibling)", {
  set.seed(1)
  dat <- scale(matrix(rnorm(30 * 4), nrow = 30)); rownames(dat) <- paste0("s", 1:30)
  km <- kmeans(dat, centers = 8)
  text_geoms <- c("GeomText", "GeomTextRepel", "GeomLabel", "GeomLabelRepel")
  for (p in list(fviz_cluster(list(data = dat, cluster = km$cluster)),
                 fviz_cluster(km, data = dat))) {
    txt <- Filter(function(l) inherits(l$geom, text_geoms), p$layers)
    expect_gt(length(txt), 0)
    expect_true(all(vapply(txt, function(l) isFALSE(l$show.legend), logical(1))))
  }
})

test_that("fviz_mca_biplot forwards `map` to ind and var (#142)", {
  skip_if_not_installed("FactoMineR")
  data(poison, package = "factoextra")
  res.mca <- FactoMineR::MCA(poison[, 5:15], graph = FALSE)
  co <- function(p) ggplot2::ggplot_build(p)$data[[1]][, c("x", "y")]

  sym <- co(fviz_mca_biplot(res.mca, map = "symmetric", geom = "point"))
  col <- co(fviz_mca_biplot(res.mca, map = "colprincipal", geom = "point"))

  # The bug: `map` was dropped, so every map produced identical coordinates.
  expect_false(isTRUE(all.equal(sym, col)))

  # No behavior change for the default symmetric map.
  def <- co(fviz_mca_biplot(res.mca, geom = "point"))
  expect_equal(def, sym)
})

test_that("get_pca_ind works for ade4 dudi.pca (data.frame $li) (#126)", {
  skip_if_not_installed("ade4")
  pca <- ade4::dudi.pca(iris[, -5], scannf = FALSE, nf = 3)

  ind <- get_pca_ind(pca)
  # The bug: a data.frame $li turned the cos2 matrix into a list (lost dims).
  expect_s3_class(ind, "factoextra")
  expect_true(all(c("coord", "cos2", "contrib") %in% names(ind)))
  expect_true(is.matrix(ind$coord) && is.matrix(ind$cos2) && is.matrix(ind$contrib))
  expect_equal(nrow(ind$coord), nrow(iris))
  expect_true(all(rowSums(ind$cos2) <= 1 + 1e-9))
  expect_true(all(abs(colSums(ind$contrib) - 100) < 1e-6))
  expect_s3_class(fviz_pca_ind(pca), "ggplot")
})

test_that("prcomp/princomp get_pca_ind unchanged by the dudi fix (#126 no-regression)", {
  pc <- prcomp(iris[, -5], scale. = TRUE)
  pr <- princomp(iris[, -5], cor = TRUE)
  for (ind in list(get_pca_ind(pc), get_pca_ind(pr))) {
    expect_true(is.matrix(ind$coord) && is.matrix(ind$cos2) && is.matrix(ind$contrib))
    expect_equal(nrow(ind$coord), nrow(iris))
    expect_true(all(rowSums(ind$cos2) <= 1 + 1e-9))
  }
})

test_that("ade4 between-/within-class PCA (bca/wca) are supported (#126)", {
  skip_if_not_installed("ade4")
  data(meaudret, package = "ade4")
  pca <- ade4::dudi.pca(meaudret$env, scannf = FALSE, nf = 3)
  fac <- meaudret$design$season
  bet <- ade4::bca(pca, fac, scannf = FALSE, nf = 2)
  wit <- ade4::wca(pca, fac, scannf = FALSE, nf = 2)

  for (obj in list(bet, wit)) {
    expect_equal(factoextra:::.get_facto_class(obj), "PCA")
    expect_s3_class(get_eig(obj), "data.frame")
    ind <- get_pca_ind(obj)
    var <- get_pca_var(obj)
    expect_true(all(c("coord", "cos2", "contrib") %in% names(ind)))
    expect_true(all(rowSums(ind$cos2) <= 1 + 1e-9))
    expect_true(all(c("coord", "cos2", "contrib") %in% names(var)))
    expect_s3_class(fviz_pca_ind(obj), "ggplot")
    expect_s3_class(fviz_pca_var(obj), "ggplot")
    expect_s3_class(fviz_pca_biplot(obj), "ggplot")
    expect_s3_class(fviz_eig(obj), "ggplot")

    # contributions must match ade4's own inertia.dudi (row.abs, in %)
    inr <- ade4::inertia.dudi(obj, row.inertia = TRUE)
    ade_contrib <- as.matrix(inr$row.abs[, seq_len(ncol(ind$contrib))])
    expect_equal(unname(as.matrix(ind$contrib)), unname(ade_contrib),
                 tolerance = 1e-4)
  }
})

test_that("fviz_dend honors an explicit k for HCPC, defaults to HCPC count (#81)", {
  skip_if_not_installed("FactoMineR")
  # PCA -> HCPC is the canonical, version-robust path (the FAMD+HCPC combo errors
  # internally in some FactoMineR versions). Default k is read from the result
  # rather than hard-coded, so the test is robust to the auto cluster count.
  res.pca <- FactoMineR::PCA(iris[, -5], ncp = 3, graph = FALSE)
  hc <- FactoMineR::HCPC(res.pca, nb.clust = 3, graph = FALSE)
  default_k <- length(unique(hc$data.clust$clust))

  n_branch_cols <- function(p) {
    b <- ggplot2::ggplot_build(p)$data
    cols <- unique(unlist(lapply(b, function(d)
      if ("colour" %in% names(d)) unique(d$colour))))
    length(setdiff(cols, c("black", "#000000", "grey50", "gray50")))
  }

  # default: unchanged -> HCPC cluster count
  expect_equal(n_branch_cols(fviz_dend(hc)), default_k)
  # explicit k now honored (previously ignored)
  expect_equal(n_branch_cols(fviz_dend(hc, k = 2)), 2)
  expect_equal(n_branch_cols(fviz_dend(hc, k = 4)), 4)
})

test_that("rotate.labels rotates variable labels, default off (#98)", {
  res.pca <- prcomp(iris[, -5], scale. = TRUE)
  text_geoms <- c("GeomText", "GeomTextRepel", "GeomLabel", "GeomLabelRepel")
  text_layer <- function(p) {
    i <- which(vapply(p$layers, function(l) inherits(l$geom, text_geoms), logical(1)))[1]
    p$layers[[i]]
  }

  # default: no angle aesthetic (unchanged behavior)
  lay0 <- text_layer(fviz_pca_var(res.pca, repel = FALSE))
  expect_false("angle" %in% names(lay0$mapping))
  expect_false(".fviz_angle" %in% names(as.data.frame(lay0$data)))

  # rotate.labels = TRUE: angle aesthetic + per-label angle column
  lay1 <- text_layer(fviz_pca_var(res.pca, repel = FALSE, rotate.labels = TRUE))
  expect_true("angle" %in% names(lay1$mapping))
  d <- as.data.frame(lay1$data)
  expect_true(".fviz_angle" %in% names(d))
  # angles match arrow directions (flipped into the readable [-90, 90] range)
  expect_true(all(d$.fviz_angle >= -90 - 1e-6 & d$.fviz_angle <= 90 + 1e-6))

  # rotation is restricted to arrow-bearing (variable) labels: fviz_pca_ind has
  # no arrows, so rotate.labels must be a no-op there.
  lay_ind <- text_layer(fviz_pca_ind(res.pca, repel = FALSE, rotate.labels = TRUE))
  expect_false("angle" %in% names(lay_ind$mapping))

  # biplot: only the variable labels are rotated, NOT the individual labels.
  pb <- fviz_pca_biplot(res.pca, repel = FALSE, rotate.labels = TRUE)
  angled <- vapply(pb$layers, function(l)
    inherits(l$geom, text_geoms) &&
      ".fviz_angle" %in% names(as.data.frame(l$data)), logical(1))
  n_rows <- vapply(pb$layers, function(l)
    if (inherits(l$geom, text_geoms)) nrow(as.data.frame(l$data)) else NA_integer_,
    integer(1))
  expect_true(any(angled))
  # angled layer holds the variables (ncol), not the individuals (nrow of data)
  expect_true(all(n_rows[angled] == ncol(iris[, -5])))
  expect_s3_class(fviz_pca_biplot(res.pca), "ggplot")
})

test_that("shape.ind maps point shape to a second factor, default unchanged (#36, #51)", {
  res.pca <- prcomp(iris[, -5], scale. = TRUE)
  set.seed(1)
  g2 <- factor(sample(c("a", "b"), nrow(iris), TRUE))

  scales_of <- function(p) {
    b <- ggplot2::ggplot_build(p)
    unlist(lapply(b$plot$scales$scales, function(s) s$aesthetics))
  }

  # default (no shape.ind): unchanged, builds fine
  expect_s3_class(fviz_pca_ind(res.pca, habillage = iris$Species, geom = "point"), "ggplot")

  # two groupings: colour by Species, shape by g2 -> both a colour and a shape scale
  p <- fviz_pca_ind(res.pca, col.ind = iris$Species, shape.ind = g2, geom = "point")
  sc <- scales_of(p)
  expect_true("colour" %in% sc)
  expect_true("shape" %in% sc)

  # a length-mismatched shape factor errors clearly
  expect_error(fviz_pca_ind(res.pca, shape.ind = factor(c("a", "b"))),
               "number of elements being plotted")

  # biplot forwards shape.ind to individuals only and still builds
  expect_s3_class(
    fviz_pca_biplot(res.pca, col.ind = iris$Species, shape.ind = g2, geom.ind = "point"),
    "ggplot")
})

test_that("numeric pointshape path is unchanged by the shape.ind feature (#36 no-regression)", {
  res.pca <- prcomp(iris[, -5], scale. = TRUE)
  # default fviz_pca_ind with a habillage still maps shape to the habillage group
  # (no Shape. column injected; pointshape stays the auto-mapped grouping)
  p <- fviz_pca_ind(res.pca, habillage = iris$Species, geom = "point")
  has_shape_col <- any(vapply(p$layers, function(l) {
    d <- tryCatch(as.data.frame(l$data), error = function(e) data.frame())
    "Shape." %in% names(d)
  }, logical(1)))
  expect_false(has_shape_col)
})

test_that("unknown label/invisible tokens warn instead of silently doing nothing (#165)", {
  res <- prcomp(iris[, -5], scale. = TRUE)

  # the reported case: label = "id" (typo for "ind") now warns and names valids.
  # (A biplot calls fviz() for ind and var, so it warns once per pass; capture
  # all warnings to assert the message without leaking the second one.)
  w <- testthat::capture_warnings(fviz_pca_biplot(res, label = c("var", "id")))
  expect_true(any(grepl("Unknown label value", w)))
  expect_warning(fviz_pca_ind(res, invisible = "ids"),
                 "Unknown invisible value")

  # valid tokens stay silent (no behavior change)
  expect_silent(fviz_pca_ind(res, label = "all"))
  expect_silent(fviz_pca_ind(res, label = c("var", "ind")))
  expect_silent(fviz_pca_ind(res, label = "none"))
  expect_silent(fviz_pca_ind(res, invisible = "none"))
  expect_silent(fviz_pca_biplot(res, label = "all"))

  # rendering for the correct keyword is unaffected: "ind" still draws labels
  text_geoms <- c("GeomText", "GeomTextRepel", "GeomLabel", "GeomLabelRepel")
  p <- suppressWarnings(fviz_pca_ind(res, label = "ind", geom = c("point", "text")))
  has_ind_text <- any(vapply(p$layers, function(l) {
    inherits(l$geom, text_geoms) &&
      nrow(as.data.frame(l$data)) == nrow(iris)
  }, logical(1)))
  expect_true(has_ind_text)
})

test_that("as_factoextra_pca() builds a fviz-ready object from coordinates (constructor bridge)", {
  # coords only (e.g. an MDS / UMAP embedding)
  mds <- stats::cmdscale(dist(scale(mtcars)), k = 3)
  o1 <- as_factoextra_pca(ind.coord = mds)
  expect_s3_class(o1, "factoextra_pca")
  expect_equal(factoextra:::.get_facto_class(o1), "PCA")

  ind <- get_pca_ind(o1)
  expect_true(all(c("coord", "cos2", "contrib") %in% names(ind)))
  expect_true(all(ind$cos2 >= 0 & ind$cos2 <= 1))
  expect_true(all(abs(colSums(ind$contrib) - 100) < 1e-6))
  expect_equal(nrow(get_eig(o1)), 3L)

  expect_s3_class(fviz_pca_ind(o1, geom = "point"), "ggplot")
  expect_s3_class(fviz_eig(o1), "ggplot")
  expect_s3_class(fviz_contrib(o1, choice = "ind"), "ggplot")
  expect_s3_class(fviz_cos2(o1, choice = "ind"), "ggplot")

  # with variable coordinates + eigenvalues (prcomp round-trip)
  pca <- prcomp(iris[, -5], scale. = TRUE)
  o2 <- as_factoextra_pca(ind.coord = pca$x, var.coord = pca$rotation, eig = pca$sdev^2)
  expect_s3_class(fviz_pca_var(o2), "ggplot")
  expect_s3_class(fviz_pca_biplot(o2), "ggplot")

  # round-trip: individual coordinates match the native prcomp plot
  co_native <- ggplot2::ggplot_build(fviz_pca_ind(pca, geom = "point"))$data[[1]][, c("x", "y")]
  co_constr <- ggplot2::ggplot_build(fviz_pca_ind(o2, geom = "point"))$data[[1]][, c("x", "y")]
  expect_equal(co_native, co_constr, tolerance = 1e-8)
})

test_that("as_factoextra_pca() validates inputs and errors clearly", {
  mds <- stats::cmdscale(dist(scale(mtcars)), k = 2)
  o <- as_factoextra_pca(ind.coord = mds)               # no var.coord
  expect_error(fviz_pca_var(o), "no variable coordinates")
  expect_error(as_factoextra_pca(data.frame(a = letters[1:3])), "must be numeric")
  expect_error(as_factoextra_pca(mds, var.coord = matrix(1, 2, 3)),
               "same number of dimensions")
  expect_error(as_factoextra_pca(mds, eig = 1),         # too few eigenvalues
               "at least 2")
})

test_that("as_factoextra_pca() does not change existing prcomp/PCA paths (no-regression)", {
  # the new class branches are gated on inherits(x, 'factoextra_pca'); ordinary
  # objects must hit the unchanged paths.
  pca <- prcomp(iris[, -5], scale. = TRUE)
  ind <- get_pca_ind(pca)
  expect_true(is.matrix(ind$coord) && is.matrix(ind$cos2) && is.matrix(ind$contrib))
  expect_equal(nrow(ind$coord), nrow(iris))
  expect_false(inherits(pca, "factoextra_pca"))
  expect_s3_class(fviz_pca_biplot(pca), "ggplot")
})

test_that("hcut()/hkmeans() let the NATIVE k > n error surface (chooseGCM revdep guard, CRAN)", {
  # The reverse dependency chooseGCM pins stats::cutree()'s native message:
  #   expect_error(hclust_gcms(..., k = 100), "elements of 'k' must be between 1 and 11")
  # A custom early stop ("k must not exceed the number of observations"), introduced during the
  # 2.1.0 cycle (#209), pre-empted cutree and CHANGED that message, breaking chooseGCM on CRAN
  # (a "change to worse in reverse dependencies"). hcut()/hkmeans() must NOT pre-empt cutree -
  # keep the native error verbatim. Do not re-introduce a custom k > n_obs stop().
  x <- matrix(stats::rnorm(11 * 5), nrow = 11)            # 11 "GCMs", like chooseGCM's case
  expect_error(hcut(x, k = 100), "elements of 'k' must be between 1 and 11")
  expect_error(hcut(stats::dist(x), k = 100, isdiss = TRUE),
               "elements of 'k' must be between 1 and 11")
  expect_error(hkmeans(x, k = 100), "elements of 'k' must be between 1 and 11")
})

#' @include get_hmfa.R
NULL
#'Visualize Hierarchical Multiple Factor Analysis
#'
#'@description Hierarchical Multiple Factor Analysis (HMFA) is, an extension of
#'  MFA, used in a situation where the data are organized into a hierarchical
#'  structure.  fviz_hmfa() provides ggplot2-based elegant visualization of HMFA
#'  outputs from the R function: HMFA [FactoMineR].\cr\cr \itemize{
#'  \item fviz_hmfa_ind(): Graph of individuals \item fviz_hmfa_var(): Graph of
#'  variables \item fviz_hmfa_quali_biplot(): Biplot of individuals and
#'  qualitative variables \item fviz_hmfa(): An alias of fviz_hmfa_ind() }
#'@param X an object of class HMFA [FactoMineR].
#'@inheritParams fviz_mca
#'@inheritParams fviz_pca
#'@inheritParams fviz
#'@param habillage an optional factor variable for coloring the observations by 
#'  groups. Default value is "none". If X is an HMFA object from FactoMineR 
#'  package, habillage can also specify the index of the factor variable in the 
#'  data.
#'@param col.ind,col.var color for individuals, partial individuals and 
#'  variables, respectively. Can be a continuous variable or a factor variable.
#'  Possible values include also : "cos2", "contrib", "coord", "x" or "y". In
#'  this case, the colors for individuals/variables are automatically controlled
#'  by their qualities ("cos2"), contributions ("contrib"), coordinates (x^2 +
#'  y^2 , "coord"), x values("x") or y values("y"). To use automatic coloring
#'  (by cos2, contrib, ....), make sure that habillage ="none".
#'@param col.partial color for partial individuals. By default, points are 
#'  colored according to the groups.
#'@param alpha.ind,alpha.var controls the transparency of individual, partial 
#'  individual and variable, respectively. The value can variate from 0 (total 
#'  transparency) to 1 (no transparency). Default value is 1. Possible values 
#'  include also : "cos2", "contrib", "coord", "x" or "y". In this case, the 
#'  transparency for individual/variable colors are automatically controlled by 
#'  their qualities ("cos2"), contributions ("contrib"), coordinates (x^2 + y^2 
#'  , "coord"), x values("x") or y values("y"). To use this, make sure that 
#'  habillage ="none".
#'@param shape.ind,shape.var point shapes of individuals and variables, 
#'  respectively.
#'@param group.names a vector containing the name of the groups (by default, 
#'  NULL and the group are named group.1, group.2 and so on).
#'@param node.level a single number indicating the HMFA node level to plot.
#'@param title the title of the graph
#'@param select.ind,select.var a selection of individuals and variables to be 
#'  drawn. Allowed values are NULL or a list containing the arguments name, cos2
#'  or contrib: \itemize{ \item name is a character vector containing 
#'  individuals/variables to be drawn \item cos2 if cos2 is in [0, 1], ex: 0.6, 
#'  then individuals/variables with a cos2 > 0.6 are drawn. if cos2 > 1, ex: 5, 
#'  then the top 5 individuals/variables with the highest cos2 are drawn. \item 
#'  contrib if contrib > 1, ex: 5,  then the top 5 individuals/variables with 
#'  the highest cos2 are drawn }
#'@param choice the graph to plot. Allowed values include one of c("quanti.var",
#'  "quali.var", "group") for plotting quantitative variables, qualitative 
#'  variables and group of variables, respectively.
#'@param ... Arguments to be passed to the function fviz() and ggpubr::ggpar()
#'@param partial list of the individuals for which the partial points should be
#'  drawn. (by default, partial = NULL and no partial points are drawn). Use
#'  partial = "all" to visualize partial points for all individuals.
#'@param col.var.sup color for supplementary variables.
#'@param repel a boolean, whether to use ggrepel to avoid overplotting text
#'  labels or not. The old \code{jitter} argument is kept for backward
#'  compatibility and is converted to \code{repel = TRUE} with a deprecation warning.
#'@return a ggplot
#'@author Fabian Mundt \email{f.mundt@inventionate.de}
#'@author Alboukadel Kassambara \email{alboukadel.kassambara@@gmail.com}
#'@references \url{https://www.sthda.com/english/}
#' @examples
#' # Hierarchical Multiple Factor Analysis
#' # ++++++++++++++++++++++++
#' # Install and load FactoMineR to compute MFA
#' # install.packages("FactoMineR")
#' library("FactoMineR")
#' data(wine)
#' hierar <- list(c(2,5,3,10,9,2), c(4,2))
#' res.hmfa <- HMFA(wine, H = hierar, type=c("n",rep("s",5)), graph = FALSE)
#'
#' # Graph of individuals
#' # ++++++++++++++++++++
#' # Color of individuals: col.ind = "#2E9FDF"
#' # Use repel = TRUE to avoid overplotting (slow if many points)
#' fviz_hmfa_ind(res.hmfa, repel = TRUE, col.ind = "#2E9FDF")
#' 
#' # Color individuals by groups, add concentration ellipses
#' # Remove labels: label = "none".
#' # Change color palette to "jco". See ?ggpubr::ggpar
#' grp <- as.factor(wine[,1])
#' p <- fviz_hmfa_ind(res.hmfa, label="none", habillage=grp,
#'        addEllipses=TRUE, palette = "jco")
#' print(p)
#'  
#'
#' # Graph of variables
#' # ++++++++++++++++++++++++++++++++++++++++
#' # Quantitative variables
#' fviz_hmfa_var(res.hmfa, "quanti.var")
#' # Graph of categorical variable categories
#' fviz_hmfa_var(res.hmfa, "quali.var")
#' # Groups of variables (correlation square)
#' fviz_hmfa_var(res.hmfa, "group")
#'
#'
#' # Biplot of categorical variable categories and individuals
#' # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#' fviz_hmfa_quali_biplot(res.hmfa)
#'     
#' # Graph of partial individuals (starplot)
#' # +++++++++++++++++++++++++++++++++++++++
#' fviz_hmfa_ind(res.hmfa, partial = "all", col.partial = "black")
#'  
#'
#'@name fviz_hmfa
#'@rdname fviz_hmfa
#'@export
fviz_hmfa_ind <- function(X,  axes = c(1,2), geom=c("point", "text"), repel = FALSE,
                          habillage="none", addEllipses=FALSE, 
                          shape.ind = 19, col.ind = "blue", col.ind.sup = "darkblue",
                          alpha.ind = 1,
                          select.ind = list(name = NULL, cos2 = NULL, contrib = NULL),
                          partial = NULL, col.partial = "group", 
                          group.names = NULL, node.level = 1,  ...)
{
  extra_args <- list(...)
  invisible <- if(is.null(extra_args$invisible)) "none" else extra_args$invisible
  
  p <- fviz (X, element = "ind", axes = axes, geom = geom, habillage = habillage, 
             addEllipses = addEllipses, pointshape = shape.ind,
             color = col.ind, alpha = alpha.ind,
             shape.sup = shape.ind, col.row.sup = col.ind.sup,
             select = select.ind, repel = repel, ...)
  
  # Add partial points
  if(!is.null(partial)){
    if(!(partial[1] %in% c("All", "all"))) select.partial = list(name = partial)
    else select.partial <- NULL
    if(col.partial %in% c("group", "groups")) col.partial <- "group.name"
    
    # Data for individuals
    ind.sum <- facto_summarize(X, element = "ind",
                               result = c("coord", "contrib", "cos2"), axes = axes)
    ind <- ind.sum
    colnames(ind)[2:3] <-  c("x", "y")
    # partial points
    if(is.null(group.names)) group.names <- rownames(X$group$coord[[node.level]])
    ind.partial <- facto_summarize(X, element = "partial.node", node.level = node.level, 
                                   group.names = group.names, result = c("coord.node.partial"), 
                                   axes = axes)
   
    
    colnames(ind.partial)[3:4] <-  c("x.partial", "y.partial")
    ind.partial <- merge(ind, ind.partial, by = "name")
    # Selection
    ind.all <- ind
    if(!is.null(select.ind)) ind <- .select(ind, select.ind)
    if(!is.null(select.partial)) {
      if(nrow(ind) != nrow(ind.all)) warning("You've already selected individuals. Partial points are only calculated for them.")
      ind.partial <-  ind.partial[ind.partial$name %in% .select(ind, select.partial)$name, , drop = FALSE]
    }
    # elements to be hidden
    hide <- .hide(invisible)
    # Plot
    if(!hide$ind && "point" %in% geom) {
      # Partial point
      p <- p + ggpubr::geom_exec(geom_point, data = ind.partial,
                                 x = "x.partial", y = "y.partial", 
                                 colour = col.partial,
                                 shape = shape.ind, size = 1)
      # Partial segments
      # FIX: ggplot2 3.4.0+ deprecation - size replaced with linewidth for geom_segment
      p <- p + ggpubr::geom_exec(geom_segment, data = ind.partial,
                                 x = "x", y = "y", xend = 'x.partial', yend = 'y.partial',
                                 linetype = "group.name", colour = col.partial, linewidth = 0.5)
    }
    # Edit plot title and legend title
    p <- p  + labs(colour = "Groups", linetype = "Groups")
  }
  
  p
}


#' @rdname fviz_hmfa
#' @export
fviz_hmfa_var <- function(X, choice = c("quanti.var", "quali.var", "group"), axes=c(1,2), geom=c("point", "text"), repel = FALSE, 
                          col.var = "red", alpha.var = 1, shape.var = 17, col.var.sup = "darkgreen", 
                          select.var = list(name = NULL, cos2 = NULL, contrib = NULL), ...)
{
  
  choice <- match.arg(choice)
  if(choice == "quanti.var") {
    if(missing(geom)) geom <- c("arrow", "text")
  }
  
  fviz (X, element = choice, axes = axes, geom = geom,
        color = col.var, alpha = alpha.var,  pointshape = shape.var, 
        shape.sup = shape.var, col.col.sup = col.var.sup, 
        select = select.var, repel = repel,  ...)
}


#' @rdname fviz_hmfa
#' @export
fviz_hmfa_quali_biplot <- function(X,  axes = c(1,2), geom=c("point", "text"), repel = FALSE, 
                                   habillage = "none",
                                   title = "Biplot of individuals and qualitative variables - HMFA",  ...)
{
  # Individuals
  p <- fviz_hmfa_ind(X,  axes = axes, geom = geom, repel = repel, habillage = habillage,  ...)
  
  # Variable
  # Add variables
  p <- fviz_hmfa_var(X, "quali.var", axes = axes, geom =  geom, repel = repel, 
                    ggp = p, ...)
  
  p+labs(title=title)
}

#' @rdname fviz_hmfa
#' @export
fviz_hmfa <- function(X, ...){
  # fviz_hmfa_ind_starplot(X, ...)
  fviz_hmfa_ind(X, ...)
}

#' Add supplementary data to a plot
#' 
#' @description
#' Add supplementary data to a plot
#'  
#' @param ggp a ggplot2 plot.
#' @param df a data frame containing the x and y coordinates
#' @param axes a numeric vector of length 2 specifying the components to be plotted.
#' @param geom a character specifying the geometry to be used for the graph
#'  Allowed values are "point" or "arrow" or "text"
#' @param color the color to be used
#' @param addlabel a logical value. If TRUE, labels are added
#' @param labelsize the size of labels. Default value is 4
#' @param pointsize the size of points
#' @param shape point shape when geom ="point"
#' @param linetype the linetype to be used when geom ="arrow"
#' @param repel a boolean, whether to use ggrepel to avoid overplotting text
#'   labels or not. The old \code{jitter} argument is kept for backward
#'   compatibility and is converted to \code{repel = TRUE} with a deprecation warning.
#' @param font.family character vector specifying font family.
#' @param ... Additional arguments, not used
#' @return a ggplot2 plot
#' @author Alboukadel Kassambara \email{alboukadel.kassambara@@gmail.com}
#' @references \url{https://www.sthda.com/english/}
#' @examples
#' \donttest{
#' # Principal component analysis
#' data(decathlon2)
#' decathlon2.active <- decathlon2[1:23, 1:10]
#' res.pca <- prcomp(decathlon2.active,  scale = TRUE)
#' 
#' # Visualize variables
#' p <- fviz_pca_var(res.pca)
#' print(p)
#' 
#' # Add supplementary variables
#' coord <- data.frame(PC1 = c(-0.7, 0.9), PC2 = c(0.25, -0.07))
#' rownames(coord) <- c("Rank", "Points")
#' print(coord)
#' fviz_add(p, coord, color ="blue", geom="arrow") 
#'  }
#'  
#' @export 
fviz_add <- function(ggp, df, axes = c(1,2), geom=c("point", "arrow"), color ="blue", 
                     addlabel = TRUE, labelsize = 4, pointsize = 2, shape=19, linetype ="dashed",
                     repel = FALSE, font.family = "", ...)
{
  # Backward compatibility: jitter argument converted with warning

  extra_args <- list(...)
  if (!is.null(extra_args$jitter)) {
    .facto_dep("jitter", "repel = TRUE", NULL)
    if(!is.null(extra_args$jitter$width) || !is.null(extra_args$jitter$height) ) repel = TRUE
  }
  
  if(!inherits(df, c("data.frame", "matrix")))
     stop("df should be a data frame or a matrix")
  if(!inherits(df, "data.frame")) df <- as.data.frame(df)
     
  if(ncol(df) < 2)
    stop("df should have at least two columns (x and y coordinates)")
  
  if(length(intersect(geom, c("point", "arrow", "text"))) == 0)
    stop("The specified value(s) for the argument geom are not allowed ")
  
  if(is.null(df$name)) df$name <- rownames(df)
  if(is.null(df$x)) df$x <- df[,axes[1]]
  if(is.null(df$y)) df$y <- df[,axes[2]]
  
  # Plot
  #%%%%%%%%%%%%%%%%%%%%%%
  hjust <- vjust <- 0.5
  if("point" %in% geom) {
    p <- ggp + ggpubr::geom_exec(geom_point, data = df, x = "x", y = "y",
                                 color = color, shape = shape, size = pointsize)
    vjust <- -0.7
  }
  else if("arrow" %in% geom){
    # FIX: ggplot2 3.0.0+ deprecation - aes_string() replaced with aes() + .data pronoun
    # See: https://github.com/kassambara/factoextra/issues/190
    p <- ggp + geom_segment(data = df,
                      aes(x = 0, y = 0, xend = .data[["x"]], yend = .data[["y"]]),
                      arrow = grid::arrow(length = grid::unit(0.2, 'cm')),
                      color=color, linetype=linetype)
    hjust <- 0.8
    vjust <- 0
  }
  else if("text" %in% geom) {
    vjust <- -0.7
    p <- ggp
  }
  
  if(addlabel || "text" %in% geom){
    if(repel){
      p <- p + ggpubr::geom_exec(ggrepel::geom_text_repel, data = df, x = "x", y = "y", 
                                 label = "name", color = color, size = labelsize,
                                 family = font.family)
    }
    else{
      p <- p + ggpubr::geom_exec(geom_text, data = df, x = "x", y = "y", 
                                 label = "name", color = color, size = labelsize,
                                 vjust=vjust, hjust = hjust, family = font.family)
    } 
  }
  
  
  return(p)
}

#' @include print.factoextra.R
NULL
#' Extract the results for individuals and variables - FAMD
#' 
#' @description
#' Extract all the results (coordinates, squared cosine and contributions) 
#' for the active individuals and variables from Factor Analysis of Mixed Data (FAMD) outputs.\cr\cr
#' \itemize{
#' \item get_famd(): Extract the results for variables and individuals
#' \item get_famd_ind(): Extract the results for individuals only
#' \item get_famd_var(): Extract the results for quantitative and qualitative variables only
#' }
#' @param res.famd an object of class FAMD [FactoMineR].
#' @param element the element to subset from the output. Possible values are
#'   "ind", "var", "quanti.var", "quali.var" or "quali.sup".
#' @return a list of matrices containing the results for the active 
#' individuals and variables, including : 
#' \item{coord}{coordinates of individuals/variables.}
#' \item{cos2}{cos2 values representing the quality of representation on the factor map.}
#' \item{contrib}{contributions of individuals / variables to the principal components.}
#' @author Alboukadel Kassambara \email{alboukadel.kassambara@@gmail.com}
#' @examples
#' 
#'  # Compute FAMD
#'  library("FactoMineR")
#'  data(wine)
#'  res.famd <- FAMD(wine[,c(1,2, 16, 22, 29, 28, 30,31)], graph = FALSE)
#'  res.famd.sup <- FAMD(wine[,c(1,2, 16, 22, 29, 28, 30,31)],
#'                       sup.var = 2, graph = FALSE)
#'  
#'  # Extract the results for qualitative variable categories
#'  quali.var <- get_famd_var(res.famd, "quali.var")
#'  print(quali.var)
#'  head(quali.var$coord) # coordinates of qualitative variables
#' 
#'  # Extract the results for supplementary qualitative variable categories
#'  quali.sup <- get_famd_var(res.famd.sup, "quali.sup")
#'  print(quali.sup)
#'  head(quali.sup$coord) # coordinates of supplementary qualitative variables
#'  
#'  # Extract the results for quantitative variables
#'  quanti.var <- get_famd_var(res.famd, "quanti.var")
#'  print(quanti.var)
#'  head(quanti.var$coord) # coordinates
#'  
#'  # Extract the results for individuals
#'  ind <- get_famd_ind(res.famd)
#'  print(ind)
#'  head(ind$coord) # coordinates of individuals
#'  
#'  
#' @name get_famd
#' 
#' @rdname get_famd
#' @export 
get_famd <- function(res.famd, element = c("ind",  "var", "quanti.var", "quali.var", "quali.sup")){
  elmt <- match.arg(element)
  switch(elmt,
         ind = get_famd_ind(res.famd),
         var = get_famd_var(res.famd, "var"),
         quanti.var = get_famd_var(res.famd, "quanti.var"),
         quali.var = get_famd_var(res.famd, "quali.var"),
         quali.sup = get_famd_var(res.famd, "quali.sup")
         )
}

#' @rdname get_famd
#' @export
get_famd_ind <- function(res.famd){
  # FactoMineR package
  if(inherits(res.famd, c("FAMD"))) ind <- res.famd$ind
  else stop("An object of class : ", paste(class(res.famd), collapse = ", "), 
            " can't be handled by the function get_famd_ind()")
  class(ind)<-c("factoextra", "famd", "famd_ind")
  attr(ind, "element") <- "individuals"
  return(ind)
}

#' @rdname get_famd
#' @export
get_famd_var <- function(res.famd, element = c("var", "quanti.var", "quali.var", "quali.sup")){
  choice <- match.arg(element)
  # FactoMineR package
  if(inherits(res.famd, "FAMD")) {
    quali.sup <- .get_factominer_quali_sup(res.famd)
    if(choice == "quanti.var" && is.null(res.famd$quanti.var))
      stop("There are no quantitative variables in this FAMD.")
    else if(choice == "quali.var" && is.null(res.famd$quali.var))
      stop("There are no qualitative variables in this FAMD.")
    else if(choice == "quali.sup" && is.null(quali.sup))
      stop("There are no supplementary qualitative variables in this FAMD.")

    vars <- switch(choice,
                   var = res.famd$var,
                   quanti.var = res.famd$quanti.var,
                   quali.var = res.famd$quali.var,
                   quali.sup = quali.sup
                   )
    element_desc <- switch(choice,
                      var = "variables",
                      quanti.var = "quantitative variables",
                      quali.var = "qualitative variable categories",
                      quali.sup = "supplementary qualitative variable categories"
    )
  }
  else stop("An object of class : ", paste(class(res.famd), collapse = ", "), 
            " can't be handled.")
  
  class(vars)<-c("factoextra",  "famd", paste0("famd_", gsub(".", "_", choice, fixed = TRUE)))
  attr(vars, "element") <- element_desc
  return(vars)
}

#' @include utilities.R
NULL
#'Visualizing Multivariate Analysis Outputs
#'@description Generic function to create a scatter plot of multivariate analysis
#'  outputs, including PCA, CA, MCA and MFA.
#'@inheritParams facto_summarize
#'@param geom a text specifying the geometry to be used for the graph. Default 
#'  value is "auto". Allowed values are the combination of c("point", "arrow", 
#'  "text"). Use "point" (to show only points); "text" to show only labels; 
#'  c("point", "text") or c("arrow", "text") to show both types.
#'@param label a text specifying the elements to be labelled. Default value is 
#'  "all". Allowed values are "none" or the combination of c("ind", "ind.sup", 
#'  "quali", "var", "quanti.sup", "group.sup"). "ind" can be used to label only 
#'  active individuals. "ind.sup" is for supplementary individuals. "quali" is 
#'  for supplementary qualitative variables. "var" is for active variables. 
#'  "quanti.sup" is for quantitative supplementary variables.
#'@param invisible a text specifying the elements to be hidden on the plot. 
#'  Default value is "none". Allowed values are the combination of c("ind", 
#'  "ind.sup", "quali", "var", "quanti.sup", "group.sup").
#'@param labelsize font size for the labels
#'@param pointsize the size of points
#'@param pointshape the shape of points
#'@param arrowsize the size of arrows. Controls the thickness of arrows.
#'@param arrow.linetype linetype of the variable arrows (e.g. "solid",
#'  "dashed", "dotted"). Default is "solid".
#'@param title the title of the graph
#'@param repel a boolean, whether to use ggrepel to avoid overplotting text
#'  labels or not. The old \code{jitter} argument is kept for backward
#'  compatibility and is converted to \code{repel = TRUE} with a deprecation warning.
#'@param habillage an optional factor variable for coloring the observations by 
#'  groups. Default value is "none". If X is a PCA object from FactoMineR 
#'  package, habillage can also specify the supplementary qualitative variable 
#'  (by its index or name) to be used for coloring individuals by groups (see 
#'  ?PCA in FactoMineR).
#'@param addEllipses logical value. If TRUE, draws ellipses around the 
#'  individuals when habillage != "none".
#'@param ellipse.level the size of the concentration ellipse in normal 
#'  probability.
#'@param ellipse.type Character specifying frame type. Possible values are 
#'  \code{"convex"}, \code{"confidence"} or types supported by 
#'  \code{\link[ggplot2]{stat_ellipse}()} including one of \code{c("t", "norm", 
#'  "euclid")} for plotting concentration ellipses.
#'  
#'  \itemize{ \item \code{"convex"}: plot convex hull of a set of points. \item 
#'  \code{"confidence"}: plot confidence ellipses around group mean points as
#'  \code{\link[FactoMineR]{coord.ellipse}()}[in FactoMineR]. \item \code{"t"}:
#'  assumes a multivariate t-distribution. \item \code{"norm"}: assumes a
#'  multivariate normal distribution. \item \code{"euclid"}: draws a circle with
#'  the radius equal to level, representing the euclidean distance from the
#'  center. This ellipse probably won't appear circular unless
#'  \code{\link[ggplot2]{coord_fixed}()} is applied.}
#'  
#'  
#'@param ellipse.alpha Alpha for ellipse specifying the transparency level of 
#'  fill color. Use alpha = 0 for no fill color.
#'@param mean.point logical value. If TRUE (default), group mean points are 
#'  added to the plot.
#'@param col.circle a color for the correlation circle. Used only when X is a 
#'  PCA output.
#'@param circlesize the size of the variable correlation circle.
#'@param add.circle logical or NULL controlling the variable correlation circle.
#'  Default NULL shows it only when meaningful: for PCA variables on
#'  unit-variance (scaled) data, and for quantitative variables of
#'  MCA/MFA/HMFA/FAMD. Use \code{add.circle = TRUE} to force the circle (e.g. a
#'  \code{prcomp(scale = FALSE)} fit on data you scaled manually) or
#'  \code{add.circle = FALSE} to suppress it.
#'@param rotate.labels logical. If \code{TRUE}, the text labels of the plotted
#'  element are rotated to the angle of their arrows (ggbiplot style); labels in
#'  the left half-plane are flipped to stay upright. Default is \code{FALSE}
#'  (no rotation). Use together with \code{repel = FALSE}, as ggrepel ignores the
#'  label angle. Most useful for variable plots/biplots (e.g.
#'  \code{fviz_pca_var}, \code{fviz_pca_biplot}).
#'@param axes.linetype linetype of x and y axes.
#'@param color color to be used for the specified geometries (point, text). Can 
#'  be a continuous variable or a factor variable. Possible values include also 
#'  : "cos2", "contrib", "coord", "x" or "y". In this case, the colors for 
#'  individuals/variables are automatically controlled by their qualities of 
#'  representation ("cos2"), contributions ("contrib"), coordinates (x^2+y^2, 
#'  "coord"), x values ("x") or y values ("y"). To use automatic coloring (by 
#'  cos2, contrib, ....), make sure that habillage ="none".
#'@param fill same as the argument \code{color}, but for point fill color. 
#'  Useful when pointshape = 21, for example.
#'@param alpha controls the transparency of individual and variable colors, 
#'  respectively. The value can variate from 0 (total transparency) to 1 (no 
#'  transparency). Default value is 1. Possible values include also : "cos2", 
#'  "contrib", "coord", "x" or "y". In this case, the transparency for the 
#'  individual/variable colors are automatically controlled by their qualities 
#'  ("cos2"), contributions ("contrib"), coordinates (x^2+y^2, "coord"), x 
#'  values("x") or y values("y"). To use this, make sure that habillage ="none".
#'@param col.col.sup,col.row.sup colors for the supplementary column and row 
#'  points, respectively.
#'@param select a selection of individuals/variables to be drawn. Allowed values
#'  are NULL or a list containing the arguments name, cos2 or contrib: \itemize{
#'  \item name: is a character vector containing individuals/variables to be 
#'  drawn \item cos2: if cos2 is in [0, 1], ex: 0.6, then individuals/variables 
#'  with a cos2 > 0.6 are drawn. if cos2 > 1, ex: 5, then the top 5 
#'  individuals/variables with the highest cos2 are drawn. \item contrib: if 
#'  contrib > 1, ex: 5,  then the top 5 individuals/variables with the highest 
#'  contrib are drawn }
#'@param ggp a ggplot. If not NULL, points are added to an existing plot.
#'@inheritParams ggpubr::ggpar
#'@param font.family character vector specifying font family.
#'@param ... Arguments to be passed to the functions ggpubr::ggscatter() & 
#'  ggpubr::ggpar().
#'  
#'@return a ggplot
#'@author Alboukadel Kassambara \email{alboukadel.kassambara@@gmail.com}
#'  
#'@rdname fviz
#' @examples
#' \donttest{
#' # Principal component analysis
#' # +++++++++++++++++++++++++++++
#' data(decathlon2)
#' decathlon2.active <- decathlon2[1:23, 1:10]
#' res.pca <- prcomp(decathlon2.active,  scale = TRUE)
#' fviz(res.pca, "ind") # Individuals plot
#' fviz(res.pca, "var") # Variables plot
#' 
#' # Correspondence Analysis
#' # ++++++++++++++++++++++++++
#' # Install and load FactoMineR to compute CA
#' # install.packages("FactoMineR")
#' library("FactoMineR")
#' data("housetasks")
#' res.ca <- CA(housetasks, graph = FALSE)
#' fviz(res.ca, "row") # Rows plot
#' fviz(res.ca, "col") # Columns plot
#' 
#' # Multiple Correspondence Analysis
#' # +++++++++++++++++++++++++++++++++
#' library(FactoMineR)
#' data(poison)
#' res.mca <- MCA(poison, quanti.sup = 1:2, 
#'               quali.sup = 3:4, graph=FALSE)
#'               
#' fviz(res.mca, "ind") # Individuals plot
#' fviz(res.mca, "var") # Variables plot
#' 
#'  }
#'@export
fviz <- function(X, element, axes = c(1, 2), geom = "auto",
                          label = "all", invisible="none", labelsize=4,
                          pointsize = 1.5, pointshape = 19, arrowsize = 0.5,
                          arrow.linetype = "solid",
                          habillage="none", addEllipses=FALSE, ellipse.level = 0.95,
                          ellipse.type = "norm", ellipse.alpha = 0.1, mean.point = TRUE,
                          color = "black", fill = "white", alpha = 1, gradient.cols = NULL,
                          col.row.sup = "darkblue", col.col.sup="darkred",
                          select = list(name = NULL, cos2 = NULL, contrib = NULL),
                          title = NULL, axes.linetype = "dashed",
                          repel = FALSE, col.circle ="grey70", circlesize = 0.5, add.circle = NULL,
                          rotate.labels = FALSE,
                          ggtheme = theme_minimal(),
                          ggp = NULL, font.family = "",
                           ...)
  {
  
  .check_axes(axes, .length = 2)
  facto.class <- .get_facto_class(X)
    
  # Backward compatibility: jitter argument converted with warning
  extra_args <- list(...)
  if(!is.null(extra_args$jitter)) {
    .facto_dep("jitter", "repel = TRUE", NULL)
    repel <- TRUE
  }
  # Elements to be labelled or hidden
  lab <- .label(label)
  hide <- .hide(invisible)
  
  # Define parameters
  #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  # Define title if NULL
  if(is.null(title)){
    element_desc <- list(ind = "Individuals", var = "Variables",
                         col = "Column points", row = "Row points",
                         mca.cor = "Variables", quanti.sup = "Quantitative variables",
                         quali.sup = "Supplementary qualitative variable categories",
                         quanti.var = "Quantitative variables",
                         quali.var = "Qualitative variable categories",
                         group = "Variable groups", partial.axes = "Partial axes")
    if(facto.class == "MCA") element_desc$var <- "Variable categories"
    title <- paste0(element_desc[[element]], " - ", facto.class)
  }
  # Define geometry if auto
  if(geom[1] == "auto"){
    geom <- c("point", "text")
    if(element == "var" && facto.class == "PCA") geom <- c("arrow", "text")
  }
  # Define color if missing
  if(facto.class %in% c("CA", "MCA")){
    if(element %in% c("row", "ind") && missing(color)) color = "blue"
    else if(element %in% c("col", "var", "mca.cor") && missing(color)) color = "red"
  }

  # Data preparation
  #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  # Data frame to be used for plotting
  summary.res <- c("coord", "contrib", "cos2")
  if(element == "partial.axes" || (element == "quali.var" && facto.class == "HMFA")) 
    summary.res <- c("coord", "contrib")
  else if(element == "group" && facto.class == "HMFA") summary.res <- "coord"
  df <- facto_summarize(X, element = element, axes = axes, result = summary.res)
  colnames(df)[2:3] <-  c("x", "y")
  # Color by grouping variables
  #::::::::::::::::::::::::::::::::::::::
  is_grouping_var_exists <- !("none" %in% habillage) || .is_grouping_var(color) || .is_grouping_var(fill)
  # Augment data, if qualitative variable is used to color points by groups
  if(!("none" %in% habillage)){
    dd <- .add_ind_groups(X, df, habillage)
    df <- dd$ind
    color <- dd$name.quali
    if(missing(pointshape)) pointshape <- dd$name.quali
  }
  # Augment data if color is a continuous variable or a factor variable
  if(length(color) > 1){
    if(nrow(df) != length(color)) stop(
      "The length of the `color` variable must equal the number of elements being ",
      "plotted (individuals for fviz_*_ind(), variables for fviz_*_var()), or be a ",
      "single value or a metric such as \"cos2\"/\"contrib\".", call. = FALSE)
    .col.name <- "Col."
    df[[.col.name]] <- color
    if(missing(pointshape) && .is_grouping_var(color)) pointshape <- .col.name
    color <- .col.name
  }
  # Augment data if fill is a continuous variable or a factor variable
  if(length(fill) > 1){
    if(nrow(df) != length(fill)) stop(
      "The length of the `fill` variable must equal the number of elements being ",
      "plotted (individuals for fviz_*_ind(), variables for fviz_*_var()), or be a ",
      "single value or a metric such as \"cos2\"/\"contrib\".", call. = FALSE)
    .col.name <- "Fill."
    df[[.col.name]] <- fill
    fill <- .col.name
  }
  # Augment the data if pointshape is a grouping (factor/character) variable,
  # so points can be shaped by a DIFFERENT factor than `color`/habillage (e.g.
  # colour individuals by one variable and set their shape by another). Gated on
  # pointshape being a grouping var; the default numeric pointshape (e.g. 19) is
  # untouched, so existing plots are byte-identical. The `if(missing(pointshape))`
  # guards above already prevent colour/habillage from overwriting it. (#36, #51)
  if(.is_grouping_var(pointshape)){
    if(nrow(df) != length(pointshape)) stop(
      "The length of the `shape`/`shape.ind` variable must equal the number of ",
      "elements being plotted (individuals for fviz_*_ind()), or be a single ",
      "value.", call. = FALSE)
    .shape.name <- "Shape."
    df[[.shape.name]] <- as.factor(pointshape)
    pointshape <- .shape.name
  }
  # Augment the data, if pointsize is a continuous variable
  if(length(pointsize) > 1){
    if(nrow(df) != length(pointsize)) pointsize <- 1.5
    df[["pointsize"]] <- pointsize
    pointsize <- "pointsize"
  }
  # Selection
  df.all <- df
  if(!is.null(select) && !is.null(select$name) && .factominer_needs_category_map(facto.class, element)){
    select$name <- map_factominer_legacy_names(X, select$name, element = element)
  }
  if(!is.null(select) && !is.null(select$contrib) && !("contrib" %in% colnames(df))){
    stop("Contributions are not available for element = '", element, "'.")
  }
  if(!is.null(select)) df <- .select(df, select)
  
  # Special cases: data transformation
  #%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  # Used by fviz_pca_biplot() only:  Multiply the variables data by scale. before biplotting
  if(facto.class == "PCA" && element == "var" && !is.null(extra_args$scale.) )
    df[, c("x", "y")] <- df[, c("x", "y")]*extra_args$scale.
  # (M)CA: scale coords according to the type of map
  if(facto.class %in% c("CA", "MCA") && !(element %in% c("mca.cor", "quanti.sup"))){
  if(!is.null(extra_args$map)) df <- .scale_ca(df, res.ca = X,  element = element, 
                                               type = extra_args$map, axes = axes)
  }
  # Main plot
  #%%%%%%%%%%%%%%%%%%%
  is.pca.var <- element == "var" && facto.class == "PCA" # We don't want meanpoint for PCA variables colored by groups
  point <- ("point" %in% geom) && (!hide[[element]]) # to show points, should be TRUE
  if(missing(mean.point))
    mean.point <- (is_grouping_var_exists && !is.pca.var) && ("point" %in% geom) && (!hide[["quali"]]) # to show mean point
  if(element == "quanti.var") mean.point <- FALSE # MFA, don't show the mean point of groups of variables
  
  label <- NULL
  if(lab[[element]] && "text" %in% geom && !hide[[element]]) label <- "name"

  # Layers already present (when chaining via ggp, e.g. biplots) so we only
  # restyle the text labels added for THIS element below (#130).
  n_layers_before <- if(is.null(ggp)) 0L else length(ggp$layers)

  p <- ggplot()
  if(hide[[element]]) {
    # FIX: ggplot2 3.0.0+ deprecation - aes_string() replaced with aes() + .data pronoun
    # See: https://github.com/kassambara/factoextra/issues/190
    if(is.null(ggp)) p <-ggplot()+geom_blank(data = df, aes(x = .data[["x"]], y = .data[["y"]]))
    else p <- ggp
  }
  else p <- ggpubr::ggscatter(data = df, x = "x", y = "y",
                         color = color, fill = fill,  alpha = alpha, shape = pointshape, 
                         point = point, size = pointsize, mean.point = mean.point,
                         label = label, font.label = labelsize*3, repel = repel,
                         ellipse = addEllipses, ellipse.type = ellipse.type,
                         ellipse.alpha = ellipse.alpha, ellipse.level = ellipse.level,
                         main = title, ggtheme = ggtheme, ggp = ggp, font.family = font.family, ...)
  if(alpha %in% c("cos2","contrib", "coord", "x", "y"))
    p <- p + scale_alpha(limits = range(df.all[, alpha]))

  # Apply a numeric alpha to this element's text labels too, not just its
  # points/arrows, so e.g. alpha.var fades the variable names as well (#130).
  # Gated on a numeric alpha < 1, so the default (alpha = 1) and metric-mapped
  # alpha ("cos2"/"contrib"/...) paths are byte-identical. Only restyles labels
  # added for this element (index > n_layers_before), preserving other layers
  # when chaining via ggp (e.g. biplots).
  if(is.numeric(alpha) && length(alpha) == 1 && !is.na(alpha) && alpha < 1){
    .text_geoms <- c("GeomText", "GeomTextRepel", "GeomLabel", "GeomLabelRepel")
    for(i in seq_along(p$layers)){
      if(i > n_layers_before && inherits(p$layers[[i]]$geom, .text_geoms))
        p$layers[[i]]$aes_params$alpha <- alpha
    }
  }

  # Optionally rotate this element's text labels to the angle of their arrows
  # (ggbiplot style). Gated on rotate.labels = TRUE, so the default is
  # byte-identical. Restricted to elements that actually draw arrows ("arrow" in
  # geom), i.e. variable plots: this keeps biplots from rotating the individual
  # labels (whose angle would be meaningless). Works with plain geom_text (use
  # repel = FALSE); ggrepel ignores the angle aesthetic. (#98)
  if(isTRUE(rotate.labels) && "arrow" %in% geom)
    p <- .rotate_text_labels(p, n_layers_before)

  if(!is.null(gradient.cols))
    p <- p + ggpubr::gradient_color(gradient.cols)
    
    
  if(is.null(extra_args$legend)) p <- p + theme(legend.position = "right" )
  # Add arrows
  if("arrow" %in% geom && !hide[[element]])
    p <- p + .arrows(data = df, color = color, alpha = alpha, linewidth = arrowsize,
                     linetype = arrow.linetype)
  # Add correlation circle. By default (add.circle = NULL) the circle is shown
  # only when it is meaningful: for PCA variables on unit-variance (scaled) data,
  # and for quanti variables of MCA/MFA/HMFA/FAMD. add.circle = TRUE/FALSE forces
  # or suppresses it (e.g. force it for a prcomp(scale = FALSE) on data the user
  # scaled manually). (#88)
  .want_circle <- function(default) if(is.null(add.circle)) isTRUE(default) else isTRUE(add.circle)
  if(facto.class == "PCA" && element == "var"){
    if(.want_circle(.get_scale_unit(X) && is.null(extra_args$scale.)))
      p <- .add_corr_circle(p, color = col.circle, size = circlesize)
  }
  else if(facto.class %in% c("MCA", "MFA", "HMFA", "FAMD") && element %in% c("quanti.sup", "quanti.var", "partial.axes")){
      if(.want_circle(TRUE))
        p <- .add_corr_circle(p, color = col.circle, size = circlesize)
  }
  # Faceting when multiple variables are used to color individuals
  # (e.g., habillage = 1:2, or data.frame)
  # in this case there is a column "facet_vars" in df
  if("facet_vars" %in% colnames(df)){
    groups <- c("facet_vars", "Groups")
    xx <- ggpubr::desc_statby(df, measure.var = "x", grps = groups)[, c(groups, "mean")]
    colnames(xx)[ncol(xx)] <- "x"
    yy <- ggpubr::desc_statby(df, measure.var = "y", grps = groups)[, c(groups, "mean")]
    xx$y <- yy$mean
    grp_coord <- xx
   
    p <- p+ggpubr::geom_exec(geom_text, data = grp_coord, x = "x", y = "y",
                             label = "Groups", color = color) 
    p <- p + facet_wrap(~facet_vars) + theme(legend.position = "none") 
  }
  
  # Supplementary elements: available only for FactoMineR
  #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  scale. <- ifelse(is.null(extra_args$scale.), 1, extra_args$scale.)
  esup <- .define_element_sup(X, element, geom = geom, lab = lab, hide = hide,
                              col.row.sup = col.row.sup, col.col.sup = col.col.sup,...) 
  ca_map = extra_args$map
  if(element == "mca.cor") ca_map = NULL
  
  if(!is.null(esup)) p <- .add_supp (p, X, element = esup$name, axes = axes, select = select,
                                geom = geom, color = esup$color, shape = esup$shape, pointsize = pointsize,
                                labelsize = labelsize, addlabel = esup$addlabel, repel = repel, linetype = 2,
                                scale. = scale., ca_map = ca_map, font.family = font.family)
  
  
  p <- .fviz_finish(p, X, axes, axes.linetype, ...) +
    labs(title = title) 
  
  p
}


#+++++++++++++++++++++
# Helper functions
#+++++++++++++++++++++


# Check if fill/color variable is continous in the context of PCA
.is_continuous_var <- function(x){
  x[1] %in% c("cos2", "contrib", "x", "y") || is.numeric(x)
}

.is_grouping_var <- function(x){
  length(x) > 1 && (is.character(x) || is.factor(x))
}
# Check if is continuous or grouping variable in the context of PCA
.is_variable <- function(x){
  .is_continuous_var(x) || .is_grouping_var(x)
}


# Check if character string is a valid color representation
.is_color <- function(x) {
  vapply(x, function(X) {
    tryCatch(is.matrix(grDevices::col2rgb(X)),
             error = function(e) FALSE)
  }, logical(1))
}


# X : an object of class PCA, princomp, prcomp, dudi, expoOutput
# Return TRUE if the data are scaled to unit variance
.get_scale_unit <-function(X){
  scale_unit <- FALSE
  if(inherits(X, 'PCA')) scale_unit <- X$call$scale.unit
  else if(inherits(X, "prcomp" )) scale_unit <- is.numeric(X$scale)
  else if(inherits(X, "princomp")) scale_unit <- length(unique(X$scale))>1
  else if(inherits(X, "factoextra_pca")) scale_unit <- isTRUE(X$scale.unit)
  else if(inherits(X, "expoOutput")) scale_unit <- !all(X$ExPosition.Data$scale==1) 
  else if(inherits(X, "pca") && inherits(X, "dudi")) scale_unit <- length(unique(X$norm)) > 1
  # ade4 between-/within-class PCA: $co holds projections (not variable
  # correlations bounded by 1), so the correlation circle does not apply.
  else if(inherits(X, c("between", "within")) && inherits(X, "dudi")) scale_unit <- FALSE
  else {
    warning(".get_scale_unit function: can't handle an object of class ",
            paste(class(X), collapse = ", "))
  }
  scale_unit
}

# Rotate the text labels of the layers added for the current element so each
# label is aligned with its arrow direction (ggbiplot style). Labels in the
# left half-plane are flipped by 180 degrees so they stay upright/readable.
# Only layers with index > n_layers_before are touched, preserving any layers
# carried in via ggp (e.g. the individuals layer of a biplot). (#98)
.rotate_text_labels <- function(p, n_layers_before = 0L){
  .text_geoms <- c("GeomText", "GeomTextRepel", "GeomLabel", "GeomLabelRepel")
  for(i in seq_along(p$layers)){
    if(i <= n_layers_before) next
    layer <- p$layers[[i]]
    if(!inherits(layer$geom, .text_geoms)) next
    ld <- as.data.frame(layer$data)
    if(is.null(ld) || !all(c("x", "y") %in% names(ld)) || nrow(ld) == 0) next
    ang <- atan2(ld$y, ld$x) * 180 / pi
    flip <- ang > 90 | ang < -90
    ang[flip] <- ang[flip] + 180
    ld[[".fviz_angle"]] <- ang
    p$layers[[i]]$data <- ld
    p$layers[[i]]$mapping <- utils::modifyList(
      layer$mapping, ggplot2::aes(angle = .data[[".fviz_angle"]]))
  }
  p
}

# Add correlation circle to variables plot
.add_corr_circle <- function(p, color = "grey70", size = 0.5){
  theta <- c(seq(-pi, pi, length = 50), seq(pi, -pi, length = 50))
  circle <- data.frame(xcircle = cos(theta), ycircle = sin(theta))
  p + 
    # FIX: ggplot2 3.0.0+ deprecation - aes_string() replaced with aes() + .data pronoun
    # FIX: ggplot2 3.4.0+ deprecation - size replaced with linewidth for line geoms
    # See: https://github.com/kassambara/factoextra/issues/190, #191
    geom_path(mapping = aes(x = .data[["xcircle"]], y = .data[["ycircle"]]), data = circle, color = color,
              linewidth = size) +
    coord_fixed()
  
}

# Add arrow to the plot
# FIX: ggplot2 3.4.0+ deprecation - size replaced with linewidth for geom_segment
.arrows <- function(data, color = "black", alpha = 1, linewidth = 0.5,
                    linetype = "solid", origin = 0, xend = "x", yend = "y"){
  origin <- rep(origin, nrow(data))
  dd <- cbind.data.frame(data, xstart = origin, ystart = origin)
  ggpubr::geom_exec(geom_segment, data = dd,
                    x = "xstart", y = "ystart", xend = xend, yend = yend,
                    arrow = grid::arrow(length = grid::unit(0.2, 'cm')),
                    color = color, alpha = alpha, linewidth = linewidth,
                    linetype = linetype)
}



# Define element sup if any
# lab,hide: element to be labelled or hidden as returned by .hide() and .label()
.define_element_sup <- function(X, element, geom, lab, hide,
                                col.row.sup = "darkblue", col.col.sup = "darkred", 
                                 ...){
  
  extra_args <- list(...)
  shape.sup <- ifelse(is.null(extra_args$shape.sup), 19, extra_args$shape.sup)
  color <- col.row.sup
  if(element %in% c("var", "mca.cor", "col", "group", "quanti.var")) color <- col.col.sup
    
  res <- NULL
  # Supplementary individuals
  if(element == "ind" && inherits(X, c("PCA", "MCA", "MFA", "FAMD")) && !hide$ind.sup)
    res <- list(name = "ind.sup", addlabel = (lab$ind.sup && "text" %in% geom))
  # Supplementary quantitative variables
  else if(element == "var" && inherits(X, "PCA") && !hide$quanti.sup)
    res <- list(name = "quanti", addlabel = (lab$quanti.sup && "text" %in% geom))
  else if(element == "mca.cor" && inherits(X, "MCA") && !hide$quanti)
    res <- list(name = c("quanti.sup", "quali.sup$eta2"), addlabel = (lab$quanti && "text" %in% geom))
  else if(element %in% "var" && inherits(X, "MCA") && !hide$quali.sup)
    res <- list(name = "quali.sup", addlabel = (lab$quali.sup && "text" %in% geom))
  else if(element %in% "quali.var" && inherits(X, "MFA") && !hide$quali.sup)
    res <- list(name = "quali.sup", addlabel = (lab$quali.sup && "text" %in% geom))
  else if(element %in% "quali.var" && inherits(X, "FAMD") && !hide$quali.sup)
    res <- list(name = "quali.sup", addlabel = (lab$quali.sup && "text" %in% geom))
  else if(element %in% "var" && inherits(X, "FAMD") && !hide$quanti.sup)
    res <- list(name = "quanti.sup", addlabel = (lab$quanti.sup && "text" %in% geom))
  # CA
  else if(element == "row" && inherits(X, c("CA", "ca")) && !hide$row.sup)
    res <- list(name = "row.sup", addlabel = (lab$row.sup && "text" %in% geom))
  else if(element == "col" && inherits(X, c("CA", "ca")) && !hide$row.sup)
    res <- list(name = "col.sup", addlabel = (lab$col.sup && "text" %in% geom))
  else if(element == "group" && inherits(X, "MFA") && !hide$group.sup)
    res <- list(name = "group", addlabel = (lab$group.sup && "text" %in% geom))
  else if(element == "quanti.var" && inherits(X, "MFA") && !hide$quanti.var.sup)
    res <- list(name = "quanti.var.sup", addlabel = (lab$quanti.var.sup && "text" %in% geom))
  
  res$color <- color
  res$shape <- shape.sup
  res
}

#'Extract and visualize the eigenvalues/variances of dimensions
#'
#'@description Eigenvalues correspond to the amount of the variation explained 
#'  by each principal component (PC). 
#'  
#'  \itemize{ \item get_eig(): Extract the eigenvalues/variances of the
#'  principal dimensions \item fviz_eig(): Plot the eigenvalues/variances
#'  against the number of dimensions \item get_eigenvalue(): an alias of
#'  get_eig() \item fviz_screeplot(): an alias of fviz_eig() }
#'  
#'  These functions support the results of Principal Component Analysis (PCA), 
#'  Correspondence Analysis (CA), Multiple Correspondence Analysis (MCA), Factor Analysis of Mixed Data (FAMD),
#'  Multiple Factor Analysis (MFA) and Hierarchical Multiple Factor Analysis
#'  (HMFA) functions. \code{fviz_eig()} validates \code{ncp},
#'  \code{parallel.iter}, and \code{parallel.seed} before plotting, accepting
#'  integer-like numeric values while still rejecting fractional inputs.
#'  
#'  
#'@param X an object of class PCA, CA, MCA, FAMD, MFA and HMFA [FactoMineR]; prcomp 
#'  and princomp [stats]; dudi, pca, coa and acm [ade4]; ca and mjca [ca 
#'  package].
#'@param choice a text specifying the data to be plotted. Allowed values are 
#'  "variance" or "eigenvalue".
#'@param geom a text specifying the geometry to be used for the graph. Allowed 
#'  values are "bar" for barplot, "line" for lineplot or c("bar", "line") to use
#'  both types.
#'@param barfill fill color for bar plot.
#'@param barcolor outline color for bar plot.
#'@param linecolor color for line plot (when geom contains "line").
#'@param ncp a single positive integer specifying the number of dimensions to
#'  be shown. Integer-like numeric values are accepted.
#'@param addlabels logical value. If TRUE, labels are added at the top of bars
#'  or points showing the information retained by each dimension.
#'@param hjust horizontal adjustment of the labels.
#'@param main,xlab,ylab plot main and axis titles.
#'@param parallel logical value. If TRUE, adds a parallel analysis threshold line
#'  (Horn's method) to help determine the number of components to retain. Components
#'  with eigenvalues above this line are considered significant. Only works when
#'  choice = "eigenvalue" and X is a prcomp or princomp object. Default is FALSE.
#'@param parallel.color color of the parallel analysis threshold line. Default is "red".
#'@param parallel.lty line type for the parallel analysis line. Default is "dashed".
#'@param parallel.iter a single positive integer giving the number of
#'  iterations for parallel analysis simulation. Integer-like numeric values
#'  are accepted. Default is 100.
#'@param parallel.seed NULL or a single non-negative integer seed for
#'  reproducible parallel analysis simulation. If NULL (default), the current
#'  RNG stream is used. Integer-like numeric values are accepted.
#' @inheritParams ggpubr::ggpar
#'@param ... optional arguments to be passed to the function \link[ggpubr]{ggpar}.
#'  
#'@return \itemize{ \item get_eig() (or get_eigenvalue()): returns a data.frame
#'  containing 3 columns: the eigenvalues, the percentage of variance and  the
#'  cumulative percentage of variance retained by each dimension.
#'  \item fviz_eig() (or fviz_screeplot()): returns a ggplot2 }
#'  
#'@author Alboukadel Kassambara \email{alboukadel.kassambara@@gmail.com}
#'@seealso \code{\link{fviz_pca}}, \code{\link{fviz_ca}},
#'  \code{\link{fviz_mca}}, \code{\link{fviz_mfa}}, \code{\link{fviz_hmfa}}
#'@references \url{https://www.sthda.com/english/}
#' @examples
#' # Principal Component Analysis
#' # ++++++++++++++++++++++++++
#' data(iris)
#' res.pca <- prcomp(iris[, -5],  scale = TRUE)
#' 
#' # Extract eigenvalues/variances
#' get_eig(res.pca)
#' 
#' # Default plot
#' fviz_eig(res.pca, addlabels = TRUE, ylim = c(0, 85))
#'   
#' # Scree plot - Eigenvalues
#' fviz_eig(res.pca, choice = "eigenvalue", addlabels=TRUE)
#' 
#' # Use only bar  or line plot: geom = "bar" or geom = "line"
#' fviz_eig(res.pca, geom="line")
#'
#' # Parallel analysis (Horn's method) to determine number of components
#' # Components with eigenvalues above the red line are significant
#' fviz_eig(res.pca, choice = "eigenvalue", parallel = TRUE,
#'          addlabels = TRUE, parallel.color = "red",
#'          parallel.iter = 10, parallel.seed = 123)
#'
#' \dontrun{         
#' # Correspondence Analysis
#' # +++++++++++++++++++++++++++++++++
#' library(FactoMineR)
#' data(housetasks)
#' res.ca <- CA(housetasks, graph = FALSE)
#' get_eig(res.ca)
#' fviz_eig(res.ca, linecolor = "#FC4E07",
#'    barcolor = "#00AFBB", barfill = "#00AFBB")
#' 
#' # Multiple Correspondence Analysis
#' # +++++++++++++++++++++++++++++++++
#' library(FactoMineR)
#' data(poison)
#' res.mca <- MCA(poison, quanti.sup = 1:2, 
#'               quali.sup = 3:4, graph=FALSE)
#' get_eig(res.mca)
#' fviz_eig(res.mca, linecolor = "#FC4E07",
#'    barcolor = "#2E9FDF", barfill = "#2E9FDF")
#' }
#' 
#'@name eigenvalue
NULL
#' @rdname eigenvalue
#' @export
get_eig<-function(X){
  
  # FactoMineR package
  if(inherits(X, c('PCA', 'CA', 'MCA', 'FAMD', 'MFA', 'HMFA', 'sPCA', 'sCA', 'sMCA', 'sMFA', 'sHMFA'))) eig <- X$eig
  else{
    # stats package
    if(inherits(X, "prcomp") || inherits(X, "princomp")) eig <- (X$sdev)^2
    # user-supplied coordinates wrapped by as_factoextra_pca()
    else if(inherits(X, "factoextra_pca")) eig <- X$eig.values
    # ade4 package (incl. between-class/within-class analyses: bca/wca)
    else if(inherits(X, c("pca", "coa", "acm", "between", "within")) && inherits(X, "dudi")) eig <- X$eig
    # ca package
    else if(inherits(X, 'ca'))  eig <- X$sv^2
    else if(inherits(X, 'mjca')) eig <- X$inertia.e
    # MASS
    else if(inherits(X, 'correspondence'))  eig <- X$cor^2
    # ExPosition package
    else if (inherits(X, "expoOutput")) eig <- X$ExPosition.Data$eigs
    else stop("An object of class : ", paste(class(X), collapse = ", "), 
              " can't be handled by the function get_eigenvalue()")
    
    variance <- eig*100/sum(eig)
    cumvar <- cumsum(variance)
    eig <- data.frame(eigenvalue = eig, variance = variance, 
                      cumvariance = cumvar)
  }
  
  colnames(eig) <- c("eigenvalue", "variance.percent", 
                     "cumulative.variance.percent")
  rownames(eig) <- paste0("Dim.", seq_len(nrow(eig)))
  
  eig 
}

#' @rdname eigenvalue
#' @export
get_eigenvalue <- function(X){
  get_eig(X)
}

#' @rdname eigenvalue
#' @export
fviz_eig<-function(X, choice=c("variance", "eigenvalue"), geom=c("bar", "line"),
                  barfill="steelblue", barcolor="steelblue", linecolor = "black",
                  ncp=10, addlabels=FALSE, hjust = 0,
                   main = NULL, xlab = NULL, ylab = NULL,
                  ggtheme = theme_minimal(),
                  parallel = FALSE, parallel.color = "red",
                  parallel.lty = "dashed", parallel.iter = 100,
                  parallel.seed = NULL, ...)
{
  ncp <- .coerce_integerish(ncp, "ncp")

  eig <- get_eigenvalue(X)
  eig <-eig[seq_len(min(ncp, nrow(eig))), , drop=FALSE]
  
  choice <- choice[1]
  if(choice=="eigenvalue") {
    eig <- eig[,1]
    text_labels <- round(eig,1)
    if(is.null(ylab)) ylab <- "Eigenvalue"
  }
  else if(choice=="variance") {
    eig <- eig[,2]
    text_labels <- paste0(round(eig,1), "%")
  }
  else stop("Allowed values for the argument choice are : 'variance' or 'eigenvalue'")
  
  if(length(intersect(geom, c("bar", "line"))) == 0)
    stop("The specified value(s) for the argument geom are not allowed ")
  
  
  
  df.eig <- data.frame(dim = factor(seq_along(eig)), eig=eig )
  
  extra_args <- list(...)
  bar_width <- extra_args$bar_width
  linetype <- extra_args$linetype
  if(is.null(linetype)) linetype <- "solid"

  p <- ggplot(df.eig, aes(dim, eig, group=1 ))
  if("bar" %in% geom) {
    bar_args <- list(stat = "identity", fill = barfill, color = barcolor)
    if(!is.null(bar_width)) bar_args$width <- bar_width
    p <- p + do.call(geom_bar, bar_args)
  }
  if("line" %in% geom) p <- p + geom_line(color = linecolor, linetype = linetype)+
    geom_point(shape=19, color=linecolor)
  if(addlabels) p <- p + geom_text(label = text_labels, vjust=-0.4, hjust = hjust)

  if(!is.null(parallel.seed)){
    parallel.seed <- .coerce_integerish(
      parallel.seed, "parallel.seed", lower = 0L, upper = .Machine$integer.max,
      value_label = "NULL or a single integer value"
    )
  }

  # Add parallel analysis line (Horn's method) if requested
  if(parallel && choice == "eigenvalue") {
    parallel.iter <- .coerce_integerish(parallel.iter, "parallel.iter")

    compute_parallel_threshold <- function(n_obs, n_var, fit_fn){
      sim_eigs <- matrix(NA_real_, nrow = parallel.iter, ncol = n_var)
      for(i in seq_len(parallel.iter)) {
        random_data <- matrix(stats::rnorm(n_obs * n_var), nrow = n_obs, ncol = n_var)
        sim_eigs[i, ] <- fit_fn(random_data)
      }
      apply(sim_eigs, 2, function(x) stats::quantile(x, 0.95))
    }

    # Parallel analysis requires the original data, available from prcomp/princomp
    if(inherits(X, "prcomp") && !is.null(X$x)) {
      n_obs <- nrow(X$x)
      n_var <- ncol(X$x)
      if(is.null(parallel.seed)) {
        parallel_threshold <- compute_parallel_threshold(
          n_obs, n_var, fit_fn = function(random_data) stats::prcomp(random_data, scale. = TRUE)$sdev^2
        )
      } else {
        parallel_threshold <- .with_preserved_seed(parallel.seed, {
          compute_parallel_threshold(
            n_obs, n_var, fit_fn = function(random_data) stats::prcomp(random_data, scale. = TRUE)$sdev^2
          )
        })
      }
      parallel_threshold <- parallel_threshold[seq_len(min(ncp, length(parallel_threshold)))]
      df.parallel <- data.frame(dim = factor(seq_along(parallel_threshold)),
                                 threshold = parallel_threshold)
      p <- p + geom_line(data = df.parallel, aes(x = .data[["dim"]], y = .data[["threshold"]]),
                         color = parallel.color, linetype = parallel.lty, linewidth = 0.8) +
               geom_point(data = df.parallel, aes(x = .data[["dim"]], y = .data[["threshold"]]),
                          color = parallel.color, shape = 4, size = 2)
    } else if(inherits(X, "princomp") && !is.null(X$scores)) {
      n_obs <- nrow(X$scores)
      n_var <- ncol(X$scores)
      if(is.null(parallel.seed)) {
        parallel_threshold <- compute_parallel_threshold(
          n_obs, n_var, fit_fn = function(random_data) stats::princomp(random_data, cor = TRUE)$sdev^2
        )
      } else {
        parallel_threshold <- .with_preserved_seed(parallel.seed, {
          compute_parallel_threshold(
            n_obs, n_var, fit_fn = function(random_data) stats::princomp(random_data, cor = TRUE)$sdev^2
          )
        })
      }
      parallel_threshold <- parallel_threshold[seq_len(min(ncp, length(parallel_threshold)))]
      df.parallel <- data.frame(dim = factor(seq_along(parallel_threshold)),
                                 threshold = parallel_threshold)
      p <- p + geom_line(data = df.parallel, aes(x = .data[["dim"]], y = .data[["threshold"]]),
                         color = parallel.color, linetype = parallel.lty, linewidth = 0.8) +
               geom_point(data = df.parallel, aes(x = .data[["dim"]], y = .data[["threshold"]]),
                          color = parallel.color, shape = 4, size = 2)
    } else {
      warning("Parallel analysis requires prcomp or princomp objects with score data. Skipping.")
    }
  } else if(parallel && choice != "eigenvalue") {
    warning("Parallel analysis only works with choice = 'eigenvalue'. Skipping.")
  }

  if(is.null(main)) main <- "Scree plot"
  if(is.null(xlab)) xlab <- "Dimensions"
  if(is.null(ylab)) ylab <- "Percentage of explained variances"
  
  p <- p + labs(title = main, x = xlab, y = ylab)
  ggpubr::ggpar(p, ggtheme = ggtheme,  ...)
}


#' @rdname eigenvalue
#' @export 
fviz_screeplot<- function(...){
  fviz_eig(...)
} 

#' @include eigenvalue.R get_pca.R hcut.R
 NULL
#'Visualize Silhouette Information from Clustering
#'@description Silhouette (Si) analysis is a cluster validation approach that
#'  measures how well an observation is clustered and it estimates the average
#'  distance between clusters. fviz_silhouette() provides ggplot2-based elegant
#'  visualization of silhouette information from i) the result of
#'  \code{\link[cluster]{silhouette}}(), \code{\link[cluster]{pam}}(),
#'  \code{\link[cluster]{clara}}() and \code{\link[cluster]{fanny}}() [in
#'  cluster package]; ii) \code{\link{eclust}}() and \code{\link{hcut}}() [in
#'  factoextra]. Results without silhouette information, such as one-cluster
#'  \code{eclust}/\code{hcut} objects, are rejected with a package-level error.
#'  
#'  Read more: 
#'  \href{https://www.datanovia.com/en/lessons/cluster-validation-statistics-must-know-methods/}{Clustering
#'   Validation Statistics}.
#'@details - Observations with a large silhouhette Si (almost 1) are very well 
#'  clustered.
#'  
#'  - A small Si (around 0) means that the observation lies between two
#'  clusters.
#'  
#'  - Observations with a negative Si are probably placed in the wrong cluster.
#'
#'  - Silhouette plots require at least two clusters and available silhouette
#'  widths.
#'  
#'@param sil.obj an object of class silhouette: pam, clara, fanny [in cluster
#'  package]; eclust and hcut [in factoextra]. For \code{eclust} and
#'  \code{hcut}, silhouette information must be available, which requires at
#'  least two clusters.
#'@param label logical value. If true, x axis tick labels are shown
#'@param print.summary logical value. If true a summary of cluster silhouettes 
#'  are printed in fviz_silhouette().
#' @param ... other arguments to be passed to the function ggpubr::ggpar().
#'  
#'@return a ggplot2 object.
#'@author Alboukadel Kassambara \email{alboukadel.kassambara@@gmail.com}
#'@seealso \code{\link{fviz_cluster}}, \code{\link{hcut}}, 
#'  \code{\link{hkmeans}},  \code{\link{eclust}}, \code{\link{fviz_dend}}
#' @examples 
#' set.seed(123)
#' 
#' # Data preparation
#' # +++++++++++++++
#' data("iris")
#' head(iris)
#' # Remove species column (5) and scale the data
#' iris.scaled <- scale(iris[, -5])
#' 
#' # K-means clustering
#' # +++++++++++++++++++++
#' km.res <- kmeans(iris.scaled, 3, nstart = 2)
#' 
#' # Visualize kmeans clustering
#' fviz_cluster(km.res, iris[, -5], ellipse.type = "norm")+
#' theme_minimal()
#' 
#' # Visualize silhouette information
#' requireNamespace("cluster", quietly = TRUE)
#' sil <- cluster::silhouette(km.res$cluster, dist(iris.scaled))
#' fviz_silhouette(sil)
#' 
#' # Identify observation with negative silhouette
#' neg_sil_index <- which(sil[, "sil_width"] < 0)
#' sil[neg_sil_index, , drop = FALSE]
#' \dontrun{
#' # PAM clustering
#' # ++++++++++++++++++++
#' requireNamespace("cluster", quietly = TRUE)
#' pam.res <- cluster::pam(iris.scaled, 3)
#' # Visualize pam clustering
#' fviz_cluster(pam.res, ellipse.type = "norm")+
#' theme_minimal()
#' # Visualize silhouette information
#' fviz_silhouette(pam.res)
#' 
#' # Hierarchical clustering
#' # ++++++++++++++++++++++++
#' # Use hcut() which compute hclust and cut the tree
#' hc.cut <- hcut(iris.scaled, k = 3, hc_method = "complete")
#' # Visualize dendrogram
#' fviz_dend(hc.cut, show_labels = FALSE, rect = TRUE)
#' # Visualize silhouette information
#' if (hc.cut$nbclust > 1) fviz_silhouette(hc.cut)
#' }
#'@export
fviz_silhouette <- function(sil.obj, label = FALSE, print.summary = TRUE, ...){
  
  if(inherits(sil.obj, c("eclust", "hcut", "pam", "clara", "fanny"))){
    if(is.null(sil.obj$silinfo) || is.null(sil.obj$silinfo$widths)) {
      stop(
        "Silhouette information is unavailable for this clustering result. ",
        "Silhouette plots require at least 2 clusters."
      )
    }
    df <- as.data.frame(sil.obj$silinfo$widths)
  }
  else if(inherits(sil.obj, "silhouette"))
    df <- as.data.frame(sil.obj[, 1:3])
  else stop("Don't support an object of class ", paste(class(sil.obj), collapse = ", "))
  
  # order by cluster and by sil_width
  df <- df[order(df$cluster, -df$sil_width), ]
  if(!is.null(rownames(df))) df$name <- factor(rownames(df), levels = rownames(df))
  else df$name <- as.factor(seq_len(nrow(df)))
  df$cluster <- as.factor(df$cluster)
  # FIX: ggplot2 3.0.0+ deprecation - aes_string() replaced with aes() + .data pronoun
  # See: https://github.com/kassambara/factoextra/issues/190
  mapping <- aes(x = .data[["name"]], y = .data[["sil_width"]],
                        color = .data[["cluster"]], fill = .data[["cluster"]])
  p <- ggplot(df, mapping) +
    geom_bar(stat = "identity") +
    labs(y = "Silhouette width Si", x = "",
         title = paste0("Clusters silhouette plot ",
                        "\n Average silhouette width: ", 
                        round(mean(df$sil_width), 2)))+
    ggplot2::ylim(c(NA, 1))+
    geom_hline(yintercept = mean(df$sil_width), linetype = "dashed", color = "red" )
  p <- ggpubr::ggpar(p, ...)
  # Labels
  if(!label) p <- p + theme(axis.text.x = element_blank(), 
                            axis.ticks.x = element_blank())
  else if(label)
    p <- p + theme(axis.text.x = element_text(angle=45))
  
  # Print summary
  ave <- tapply(df$sil_width, df$cluster, mean)
  n <- tapply(df$cluster, df$cluster, length)
  sil.sum <- data.frame(cluster = names(ave), size = n,
                      ave.sil.width = round(ave,2))
  if(print.summary) print(sil.sum)
  
  p
}

#' @include utilities.R dist.R
NULL
#' Assessing Clustering Tendency
#' 
#' @description Before applying cluster methods, the first step is to assess 
#'   whether the data is clusterable, a process defined as the \strong{assessing
#'   of clustering tendency}. get_clust_tendency() assesses clustering tendency 
#'   using Hopkins' statistic and a visual approach. An ordered dissimilarity 
#'   image (ODI) is shown. Objects belonging to the same cluster are displayed 
#'   in consecutive order using hierarchical clustering. For more details and 
#'   interpretation, see 
#'   \href{https://www.datanovia.com/en/lessons/assessing-clustering-tendency/}{STHDA
#'    website: Assessing clustering tendency}.
#' @param data a numeric data frame or matrix. Columns are variables and rows 
#'   are samples. Computation are done on rows (samples) by default. If you want
#'   to calculate Hopkins statistic on variables, transpose the data before.
#' @param n a positive integer specifying the number of points selected from
#'   sample space and from the observed data. Must be smaller than the number
#'   of complete observations.
#' @param graph logical value; if TRUE the ordered dissimilarity image (ODI) is 
#'   shown.
#' @param gradient a list containing three elements specifying the colors for 
#'   low, mid and high values in the ordered dissimilarity image. The element 
#'   "mid" can take the value of NULL.
#' @param seed an integer seed for reproducibility, or NULL to use the current
#'   RNG stream. When non-NULL, the function restores the caller RNG state on
#'   exit.
#' @details
#' 
#' \strong{Hopkins statistic}: If the value of Hopkins statistic is close to
#' 1 (far above 0.5), then we can conclude that the dataset is significantly
#' clusterable. The statistic is calculated using the correct formula from
#' Cross and Jain (1982) with exponent d=D where D is the dimensionality
#' (number of columns) of the data. Under the null hypothesis of spatial
#' randomness, the Hopkins statistic follows a Beta(n, n) distribution.
#'
#' \strong{Note on interpretation}: This function returns the Hopkins statistic H
#' where values close to 1 indicate clusterable data. Some other packages (e.g.,
#' \code{performance::check_clusterstructure}) return 1-H, where values close to
#' 0 indicate clusterability. Always check the documentation of the specific
#' implementation you are using.
#'
#' \strong{Breaking change}: factoextra uses the corrected Hopkins statistic
#' formula (Wright 2022). Results differ from legacy factoextra and a one-time
#' warning is emitted. Set \code{options(factoextra.warn_hopkins = FALSE)} to
#' silence the warning.
#'
#' For large datasets, nearest-neighbor distances are computed with a low-memory
#' fallback when the full pairwise matrix would exceed
#' \code{getOption("factoextra.hopkins.max_matrix_cells", 2e7)} cells.
#' 
#' \strong{VAT (Visual Assessment of cluster Tendency)}: The VAT detects the
#' clustering tendency in a visual form by counting the number of square shaped
#' dark (or colored) blocks along the diagonal in a VAT image.
#'  
#' @return A list containing the elements:
#'   
#'   - hopkins_stat for Hopkins statistic value
#'   
#'   - plot for ordered dissimilarity image. This is generated using the 
#'   function \code{\link{fviz_dist}}(dist.obj).
#' @seealso \code{\link{fviz_dist}}
#' @author Alboukadel Kassambara \email{alboukadel.kassambara@@gmail.com}
#' @examples 
#' data(iris)
#' 
#' # Silence the one-time compatibility warning in examples
#' old_hopkins_warn <- getOption("factoextra.warn_hopkins")
#' options(factoextra.warn_hopkins = FALSE)
#' 
#' # Clustering tendency
#' gradient_col = list(low = "steelblue", high = "white")
#' get_clust_tendency(iris[,-5], n = 50, gradient = gradient_col)
#'    
#' # Random uniformly distributed dataset
#' # (without any inherent clusters)
#' set.seed(123)
#' random_df <- apply(iris[, -5], 2, 
#'                    function(x){runif(length(x), min(x), max(x))}
#'                    )
#' get_clust_tendency(random_df, n = 50, gradient = gradient_col)
#' options(factoextra.warn_hopkins = old_hopkins_warn)
#' 
#' @export
get_clust_tendency <- function(data, n, graph = TRUE,
                               gradient = list(low = "red", mid = "white", high = "blue"),
                               seed = NULL) 
{
  if (is.data.frame(data)) 
    data <- as.matrix(data)
  if (!(is.matrix(data))) 
    stop("data must be data.frame or matrix")
  if (!is.numeric(data))
    stop("data must contain numeric values only")
  if (!is.numeric(n) || length(n) != 1L || is.na(n) || n < 1 || n %% 1 != 0)
    stop("n must be a positive integer")

  data <- na.omit(data)
  if (nrow(data) < 2)
    stop("data must contain at least two complete rows")
  if (n >= nrow(data))
    stop("n must be no larger than num of samples")
  rownames(data) <- paste0("r", seq_len(nrow(data)))

  if (!is.null(seed)) {
    has_seed <- exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)
    if (has_seed) old_seed <- get(".Random.seed", envir = .GlobalEnv)
    on.exit({
      if (!has_seed && exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)) {
        rm(".Random.seed", envir = .GlobalEnv)
      } else if (has_seed) {
        assign(".Random.seed", old_seed, envir = .GlobalEnv)
      }
    }, add = TRUE)
    set.seed(seed)
  }

  if (isTRUE(getOption("factoextra.warn_hopkins", TRUE)) &&
      !isTRUE(.factoextra_state$hopkins_warned)) {
    warning(
      "Hopkins statistic uses the corrected formula (Wright 2022); results ",
      "differ from legacy factoextra. Set options(factoextra.warn_hopkins = FALSE) ",
      "to silence this warning.",
      call. = FALSE
    )
    .factoextra_state$hopkins_warned <- TRUE
  }
  plot <- NULL
  if(graph){
    plot <- fviz_dist(stats::dist(data), order = TRUE, 
                      show_labels = FALSE, gradient = gradient)
  }
  
  # Hopkins statistic
  # Sample synthetic points uniformly within the observed ranges.
  mins <- vapply(seq_len(ncol(data)), function(j) min(data[, j]), numeric(1))
  maxs <- vapply(seq_len(ncol(data)), function(j) max(data[, j]), numeric(1))
  p <- vapply(seq_len(ncol(data)), function(j) {
    runif(n, min = mins[j], max = maxs[j])
  }, numeric(n))

  # Use sample() for uniform row selection (fixes biased sampling from PR #133).
  k <- sample(seq_len(nrow(data)), n, replace = TRUE)
  q <- data[k, , drop = FALSE]

  max_matrix_cells <- getOption("factoextra.hopkins.max_matrix_cells", 2e7)
  if(!is.numeric(max_matrix_cells) || length(max_matrix_cells) != 1L ||
     is.na(max_matrix_cells) || max_matrix_cells <= 0){
    max_matrix_cells <- 2e7
  }

  use_vectorized <- (nrow(p) * nrow(data) <= max_matrix_cells) &&
                    (nrow(q) * nrow(data) <= max_matrix_cells)

  euclid_sq <- function(a, b){
    a_sq <- rowSums(a^2)
    b_sq <- rowSums(b^2)
    # Squared Euclidean distances between all rows of a and b.
    pmax(outer(a_sq, b_sq, "+") - 2 * tcrossprod(a, b), 0)
  }

  min_sq_dist <- function(a, b, exclude_identical = FALSE){
    if(use_vectorized){
      d2 <- euclid_sq(a, b)
      if(exclude_identical) d2[d2 <= .Machine$double.eps] <- Inf
      mins <- apply(d2, 1, min)
      mins[!is.finite(mins)] <- 0
      return(mins)
    }

    # Chunked fallback keeps memory bounded while preserving vectorized math.
    chunk_size <- floor(max_matrix_cells / nrow(b))
    if(!is.finite(chunk_size) || chunk_size < 1) chunk_size <- 1
    chunk_size <- as.integer(chunk_size)
    mins <- numeric(nrow(a))
    starts <- seq.int(1L, nrow(a), by = chunk_size)
    for(start in starts){
      end <- min(start + chunk_size - 1L, nrow(a))
      idx <- start:end
      d2 <- euclid_sq(a[idx, , drop = FALSE], b)
      if(exclude_identical) d2[d2 <= .Machine$double.eps] <- Inf
      block_mins <- apply(d2, 1, min)
      block_mins[!is.finite(block_mins)] <- 0
      mins[idx] <- block_mins
    }
    mins
  }

  minp <- sqrt(min_sq_dist(p, data, exclude_identical = FALSE))
  minq <- sqrt(min_sq_dist(q, data, exclude_identical = TRUE))
  
  # Hopkins statistic formula fix: use exponent d=D (dimensionality) as per

  # Cross & Jain (1982) "Measurement of Clustering Tendency" and
  # Wright (2022) "Will the Real Hopkins Statistic Please Stand Up?" R Journal.
  # Previous implementation incorrectly used d=1; correct formula uses d=ncol(data).
  d <- ncol(data)
  list(hopkins_stat = sum(minp^d)/(sum(minp^d) + sum(minq^d)), plot = plot)
}

#' @include print.factoextra.R
NULL
#' Extract the results for rows/columns - CA
#' 
#' @description
#' Extract all the results (coordinates, squared cosine, contributions and inertia) 
#' for the active row/column variables from Correspondence Analysis (CA) outputs.\cr\cr
#' \itemize{
#' \item get_ca(): Extract the results for rows and columns
#' \item get_ca_row(): Extract the results for rows only
#' \item get_ca_col(): Extract the results for columns only
#' }
#' @param res.ca an object of class CA [FactoMineR], ca [ca], coa [ade4];
#'  correspondence [MASS].
#' @param element the element to subset from the output. Possible values are "row" or "col".
#' @return a list of matrices containing the results for the active rows/columns including : 
#' \item{coord}{coordinates for the rows/columns}
#' \item{cos2}{cos2 for the rows/columns}
#' \item{contrib}{contributions of the rows/columns}
#' \item{inertia}{inertia of the rows/columns}
#' @author Alboukadel Kassambara \email{alboukadel.kassambara@@gmail.com}
#' @references \url{https://www.sthda.com/english/}
#' @examples
#' \donttest{
#' # Install and load FactoMineR to compute CA
#' # install.packages("FactoMineR")
#'  library("FactoMineR")
#'  data("housetasks")
#'  res.ca <- CA(housetasks, graph = FALSE)
#'  
#' # Result for column variables
#'  col <- get_ca_col(res.ca)
#'  col # print
#'  head(col$coord) # column coordinates
#'  head(col$cos2) # column cos2
#'  head(col$contrib) # column contributions
#'  
#' # Result for row variables
#'  row <- get_ca_row(res.ca)
#'  row # print
#'  head(row$coord) # row coordinates
#'  head(row$cos2) # row cos2
#'  head(row$contrib) # row contributions
#'  
#'  # You can also use the function get_ca()
#'  get_ca(res.ca, "row") # Results for rows
#'  get_ca(res.ca, "col") # Results for columns
#'  }
#' @name get_ca
#' 
#' @rdname get_ca
#' @export 
get_ca <- function(res.ca, element = c("row", "col")){
 elmt <- match.arg(element)
 if(elmt =="row") get_ca_row(res.ca)
 else if(elmt == "col") get_ca_col(res.ca)
}


#' @rdname get_ca
#' @export
get_ca_col <- function(res.ca){
  # FactoMineR package
  if(inherits(res.ca, c("CA"))) cols <- res.ca$col
  
  # ca package
  else if(inherits(res.ca, "ca")){
    # principal coord = standard coord X sqrt(eig)
    coord <- sweep(res.ca$colcoord, 2, res.ca$sv, "*")
    cos2 <- sweep(coord^2, 1, res.ca$coldist^2, "/")
    # col.contrib <- res.ca$colmass * col.coord^2/res.ca$sv^2
    cc <- sweep(coord^2, 1, res.ca$colmass, "*")
    contrib <- sweep(cc, 2, res.ca$sv^2, "/") * 100
    inertia <- res.ca$colinertia
    colnames(coord) <- colnames(cos2) <- colnames(contrib) <- paste0("Dim.", seq_len(ncol(coord))) 
    # remove supplementary points
    index <- setdiff(seq_len(nrow(res.ca$colcoord)), res.ca$colsup)
    cols <- list(coord = coord[index, , drop = FALSE], 
                 contrib = contrib[index, , drop = FALSE],
                 cos2 = cos2[index, , drop = FALSE], inertia = inertia[index]) 
  }
  # Mass package
  else if(inherits(res.ca, "correspondence")){
    # principal coord = standard coord X sqrt(eig)
    coord <- sweep(res.ca$cscore, 2, res.ca$cor, "*")
    # cos2 = coord^2/d^2
    row.sum <- rowSums(res.ca$Freq)
    col.sum <- colSums(res.ca$Freq)
    n <- sum(res.ca$Freq)
    profile <- sweep(res.ca$Freq, 2, col.sum, "/")
    average.profile <- row.sum/n
    centered <- sweep(profile, 1, average.profile, "-")
    d2 <- colSums((centered^2) / average.profile)
    cos2 <- sweep(coord^2, 1, d2, "/")
    # contrib <- mass * coord^2/eig
    mass <- col.sum/n
    cc <- sweep(coord^2, 1, mass, "*")
    contrib <- sweep(cc, 2, res.ca$cor^2, "/") * 100
    # inertia = mass * d^2
    inertia <- mass * d2
    colnames(coord) <- colnames(cos2) <- colnames(contrib) <- paste0("Dim.", seq_len(ncol(coord))) 
    cols <- list(coord = coord, contrib = contrib, cos2 = cos2, inertia = inertia)
  }
  # ade4 package
  else if(inherits(res.ca, "coa") && inherits(res.ca, "dudi")){
    if (!requireNamespace("ade4", quietly = TRUE)) {
      stop("ade4 package needed for this function to work. Please install it.")
    }
    coord <- res.ca$co
    inertia <- ade4::inertia.dudi(res.ca, row.inertia = FALSE, col.inertia = TRUE)
    vv <- as.character(utils::packageVersion("ade4"))
    cc <- utils::compareVersion(vv, "1.7.4") > 0
    if(cc){
      # "v>1.7.4"
      cos2 <- abs(inertia$col.rel/100)[, seq_len(ncol(coord))]
      contrib <- (inertia$col.abs)[, seq_len(ncol(coord))]
    }
    # v<=1.7.4
    else {
    cos2 <- abs(inertia$col.rel/10000)[, seq_len(ncol(coord))]
    contrib <- (inertia$col.abs/100)[, seq_len(ncol(coord))]
    }
    colnames(coord) <- colnames(cos2) <- colnames(contrib) <- paste0("Dim.", seq_len(ncol(coord))) 
    cols <- list(coord = coord, contrib = contrib, cos2 = cos2, inertia = NA)
  }
  # ExPosition package
  else if (inherits(res.ca, "expoOutput") && inherits(res.ca$ExPosition.Data, "epCA")) {
    coord <- res.ca$ExPosition.Data$fj
    inertia <- res.ca$ExPosition.Data$dj*res.ca$ExPosition.Data$W
    cos2 <- res.ca$ExPosition.Data$rj
    contrib <- res.ca$ExPosition.Data$cj*100
    colnames(coord) <- colnames(cos2) <- colnames(contrib) <- paste0("Dim.", 
                                                                     seq_len(ncol(coord)))
    cols <- list(coord = coord, contrib = contrib, cos2 = cos2, 
                 inertia = inertia)
  }
  
  else stop("An object of class : ", paste(class(res.ca), collapse = ", "), 
            " can't be handled by the function get_ca_col()")
  class(cols)<-c("factoextra", "ca_col")
  return(cols)
}

#' @rdname get_ca
#' @export
get_ca_row <- function(res.ca){
  
  # FactoMineR package
  if(inherits(res.ca, c("CA"))) row <- res.ca$row
  
  # ca package
  else if(inherits(res.ca, "ca")){
    # principal coord = standard coord X sqrt(eig)
    coord <- sweep(res.ca$rowcoord, 2, res.ca$sv, "*")
    cos2 <- sweep(coord^2, 1, res.ca$rowdist^2, "/")
    # contrib <- res.ca$rowmass * coord^2/res.ca$sv^2
    cc <- sweep(coord^2, 1, res.ca$rowmass, "*")
    contrib <- sweep(cc, 2, res.ca$sv^2, "/") * 100
    inertia <- res.ca$rowinertia
    colnames(coord) <- colnames(cos2) <- colnames(contrib) <- paste0("Dim.", seq_len(ncol(coord))) 
    # remove supplementary points
    index <- setdiff(seq_len(nrow(res.ca$rowcoord)), res.ca$rowsup)
    row <- list(coord = coord[index, , drop = FALSE], 
                contrib = contrib[index, , drop = FALSE],
                cos2 = cos2[index, , drop = FALSE], inertia = inertia[index])  
  }
  # Mass package
  else if(inherits(res.ca, "correspondence")){
    # principal coord = standard coord X sqrt(eig)
    coord <- sweep(res.ca$rscore, 2, res.ca$cor, "*")
    # cos2 = coord^2/d^2
    row.sum <- rowSums(res.ca$Freq)
    col.sum <- colSums(res.ca$Freq)
    n <- sum(res.ca$Freq)
    profile <- sweep(res.ca$Freq, 1, row.sum, "/")
    average.profile <- col.sum/n
    centered <- sweep(profile, 2, average.profile, "-")
    d2 <- rowSums(sweep(centered^2, 2, average.profile, "/"))
    cos2 <- sweep(coord^2, 1, d2, "/")
    # contrib <- mass * coord^2/eig
    mass <- row.sum/n
    cc <- sweep(coord^2, 1, mass, "*")
    contrib <- sweep(cc, 2, res.ca$cor^2, "/") * 100
    # inertia = mass * d^2
    inertia <- mass * d2
    colnames(coord) <- colnames(cos2) <- colnames(contrib) <- paste0("Dim.", seq_len(ncol(coord))) 
    row <- list(coord = coord, contrib = contrib, cos2 = cos2, inertia = inertia)
  }
  
  # ade4 package
  else if(inherits(res.ca, "coa") && inherits(res.ca, "dudi")){
    if (!requireNamespace("ade4", quietly = TRUE)) {
      stop("ade4 package needed for this function to work. Please install it.")
    }
    coord <- res.ca$li
    inertia <- ade4::inertia.dudi(res.ca, row.inertia = TRUE, col.inertia = FALSE)
    vv <- as.character(utils::packageVersion("ade4"))
    cc <- utils::compareVersion(vv, "1.7.4") > 0
    if(cc){
      # "v>1.7.4"
      cos2 <- abs(inertia$row.rel/100)[, seq_len(ncol(coord))]
      contrib <- (inertia$row.abs)[, seq_len(ncol(coord))]
    }
    # v<=1.7.4
    else {
      cos2 <- abs(inertia$row.rel/10000)[, seq_len(ncol(coord))]
      contrib <- (inertia$row.abs/100)[, seq_len(ncol(coord))]
    }
    colnames(coord) <- colnames(cos2) <- colnames(contrib) <- paste0("Dim.", seq_len(ncol(coord))) 
    row <- list(coord = coord, contrib = contrib, cos2 = cos2, inertia = NA)
  }
  # ExPosition package
  else if (inherits(res.ca, "expoOutput") && inherits(res.ca$ExPosition.Data, "epCA")) {
    coord <- res.ca$ExPosition.Data$fi
    inertia <- res.ca$ExPosition.Data$di*res.ca$ExPosition.Data$M
    cos2 <- res.ca$ExPosition.Data$ri
    contrib <- res.ca$ExPosition.Data$ci*100
    colnames(coord) <- colnames(cos2) <- colnames(contrib) <- paste0("Dim.", 
                                                                     seq_len(ncol(coord)))
    row <- list(coord = coord, contrib = contrib, cos2 = cos2, 
                inertia = inertia)
  }
  else stop("An object of class : ", paste(class(res.ca), collapse = ", "), 
            " can't be handled by the function get_ca_row()")
  class(row)<-c("factoextra", "ca_row")
  return(row)
}

test_that("CA extractors work with ca package outputs", {
  skip_if_not_installed("ca")

  data("housetasks", package = "factoextra")
  res <- ca::ca(housetasks)

  cols <- get_ca_col(res)
  rows <- get_ca_row(res)

  expect_s3_class(cols, "factoextra")
  expect_s3_class(rows, "factoextra")
  expect_true(all(c("coord", "cos2", "contrib") %in% names(cols)))
  expect_true(all(c("coord", "cos2", "contrib") %in% names(rows)))
  expect_equal(nrow(cols$coord), ncol(housetasks))
  expect_equal(nrow(rows$coord), nrow(housetasks))
})

test_that("CA extractors work with MASS correspondence outputs", {
  skip_if_not_installed("MASS")

  data("housetasks", package = "factoextra")
  res <- MASS::corresp(housetasks, nf = 2)

  cols <- get_ca_col(res)
  rows <- get_ca_row(res)

  expect_s3_class(cols, "factoextra")
  expect_s3_class(rows, "factoextra")
  expect_equal(ncol(cols$coord), 2)
  expect_equal(ncol(rows$coord), 2)
  expect_false(any(is.na(cols$cos2)))
  expect_false(any(is.na(rows$cos2)))
})

#' @include eigenvalue.R get_pca.R hcut.R
 NULL
#'Visualize Clustering Results
#'@description Provides ggplot2-based elegant visualization of partitioning
#'  methods including kmeans [stats package]; pam, clara and fanny [cluster
#'  package]; dbscan [fpc package]; Mclust [mclust package]; HCPC [FactoMineR];
#'  hkmeans [factoextra]. Observations are represented by points in the plot,
#'  using principal components if ncol(data) > 2. An ellipse is drawn around
#'  each cluster. When \code{stand = TRUE}, the plotting data must remain
#'  finite after scaling.
#'@param object an object of class "partition" created by the functions pam(),
#'  clara() or fanny() in cluster package; "kmeans" [in stats package]; "dbscan"
#'  [in fpc package]; "Mclust" [in mclust]; "hkmeans", "eclust" [in factoextra].
#'  Possible value are also any list object with data and cluster components
#'  (e.g.: object = list(data = mydata, cluster = myclust)).
#'@param data the data that has been used for clustering. Required only when
#'  object is a class of kmeans or dbscan.
#'@param choose.vars a character vector containing variables to be considered
#'  for plotting.
#'@param stand logical value; if TRUE, data is standardized before principal
#'  component analysis. If scaling produces \code{NA} values,
#'  \code{fviz_cluster()} stops with a package-level error.
#'@param axes a numeric vector of length 2 specifying the dimensions to be
#'  plotted.
#'@param geom a text specifying the geometry to be used for the graph. Allowed
#'  values are the combination of c("point", "text"). Use "point" (to show only
#'  points);  "text" to show only labels; c("point", "text") to show both types.
#'@param repel a boolean, whether to use ggrepel to avoid overplotting text
#'  labels or not. The old \code{jitter} argument is kept for backward
#'  compatibility and is converted to \code{repel = TRUE} with a deprecation warning.
#'@param show.clust.cent logical; if TRUE, shows cluster centers
#'@param ellipse logical value; if TRUE, draws outline around points of each
#'  cluster
#'@param ellipse.type Character specifying frame type. Possible values are
#'  'convex', 'confidence' or types supported by
#'  \code{\link[ggplot2]{stat_ellipse}} including one of c("t", "norm",
#'  "euclid").
#'@param ellipse.level the size of the concentration ellipse in normal
#'  probability. Passed for \code{ggplot2::stat_ellipse} 's level. Ignored in
#'  'convex'. Default value is 0.95.
#'@param ellipse.alpha Alpha for frame specifying the transparency level of fill
#'  color. Use alpha = 0 for no fill color.
#'@param labelsize font size for the labels
#'@param shape the shape of points.
#'@param pointsize the size of points
#'@param outlier.pointsize,outlier.color,outlier.shape,outlier.labelsize
#'  arguments for customizing outliers, which can be detected only in DBSCAN
#'  clustering.
#'@param main plot main title.
#'@param xlab,ylab character vector specifying x and y axis labels,
#'  respectively. Use xlab = FALSE and ylab = FALSE to hide xlab and ylab,
#'  respectively.
#'@inheritParams ggpubr::ggpar
#'@param ... other arguments to be passed to the functions
#'  \code{\link[ggpubr]{ggscatter}} and \code{\link[ggpubr]{ggpar}}.
#'
#'@return a ggplot2 object.
#'@author Alboukadel Kassambara \email{alboukadel.kassambara@@gmail.com}
#'@seealso \code{\link{fviz_silhouette}}, \code{\link{hcut}},
#'  \code{\link{hkmeans}},  \code{\link{eclust}}, \code{\link{fviz_dend}}
#' @examples
#' set.seed(123)
#'
#' # Data preparation
#' # +++++++++++++++
#' data("iris")
#' head(iris)
#' # Remove species column (5) and scale the data
#' iris.scaled <- scale(iris[, -5])
#'
#' # K-means clustering
#' # +++++++++++++++++++++
#' km.res <- kmeans(iris.scaled, 3, nstart = 10)
#'
#' # Visualize kmeans clustering
#' # use repel = TRUE to avoid overplotting
#' fviz_cluster(km.res, iris[, -5], ellipse.type = "norm")
#'
#'
#'# Change the color palette and theme
#'fviz_cluster(km.res, iris[, -5],
#'    palette = "Set2", ggtheme = theme_minimal())
#'
#'  \dontrun{
#' # Show points only
#' fviz_cluster(km.res, iris[, -5], geom = "point")
#' # Show text only
#' fviz_cluster(km.res, iris[, -5], geom = "text")
#'
#' # PAM clustering
#' # ++++++++++++++++++++
#' requireNamespace("cluster", quietly = TRUE)
#' pam.res <- pam(iris.scaled, 3)
#'  # Visualize pam clustering
#' fviz_cluster(pam.res, geom = "point", ellipse.type = "norm")
#'
#' # Hierarchical clustering
#' # ++++++++++++++++++++++++
#' # Use hcut() which compute hclust and cut the tree
#' hc.cut <- hcut(iris.scaled, k = 3, hc_method = "complete")
#' # Visualize dendrogram
#' fviz_dend(hc.cut, show_labels = FALSE, rect = TRUE)
#' # Visualize cluster
#' fviz_cluster(hc.cut, ellipse.type = "convex")
#'
#' }
#'
#'
#'
#'@name fviz_cluster
#'@rdname fviz_cluster
#'@export
fviz_cluster <- function(object, data = NULL, choose.vars = NULL, stand = TRUE, 
                         axes = c(1, 2),
                         geom = c("point", "text"), repel = FALSE,
                         show.clust.cent = TRUE,
                         ellipse = TRUE, ellipse.type = "convex", ellipse.level = 0.95,
                         ellipse.alpha = 0.2,
                         shape = NULL, pointsize = 1.5, labelsize = 12,
                         main = "Cluster plot",  xlab = NULL, ylab = NULL,
                         outlier.color = "black", outlier.shape = 19,
                         outlier.pointsize = pointsize, outlier.labelsize = labelsize,
                         ggtheme = theme_grey(), ...){
  
  # Backward compatibility: deprecated arguments converted with warning
  extra_args <- list(...)
  jitter_arg <- extra_args[["jitter"]]
  frame_arg <- extra_args[["frame"]]
  frame_type_arg <- extra_args[["frame.type"]]
  frame_level_arg <- extra_args[["frame.level"]]
  frame_alpha_arg <- extra_args[["frame.alpha"]]
  title_arg <- extra_args[["title"]]
  .check_axes(axes, .length = 2)

  # jitter -> repel (converted with warning)
  if (!is.null(jitter_arg)) {
    .facto_dep("jitter", "repel = TRUE", NULL)
    if(is.list(jitter_arg)) {
      if(!is.null(jitter_arg$width) || !is.null(jitter_arg$height)) repel <- TRUE
    } else if(is.logical(jitter_arg) && length(jitter_arg) == 1L && !is.na(jitter_arg) && jitter_arg) {
      repel <- TRUE
    }
  }

  # frame -> ellipse (converted with warning)
  if(!is.null(frame_arg)) {
    .facto_dep("frame", "ellipse", NULL)
    ellipse <- frame_arg
  }
  if(!is.null(frame_type_arg)) {
    .facto_dep("frame.type", "ellipse.type", NULL)
    ellipse.type <- frame_type_arg
  }
  if(!is.null(frame_level_arg)) {
    .facto_dep("frame.level", "ellipse.level", NULL)
    ellipse.level <- frame_level_arg
  }
  if(!is.null(frame_alpha_arg)) {
    .facto_dep("frame.alpha", "ellipse.alpha", NULL)
    ellipse.alpha <- frame_alpha_arg
  }
  if(!is.null(title_arg)) {
    .facto_dep("title", "main", NULL)
    main <- title_arg
  }
  extra_args <- extra_args[setdiff(names(extra_args),
                                   c("jitter", "frame", "frame.type", "frame.level", "frame.alpha", "title"))]
  
  
  # object from cluster package
  if(inherits(object, c("partition", "hkmeans", "eclust"))){
    # Use the object's own data when present; otherwise keep the user-supplied
    # `data=`. pam()/fanny() fitted on a dissimilarity matrix store no
    # coordinates (object$data is NULL), so the original data is needed for the
    # 2-D layout; the cluster assignments still come from the object (#128).
    if(!is.null(object$data)) data <- object$data
    else if(!is.null(data)){
      # Align the object's clustering to the supplied data's rows by name, so
      # points are not silently mis-coloured if `data` is ordered differently
      # from the dissimilarity. Error on a genuine set mismatch.
      cl <- object$clustering
      if(!is.null(cl) && !is.null(names(cl)) && !is.null(rownames(data))){
        if(!setequal(names(cl), rownames(data)))
          stop("The row names of `data` do not match the observations used to ",
               "build the clustering. Pass the same data used for the ",
               "dissimilarity matrix.", call. = FALSE)
        object$clustering <- cl[rownames(data)]
      }
    }
    if(is.null(data))
      stop("This clustering has no stored data (e.g. pam()/fanny() on a ",
           "dissimilarity matrix). Supply the original data via 'data=' - it is ",
           "used only for the 2-D layout; the cluster assignments come from the object.",
           call. = FALSE)
  }
  # Object from kmeans (stats package)
  else if((inherits(object, "kmeans") && !inherits(object, "eclust")) || inherits(object, "dbscan")){
    if(is.null(data)) stop("data is required for plotting kmeans/dbscan clusters")
  } 
  # Object from mclust package
  else if(inherits(object, "Mclust")) {
    object$cluster <- object$classification
    data <- object$data
  }
  # HCPC in FactoMineR
  else if(inherits(object, "HCPC")) {
    object$cluster <- object$call$X$clust
    data <- res.hcpc <- object
    stand <- FALSE # to avoid trying to standardize HCPC results
#     data <- object$data.clust[, -ncol(object$data.clust), drop = FALSE]
#     object$cluster <- as.vector(object$data.clust$clust)
  }
  else if(inherits(object, "hcut")){
    if(inherits(object$data, "dist")){
      if(is.null(data)) stop("The option 'data' is required for an object of class hcut." )
    }
    else data <- object$data
  }
  # Any obects containing data and cluster elements
  else if(!is.null(object$data) && !is.null(object$cluster)){
    data <- object$data
    cluster <- object$cluster
  }
  else stop("Can't handle an object of class ", paste(class(object), collapse = ", "))
  
  # Choose variables
  if(!is.null(choose.vars))
    data <- data[, choose.vars, drop = FALSE]
  if(stand) {
    data <- scale(data)
    if(anyNA(data))
      stop("Scaling produced NA values. Check for constant columns or non-finite values.")
  }
  cluster <- as.factor(object$cluster)
  
  pca_performed <- FALSE
  
  # Prepare the data for plotting
  # ++++++++++++++++++++++++
  # PCA is performed depending on the number of variables
  if(inherits(data, c("matrix", "data.frame"))){
    # ncol(data) > 2 --> PCA
    if(ncol(data)>2){
    pca <- stats::prcomp(data, scale = FALSE, center = FALSE)
    ind <- facto_summarize(pca, element = "ind", result = "coord", axes = axes)
    eig <- get_eigenvalue(pca)[axes,2]
    if(is.null(xlab)) xlab = paste0("Dim", axes[1], " (", round(eig[1],1), "%)") 
    if(is.null(ylab)) ylab = paste0("Dim", axes[2], " (", round(eig[2], 1),"%)")
    }
    # PCA is not performed
    else if(ncol(data) == 2){
      ind <- as.data.frame(data)
      ind <- cbind.data.frame(name = rownames(ind), ind)
      if(is.null(xlab)) xlab <- colnames(data)[1]
      if(is.null(ylab)) ylab <- colnames(data)[2]
      
      if(xlab=="x") xlab <- "x value"
      if(ylab == "y") ylab <- "y value"
    }
    else{
      stop("The dimension of the data < 2! No plot.")
    }
    colnames(ind)[2:3] <-  c("x", "y")
    label_coord <- ind
  }
  else if(inherits(data, "HCPC")){
    ind <- res.hcpc$call$X[, c(axes, ncol(res.hcpc$call$X))]
    colnames(ind) <- c("Dim.1", "Dim.2", "clust")
    ind <- cbind.data.frame(name = rownames(ind), ind)
    colnames(ind)[2:3] <-  c("x", "y")
    label_coord <- ind
    eig <- get_eigenvalue(res.hcpc$call$t$res)[axes,2]
    if(is.null(xlab)) xlab = paste0("Dim", axes[1], " (", round(eig[1], 1), "%)") 
    if(is.null(ylab)) ylab = paste0("Dim", axes[2], " (", round(eig[2], 1),"%)")
  }
  else stop("A data of class ", paste(class(data), collapse = ", "), " is not supported.")
  
  # Plot data and labels
  # ++++++++++++++++++++++++
  label = FALSE
  if("text" %in% geom) label = TRUE
  if(!("point" %in% geom)) pointsize = 0
  
  plot.data <- cbind.data.frame(ind, cluster = cluster)
  label_coord <- cbind.data.frame(label_coord, cluster = cluster)
  # Augment data
  if(inherits(object, "Mclust")){
    plot.data$uncertainty <- object$uncertainty
    label_coord$uncertainty <- object$uncertainty
  }
  
  # IF DBSCAN: cluster 0 is outliers. We don't want to make ellipse around
  # these observations. Let's remove them. They will be added to the plot later
  is_outliers = FALSE
  if(inherits(object, c("dbscan", "Mclust"))){
    outliers <- which(cluster == 0)
    if(length(outliers) > 0){
      is_outliers = TRUE
      outliers_data <- plot.data[outliers, , drop = FALSE]
      outliers_labs <- label_coord[outliers, , drop = FALSE]
      
      ind <- ind[-outliers, , drop = FALSE]
      cluster <- cluster[-outliers]
      plot.data <- plot.data[-outliers, , drop = FALSE]
      label_coord <- label_coord[-outliers, , drop = FALSE]
    }
  }
  
  # Plot
  # ++++++++++++++++++++++++
  lab <- NULL
  if("text" %in% geom) lab <- "name"
  if(is.null(shape)) shape <- "cluster"
  
  if(inherits(object, "partition") && missing(show.clust.cent))
    show.clust.cent <- FALSE # hide mean point for PAM, CLARA
  
  ggscatter_args <- c(
    list(
      data = plot.data, x = "x", y = "y",
      color = "cluster", shape = shape, size = pointsize,
      point = "point" %in% geom,
      label = lab,
      font.label = labelsize, repel = repel,
      mean.point = show.clust.cent,
      ellipse = ellipse, ellipse.type = ellipse.type,
      ellipse.alpha = ellipse.alpha, ellipse.level = ellipse.level,
      main = main, xlab = xlab, ylab = ylab,
      ggtheme = ggtheme
    ),
    extra_args
  )
  p <- do.call(ggpubr::ggscatter, ggscatter_args)
  
  # Add outliers (can exist only in dbscan)
  if(is_outliers)
    p <- .add_outliers(p, outliers_data, outliers_labs, outlier.color, outlier.shape,
                  outlier.pointsize, outlier.labelsize/3, geom, repel = repel)

  # Keep the point labels out of the legend (no stray "a" glyph); the cluster
  # colour/shape legend comes from the point layer. Mirrors .fviz_finish() (#14).
  p <- .hide_text_legend(p)
  p
}



# Add outliers to cluster plot (for dbscan only)
.add_outliers <-function(p, outliers_data, outliers_labs, 
                         outlier.color = "black", outlier.shape = 19,
                         pointsize = 2, labelsize = 4, geom = c("point", "text"), repel = FALSE)
  {
  
  # FIX: ggplot2 3.0.0+ deprecation - aes_string() replaced with aes() + .data pronoun
  # See: https://github.com/kassambara/factoextra/issues/190
  if("point" %in% geom)
    p <-  p + geom_point(data = outliers_data,
                      aes(x = .data[["x"]], y = .data[["y"]]),
                      size = pointsize, color = outlier.color, shape = outlier.shape)
  if("text" %in% geom){
    if(repel)
      p <- p +ggrepel::geom_text_repel(data = outliers_labs,
                                       aes(x = .data[["x"]], y = .data[["y"]], label = .data[["name"]]),
                                       size = labelsize, color = outlier.color)
    else
      p <- p + geom_text(data = outliers_labs,
                         aes(x = .data[["x"]], y = .data[["y"]], label = .data[["name"]]),
                         size = labelsize, vjust = -0.7, color = outlier.color)
  }

  return(p)
}

#' @include utilities.R get_mca.R
NULL
#'Visualize Multiple Correspondence Analysis
#'
#'@description Multiple Correspondence Analysis (MCA) is an extension of simple 
#'  CA to analyse a data table containing more than two categorical variables. 
#'  fviz_mca() provides ggplot2-based elegant visualization of MCA outputs from 
#'  the R functions: MCA [in FactoMineR], acm [in ade4], and expOutput/epMCA [in
#'  ExPosition]. Read more: 
#'  \href{https://www.sthda.com/english/articles/31-principal-component-methods-in-r-practical-guide/114-mca-multiple-correspondence-analysis-in-r-essentials/}{Multiple
#'   Correspondence Analysis Essentials.}
#'  
#'  \itemize{ \item fviz_mca_ind(): Graph of individuals \item fviz_mca_var():
#'  Graph of variables \item fviz_mca_biplot(): Biplot of individuals and
#'  variables \item fviz_mca(): An alias of fviz_mca_biplot() }
#'@param X an object of class MCA [FactoMineR], acm [ade4] and expOutput/epMCA
#'  [ExPosition].
#'@inheritParams fviz_pca
#' @param geom.ind,geom.var as \code{geom} but for individuals and variables,
#'   respectively. Default is geom.ind = c("point", "text), geom.var =
#'   c("point", "text").
#'@param label a text specifying the elements to be labelled. Default value is 
#'  "all". Allowed values are "none" or the combination of c("ind", 
#'  "ind.sup","var", "quali.sup",  "quanti.sup"). "ind" can be used to label 
#'  only active individuals. "ind.sup" is for supplementary individuals. "var" 
#'  is for active variable categories. "quali.sup" is for supplementary 
#'  qualitative variable categories. "quanti.sup" is for quantitative 
#'  supplementary variables.
#'@param invisible a text specifying the elements to be hidden on the plot. 
#'  Default value is "none". Allowed values are the combination of c("ind", 
#'  "ind.sup","var", "quali.sup",  "quanti.sup").
#'@param habillage an optional factor variable for coloring the observations by 
#'  groups. Default value is "none". If X is an MCA object from FactoMineR 
#'  package, habillage can also specify the index of the factor variable in the 
#'  data.
#'@param col.ind,col.var color for individuals and variables, respectively. Can 
#'  be a continuous variable or a factor variable. Possible values include also
#'  : "cos2", "contrib", "coord", "x" or "y". In this case, the colors for
#'  individuals/variables are automatically controlled by their qualities
#'  ("cos2"), contributions ("contrib"), coordinates (x^2 + y^2 , "coord"), x
#'  values("x") or y values("y"). To use automatic coloring (by cos2, contrib,
#'  ....), make sure that habillage ="none".
#'@param alpha.ind,alpha.var controls the transparency of individual and 
#'  variable colors, respectively. The value can variate from 0 (total 
#'  transparency) to 1 (no transparency). Default value is 1. Possible values 
#'  include also : "cos2", "contrib", "coord", "x" or "y". In this case, the 
#'  transparency for individual/variable colors are automatically controlled by 
#'  their qualities ("cos2"), contributions ("contrib"), coordinates (x^2 + y^2 
#'  , "coord"), x values("x") or y values("y"). To use this, make sure that 
#'  habillage ="none".
#'@param shape.ind,shape.var point shapes of individuals and variables.
#'@param col.quanti.sup,col.quali.sup a color for the quantitative/qualitative 
#'  supplementary variables.
#'@param repel a boolean, whether to use ggrepel to avoid overplotting text
#'  labels or not. The old \code{jitter} argument is kept for backward
#'  compatibility and is converted to \code{repel = TRUE} with a deprecation warning.
#'@param choice the graph to plot. Allowed values include: i) "var" and 
#'  "mca.cor" for plotting the correlation between variables and principal 
#'  dimensions; ii) "var.cat" for variable categories and iii) "quanti.sup" for
#'  the supplementary quantitative variables.
#'@param title the title of the graph
#'@param select.ind,select.var a selection of individuals/variables to be drawn.
#'  Allowed values are NULL or a list containing the arguments name, cos2 or 
#'  contrib: \itemize{ \item name is a character vector containing 
#'  individuals/variables to be drawn \item cos2 if cos2 is in [0, 1], ex: 0.6, 
#'  then individuals/variables with a cos2 > 0.6 are drawn. if cos2 > 1, ex: 5, 
#'  then the top 5 individuals/variables with the highest cos2 are drawn. \item 
#'  contrib if contrib > 1, ex: 5,  then the top 5 individuals/variables with 
#'  the highest contrib are drawn }
#'@inheritParams ggpubr::ggpar
#'@inheritParams fviz
#'@param ... Additional arguments. \itemize{ \item in fviz_mca_ind(), 
#'  fviz_mca_var() and fviz_mca_cor(): Additional arguments are passed to the 
#'  functions fviz() and ggpubr::ggpar(). \item in fviz_mca_biplot() and 
#'  fviz_mca(): Additional arguments are passed to fviz_mca_ind() and 
#'  fviz_mca_var().}
#'@param map character string specifying the map type. Allowed options include: 
#'  "symmetric", "rowprincipal", "colprincipal", "symbiplot", "rowgab", 
#'  "colgab", "rowgreen" and "colgreen". See details
#'@param arrows Vector of two logicals specifying if the plot should contain 
#'  points (FALSE, default) or arrows (TRUE). First value sets the rows and the 
#'  second value sets the columns.
#'@details The default plot of MCA is a "symmetric" plot in which both rows and 
#'  columns are in principal coordinates. In this situation, it's not possible 
#'  to interpret the distance between row points and column points. To overcome 
#'  this problem, the simplest way is to make an asymmetric plot. The argument 
#'  "map" can be used to change the plot type. For more explanation, read the 
#'  details section of fviz_ca documentation.
#'  
#'@return a ggplot
#'@author Alboukadel Kassambara \email{alboukadel.kassambara@@gmail.com}
#'@seealso \code{\link{get_mca}}, \code{\link{fviz_pca}}, \code{\link{fviz_ca}},
#'  \code{\link{fviz_mfa}}, \code{\link{fviz_hmfa}}
#' @examples
#' # Multiple Correspondence Analysis
#' # ++++++++++++++++++++++++++++++
#' # Install and load FactoMineR to compute MCA
#' # install.packages("FactoMineR")
#' library("FactoMineR")
#' data(poison)
#' poison.active <- poison[1:55, 5:15]
#' head(poison.active)
#' res.mca <- MCA(poison.active, graph=FALSE)
#' 
#' # Graph of individuals
#' # +++++++++++++++++++++
#' 
#' # Default Plot
#' # Color of individuals: col.ind = "steelblue"
#' fviz_mca_ind(res.mca, col.ind = "steelblue")
#' 
#' # 1. Control automatically the color of individuals 
#'    # using the "cos2" or the contributions "contrib"
#'    # cos2 = the quality of the individuals on the factor map
#' # 2. To keep only point or text use geom = "point" or geom = "text".
#' # 3. Change themes: http://www.sthda.com/english/wiki/ggplot2-themes
#' 
#' fviz_mca_ind(res.mca, col.ind = "cos2", repel = TRUE)
#'
#' \dontrun{     
#' # You can also control the transparency 
#' # of the color by the cos2
#' fviz_mca_ind(res.mca, alpha.ind="cos2") 
#'}
#'      
#' # Color individuals by groups, add concentration ellipses
#' # Remove labels: label = "none".
#' grp <- as.factor(poison.active[, "Vomiting"])
#' p <- fviz_mca_ind(res.mca, label="none", habillage=grp,
#'        addEllipses=TRUE, ellipse.level=0.95)
#' print(p)
#'       
#'     
#' # Change group colors using RColorBrewer color palettes
#' # Read more: http://www.sthda.com/english/wiki/ggplot2-colors
#' p + scale_color_brewer(palette="Dark2") +
#'     scale_fill_brewer(palette="Dark2") 
#'      
#' # Change group colors manually
#' # Read more: http://www.sthda.com/english/wiki/ggplot2-colors
#' p + scale_color_manual(values=c("#999999", "#E69F00"))+
#'  scale_fill_manual(values=c("#999999", "#E69F00"))
#'              
#'              
#' # Select and visualize some individuals (ind) with select.ind argument.
#'  # - ind with cos2 >= 0.4: select.ind = list(cos2 = 0.4)
#'  # - Top 20 ind according to the cos2: select.ind = list(cos2 = 20)
#'  # - Top 20 contributing individuals: select.ind = list(contrib = 20)
#'  # - Select ind by names: select.ind = list(name = c("44", "38", "53",  "39") )
#'  
#' # Example: Select the top 40 according to the cos2
#' fviz_mca_ind(res.mca, select.ind = list(cos2 = 20))
#' 
#'  
#' # Graph of variable categories
#' # ++++++++++++++++++++++++++++
#' # Default plot: use repel = TRUE to avoid overplotting
#' fviz_mca_var(res.mca, col.var = "#FC4E07")
#' 
#' # Control variable colors using their contributions
#' # use repel = TRUE to avoid overplotting
#' fviz_mca_var(res.mca, col.var = "contrib",
#'   gradient.cols = c("#00AFBB", "#E7B800", "#FC4E07"))
#'         
#'     
#' # Biplot
#' # ++++++++++++++++++++++++++
#' grp <- as.factor(poison.active[, "Vomiting"])
#' fviz_mca_biplot(res.mca, repel = TRUE, col.var = "#E7B800",
#'  habillage = grp, addEllipses = TRUE, ellipse.level = 0.95)
#'  
#'  \dontrun{
#' # Keep only the labels for variable categories: 
#' fviz_mca_biplot(res.mca, label ="var")
#' 
#' # Keep only labels for individuals
#' fviz_mca_biplot(res.mca, label ="ind")
#' 
#' # Hide variable categories
#' fviz_mca_biplot(res.mca, invisible ="var")
#' 
#' # Hide individuals
#' fviz_mca_biplot(res.mca, invisible ="ind")
#' 
#'# Control automatically the color of individuals using the cos2
#' fviz_mca_biplot(res.mca, label ="var", col.ind="cos2")
#'        
#' # Change the color by groups, add ellipses
#' fviz_mca_biplot(res.mca, label="var", col.var ="blue",
#'    habillage=grp, addEllipses=TRUE, ellipse.level=0.95)
#'                
#' # Select the top 30 contributing individuals
#' # And the top 10 variables
#' fviz_mca_biplot(res.mca,  
#'                select.ind = list(contrib = 30),
#'                select.var = list(contrib = 10)) 
#' }
#' 
#'@name fviz_mca
#'@rdname fviz_mca
#'@export
fviz_mca_ind <- function(X,  axes = c(1,2), geom=c("point", "text"), geom.ind = geom, repel = FALSE,
                         habillage = "none", palette = NULL, addEllipses = FALSE, 
                         col.ind = "blue", col.ind.sup = "darkblue", alpha.ind = 1,
                         shape.ind = 19, map ="symmetric", 
                         select.ind = list(name = NULL, cos2 = NULL, contrib = NULL),
                         ...)
{
  
  fviz (X, element = "ind", axes = axes, geom = geom.ind, habillage = habillage, 
        addEllipses = addEllipses, palette = palette, pointshape = shape.ind,
        color = col.ind, alpha = alpha.ind,
        shape.sup = shape.ind, col.row.sup = col.ind.sup,
        select = select.ind,  map = map, repel = repel, ...)

  
}


#' @rdname fviz_mca
#' @export 
fviz_mca_var <- function(X, choice = c("var.cat", "mca.cor", "var", "quanti.sup"), 
                         axes=c(1,2), geom = c("point", "text"), geom.var = geom, repel = FALSE, 
                         col.var="red", alpha.var=1, shape.var = 17, col.quanti.sup = "blue", 
                         col.quali.sup = "darkgreen",  map = "symmetric", 
                         select.var = list(name = NULL, cos2 = NULL, contrib = NULL), ...)
{
  
  col.col.sup <- col.quanti.sup
  extra_args <- list(...)
  if(missing(choice) && !is.null(extra_args$choix))
    choice <- extra_args$choix
  
  # Define plot types
  choice <- match.arg(choice)
  if(choice == "var.cat") choice <- "var"
  else if(choice == "var") choice <- "mca.cor"
  # Define color for supplementary element
  if(choice == "mca.cor") col.col.sup <- col.quanti.sup 
  else if(choice == "var") col.col.sup <- col.quali.sup
  # Define geometry for supplementary quantitative variables
  if(choice == "quanti.sup") {
    if(missing(geom)) geom <- c("arrow", "text")
    if(missing(col.var)) col.var <- col.quanti.sup
  }
  fviz (X, element = choice, axes = axes, geom = geom.var,
        color = col.var, alpha = alpha.var,  pointshape = shape.var, 
        shape.sup = shape.var, col.col.sup = col.col.sup, 
        select = select.var, repel = repel,  map = map, ...)
}



#' @rdname fviz_mca
#' @export
fviz_mca_biplot <- function(X,  axes = c(1,2), geom = c("point", "text"),
                            geom.ind = geom, geom.var = geom,
                            repel = FALSE, label = "all", invisible="none",
                            habillage="none", addEllipses=FALSE, palette = NULL,
                            arrows = c(FALSE, FALSE), map ="symmetric", 
                            title = "MCA - Biplot", ...)
{
  
  # Individuals
  geom2 <- geom.ind
  if(arrows[1]==TRUE) geom2 <- setdiff(unique(c(geom2, "arrow")), "point")
  p <- fviz_mca_ind(X,  axes = axes, geom = geom2, repel = repel,
                    label = label, invisible=invisible, habillage = habillage,
                    addEllipses = addEllipses, palette = palette, map = map, ...)

  # Variable
  geom2 <- geom.var
  if(arrows[2]==TRUE) geom2 <- setdiff(unique(c(geom2, "arrow")), "point")
  # Add variables
  p <- fviz_mca_var(X, axes = axes, geom =  geom2, repel = repel,
                    label = label, invisible = invisible, ggp = p, map = map, ...)
  
  p+labs(title=title)
}

#' @rdname fviz_mca
#' @export
fviz_mca <- function(X, ...){
  fviz_mca_biplot(X, ...)
}

#' @include get_mfa.R
NULL
#' Visualize Factor Analysis of Mixed Data
#' 
#' @description Factor analysis of mixed data (FAMD) is, a particular case of 
#'   MFA, used to analyze a data set containing both quantitative and
#'   qualitative variables. fviz_famd() provides ggplot2-based elegant
#'   visualization of FAMD outputs from the R function: FAMD [FactoMineR].\cr\cr
#'   \itemize{ \item fviz_famd_ind(): Graph of individuals
#'   \item fviz_famd_var(): Graph of variables \item fviz_famd(): An alias of
#'   fviz_famd_ind(res.famd) }
#'   
#' @param X an object of class FAMD [FactoMineR].
#' @inheritParams fviz_mca
#' @inheritParams fviz_pca
#' @inheritParams fviz
#' @inheritParams ggpubr::ggpar
#' @param choice The graph to plot in fviz_famd_var(). Allowed values include
#'   one of c("var", "quanti.var", "quali.var", "quali.sup").
#' @param habillage an optional factor variable for coloring the observations by
#'   groups. Default value is "none". If X is an MFA object from FactoMineR 
#'   package, habillage can also specify the index of the factor variable in the
#'   data.
#' @param col.ind,col.var color for individuals and variables, respectively. Can
#'   be a continuous variable or a factor variable. Possible values include also
#'   : "cos2", "contrib", "coord", "x" or "y". In this case, the colors for
#'   individuals/variables are automatically controlled by their qualities
#'   ("cos2"), contributions ("contrib"), coordinates (x^2 + y^2 , "coord"), x
#'   values("x") or y values("y"). To use automatic coloring (by cos2, contrib,
#'   ....), make sure that habillage ="none".
#' @param col.var.sup color for supplementary variables.
#' @param col.quali.var color for qualitative variables in fviz_famd_ind(). 
#'   Default is "black".
#' @param alpha.ind,alpha.var controls the transparency of individuals and 
#'   variables, respectively. The value can variate from 0 (total transparency) 
#'   to 1 (no transparency). Default value is 1. Possible values include also : 
#'   "cos2", "contrib", "coord", "x" or "y". In this case, the transparency for 
#'   individual/variable colors are automatically controlled by their qualities 
#'   ("cos2"), contributions ("contrib"), coordinates (x^2 + y^2 , "coord"), x 
#'   values("x") or y values("y"). To use this, make sure that habillage 
#'   ="none".
#' @param shape.ind,shape.var point shapes of individuals, variables, groups and
#'   axes
#' @param select.ind,select.var a selection of individuals and variables to be 
#'   drawn. Allowed values are NULL or a list containing the arguments name, 
#'   cos2 or contrib: \itemize{ \item name is a character vector containing 
#'   individuals/variables to be drawn \item cos2 if cos2 is in [0, 1], ex: 0.6,
#'   then individuals/variables with a cos2 > 0.6 are drawn. if cos2 > 1, ex: 5,
#'   then the top 5 individuals/variables with the highest cos2 are drawn. \item
#'   contrib if contrib > 1, ex: 5,  then the top 5 individuals/variables with 
#'   the highest cos2 are drawn }
#' @param ... Arguments to be passed to the function fviz()
#' @param repel a boolean, whether to use ggrepel to avoid overplotting text
#'   labels or not. The old \code{jitter} argument is kept for backward
#'   compatibility and is converted to \code{repel = TRUE} with a deprecation warning.
#' @return a ggplot
#' @author Alboukadel Kassambara \email{alboukadel.kassambara@@gmail.com}
#' @examples
#' # Compute FAMD
#'  library("FactoMineR")
#'  data(wine)
#'  res.famd <- FAMD(wine[,c(1,2, 16, 22, 29, 28, 30,31)], graph = FALSE)
#'  res.famd.sup <- FAMD(wine[,c(1,2, 16, 22, 29, 28, 30,31)],
#'                       sup.var = 2, graph = FALSE)
#'                
#' # Eigenvalues/variances of dimensions
#' fviz_screeplot(res.famd)
#' # Graph of variables
#' fviz_famd_var(res.famd)
#' # Quantitative variables
#' fviz_famd_var(res.famd, "quanti.var", repel = TRUE, col.var = "black")
#' # Qualitative variables
#' fviz_famd_var(res.famd, "quali.var", col.var = "black")
#' # Supplementary qualitative variable categories
#' fviz_famd_var(res.famd.sup, "quali.sup", col.var = "darkgreen")
#' # Graph of individuals colored by cos2
#' fviz_famd_ind(res.famd, col.ind = "cos2", 
#'   gradient.cols = c("#00AFBB", "#E7B800", "#FC4E07"),
#'   repel = TRUE)
#' 
#' 
#' @name fviz_famd
#' @rdname fviz_famd
#' @export
fviz_famd_ind <- function(X,  axes = c(1,2), geom=c("point", "text"), repel = FALSE,
                         habillage = "none", palette = NULL, addEllipses = FALSE, 
                         col.ind = "blue", col.ind.sup = "darkblue", alpha.ind = 1,
                         shape.ind = 19, col.quali.var = "black",
                         select.ind = list(name = NULL, cos2 = NULL, contrib = NULL),
                         gradient.cols = NULL,
                         ...)
{
  extra_args <- list(...)
  invisible <- if(is.null(extra_args$invisible)) "none" else extra_args$invisible
  quali.sup <- .get_factominer_quali_sup(X)
  
  if(col.ind %in% c("cos2","contrib", "coord")) partial = NULL
  
  show.quali.var <- !("quali.var" %in% invisible) && !is.null(X$quali.var)
  show.quali.sup <- !("quali.sup" %in% invisible) && !is.null(quali.sup)
  p <- NULL
  
  if(show.quali.var)
    p <- fviz_famd_var(X, "quali.var", axes = axes, geom =  geom, repel = repel,
                       col.var = col.quali.var,  ...)
  else if(show.quali.sup)
    p <- fviz_famd_var(X, "quali.sup", axes = axes, geom = geom, repel = repel,
                       col.var = col.quali.var, ...)
  
  # Individuals
  p <- fviz (X, element = "ind", axes = axes, geom = geom, habillage = habillage, 
        addEllipses = addEllipses, palette = palette, pointshape = shape.ind,
        color = col.ind, alpha = alpha.ind,
        shape.sup = shape.ind, col.row.sup = col.ind.sup,
        select = select.ind, repel = repel, ggp = p,  ...)
  
  if(!is.null(gradient.cols))
    p <- p + ggpubr::gradient_color(gradient.cols)
  
 
  p
  
}


#' @rdname fviz_famd
#' @export
fviz_famd_var <- function(X, choice = c("var", "quanti.var", "quali.var", "quali.sup"), axes = c(1,2),
                         geom = c("point", "text"), repel = FALSE, 
                         col.var ="red", alpha.var=1, shape.var = 17, 
                         col.var.sup = "darkgreen", 
                         select.var = list(name = NULL, cos2 = NULL, contrib = NULL), ...)
{
  extra_args <- list(...)
  if(missing(choice) && !is.null(extra_args$choix)) choice <- extra_args$choix
  choice <- match.arg(choice)

  if(choice == "quanti.var") {
    if(missing(geom)) geom <- c("arrow", "text")
  }
  else if(choice == "quali.var" && is.null(X$quali.var)) {
    stop("There are no qualitative variables to plot.")
  }
  else if(choice == "quali.sup" && is.null(.get_factominer_quali_sup(X))) {
    stop("There are no supplementary qualitative variables to plot.")
  }
  # Main plot
  fviz (X, element = choice, axes = axes, geom = geom,
        color = col.var, alpha = alpha.var,  pointshape = shape.var, 
        shape.sup = shape.var, col.col.sup = col.var.sup, 
        select = select.var, repel = repel,  ...)
}



#' @rdname fviz_famd
#' @export
fviz_famd <- function(X,  ...){
  fviz_famd_ind(X,  ...)
}

#' @include get_mfa.R
NULL
#'Visualize Multiple Factor Analysis
#'
#'@description Multiple factor analysis (MFA) is used to analyze a data set in 
#'  which individuals are described by several sets of variables (quantitative 
#'  and/or qualitative) structured into groups. fviz_mfa() provides
#'  ggplot2-based elegant visualization of MFA outputs from the R function: MFA
#'  [FactoMineR].\cr\cr \itemize{ \item fviz_mfa_ind(): Graph of individuals
#'  \item fviz_mfa_var(): Graph of variables \item fviz_mfa_axes(): Graph of
#'  partial axes \item fviz_mfa(): An alias of fviz_mfa_ind(res.mfa, partial =
#'  "all") \item fviz_mfa_quali_biplot(): Biplot of individuals and qualitative
#'  variables }
#'  
#'@param X an object of class MFA [FactoMineR].
#'@inheritParams fviz_mca
#'@inheritParams fviz_pca
#'@inheritParams fviz
#'@inheritParams ggpubr::ggpar
#'@param choice the graph to plot. Allowed values include one of c("quanti.var",
#'  "quali.var", "quali.sup", "group") for plotting quantitative variables,
#'  qualitative variables, supplementary qualitative variables and group of
#'  variables, respectively.
#'@param habillage an optional factor variable for coloring the observations by 
#'  groups. Default value is "none". If X is an MFA object from FactoMineR 
#'  package, habillage can also specify the index of the factor variable in the 
#'  data.
#'@param col.ind,col.var,col.axes color for individuals, variables and col.axes 
#'  respectively. Can be a continuous variable or a factor variable. Possible
#'  values include also : "cos2", "contrib", "coord", "x" or "y". In this case,
#'  the colors for individuals/variables are automatically controlled by their
#'  qualities ("cos2"), contributions ("contrib"), coordinates (x^2 + y^2 ,
#'  "coord"), x values("x") or y values("y"). To use automatic coloring (by
#'  cos2, contrib, ....), make sure that habillage ="none".
#'@param col.partial color for partial individuals. By default, points are 
#'  colored according to the groups.
#'@param col.var.sup color for supplementary variables.
#'@param alpha.ind,alpha.var,alpha.axes controls the transparency of individual,
#'  variable, group and axes colors, respectively. The value can variate from 0 
#'  (total transparency) to 1 (no transparency). Default value is 1. Possible 
#'  values include also : "cos2", "contrib", "coord", "x" or "y". In this case, 
#'  the transparency for individual/variable colors are automatically controlled
#'  by their qualities ("cos2"), contributions ("contrib"), coordinates (x^2 + 
#'  y^2 , "coord"), x values("x") or y values("y"). To use this, make sure that 
#'  habillage ="none".
#'@param shape.ind,shape.var point shapes of individuals, variables, groups and 
#'  axes
#'@param col.quali.var.sup color for supplementary qualitative variables. 
#'  Default is "black".
#'@param title the title of the graph
#'@param select.ind,select.var,select.axes a selection of individuals/partial 
#'  individuals/ variables/groups/axes to be drawn. Allowed values are NULL or a
#'  list containing the arguments name, cos2 or contrib: \itemize{ \item name is
#'  a character vector containing individuals/variables to be drawn \item cos2 
#'  if cos2 is in [0, 1], ex: 0.6, then individuals/variables with a cos2 > 0.6 
#'  are drawn. if cos2 > 1, ex: 5, then the top 5 individuals/variables with the
#'  highest cos2 are drawn. \item contrib if contrib > 1, ex: 5,  then the top 5
#'  individuals/variables with the highest cos2 are drawn }
#'@param ... Arguments to be passed to the function fviz()
#'@param repel a boolean, whether to use ggrepel to avoid overplotting text
#'  labels or not. The old \code{jitter} argument is kept for backward
#'  compatibility and is converted to \code{repel = TRUE} with a deprecation warning.
#'@param partial list of the individuals for which the partial points should be
#'  drawn. (by default, partial = NULL and no partial points are drawn). Use
#'  partial = "all" to visualize partial points for all individuals.
#'  
#'@return a ggplot2 plot
#'@author Fabian Mundt \email{f.mundt@inventionate.de}
#'@author Alboukadel Kassambara \email{alboukadel.kassambara@@gmail.com}
#'@references \url{https://www.sthda.com/english/}
#' @examples
#' # Compute Multiple Factor Analysis
#' library("FactoMineR")
#' data(wine)
#' res.mfa <- MFA(wine, group=c(2,5,3,10,9,2), type=c("n",rep("s",5)),
#'                ncp=5, name.group=c("orig","olf","vis","olfag","gust","ens"),
#'                num.group.sup=c(1,6), graph=FALSE)
#'                
#' # Eigenvalues/variances of dimensions
#' fviz_screeplot(res.mfa)
#' # Group of variables
#' fviz_mfa_var(res.mfa, "group")
#' # Quantitative variables
#' fviz_mfa_var(res.mfa, "quanti.var", palette = "jco", 
#'   col.var.sup = "violet", repel = TRUE)
#' # Graph of individuals colored by cos2
#' fviz_mfa_ind(res.mfa, col.ind = "cos2", 
#'   gradient.cols = c("#00AFBB", "#E7B800", "#FC4E07"),
#'   repel = TRUE)
#' # Partial individuals
#' fviz_mfa_ind(res.mfa, partial = rownames(wine)[1:3], col.partial = "black") 
#' # Partial axes
#' fviz_mfa_axes(res.mfa)
#' 
#'
#' # Graph of categorical variable categories
#' # ++++++++++++++++++++++++++++++++++++++++
#' data(poison)
#' res.mfa <- MFA(poison, group=c(2,2,5,6), type=c("s","n","n","n"),
#'                name.group=c("desc","desc2","symptom","eat"),
#'                num.group.sup=1:2, graph=FALSE)
#'
#' # Plot of qualitative variables
#' fviz_mfa_var(res.mfa, "quali.var")
#' # Plot of supplementary qualitative variables
#' fviz_mfa_var(res.mfa, "quali.sup")
#'  
#'  
#' 
#' # Biplot of categorical variable categories and individuals
#' # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#'  # Use repel = TRUE to avoid overplotting
#' grp <- as.factor(poison[, "Vomiting"])
#' fviz_mfa_quali_biplot(res.mfa, repel = FALSE, col.var = "#E7B800",
#'    habillage = grp, addEllipses = TRUE, ellipse.level = 0.95)
#' 
#'@name fviz_mfa
#'@rdname fviz_mfa
#'@export
fviz_mfa_ind <- function(X,  axes = c(1,2), geom=c("point", "text"), repel = FALSE,
                         habillage = "none", palette = NULL, addEllipses = FALSE, 
                         col.ind = "blue", col.ind.sup = "darkblue", alpha.ind = 1,
                         shape.ind = 19, col.quali.var.sup = "black",
                         select.ind = list(name = NULL, cos2 = NULL, contrib = NULL),
                         partial = NULL, col.partial = "group",
                         ...)
{
  extra_args <- list(...)
  invisible <- if(is.null(extra_args$invisible)) "none" else extra_args$invisible
  
  if(col.ind %in% c("cos2","contrib", "coord")) partial = NULL
  if(!is.null(partial) && missing(col.ind)) col.ind <- "black"
  
  p <- fviz (X, element = "ind", axes = axes, geom = geom, habillage = habillage, 
        addEllipses = addEllipses, palette = palette, pointshape = shape.ind,
        color = col.ind, alpha = alpha.ind,
        shape.sup = shape.ind, col.row.sup = col.ind.sup,
        select = select.ind, repel = repel, ...)
  
  # Add supplementary qualitative variables if they exist
  show.quali.sup <- !("quali.sup" %in% invisible) && is.null(partial)
  is_habillage <- habillage[1] != "none" 
  if(!is.null(X$quali.var.sup) && show.quali.sup){
    if(!(col.ind %in% c("cos2", "contrib")) && !is_habillage) col.quali.var.sup  = "quali.sup"
    quali.sup <- .get_supp(X, element = "quali.var.sup", axes = axes)
    quali.sup$quali.sup <- .get_quali_var_sup_names(X)
    colnames(quali.sup)[2:3] <-  c("x", "y")
    p <- fviz_add(p, df = quali.sup[, -1], 
                  geom = c("point", "text"), color = col.quali.var.sup, shape = 0, 
                  repel = repel, addlabel = TRUE)
    
  }
  
  # Add partial points
  if(!is.null(partial)){
    if(!(partial[1] %in% c("All", "all"))) select.partial = list(name = partial)
    else select.partial <- NULL
    if(col.partial %in% c("group", "groups")) col.partial <- "group.name"
    
    # Data for individuals
    ind.sum <- facto_summarize(X, element = "ind",
                               result = c("coord", "contrib", "cos2", "coord.partial"), axes = axes)
    ind <- ind.sum$res
    colnames(ind)[2:3] <-  c("x", "y")
    # partial points
    ind.partial <- ind.sum$res.partial
    colnames(ind.partial)[3:4] <-  c("x.partial", "y.partial")
    ind.partial <- merge(ind, ind.partial, by = "name")
    # Selection
    ind.all <- ind
    if(!is.null(select.ind)) ind <- .select(ind, select.ind)
    if(!is.null(select.partial)) {
      if(nrow(ind) != nrow(ind.all)) warning("You've already selected individuals. Partial points are only calculated for them.")
      ind.partial <-  ind.partial[ind.partial$name %in% .select(ind, select.partial)$name, , drop = FALSE]
    }
    # elements to be hidden
    hide <- .hide(invisible)
    # Plot
    if(!hide$ind && "point" %in% geom) {
      # Partial point
      p <- p + ggpubr::geom_exec(geom_point, data = ind.partial,
                                 x = "x.partial", y = "y.partial", 
                                 colour = col.partial,
                                 shape = shape.ind, size = 1)
      # Partial segments
      # FIX: ggplot2 3.4.0+ deprecation - size replaced with linewidth for geom_segment
      p <- p + ggpubr::geom_exec(geom_segment, data = ind.partial,
                                 x = "x", y = "y", xend = 'x.partial', yend = 'y.partial',
                                 linetype = "group.name", colour = col.partial, linewidth = 0.5)
    }
    # Edit plot title and legend title
    p <- p  + labs(colour = "Groups", linetype = "Groups")
  }
  
  p
  
}



#' @rdname fviz_mfa
#' @export
fviz_mfa_quali_biplot <- function(X,  axes = c(1,2), geom=c("point", "text"), repel = FALSE,
                                  title = "Biplot of individuals and qualitative variables - MFA", ...)
{
  extra_args <- list(...)
  invisible.ind <- extra_args$invisible
  invisible.var <- if(is.null(invisible.ind)) "quali.sup" else unique(c(invisible.ind, "quali.sup"))
  extra_args[c("invisible", "choice", "ggp")] <- NULL

  # Individuals
  ind.args <- c(list(X = X, axes = axes, geom = geom, repel = repel), extra_args)
  if(!is.null(invisible.ind)) ind.args$invisible <- invisible.ind
  p <- do.call(fviz_mfa_ind, ind.args)
  
  # Variable
  # Add variables
  var.args <- c(
    list(X = X, choice = "quali.var", axes = axes, geom = geom, repel = repel,
         ggp = p, invisible = invisible.var),
    extra_args
  )
  p <- do.call(fviz_mfa_var, var.args)
  
  p+labs(title=title)
}




#' @rdname fviz_mfa
#' @export
fviz_mfa_var <- function(X, choice = c("quanti.var", "group", "quali.var", "quali.sup"), axes = c(1,2), 
                         geom = c("point", "text"), repel = FALSE, habillage = "none",
                         col.var ="red", alpha.var=1, shape.var = 17, 
                         col.var.sup = "darkgreen", palette = NULL,
                         select.var = list(name = NULL, cos2 = NULL, contrib = NULL), ...)
{
  
  extra_args <- list(...)
  if(missing(choice) && !is.null(extra_args$choix)) choice <- extra_args$choix
  choice <- match.arg(choice)
 
 
  # Define habillage if quanti.var. Quantitative variables are colored by groups
  if(choice == "quanti.var") {
    .check_if_quanti_exists(X)
    if(missing(geom)) geom <- c("arrow", "text")
    habillage <- .get_quanti_var_groups(X)
  }
  else if(choice== "quali.var"){
    .check_if_quali_exists(X)
  }
  else if(choice == "quali.sup"){
    if(is.null(.get_factominer_quali_sup(X)))
      stop("There are no supplementary qualitative variables to plot.")
  }
  if(!missing(col.var)) habillage = "none"
  # Main plot
  fviz (X, element = choice, axes = axes, geom = geom,
        color = col.var, alpha = alpha.var,  pointshape = shape.var, 
        shape.sup = shape.var, col.col.sup = col.var.sup, 
        habillage = habillage,  palette = palette,
        select = select.var, repel = repel,  ...)
}



#' @rdname fviz_mfa
#' @export
fviz_mfa_axes <- function(X,  axes = c(1,2), geom=c("arrow", "text"),
                          col.axes = NULL, alpha.axes=1, col.circle ="grey70",
                          select.axes = list(name = NULL, contrib = NULL),
                          repel = FALSE, ...)
{
  
  # Color partial axes by groups
  habillage <- "none"
  if(is.null(col.axes)){
    axes.names <- rownames(X$partial.axes$coord)
    axes.groups <- vapply(
      axes.names,
      function(x) gsub("^Dim[0-9]+\\.",  "", x, perl = TRUE),
      character(1)
    )
    habillage <- as.factor(axes.groups)
  }
  fviz (X, element = "partial.axes", axes = axes, geom = geom,
        color = col.axes, alpha = alpha.axes,  select = select.axes,
        repel = repel, col.circle = col.circle, habillage = habillage, ...)
}


#' @rdname fviz_mfa
#' @export
fviz_mfa <- function(X, partial = "all", ...){
  fviz_mfa_ind(X, partial = partial, ...)
}



#+++++++++++++++++++++
# Helper functions
#+++++++++++++++++++++

# Check if there are quantitative variables.
.check_if_quanti_exists <- function(X){
  map <- .mfa_group_map(X)
  if(is.null(map) || !any(map$var.type == "quanti" & !map$is.group.sup))
    stop("There are no quantitative variables to plot.")
  
}

# Check if qualitative variables exists
.check_if_quali_exists <- function (X){
  map <- .mfa_group_map(X)
  if(is.null(map) || !any(map$var.type == "quali" & !map$is.group.sup))
    stop("There are no qualitative variables to plot.")
}

#'House tasks contingency table
#'
#'@description A data frame containing the frequency of execution of 13 house
#'  tasks in the couple. This table is also available in ade4 package.
#'@name housetasks
#'@docType data
#'@usage data("housetasks")
#'@format  
#' A data frame with 13 observations (house tasks) on the following 4 columns.
#' \describe{
#'  \item{\code{Wife}}{a numeric vector}
#'  \item{\code{Alternating}}{a numeric vector}
#'  \item{\code{Husband}}{a numeric vector}
#'  \item{\code{Jointly}}{a numeric vector}
#' }
#'@source This data is from FactoMineR package.
#'
#' @examples
#' library(FactoMineR)
#' data(housetasks)
#' res.ca <- CA(housetasks, graph=FALSE)
#' fviz_ca_biplot(res.ca, repel = TRUE)+
#' theme_minimal()
#' 
NULL

#' @include  hcut.R
 NULL
#' Determining and Visualizing the Optimal Number of Clusters
#' @description Partitioning methods, such as k-means clustering require the 
#'   users to specify the number of clusters to be generated. \itemize{
#'   \item fviz_nbclust(): Determines and visualizes the optimal number of
#'   clusters using different methods: \strong{within cluster sums of squares},
#'   \strong{average silhouette} and \strong{gap statistics}. Silhouette values
#'   are evaluated only for \code{k = 2, \ldots, k.max} because average
#'   silhouette width is undefined for a one-cluster partition. The dashed
#'   guide line marks the best displayed silhouette width among those
#'   \code{k >= 2} candidates.
#'   \item fviz_gap_stat(): Visualizes the gap statistic generated by the
#'   function \code{\link[cluster]{clusGap}}() [in cluster package]. The optimal
#'   number of clusters is specified using the "firstmax" method
#'   (?cluster::clustGap). }
#'
#'   For \code{method = "wss"}, \code{factoextra} computes the \code{k = 1}
#'   baseline internally so helper functions such as \code{hcut()} and
#'   \code{hkmeans()} can keep rejecting direct \code{k = 1} inputs.
#'
#'   Read more:
#'   \href{https://www.datanovia.com/en/lessons/determining-the-optimal-number-of-clusters-3-must-know-methods/}{Determining
#'    the optimal number of clusters}
#'   
#' @param x numeric matrix or data frame. In the function fviz_nbclust(), x can
#'   be the results of the function NbClust(). For \code{method = "silhouette"}
#'   or \code{"wss"}, x may also be a precomputed dissimilarity (an object of
#'   class \code{"dist"}); it is then passed directly to \code{FUNcluster},
#'   which must be a dissimilarity-capable method (e.g. \code{cluster::pam} or
#'   \code{factoextra::hcut}). \code{method = "gap_stat"} still needs the raw data.
#' @param method the method to be used for estimating the optimal number of 
#'   clusters. Possible values are "silhouette" (for average silhouette width), 
#'   "wss" (for total within sum of square) and "gap_stat" (for gap statistics).
#' @param FUNcluster a partitioning function which accepts as first argument a
#'   (data) matrix like \code{x}, second argument, say \code{k >= 2}, the
#'   number of clusters desired, and returns a list with a component named
#'   \code{cluster} which contains the grouping of observations. Allowed values
#'   include: \code{kmeans}, \code{cluster::pam}, \code{cluster::clara},
#'   \code{cluster::fanny}, \code{hcut}, etc. In \code{method = "wss"} mode,
#'   \code{fviz_nbclust()} computes the \code{k = 1} baseline internally instead
#'   of calling \code{FUNcluster(x, 1, ...)}. This argument is not required
#'   when \code{x} is an output of the function \code{NbClust::NbClust()}.
#' @param diss dist object as produced by dist(), i.e.: diss = dist(x, method = 
#'   "euclidean"). Used to compute the average silhouette width of clusters, the
#'   within sum of square and hierarchical clustering. If NULL, dist(x) is 
#'   computed with the default method = "euclidean"
#' @param k.max the maximum number of clusters to consider, must be at least two.
#' @param nboot integer, number of Monte Carlo ("bootstrap") samples. Used only for determining the number of clusters 
#' using gap statistic.
#' @param verbose logical value. If TRUE, the result of progress is printed.
#' @param barfill,barcolor fill color and outline color for bars
#' @param linecolor color for lines
#' @param print.summary logical value. If true, the optimal number of clusters 
#'   are printed in fviz_nbclust().
#' @param ... optionally further arguments:
#'   arguments for FUNcluster() in "wss"/"silhouette" modes; arguments for
#'   \code{\link[cluster]{clusGap}}() in "gap_stat" mode. A \code{maxSE} list
#'   can also be supplied in "gap_stat" mode and is forwarded to
#'   \code{fviz_gap_stat()}.
#'   
#' @return \itemize{ \item fviz_nbclust, fviz_gap_stat: return a ggplot2 }
#' @seealso \code{\link{fviz_cluster}}, \code{\link{eclust}}
#' @author Alboukadel Kassambara \email{alboukadel.kassambara@@gmail.com}
#'   
#' @examples 
#' set.seed(123)
#' 
#' # Data preparation
#' # +++++++++++++++
#' data("iris")
#' head(iris)
#' # Remove species column (5) and scale the data
#' iris.scaled <- scale(iris[, -5])
#' 
#' 
#' # Optimal number of clusters in the data
#' # ++++++++++++++++++++++++++++++++++++++
#' # Examples are provided only for kmeans, but
#' # you can also use cluster::pam (for pam) or
#' #  hcut (for hierarchical clustering)
#'  
#' ### Elbow method (look at the knee)
#' # Elbow method for kmeans
#' fviz_nbclust(iris.scaled, kmeans, method = "wss") +
#' geom_vline(xintercept = 3, linetype = 2)
#'
#' # WSS with hierarchical clustering keeps the internal k = 1 baseline
#' fviz_nbclust(iris.scaled, hcut, method = "wss", hc_method = "complete")
#' 
#' # Average silhouette for kmeans
#' fviz_nbclust(iris.scaled, kmeans, method = "silhouette")
#' 
#' ### Gap statistic
#' library(cluster)
#' set.seed(123)
#' # Compute gap statistic for kmeans
#' # we used B = 10 for demo. Recommended value is ~500
#' gap_stat <- clusGap(iris.scaled, FUN = kmeans, nstart = 25,
#'  K.max = 10, B = 10)
#'  print(gap_stat, method = "firstmax")
#' fviz_gap_stat(gap_stat)
#'  
#' # Gap statistic for hierarchical clustering
#' gap_stat <- clusGap(iris.scaled, FUN = hcut, K.max = 10, B = 10)
#' fviz_gap_stat(gap_stat)
#' 
#'  
#' @name fviz_nbclust
#' @rdname fviz_nbclust
#' @export
fviz_nbclust <- function (x, FUNcluster = NULL, method = c("silhouette", "wss", "gap_stat"),
                          diss = NULL, k.max = 10, nboot = 100, verbose = interactive(),
                          barfill="steelblue", barcolor="steelblue", 
                          linecolor = "steelblue", print.summary = TRUE,  ...) 
  {
  if(k.max < 2) stop("k.max must be >= 2")
  method = match.arg(method)
  # x may also be a precomputed dissimilarity ("dist") for diss-capable methods (#90).
  is_dist <- inherits(x, "dist")
  if(!inherits(x, c("data.frame", "matrix", "dist")) && !("Best.nc" %in% names(x)))
    stop("x should be an object of class matrix/data.frame/dist or ",
         "an object created by the function NbClust() [NbClust package].")
  
  # x is an object created by the function NbClust() [NbClust package]
  if(inherits(x, "list") && "Best.nc" %in% names(x)){
      best_nc <- x$Best.nc
      # FIX: R 4.0.0+ deprecation - class() can return multiple values for matrices
      # Using inherits() or is.X() functions instead of class() == comparison
      # See: https://github.com/kassambara/factoextra/issues/171
      if(is.numeric(best_nc) && !is.matrix(best_nc)) print(best_nc)
      else if(is.matrix(best_nc))
        .viz_NbClust(x, print.summary, barfill, barcolor)
  }
  else if(is.null(FUNcluster)) stop("The argument FUNcluster is required. ",
                                    "Possible values are kmeans, pam, hcut, clara, ...")
  else if(!is.function(FUNcluster)){
    stop(
      "The argument FUNcluster should be a function. ",
      "Check if you're not overriding the specified function name somewhere."
      )
  }
  else if(method %in% c("silhouette", "wss")) {

      if (is.data.frame(x)) x <- as.matrix(x)
      # When x is a dissimilarity, use it directly (don't recompute) and pass it
      # to FUNcluster, which must be a dissimilarity-capable method (#90).
      # Detect capability via a 'diss'/'isdiss' formal (pam/fanny/hcut have one;
      # kmeans/clara do not and would silently mis-cluster a coerced dist).
      if(is_dist){
        fmls <- tryCatch(names(formals(FUNcluster)), error = function(e) character(0))
        if(!any(c("diss", "isdiss") %in% fmls))
          stop("FUNcluster does not accept a distance matrix (it has no 'diss'/'isdiss' ",
               "argument). Use a dissimilarity-capable method such as cluster::pam, ",
               "cluster::fanny, or factoextra::hcut, or pass the raw data instead of a 'dist'.",
               call. = FALSE)
      }
      if(is.null(diss)) diss <- if(is_dist) x else stats::dist(x)
      .cluster_at <- function(i) FUNcluster(x, i, ...)

      v <- rep(0, k.max)
      if(method == "silhouette"){
        for(i in 2:k.max){
          clust <- .cluster_at(i)
          v[i] <- .get_ave_sil_width(diss, clust$cluster)
        }
      }
      else if(method == "wss"){
        n_obs <- attr(diss, "Size")
        if(is.null(n_obs) || !is.numeric(n_obs))
          stop("Unable to determine the number of observations from diss")
        # Compute the one-cluster baseline internally so callers may reject k = 1.
        v[1] <- .get_withinSS(diss, rep(1L, n_obs))
        for(i in 2:k.max){
          clust <- .cluster_at(i)
          v[i] <- .get_withinSS(diss, clust$cluster)
        }
        
      }
      
      if(method == "silhouette"){
        silhouette_clusters <- seq.int(2, k.max)
        df <- data.frame(clusters = as.factor(silhouette_clusters), y = v[silhouette_clusters])
      }
      else{
        df <- data.frame(clusters = as.factor(1:k.max), y = v)
      }
      
      ylab <- "Total Within Sum of Square"
      main_title <- "Optimal number of clusters"
      if(method == "silhouette") {
        ylab <- "Average silhouette width"
        main_title <- "Optimal number of clusters (method = \"silhouette\")"
      }

      p <- ggpubr::ggline(df, x = "clusters", y = "y", group = 1,
                          color = linecolor, ylab = ylab,
                          xlab = "Number of clusters k",
                          main = main_title
                          )
      if(method == "silhouette") {
        valid_clusters <- which(!is.na(v[-1])) + 1L
        if(length(valid_clusters) > 0) {
          best_k <- valid_clusters[which.max(v[valid_clusters])]
          p <- p + geom_vline(
            xintercept = match(as.character(best_k), levels(df$clusters)),
            linetype=2, color = linecolor
          )
        }
      }
      
      return(p) 
  }
  
  else if(method == "gap_stat"){
    if(is_dist)
      stop("method = 'gap_stat' requires the original data, not a distance matrix ",
           "(the gap statistic simulates reference datasets). Use method = 'silhouette' or ",
           "'wss' with a 'dist', or pass the raw data.", call. = FALSE)
    extra_args <- list(...)
    if(!is.null(extra_args$maxSE)) {
      maxSE <- extra_args$maxSE
      extra_args$maxSE <- NULL
    } else {
      maxSE <- list(method = "firstSEmax", SE.factor = 1)
    }
    gap_stat <- do.call(cluster::clusGap, c(list(x = x, FUNcluster = FUNcluster, K.max = k.max, B = nboot,
                                                  verbose = verbose), extra_args))
    p <- fviz_gap_stat(gap_stat,  linecolor = linecolor, maxSE = maxSE)
    return(p) 
  }
  

}

#' @rdname fviz_nbclust
#' @param gap_stat an object of class "clusGap" returned by the function
#'   clusGap() [in cluster package]
#' @param maxSE a list containing the parameters (method and SE.factor) for
#'   determining the location of the maximum of the gap statistic (Read the
#'   documentation ?cluster::maxSE). Allowed values for maxSE$method include:
#'   \itemize{ \item "globalmax": simply corresponds to the global maximum,
#'   i.e., is which.max(gap) \item "firstmax": gives the location of the first
#'   local maximum \item "Tibs2001SEmax": uses the criterion, Tibshirani et al
#'   (2001) proposed: "the smallest k such that gap(k) >= gap(k+1) - s(k+1)".
#'   It's also possible to use "the smallest k such that gap(k) >= gap(k+1) -
#'   SE.factor*s(k+1)" where SE.factor is a numeric value which can be 1
#'   (default), 2, 3, etc. \item "firstSEmax": location of the first f() value
#'   which is not larger than the first local maximum minus SE.factor * SE.f,
#'   i.e, within an "f S.E." range of that maximum. \item
#'   see ?cluster::maxSE for more options }
#'
#' @section Method selection for gap statistic:
#'   The default method "firstSEmax" (developed by Martin Maechler, 2012) is
#'   recommended as a robust alternative to "Tibs2001SEmax". The original
#'   Tibshirani method can be overly conservative and often returns k=1 when
#'   standard deviations are large relative to gap differences. The "firstSEmax"
#'   method finds the smallest k within one standard error of the first local
#'   maximum, providing more stable results in practice.
#'
#'
#' @export
fviz_gap_stat <- function(gap_stat,  linecolor = "steelblue",
                          maxSE = list(method = "firstSEmax", SE.factor = 1)){
  if(!inherits(gap_stat, "clusGap"))
    stop("Only an object of class clusGap is allowed. (cluster package)")
  if(is.list(maxSE)){
    if(is.null(maxSE$method)) maxSE$method = "firstmax"
    if(is.null(maxSE$SE.factor)) maxSE$SE.factor = 1
  }
  else stop("The argument maxSE must be a list containing the parameters method and SE.factor")
  
  # first local max
  gap <- gap_stat$Tab[, "gap"]
  se <- gap_stat$Tab[, "SE.sim"]
  k <- .maxSE(gap, se, method = maxSE$method, SE.factor = maxSE$SE.factor)

  df <- as.data.frame(gap_stat$Tab)
  df$clusters <- as.factor(seq_len(nrow(df)))
  df$ymin <- gap-se
  df$ymax <- gap + se
  p <- ggpubr::ggline(df, x = "clusters", y = "gap", group = 1, color = linecolor)+
    # FIX: ggplot2 3.0.0+ deprecation - aes_string() replaced with aes() + .data pronoun
    # See: https://github.com/kassambara/factoextra/issues/190
    ggplot2::geom_errorbar(aes(ymin = .data[["ymin"]], ymax = .data[["ymax"]]), width=.2, color = linecolor)+
    geom_vline(xintercept = k, linetype=2, color = linecolor)+
    labs(y = "Gap statistic (k)", x = "Number of clusters k",
         title = "Optimal number of clusters (method = \"gap_stat\")")
  p
}




# Get the average silhouette width
# ++++++++++++++++++++++++++
# Cluster package required
# d: dist object
# cluster: cluster number of observation
# Returns NA if silhouette cannot be computed (e.g., k <= 1 or k >= n)
# Fixes GitHub issues #113 and #147
.get_ave_sil_width <- function(d, cluster){
  if (!requireNamespace("cluster", quietly = TRUE)) {
    stop("cluster package needed for this function to work. Please install it.")
  }
  ss <- cluster::silhouette(cluster, d)
  # Handle case where silhouette() returns NA (when k <= 1 or k >= n)
  if (length(ss) == 1 && is.na(ss)) {
    return(NA_real_)
  }
  mean(ss[, 3])
}

# Get total within sum of square
# +++++++++++++++++++++++++++++
# d: dist object
# cluster: cluster number of observation
#
# OPTIMIZATION: Replaced explicit loops with vapply() for better performance
# - Pre-compute cluster indices once using split()
# - Use vapply for type-safe vectorized computation
# - Avoid repeated subsetting of distance matrix
# - Maintains identical econometric results
.get_withinSS <- function(d, cluster){
  d <- stats::as.dist(d)
  dmat <- as.matrix(d)

  # Handle cluster renumbering if needed
  clusterf <- as.factor(cluster)
  clusterl <- levels(clusterf)
  cn <- length(clusterl)

  if (max(cluster) != cn) {
    warning("cluster renumbered because maximum != number of clusters")
    cluster <- as.integer(clusterf)
  }

  # OPTIMIZED: Pre-compute cluster membership indices
  # split() creates a list of indices for each cluster - computed once
  cluster_indices <- split(seq_along(cluster), cluster)

  # OPTIMIZED: Compute within-cluster SS using vapply (faster than for loop)
  # For each cluster: sum(d[i,j]^2) / cluster_size for all pairs in cluster
  within_ss <- vapply(cluster_indices, function(idx) {
    if (length(idx) <= 1) return(0)
    # Extract submatrix for this cluster and compute sum of squared distances
    di <- dmat[idx, idx, drop = FALSE]
    # Only use lower triangle (as.dist) to avoid double counting
    sum(di[lower.tri(di)]^2) / length(idx)
  }, FUN.VALUE = numeric(1))

  sum(within_ss)
}





# Visualization of the output returned by the function
# NbClust()
# x : an object generated by the function NbClust()
.viz_NbClust <- function(x, print.summary = TRUE,
                         barfill = "steelblue", barcolor = "steelblue")
  {
     best_nc <- x$Best.nc
    # FIX: R 4.0.0+ deprecation - class() can return multiple values for matrices
    # Using inherits() or is.X() functions instead of class() == comparison
    # See: https://github.com/kassambara/factoextra/issues/171
    if(is.numeric(best_nc) && !is.matrix(best_nc)) print(best_nc)
     else if(is.matrix(best_nc)){
    best_nc <- as.data.frame(t(best_nc))
    best_nc$Number_clusters <- as.factor(best_nc$Number_clusters)
    ss <- summary(best_nc$Number_clusters)
    
    # Summary
    if(print.summary){
      cat ("Among all indices: \n===================\n")
      for(i in seq_along(ss)){
        cat("*", ss[i], "proposed ", names(ss)[i], "as the best number of clusters\n" )
      }
      cat("\nConclusion\n=========================\n")
      cat("* According to the majority rule, the best number of clusters is ",
          names(which.max(ss)),  ".\n\n")
    }
    # Fix for Issue #131: ensure numeric ordering of clusters (not alphabetical)
    # When clusters > 9, alphabetical ordering would put "10" before "2"
    cluster_names <- names(ss)
    cluster_order <- order(as.numeric(cluster_names))
    df <- data.frame(
      Number_clusters = factor(cluster_names, levels = cluster_names[cluster_order]),
      freq = ss,
      stringsAsFactors = FALSE
    )
    p <- ggpubr::ggbarplot(df,  x = "Number_clusters", y = "freq", fill = barfill, color = barcolor)+
      labs(x = "Number of clusters k", y = "Frequency among all indices",
           title = paste0("Optimal number of clusters - k = ", names(which.max(ss)) ))
    
    return(p)
  }
}


#  Determines the location of the maximum of f see ?cluster::maxSE
# +++++++++++++++++++++++++++++++++++++++++++
# f: numeric vector containing the gap statistic
# SE.f : standard error of the gap statistic
# method : character string indicating how the "optimal" number of clusters, k, 
  # is computed from the gap statistics (and their standard deviations), 
  # or more generally how the location k^ of the maximum of f[k] should be determined.
# SE.factor:  Determining the optimal number of clusters, Tibshirani et al. proposed the "1 S.E."-rule.
.maxSE <- function (f, SE.f, method = c("firstSEmax", "Tibs2001SEmax", 
                                        "globalSEmax", "firstmax", "globalmax"), SE.factor = 1) 
{
  method <- match.arg(method)
  stopifnot((K <- length(f)) >= 1, K == length(SE.f), SE.f >= 
              0, SE.factor >= 0)
  fSE <- SE.factor * SE.f
  switch(method, firstmax = {
    decr <- diff(f) <= 0
    if (any(decr)) which.max(decr) else K
  }, globalmax = {
    which.max(f)
  }, Tibs2001SEmax = {
    g.s <- f - fSE
    if (any(mp <- f[-K] >= g.s[-1])) which.max(mp) else K
  }, firstSEmax = {
    decr <- diff(f) <= 0
    nc <- if (any(decr)) which.max(decr) else K
    if (any(mp <- f[seq_len(nc - 1)] >= f[nc] - fSE[nc])) which(mp)[1] else nc
  }, globalSEmax = {
    nc <- which.max(f)
    if (any(mp <- f[seq_len(nc - 1)] >= f[nc] - fSE[nc])) which(mp)[1] else nc
  })
}

#' @include facto_summarize.R
NULL
#' Visualize the contributions of row/column elements
#'
#' @description
#' This function can be used to visualize the contribution of rows/columns
#' from the results of Principal Component Analysis (PCA),
#' Correspondence Analysis (CA), Multiple Correspondence Analysis (MCA), Factor Analysis of Mixed Data (FAMD),
#' and Multiple Factor Analysis (MFA) functions.
#' @param sort.val a string specifying whether the value should be sorted. 
#' Allowed values are "none" (no sorting), "asc" (for ascending) or "desc" (for descending).
#' @param ... other arguments to be passed to the function \link[ggpubr]{ggpar}.
#' @inheritParams fviz_cos2
#' @inheritParams ggpubr::ggpar
#' @details
#' The function fviz_contrib() creates a barplot of row/column contributions.
#' A reference dashed line is also shown on the barplot. This reference line
#' corresponds to the expected value if the contribution were uniform.\cr\cr
#' For a given dimension, any row/column with a contribution above the reference line could be
#' considered as important in contributing to the dimension.
#'
#' @return a ggplot2 plot
#' @author Alboukadel Kassambara \email{alboukadel.kassambara@@gmail.com}
#' @references \url{https://www.sthda.com/english/}
#' @examples
#' \donttest{
#' # Principal component analysis
#' # ++++++++++++++++++++++++++
#' data(decathlon2)
#' decathlon2.active <- decathlon2[1:23, 1:10]
#' res.pca <- prcomp(decathlon2.active,  scale = TRUE)
#'
#' # variable contributions on axis 1
#' fviz_contrib(res.pca, choice="var", axes = 1, top = 10 )
#' 
#' # Change theme and color
#' fviz_contrib(res.pca, choice="var", axes = 1,
#'          fill = "lightgray", color = "black") +
#'          theme_minimal() +
#'          theme(axis.text.x = element_text(angle=45))
#'
#' # Variable contributions on axis 2
#' fviz_contrib(res.pca, choice="var", axes = 2)
#' # Variable contributions on axes 1 + 2
#' fviz_contrib(res.pca, choice="var", axes = 1:2)
#'
#' # Contributions of individuals on axis 1
#' fviz_contrib(res.pca, choice="ind", axes = 1)
#'
#'\dontrun{
#' # Correspondence Analysis
#' # ++++++++++++++++++++++++++
#' # Install and load FactoMineR to compute CA
#' # install.packages("FactoMineR")
#' library("FactoMineR")
#' data("housetasks")
#' res.ca <- CA(housetasks, graph = FALSE)
#'
#' # Visualize row contributions on axes 1
#' fviz_contrib(res.ca, choice ="row", axes = 1)
#' # Visualize column contributions on axes 1
#' fviz_contrib(res.ca, choice ="col", axes = 1)
#'
#' # Multiple Correspondence Analysis
#' # +++++++++++++++++++++++++++++++++
#' library(FactoMineR)
#' data(poison)
#' res.mca <- MCA(poison, quanti.sup = 1:2,
#'               quali.sup = 3:4, graph=FALSE)
#'
#' # Visualize individual contributions on axes 1
#' fviz_contrib(res.mca, choice ="ind", axes = 1)
#' # Visualize variable category contributions on axes 1
#' fviz_contrib(res.mca, choice ="var", axes = 1)
#'
#' # Multiple Factor Analysis
#' # ++++++++++++++++++++++++
#' library(FactoMineR)
#' data(poison)
#' res.mfa <- MFA(poison, group=c(2,2,5,6), type=c("s","n","n","n"),
#'                name.group=c("desc","desc2","symptom","eat"),
#'                num.group.sup=1:2, graph=FALSE)
#'
#' # Visualize individual contributions on axes 1
#' fviz_contrib(res.mfa, choice ="ind", axes = 1, top = 20)
#' # Visualize categorical variable category contributions on axes 1
#' fviz_contrib(res.mfa, choice ="quali.var", axes = 1)
#' }
#'
#'  }
#' @export
fviz_contrib <- function(X, choice = c("row", "col", "var", "ind", "quanti.var", "quali.var", "group", "partial.axes"),
                         axes=1, fill="steelblue", color = "steelblue", 
                         sort.val = c("desc", "asc", "none"), top = Inf,
                         xtickslab.rt = 45, ggtheme = theme_minimal(), ...)
{

  sort.val <- match.arg(sort.val)
  choice = match.arg(choice)
  
  title <- .build_title(choice[1], "Contribution", axes)

  dd <- facto_summarize(X, element = choice, result = "contrib", axes = axes)
  contrib <- dd$contrib
  names(contrib) <-rownames(dd)

  # expected Average contribution
  theo_contrib <- 100/length(contrib)
  if(length(axes) > 1) {
    # Adjust variable contributions by the Dimension eigenvalues
    eig <- get_eigenvalue(X)[axes,1]
    theo_contrib <- sum(theo_contrib*eig)/sum(eig)
  }
  df <- data.frame(name = factor(names(contrib), levels = names(contrib)), contrib = contrib)
  
  # Define color if quanti.var
  if(choice == "quanti.var") {
    df$Groups <- .get_quanti_var_groups (X)
    if(missing(fill)) fill <- "Groups"
    if(missing(color)) color <- "Groups"
  }
  
  p <- ggpubr::ggbarplot(df, x = "name", y = "contrib", fill = fill, color = color,
                         sort.val = sort.val, top = top,
                         main = title, xlab = FALSE, ylab ="Contributions (%)",
                         xtickslab.rt = xtickslab.rt, ggtheme = ggtheme,
                         sort.by.groups = FALSE, ...
                         )+
    geom_hline(yintercept=theo_contrib, linetype=2, color="red")

  p
}


#' @describeIn fviz_contrib deprecated function. Use fviz_contrib()
#' @param sortcontrib see the argument sort.val
#' @export
fviz_pca_contrib <- function(X, choice = c("var", "ind"), axes=1,
                             fill="steelblue", color = "steelblue",
                             sortcontrib = c("desc", "asc", "none"), top = Inf,...)
{

  warning("The function fviz_pca_contrib() is deprecated. ",
          "Please use the function fviz_contrib() which can handle outputs ",
          " of PCA, CA and MCA functions.")

  p <- fviz_contrib(X = X, choice = choice, axes = axes,
               fill = fill, color = color, sort.val = sortcontrib,
               top = top, ...)
  p
}

#' @include utilities.R
NULL
#' Draw confidence ellipses around the categories
#' @description Draw confidence ellipses around the categories
#' @inheritParams fviz
#' @inheritParams ggpubr::ggpar
#' @param X an object of class MCA, PCA or MFA.
#' @param habillage a numeric vector of indexes of variables or a
#'   character vector of names of variables. Can be also a data frame containing grouping variables.
#' @param geom a text specifying the geometry to be used for the graph.  Allowed
#'   values are the combination of c("point", "text"). Use "point" (to show only
#'   points); "text" to show only labels; c("point", "text") to show both types.
#' @param ... Arguments to be passed to the functions ggpubr::ggscatter()  &
#'   ggpubr::ggpar().
#'   
#' @return a ggplot
#' @author Alboukadel Kassambara \email{alboukadel.kassambara@@gmail.com}
#'   
#' @rdname fviz_ellipses
#' @examples
#' \donttest{
#' 
#' # Multiple Correspondence Analysis
#' # +++++++++++++++++++++++++++++++++
#' library(FactoMineR)
#' data(poison)
#' res.mca <- MCA(poison, quanti.sup = 1:2, 
#'               quali.sup = 3:4, graph=FALSE)
#'               
#' fviz_ellipses(res.mca, 3:4, geom = "point",
#' palette = "jco") 
#' 
#'  }
#' @export
fviz_ellipses <- function(X, habillage, axes = c(1,2), 
                          addEllipses = TRUE, ellipse.type = "confidence",
                          palette = NULL, pointsize = 1, geom = c("point", "text"),
                          ggtheme = theme_bw(),...){
  
  df <- facto_summarize(X, element = "ind", axes = axes, result = c("coord"))
  colnames(df)[2:3] <-  c("x", "y")
  # augment data, if qualitative variable is used to color points by groups
  dd <- .add_ind_groups(X, df, habillage)
  df <- dd$ind
  color <- dd$name.quali
  
  if(length(habillage) == 1 || is.factor(habillage)) legend. <- "right" else legend. = "none"
  
  label <- NULL
  if("text" %in% geom) label <- "name"
  p <- ggpubr::ggscatter(df, x = "x", y = "y", color = color, palette = palette,
                    ellipse = addEllipses, ellipse.type = ellipse.type,
                    legend = legend., ggtheme = ggtheme, mean.point = TRUE,
                    label = label, size = pointsize, ...)
  
  # Faceting when multiple variables are used to color individuals
  # in this case there is a column "facet_vars" in df
  if("facet_vars" %in% colnames(df)){
    groups <- c("facet_vars", "Groups")
    xx <- ggpubr::desc_statby(df, measure.var = "x", grps = groups)[, c(groups, "mean")]
    colnames(xx)[ncol(xx)] <- "x"
    yy <- ggpubr::desc_statby(df, measure.var = "y", grps = groups)[, c(groups, "mean")]
    xx$y <- yy$mean
    grp_coord <- xx
    p <- ggpubr::ggtext(grp_coord, x = "x", y = "y", color = color,
                             palette = palette,
                           legend = legend., ggtheme = ggtheme,
                           label = "Groups",  ggp = p, ...) 
    p <- p + facet_wrap(~facet_vars) 
  }
  
  p <- .fviz_finish(p, X, axes,  ...) 
  p
}

#' Hierarchical k-means clustering
#' 
#' @description 
#' The final k-means clustering solution is very sensitive to the initial random selection 
#' of cluster centers. This function provides a solution using an hybrid approach by combining 
#' the hierarchical clustering and the k-means methods. The procedure is explained in "Details" section. Read more:
#'   \href{https://www.datanovia.com/en/lessons/hierarchical-k-means-clustering-optimize-clusters/}{Hybrid hierarchical k-means clustering for optimizing clustering outputs}.
#' \itemize{
#' \item hkmeans(): compute hierarchical k-means clustering
#' \item print.hkmeans(): prints the result of hkmeans
#' \item hkmeans_tree(): plots the initial dendrogram
#' }
#' 
#' @param x a numeric matrix, data frame or vector
#' @param k a single integer specifying the number of clusters to be generated.
#'   Must be at least 2 and smaller than \code{nrow(x)}.
#' @param hc.metric the distance measure to be used. Possible values are "euclidean", "maximum", "manhattan", 
#' "canberra", "binary" or "minkowski" (see ?dist).
#' @param hc.method the agglomeration method to be used. Possible values include "ward.D", "ward.D2", "single", 
#' "complete", "average", "mcquitty", "median"or "centroid" (see ?hclust).
#' @param iter.max the maximum number of iterations allowed for k-means.
#' @param km.algorithm the algorithm to be used for kmeans (see ?kmeans).
#' @details 
#' The procedure is as follow:   
#'      
#' 1. Compute hierarchical clustering
#'    
#' 2. Cut the tree in k-clusters
#'    
#' 3. compute the center (i.e the mean) of each cluster
#'      
#' 4. Do k-means by using the set of cluster centers (defined in step 3) as the initial cluster centers. Optimize the clustering.  
#'     
#' This means that the final optimized partitioning obtained at step 4 might be different from the initial partitioning obtained at step 2. 
#' Consider mainly the result displayed by \code{fviz_cluster()}.
#' 
#' @return hkmeans returns an object of class "hkmeans" containing the following components:
#' \itemize{
#' \item The elements returned by the standard function kmeans() (see ?kmeans)
#' \item data: the data used for the analysis
#' \item hclust: an object of class "hclust" generated by the function hclust()
#' }
#' @examples
#' \donttest{
#' # Load data
#' data(USArrests)
#' # Scale the data
#' df <- scale(USArrests)
#' 
#' # Compute hierarchical k-means clustering
#' res.hk <-hkmeans(df, 4)
#' 
#' # Elements returned by hkmeans()
#' names(res.hk)
#' 
#' # Print the results
#' res.hk
#' 
#' # Visualize the tree
#' hkmeans_tree(res.hk, cex = 0.6)
#' # or use this
#' fviz_dend(res.hk, cex = 0.6)
#' 
#' 
#' # Visualize the hkmeans final clusters
#' fviz_cluster(res.hk, ellipse.type = "norm", ellipse.level = 0.68)
#' }
#' @name hkmeans
#' @rdname hkmeans
#' @export
hkmeans <- function(x, k, hc.metric = "euclidean", hc.method = "ward.D2",
                    iter.max = 10, km.algorithm = "Hartigan-Wong"){
  if(!inherits(x, c("matrix", "data.frame")))
    stop("x must be a matrix or data.frame")
  if(nrow(x) < 2 || ncol(x) < 1)
    stop("x must have at least 2 rows and 1 column")
  if(!is.numeric(k) || length(k) != 1L || is.na(k) || k %% 1 != 0 || k < 2)
    stop("k must be a single integer >= 2")
  # NOTE: do NOT pre-empt k > n here. stats::cutree() below errors natively
  # ("elements of 'k' must be between 1 and N"); a custom early error broke the
  # chooseGCM reverse dependency on CRAN (mirrors the hcut() fix).
  res.hc <- stats::hclust(stats::dist(x, method = hc.metric), method = hc.method)
  grp <- stats::cutree(res.hc, k = k)
  clus.centers <- stats::aggregate(x, list(grp), mean)[, -1]
  res.km <- kmeans(x, centers = clus.centers, 
                   iter.max = iter.max, algorithm = km.algorithm)
  class(res.km) <- "hkmeans"
  res.km$data <- x
  res.km$hclust <- res.hc
  res.km
}

 

#' @rdname hkmeans
#' @export
print.hkmeans <- function (x, ...) 
  {
  cat("Hierarchical K-means clustering with ", length(x$size), " clusters of sizes ", 
      paste(x$size, collapse = ", "), "\n", sep = "")
  cat("\nCluster means:\n")
  print(x$centers, ...)
  cat("\nClustering vector:\n")
  print(x$cluster, ...)
  cat("\nWithin cluster sum of squares by cluster:\n")
  print(x$withinss, ...)
  ratio <- sprintf(" (between_SS / total_SS = %5.1f %%)\n", 
                   100 * x$betweenss/x$totss)
  cat(sub(".", getOption("OutDec"), ratio, fixed = TRUE), "Available components:\n", 
      sep = "\n")
  print(names(x))
  if (!is.null(x$ifault) && x$ifault == 2L) 
    cat("Warning: did *not* converge in specified number of iterations\n")
  invisible(x)
}


#' @rdname hkmeans
#' @param hkmeans an object of class hkmeans (returned by the function hkmeans())
#' @param rect.col Vector with border colors for the rectangles around clusters in dendrogram
#' @param ... others arguments to be passed to the function plot.hclust(); (see ? plot.hclust) 
#' @export
hkmeans_tree <- function(hkmeans, rect.col = NULL, ...)
  {
  res.hk <- hkmeans
  if(is.null(rect.col)) rect.col <- unique(res.hk$cluster)
  plot(res.hk$hclust, hang = -1, sub = "", xlab = "", ...)
  k <- length(unique(res.hk$cluster))
  stats::rect.hclust(res.hk$hclust, k = k, border = rect.col)
}

#' @include facto_summarize.R
NULL
#' Visualize the quality of representation of rows/columns
#' 
#' @description This function can be used to visualize the quality of
#' representation (cos2) of rows/columns from the results of Principal Component
#' Analysis (PCA), Correspondence Analysis (CA), Multiple Correspondence
#' Analysis (MCA), Factor Analysis of Mixed Data (FAMD), Multiple Factor
#' Analysis (MFA) and Hierarchical Multiple Factor Analysis (HMFA) functions.
#' @param X an object of class PCA, CA, MCA, FAMD, MFA and HMFA [FactoMineR];
#'   prcomp and princomp [stats]; dudi, pca, coa and acm [ade4]; ca [ca
#'   package].
#' @param choice allowed values are "row" and "col" for CA;  "var" and "ind" for
#'   PCA or MCA; "var", "ind", "quanti.var", "quali.var" and "group" for FAMD, MFA and HMFA.
#' @param axes a numeric vector specifying the dimension(s) of interest.
#' @param fill a fill color for the bar plot.
#' @param color an outline color for the bar plot.
#' @param sort.val a string specifying whether the value should be sorted. 
#'   Allowed values are "none" (no sorting), "asc" (for ascending) or "desc"
#'   (for descending).
#' @param top a numeric value specifying the number of top elements to be shown.
#' @param xtickslab.rt rotation angle for x axis tick labels. Default is 45 degrees.
#' @inheritParams ggpubr::ggpar
#'   
#' @return a ggplot
#' @author Alboukadel Kassambara \email{alboukadel.kassambara@@gmail.com}
#' @references \url{https://www.sthda.com/english/}
#' @examples
#' \donttest{
#' # Principal component analysis
#' # ++++++++++++++++++++++++++
#' data(decathlon2)
#' decathlon2.active <- decathlon2[1:23, 1:10]
#' res.pca <- prcomp(decathlon2.active,  scale = TRUE)
#' 
#' # variable cos2 on axis 1
#' fviz_cos2(res.pca, choice="var", axes = 1, top = 10 )
#' 
#' # Change color
#' fviz_cos2(res.pca, choice="var", axes = 1,
#'          fill = "lightgray", color = "black") 
#'          
#' # Variable cos2 on axes 1 + 2
#' fviz_cos2(res.pca, choice="var", axes = 1:2)
#' 
#' # cos2 of individuals on axis 1
#' fviz_cos2(res.pca, choice="ind", axes = 1)
#' 
#' \dontrun{
#' # Correspondence Analysis
#' # ++++++++++++++++++++++++++
#' library("FactoMineR")
#' data("housetasks")
#' res.ca <- CA(housetasks, graph = FALSE)
#' 
#' # Visualize row cos2 on axes 1
#' fviz_cos2(res.ca, choice ="row", axes = 1)
#' # Visualize column cos2 on axes 1
#' fviz_cos2(res.ca, choice ="col", axes = 1)
#' 
#' # Multiple Correspondence Analysis
#' # +++++++++++++++++++++++++++++++++
#' library(FactoMineR)
#' data(poison)
#' res.mca <- MCA(poison, quanti.sup = 1:2, 
#'               quali.sup = 3:4, graph=FALSE)
#'               
#' # Visualize individual cos2 on axes 1
#' fviz_cos2(res.mca, choice ="ind", axes = 1, top = 20)
#' # Visualize variable category cos2 on axes 1
#' fviz_cos2(res.mca, choice ="var", axes = 1)
#' 
#' # Multiple Factor Analysis
#' # ++++++++++++++++++++++++
#' library(FactoMineR)
#' data(poison)
#' res.mfa <- MFA(poison, group=c(2,2,5,6), type=c("s","n","n","n"),
#'                name.group=c("desc","desc2","symptom","eat"),
#'                num.group.sup=1:2, graph=FALSE)
#' # Visualize individual cos2 on axes 1
#' # Select the top 20
#' fviz_cos2(res.mfa, choice ="ind", axes = 1, top = 20)
#' # Visualize categorical variable category cos2 on axes 1
#' fviz_cos2(res.mfa, choice ="quali.var", axes = 1)
#' }
#'                
#'  }
#' @export
fviz_cos2 <- function(X, choice = c("row", "col", "var", "ind", "quanti.var", "quali.var", "group"), axes=1,
                   fill="steelblue", color = "steelblue",  
                   sort.val = c("desc", "asc", "none"), top = Inf, 
                   xtickslab.rt = 45, ggtheme = theme_minimal(), ...)
{
   sort.val <- match.arg(sort.val)
   title <- .build_title(choice[1], "Cos2", axes)
  
   dd <- facto_summarize(X, element = choice, result = "cos2", axes = axes)
   cos2 <- dd$cos2
   
   names(cos2) <-rownames(dd)
   
   df <- data.frame(name = factor(names(cos2), levels = names(cos2)), cos2 = cos2)
   
   p <- ggpubr::ggbarplot(df, x = "name", y = "cos2", fill = fill, color = color,
                          sort.val = sort.val, top = top,
                          main = title, xlab = FALSE, ylab ="Cos2 - Quality of representation",
                          xtickslab.rt = xtickslab.rt, ggtheme = ggtheme, ...
   )
  p 
}

---
title: "Extending factoextra to support new analysis backends"
author: "Alboukadel Kassambara"
output:
  rmarkdown::html_vignette:
    toc: true
    toc_depth: 3
vignette: >
  %\VignetteIndexEntry{Extending factoextra to support new analysis backends}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  # All chunks are illustrative: they document the extension contract rather
  # than execute backend code, so the vignette builds without any optional
  # backend package (ExPosition, ade4, vegan, ...) installed.
  eval = FALSE
)
```

## Why this guide exists

`factoextra` visualizes the output of many different multivariate-analysis
packages (FactoMineR, stats, ade4, ca, MASS, ExPosition) through **one unified,
ggplot2-based interface**. The same `fviz_pca_*()`, `fviz_ca_*()`,
`fviz_mca_*()` functions work whether your PCA came from `stats::prcomp()`,
`FactoMineR::PCA()`, `ade4::dudi.pca()` or `ExPosition::epPCA()`.

This is possible because of a strict two-layer design. Adding a **new backend**
(say, an analysis object from another package) almost never requires touching the
plotting code: you only teach a handful of *extractor* functions how to read the
new object. This vignette documents that contract precisely so you can add a
backend yourself and submit it as a pull request.

## Quick start: bring your own coordinates with `as_factoextra_pca()`

Before writing any backend code, check whether the **constructor** is all you
need. If you already have coordinates from *any* method, wrap them with
`as_factoextra_pca()` and the whole `fviz_pca_*()` family works immediately — no
source edits, no S3 methods.

The constructor takes individual coordinates (`ind.coord`) and, optionally,
variable coordinates/loadings (`var.coord`) and eigenvalues (`eig`). Squared
cosines and contributions are derived from the coordinates when you don't supply
them.

Here we *illustrate* it with a base-R PCA (`stats::prcomp()`) — passing its
scores, loadings and eigenvalues through the constructor reproduces a standard
factoextra biplot:

```{r quick-start-prcomp}
library(factoextra)

# A base-R PCA (any analysis that yields coordinates would do)
pca <- prcomp(iris[, -5], scale. = TRUE)

# Wrap the coordinates into a factoextra-ready object
res <- as_factoextra_pca(
  ind.coord = pca$x,         # individual scores  (n x k)
  var.coord = pca$rotation,  # variable loadings  (p x k)
  eig       = pca$sdev^2     # eigenvalues        (optional)
)

# Now the full fviz_pca_* family works on `res`
fviz_pca_biplot(res, label = "var", habillage = iris$Species,
                addEllipses = TRUE)
fviz_eig(res)
fviz_contrib(res, choice = "var", axes = 1)
```

The same pattern turns the output of methods factoextra doesn't natively know
about into elegant biplots — e.g. classical MDS or a UMAP/t-SNE embedding
(coordinates only; supply `var.coord` only if your method has variable loadings):

```{r quick-start-mds}
mds <- stats::cmdscale(dist(scale(mtcars)), k = 3)   # classical MDS
fviz_pca_ind(as_factoextra_pca(ind.coord = mds), repel = TRUE)
```

When you don't pass `eig`, it defaults to the variance of each coordinate column
(the natural eigenvalue), so the scree plot and axis percentages still work. When
you don't pass `cos2`/`contrib`, they are computed from the coordinates
(`cos2` is then the quality of representation *within the supplied dimensions*).

Use the constructor when you have results in hand. Read on if instead you want
factoextra to recognise a **class** of analysis objects directly (so users of
your package can call `fviz_pca_biplot(my_object)` without the constructor).

## The two-layer architecture

For every analysis family, factoextra has two layers:

1. **`get_<method>()` — extract.** Reads coordinates, squared cosines (`cos2`)
   and contributions from a backend object and returns them as a *standardized*
   list. This is where backend-specific knowledge lives.
2. **`fviz_<method>()` — visualize.** Builds the ggplot from that standardized
   list. It is backend-agnostic.

The glue is `facto_summarize()`, which calls `.get_facto_class()` to identify the
analysis type, dispatches to the right `get_*()` extractor, and hands a tidy data
frame to `fviz()`.

> **Key insight.** Once `.get_facto_class()`, `get_eig()` and the relevant
> `get_*()` extractor recognise your class, **every** `fviz_*()`,
> `fviz_contrib()`, `fviz_cos2()` and screeplot function works automatically —
> because none of them hard-code backend class names; they only read `$coord`,
> `$cos2` and `$contrib`.

So extending factoextra means editing **three or four functions**, not fifty.

## The standardized return contract

Every extractor returns a list whose elements are matrices with:

- columns named `Dim.1`, `Dim.2`, ... (one per retained axis), and
- row names equal to the individual / variable / category names.

| Extractor | Required list elements |
|-----------|------------------------|
| `get_pca_ind()` | `$coord`, `$cos2`, `$contrib` |
| `get_pca_var()` | `$coord`, `$cor`, `$cos2`, `$contrib` |
| `get_ca_row()` / `get_ca_col()` | `$coord`, `$cos2`, `$contrib`, `$inertia` |
| `get_mca_ind()` / `get_mca_var()` | `$coord`, `$cos2`, `$contrib` |

The list is given S3 class `c("factoextra", ...)`. `get_eig()` returns a data
frame (see below). Contributions are expressed as **percentages** (note the
`* 100` in the examples).

## Step 1 — Teach `.get_facto_class()` your class

`.get_facto_class()` (in `R/utilities.R`) maps a backend object to one canonical
analysis token: `"PCA"`, `"CA"`, `"MCA"`, `"FAMD"`, `"MFA"` or `"HMFA"`. Add one
`else if` branch before the final `stop()`:

```{r}
# inside .get_facto_class()
else if (inherits(X, "myPCA"))            # your new class
  facto_class <- "PCA"
```

For a *wrapper* object (a list that carries the real result in a slot), test the
inner class — this is exactly how ExPosition is handled:

```{r}
else if (inherits(X, "expoOutput")) {
  if      (inherits(X$ExPosition.Data, "epCA"))  facto_class <- "CA"
  else if (inherits(X$ExPosition.Data, "epPCA")) facto_class <- "PCA"
  else if (inherits(X$ExPosition.Data, "epMCA")) facto_class <- "MCA"
}
```

## Step 2 — Teach `get_eig()` your eigenvalues

`get_eig()` (in `R/eigenvalue.R`) returns a data frame with exactly these three
columns and `Dim.N` row names:

```
              eigenvalue variance.percent cumulative.variance.percent
Dim.1            2.918             72.96                       72.96
Dim.2            0.914             22.85                       95.81
...
```

You only need to supply the raw eigenvalues; the helper normalizes them to
percentages and cumulative percentages. Add a branch:

```{r}
# inside get_eig(), before the final stop()
else if (inherits(X, "myPCA"))
  eig <- X$eigenvalues          # a numeric vector of eigenvalues
```

If your object already stores a FactoMineR-style `$eig` data frame, it is used
as-is. The ExPosition branch is a one-liner:

```{r}
else if (inherits(X, "expoOutput"))
  eig <- X$ExPosition.Data$eigs
```

## Step 3 — Teach the `get_*()` extractor(s)

Add one `else if` branch per extractor for your method. Below are the real
templates you can copy.

### PCA

`get_pca_ind()` and `get_pca_var()` live in `R/get_pca.R`. The two existing
templates are `prcomp` (stats) and `dudi` (ade4):

```{r}
# get_pca_ind(): stats::prcomp template
else if (inherits(res.pca, "prcomp")) {
  ind.coord <- res.pca$x
  data      <- .prcomp_reconst(res.pca)
  ind <- .get_pca_ind_results(ind.coord, data, res.pca$sdev^2,
                              res.pca$center, res.pca$scale)
}

# get_pca_ind(): ade4 dudi template
else if (inherits(res.pca, "pca") && inherits(res.pca, "dudi")) {
  ind.coord <- res.pca$li
  data <- sweep(res.pca$tab, 2, res.pca$norm, "*")
  data <- sweep(data,        2, res.pca$cent, "+")
  ind <- .get_pca_ind_results(ind.coord, data, res.pca$eig,
                              res.pca$cent, res.pca$norm)
}
```

```{r}
# get_pca_var(): stats::prcomp template
else if (inherits(res.pca, "prcomp")) {
  var.cor <- sweep(res.pca$rotation, 2, res.pca$sdev, "*")
  var <- .get_pca_var_results(var.cor)
}

# get_pca_var(): ade4 dudi template
else if (inherits(res.pca, "pca") && inherits(res.pca, "dudi")) {
  var <- .get_pca_var_results(res.pca$co)
}
```

The helpers `.get_pca_ind_results()` and `.get_pca_var_results()` compute `cos2`
and `contrib` for you, so if you can produce coordinates you are nearly done.

### CA (and the mass helper you must not forget)

For CA backends edit `get_ca_row()` and `get_ca_col()` in `R/get_ca.R`, returning
`$coord`, `$cos2`, `$contrib` and `$inertia` with `Dim.N` columns:

```{r}
# get_ca_row(): build the standardized list directly
else if (inherits(res.ca, "myCA")) {
  coord   <- res.ca$row.coord
  cos2    <- res.ca$row.cos2
  contrib <- res.ca$row.contrib * 100
  inertia <- res.ca$row.inertia
  colnames(coord) <- colnames(cos2) <- colnames(contrib) <-
    paste0("Dim.", seq_len(ncol(coord)))
  row <- list(coord = coord, cos2 = cos2, contrib = contrib, inertia = inertia)
}
```

**Don't forget `.get_ca_mass()`** (in `R/utilities.R`). CA biplot scaling
(symmetric / row-/col-principal maps) needs row and column masses. Add a branch
returning a named numeric vector of masses:

```{r}
# inside .get_ca_mass()
else if (inherits(res.ca, "myCA")) {
  if      (element == "row") mass <- res.ca$row.mass
  else if (element == "col") mass <- res.ca$col.mass
}
```

### MCA

Edit `get_mca_ind()` and `get_mca_var()` in `R/get_mca.R`, returning `$coord`,
`$cos2`, `$contrib` (same `Dim.N` convention as above).

## A complete worked example: ExPosition

ExPosition is the most recent multi-family backend (PCA + CA + MCA) and is the
best template to study. Every place it is wired in:

| Extension point | Location (file / line) |
|-----------------|-----------|
| Class dispatch (wrapper) | `R/utilities.R:103-107` |
| Eigenvalues | `R/eigenvalue.R:123-124` |
| `get_pca_ind()` | `R/get_pca.R:92-95` |
| `get_pca_var()` | `R/get_pca.R:133-139` |
| `get_ca_col()` | `R/get_ca.R:127-136` |
| `get_ca_row()` | `R/get_ca.R:213-221` |
| `.get_ca_mass()` | `R/utilities.R:473-476` |
| `get_mca_var()` | `R/get_mca.R:120-129` |
| `get_mca_ind()` | `R/get_mca.R:168-175` |

For instance the PCA-individual branch simply maps ExPosition's slots onto the
standardized names:

```{r}
# R/get_pca.R — ExPosition epPCA, individuals
else if (inherits(res.pca, "expoOutput") &&
         inherits(res.pca$ExPosition.Data, "epPCA")) {
  res <- res.pca$ExPosition.Data
  ind <- list(coord = res$fi, cos2 = res$ri, contrib = res$ci * 100)
}
```

Once these branches exist, the full ExPosition workflow plots like any other:

```{r}
library(ExPosition)
library(factoextra)

res.pca <- epPCA(iris[, -5], graph = FALSE)
fviz_pca_ind(res.pca, habillage = iris$Species, addEllipses = TRUE)
fviz_eig(res.pca)
fviz_contrib(res.pca, choice = "var")
```

## A forward-looking template: vegan (RDA / CCA)

Ecology users often ask for `vegan::rda()` / `vegan::cca()` support. These are
not bundled (they are method-specific), but they are a good illustration of how
you would add a backend yourself. A constrained ordination is CA-like, so you
would map vegan's scores onto the CA extractors. **This is a sketch, not a
supported path** — treat it as a starting point for a contribution:

```{r}
# .get_facto_class(): treat an unconstrained CA component as CA
else if (inherits(X, c("cca", "rda")))
  facto_class <- "CA"

# get_eig(): vegan stores eigenvalues per axis
else if (inherits(X, c("cca", "rda")))
  eig <- vegan::eigenvals(X)

# get_ca_row()/get_ca_col(): use vegan::scores() for sites / species
else if (inherits(res.ca, c("cca", "rda"))) {
  sc      <- vegan::scores(res.ca, display = "sites", scaling = "sites")
  coord   <- as.matrix(sc)
  colnames(coord) <- paste0("Dim.", seq_len(ncol(coord)))
  # cos2 / contrib / inertia derived from the ordination as appropriate
  row <- list(coord = coord, cos2 = cos2, contrib = contrib, inertia = inertia)
}
```

The exact mapping (scaling choice, how to derive `cos2`/`contrib`, constrained
vs. unconstrained axes) is the real work and is method-specific — which is why it
belongs in a focused contribution rather than the core package.

## Testing your backend

Mirror the existing smoke tests in
`tests/testthat/test-ca-backends-smoke.R`. They assert the standardized contract
holds, which is exactly what a new backend must satisfy:

```{r}
test_that("CA extractors work with <your package> outputs", {
  skip_if_not_installed("yourpkg")
  data("housetasks", package = "factoextra")
  res <- yourpkg::your_ca(housetasks)

  cols <- get_ca_col(res)
  rows <- get_ca_row(res)

  expect_s3_class(cols, "factoextra")
  expect_true(all(c("coord", "cos2", "contrib") %in% names(cols)))
  expect_equal(nrow(cols$coord), ncol(housetasks))
  expect_equal(nrow(rows$coord), nrow(housetasks))
})
```

Use `skip_if_not_installed()` so the test is skipped when the optional backend is
absent (backends belong in `Suggests`, never `Imports`).

## Submitting a backend: checklist

1. Add the needed `else if` branches: `.get_facto_class()`, `get_eig()`, the
   relevant `get_*()` extractor(s), and `.get_ca_mass()` for CA.
2. Keep changes **gated** — each new branch only fires for the new class, so all
   existing backends are byte-identical (factoextra's non-regression policy).
3. Add a smoke test mirroring `test-ca-backends-smoke.R`, guarded with
   `skip_if_not_installed()`.
4. Put the backend package in `Suggests` (not `Imports`).
5. Update `NEWS.md` and open a pull request at
   <https://github.com/kassambara/factoextra/issues>.

That's it — no `fviz_*()` changes required.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eigenvalue.R
\name{eigenvalue}
\alias{eigenvalue}
\alias{get_eig}
\alias{get_eigenvalue}
\alias{fviz_eig}
\alias{fviz_screeplot}
\title{Extract and visualize the eigenvalues/variances of dimensions}
\usage{
get_eig(X)

get_eigenvalue(X)

fviz_eig(
  X,
  choice = c("variance", "eigenvalue"),
  geom = c("bar", "line"),
  barfill = "steelblue",
  barcolor = "steelblue",
  linecolor = "black",
  ncp = 10,
  addlabels = FALSE,
  hjust = 0,
  main = NULL,
  xlab = NULL,
  ylab = NULL,
  ggtheme = theme_minimal(),
  parallel = FALSE,
  parallel.color = "red",
  parallel.lty = "dashed",
  parallel.iter = 100,
  parallel.seed = NULL,
  ...
)

fviz_screeplot(...)
}
\arguments{
\item{X}{an object of class PCA, CA, MCA, FAMD, MFA and HMFA [FactoMineR]; prcomp 
and princomp [stats]; dudi, pca, coa and acm [ade4]; ca and mjca [ca 
package].}

\item{choice}{a text specifying the data to be plotted. Allowed values are 
"variance" or "eigenvalue".}

\item{geom}{a text specifying the geometry to be used for the graph. Allowed 
values are "bar" for barplot, "line" for lineplot or c("bar", "line") to use
both types.}

\item{barfill}{fill color for bar plot.}

\item{barcolor}{outline color for bar plot.}

\item{linecolor}{color for line plot (when geom contains "line").}

\item{ncp}{a single positive integer specifying the number of dimensions to
be shown. Integer-like numeric values are accepted.}

\item{addlabels}{logical value. If TRUE, labels are added at the top of bars
or points showing the information retained by each dimension.}

\item{hjust}{horizontal adjustment of the labels.}

\item{main, xlab, ylab}{plot main and axis titles.}

\item{ggtheme}{function, ggplot2 theme name. Default value is theme_pubr().
Allowed values include ggplot2 official themes: theme_gray(), theme_bw(),
theme_minimal(), theme_classic(), theme_void(), ....}

\item{parallel}{logical value. If TRUE, adds a parallel analysis threshold line
(Horn's method) to help determine the number of components to retain. Components
with eigenvalues above this line are considered significant. Only works when
choice = "eigenvalue" and X is a prcomp or princomp object. Default is FALSE.}

\item{parallel.color}{color of the parallel analysis threshold line. Default is "red".}

\item{parallel.lty}{line type for the parallel analysis line. Default is "dashed".}

\item{parallel.iter}{a single positive integer giving the number of
iterations for parallel analysis simulation. Integer-like numeric values
are accepted. Default is 100.}

\item{parallel.seed}{NULL or a single non-negative integer seed for
reproducible parallel analysis simulation. If NULL (default), the current
RNG stream is used. Integer-like numeric values are accepted.}

\item{...}{optional arguments to be passed to the function \link[ggpubr]{ggpar}.}
}
\value{
\itemize{ \item get_eig() (or get_eigenvalue()): returns a data.frame
 containing 3 columns: the eigenvalues, the percentage of variance and  the
 cumulative percentage of variance retained by each dimension.
 \item fviz_eig() (or fviz_screeplot()): returns a ggplot2 }
}
\description{
Eigenvalues correspond to the amount of the variation explained 
 by each principal component (PC). 
 
 \itemize{ \item get_eig(): Extract the eigenvalues/variances of the
 principal dimensions \item fviz_eig(): Plot the eigenvalues/variances
 against the number of dimensions \item get_eigenvalue(): an alias of
 get_eig() \item fviz_screeplot(): an alias of fviz_eig() }
 
 These functions support the results of Principal Component Analysis (PCA), 
 Correspondence Analysis (CA), Multiple Correspondence Analysis (MCA), Factor Analysis of Mixed Data (FAMD),
 Multiple Factor Analysis (MFA) and Hierarchical Multiple Factor Analysis
 (HMFA) functions. \code{fviz_eig()} validates \code{ncp},
 \code{parallel.iter}, and \code{parallel.seed} before plotting, accepting
 integer-like numeric values while still rejecting fractional inputs.
}
\examples{
# Principal Component Analysis
# ++++++++++++++++++++++++++
data(iris)
res.pca <- prcomp(iris[, -5],  scale = TRUE)

# Extract eigenvalues/variances
get_eig(res.pca)

# Default plot
fviz_eig(res.pca, addlabels = TRUE, ylim = c(0, 85))
  
# Scree plot - Eigenvalues
fviz_eig(res.pca, choice = "eigenvalue", addlabels=TRUE)

# Use only bar  or line plot: geom = "bar" or geom = "line"
fviz_eig(res.pca, geom="line")

# Parallel analysis (Horn's method) to determine number of components
# Components with eigenvalues above the red line are significant
fviz_eig(res.pca, choice = "eigenvalue", parallel = TRUE,
         addlabels = TRUE, parallel.color = "red",
         parallel.iter = 10, parallel.seed = 123)

\dontrun{         
# Correspondence Analysis
# +++++++++++++++++++++++++++++++++
library(FactoMineR)
data(housetasks)
res.ca <- CA(housetasks, graph = FALSE)
get_eig(res.ca)
fviz_eig(res.ca, linecolor = "#FC4E07",
   barcolor = "#00AFBB", barfill = "#00AFBB")

# Multiple Correspondence Analysis
# +++++++++++++++++++++++++++++++++
library(FactoMineR)
data(poison)
res.mca <- MCA(poison, quanti.sup = 1:2, 
              quali.sup = 3:4, graph=FALSE)
get_eig(res.mca)
fviz_eig(res.mca, linecolor = "#FC4E07",
   barcolor = "#2E9FDF", barfill = "#2E9FDF")
}

}
\references{
\url{https://www.sthda.com/english/}
}
\seealso{
\code{\link{fviz_pca}}, \code{\link{fviz_ca}},
 \code{\link{fviz_mca}}, \code{\link{fviz_mfa}}, \code{\link{fviz_hmfa}}
}
\author{
Alboukadel Kassambara \email{alboukadel.kassambara@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fviz.R
\name{fviz}
\alias{fviz}
\title{Visualizing Multivariate Analysis Outputs}
\usage{
fviz(
  X,
  element,
  axes = c(1, 2),
  geom = "auto",
  label = "all",
  invisible = "none",
  labelsize = 4,
  pointsize = 1.5,
  pointshape = 19,
  arrowsize = 0.5,
  arrow.linetype = "solid",
  habillage = "none",
  addEllipses = FALSE,
  ellipse.level = 0.95,
  ellipse.type = "norm",
  ellipse.alpha = 0.1,
  mean.point = TRUE,
  color = "black",
  fill = "white",
  alpha = 1,
  gradient.cols = NULL,
  col.row.sup = "darkblue",
  col.col.sup = "darkred",
  select = list(name = NULL, cos2 = NULL, contrib = NULL),
  title = NULL,
  axes.linetype = "dashed",
  repel = FALSE,
  col.circle = "grey70",
  circlesize = 0.5,
  add.circle = NULL,
  rotate.labels = FALSE,
  ggtheme = theme_minimal(),
  ggp = NULL,
  font.family = "",
  ...
)
}
\arguments{
\item{X}{an object of class PCA, CA, MCA, FAMD, MFA and HMFA [FactoMineR]; prcomp
and princomp [stats]; dudi, pca, coa and acm [ade4]; ca [ca package]; expoOutput [ExPosition].}

\item{element}{the element to subset from the output. Possible values are
"row" or "col" for CA; "var", "ind", "mca.cor" or "quanti.sup" for MCA;
"var" or "ind" for PCA; and 'quanti.var', 'quali.var', 'quali.sup',
'group' or 'ind' for FAMD, MFA and HMFA.}

\item{axes}{a numeric vector specifying the axes of interest. Values must be
positive integer indices within the available dimensions. Default values
are 1:2 for axes 1 and 2.}

\item{geom}{a text specifying the geometry to be used for the graph. Default 
value is "auto". Allowed values are the combination of c("point", "arrow", 
"text"). Use "point" (to show only points); "text" to show only labels; 
c("point", "text") or c("arrow", "text") to show both types.}

\item{label}{a text specifying the elements to be labelled. Default value is 
"all". Allowed values are "none" or the combination of c("ind", "ind.sup", 
"quali", "var", "quanti.sup", "group.sup"). "ind" can be used to label only 
active individuals. "ind.sup" is for supplementary individuals. "quali" is 
for supplementary qualitative variables. "var" is for active variables. 
"quanti.sup" is for quantitative supplementary variables.}

\item{invisible}{a text specifying the elements to be hidden on the plot. 
Default value is "none". Allowed values are the combination of c("ind", 
"ind.sup", "quali", "var", "quanti.sup", "group.sup").}

\item{labelsize}{font size for the labels}

\item{pointsize}{the size of points}

\item{pointshape}{the shape of points}

\item{arrowsize}{the size of arrows. Controls the thickness of arrows.}

\item{arrow.linetype}{linetype of the variable arrows (e.g. "solid",
"dashed", "dotted"). Default is "solid".}

\item{habillage}{an optional factor variable for coloring the observations by 
groups. Default value is "none". If X is a PCA object from FactoMineR 
package, habillage can also specify the supplementary qualitative variable 
(by its index or name) to be used for coloring individuals by groups (see 
?PCA in FactoMineR).}

\item{addEllipses}{logical value. If TRUE, draws ellipses around the 
individuals when habillage != "none".}

\item{ellipse.level}{the size of the concentration ellipse in normal 
probability.}

\item{ellipse.type}{Character specifying frame type. Possible values are 
\code{"convex"}, \code{"confidence"} or types supported by 
\code{\link[ggplot2]{stat_ellipse}()} including one of \code{c("t", "norm", 
"euclid")} for plotting concentration ellipses.

\itemize{ \item \code{"convex"}: plot convex hull of a set of points. \item 
\code{"confidence"}: plot confidence ellipses around group mean points as
\code{\link[FactoMineR]{coord.ellipse}()}[in FactoMineR]. \item \code{"t"}:
assumes a multivariate t-distribution. \item \code{"norm"}: assumes a
multivariate normal distribution. \item \code{"euclid"}: draws a circle with
the radius equal to level, representing the euclidean distance from the
center. This ellipse probably won't appear circular unless
\code{\link[ggplot2]{coord_fixed}()} is applied.}}

\item{ellipse.alpha}{Alpha for ellipse specifying the transparency level of 
fill color. Use alpha = 0 for no fill color.}

\item{mean.point}{logical value. If TRUE (default), group mean points are 
added to the plot.}

\item{color}{color to be used for the specified geometries (point, text). Can 
be a continuous variable or a factor variable. Possible values include also 
: "cos2", "contrib", "coord", "x" or "y". In this case, the colors for 
individuals/variables are automatically controlled by their qualities of 
representation ("cos2"), contributions ("contrib"), coordinates (x^2+y^2, 
"coord"), x values ("x") or y values ("y"). To use automatic coloring (by 
cos2, contrib, ....), make sure that habillage ="none".}

\item{fill}{same as the argument \code{color}, but for point fill color. 
Useful when pointshape = 21, for example.}

\item{alpha}{controls the transparency of individual and variable colors, 
respectively. The value can variate from 0 (total transparency) to 1 (no 
transparency). Default value is 1. Possible values include also : "cos2", 
"contrib", "coord", "x" or "y". In this case, the transparency for the 
individual/variable colors are automatically controlled by their qualities 
("cos2"), contributions ("contrib"), coordinates (x^2+y^2, "coord"), x 
values("x") or y values("y"). To use this, make sure that habillage ="none".}

\item{gradient.cols}{vector of colors to use for n-colour gradient. Allowed
values include brewer and ggsci color palettes.}

\item{col.col.sup, col.row.sup}{colors for the supplementary column and row 
points, respectively.}

\item{select}{a selection of individuals/variables to be drawn. Allowed values
are NULL or a list containing the arguments name, cos2 or contrib: \itemize{
\item name: is a character vector containing individuals/variables to be 
drawn \item cos2: if cos2 is in [0, 1], ex: 0.6, then individuals/variables 
with a cos2 > 0.6 are drawn. if cos2 > 1, ex: 5, then the top 5 
individuals/variables with the highest cos2 are drawn. \item contrib: if 
contrib > 1, ex: 5,  then the top 5 individuals/variables with the highest 
contrib are drawn }}

\item{title}{the title of the graph}

\item{axes.linetype}{linetype of x and y axes.}

\item{repel}{a boolean, whether to use ggrepel to avoid overplotting text
labels or not. The old \code{jitter} argument is kept for backward
compatibility and is converted to \code{repel = TRUE} with a deprecation warning.}

\item{col.circle}{a color for the correlation circle. Used only when X is a 
PCA output.}

\item{circlesize}{the size of the variable correlation circle.}

\item{add.circle}{logical or NULL controlling the variable correlation circle.
Default NULL shows it only when meaningful: for PCA variables on
unit-variance (scaled) data, and for quantitative variables of
MCA/MFA/HMFA/FAMD. Use \code{add.circle = TRUE} to force the circle (e.g. a
\code{prcomp(scale = FALSE)} fit on data you scaled manually) or
\code{add.circle = FALSE} to suppress it.}

\item{rotate.labels}{logical. If \code{TRUE}, the text labels of the plotted
element are rotated to the angle of their arrows (ggbiplot style); labels in
the left half-plane are flipped to stay upright. Default is \code{FALSE}
(no rotation). Use together with \code{repel = FALSE}, as ggrepel ignores the
label angle. Most useful for variable plots/biplots (e.g.
\code{fviz_pca_var}, \code{fviz_pca_biplot}).}

\item{ggtheme}{function, ggplot2 theme name. Default value is theme_pubr().
Allowed values include ggplot2 official themes: theme_gray(), theme_bw(),
theme_minimal(), theme_classic(), theme_void(), ....}

\item{ggp}{a ggplot. If not NULL, points are added to an existing plot.}

\item{font.family}{character vector specifying font family.}

\item{...}{Arguments to be passed to the functions ggpubr::ggscatter() & 
ggpubr::ggpar().}
}
\value{
a ggplot
}
\description{
Generic function to create a scatter plot of multivariate analysis
 outputs, including PCA, CA, MCA and MFA.
}
\examples{
\donttest{
# Principal component analysis
# +++++++++++++++++++++++++++++
data(decathlon2)
decathlon2.active <- decathlon2[1:23, 1:10]
res.pca <- prcomp(decathlon2.active,  scale = TRUE)
fviz(res.pca, "ind") # Individuals plot
fviz(res.pca, "var") # Variables plot

# Correspondence Analysis
# ++++++++++++++++++++++++++
# Install and load FactoMineR to compute CA
# install.packages("FactoMineR")
library("FactoMineR")
data("housetasks")
res.ca <- CA(housetasks, graph = FALSE)
fviz(res.ca, "row") # Rows plot
fviz(res.ca, "col") # Columns plot

# Multiple Correspondence Analysis
# +++++++++++++++++++++++++++++++++
library(FactoMineR)
data(poison)
res.mca <- MCA(poison, quanti.sup = 1:2, 
              quali.sup = 3:4, graph=FALSE)
              
fviz(res.mca, "ind") # Individuals plot
fviz(res.mca, "var") # Variables plot

 }
}
\author{
Alboukadel Kassambara \email{alboukadel.kassambara@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_pca.R
\name{get_pca}
\alias{get_pca}
\alias{get_pca_ind}
\alias{get_pca_var}
\title{Extract the results for individuals/variables - PCA}
\usage{
get_pca(res.pca, element = c("var", "ind"))

get_pca_ind(res.pca, ...)

get_pca_var(res.pca)
}
\arguments{
\item{res.pca}{an object of class PCA [FactoMineR]; 
prcomp and princomp [stats]; pca, dudi [adea4]; epPCA [ExPosition].}

\item{element}{the element to subset from the output. Allowed values are 
"var" (for active variables) or "ind" (for active individuals).}

\item{...}{not used}
}
\value{
a list of matrices containing all the results for the active individuals/variables including: 
\item{coord}{coordinates for the individuals/variables}
\item{cos2}{cos2 for the individuals/variables}
\item{contrib}{contributions of the individuals/variables}
}
\description{
Extract all the results (coordinates, squared cosine, contributions) for 
the active individuals/variables from Principal Component Analysis (PCA) outputs.\cr\cr
\itemize{
\item get_pca(): Extract the results for variables and individuals
\item get_pca_ind(): Extract the results for individuals only
\item get_pca_var(): Extract the results for variables only
}
}
\examples{
\donttest{
# Principal Component Analysis
# +++++++++++++++++++++++++++++
 data(iris)
 res.pca <- prcomp(iris[, -5],  scale = TRUE)
 # Extract the results for individuals
 ind <- get_pca_ind(res.pca)
 print(ind)
 head(ind$coord) # coordinates of individuals
 head(ind$cos2) # cos2 of individuals
 head(ind$contrib) # contributions of individuals
 
 # Extract the results for variables
 var <- get_pca_var(res.pca)
 print(var)
 head(var$coord) # coordinates of variables
 head(var$cos2) # cos2 of variables
 head(var$contrib) # contributions of variables
 
 # You can also use the function get_pca()
 get_pca(res.pca, "ind") # Results for individuals
 get_pca(res.pca, "var") # Results for variable categories
 }
}
\references{
\url{https://www.sthda.com/english/}
}
\author{
Alboukadel Kassambara \email{alboukadel.kassambara@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fviz_cos2.R
\name{fviz_cos2}
\alias{fviz_cos2}
\title{Visualize the quality of representation of rows/columns}
\usage{
fviz_cos2(
  X,
  choice = c("row", "col", "var", "ind", "quanti.var", "quali.var", "group"),
  axes = 1,
  fill = "steelblue",
  color = "steelblue",
  sort.val = c("desc", "asc", "none"),
  top = Inf,
  xtickslab.rt = 45,
  ggtheme = theme_minimal(),
  ...
)
}
\arguments{
\item{X}{an object of class PCA, CA, MCA, FAMD, MFA and HMFA [FactoMineR];
prcomp and princomp [stats]; dudi, pca, coa and acm [ade4]; ca [ca
package].}

\item{choice}{allowed values are "row" and "col" for CA;  "var" and "ind" for
PCA or MCA; "var", "ind", "quanti.var", "quali.var" and "group" for FAMD, MFA and HMFA.}

\item{axes}{a numeric vector specifying the dimension(s) of interest.}

\item{fill}{a fill color for the bar plot.}

\item{color}{an outline color for the bar plot.}

\item{sort.val}{a string specifying whether the value should be sorted. 
Allowed values are "none" (no sorting), "asc" (for ascending) or "desc"
(for descending).}

\item{top}{a numeric value specifying the number of top elements to be shown.}

\item{xtickslab.rt}{rotation angle for x axis tick labels. Default is 45 degrees.}

\item{ggtheme}{function, ggplot2 theme name. Default value is theme_pubr().
Allowed values include ggplot2 official themes: theme_gray(), theme_bw(),
theme_minimal(), theme_classic(), theme_void(), ....}

\item{...}{not used}
}
\value{
a ggplot
}
\description{
This function can be used to visualize the quality of
representation (cos2) of rows/columns from the results of Principal Component
Analysis (PCA), Correspondence Analysis (CA), Multiple Correspondence
Analysis (MCA), Factor Analysis of Mixed Data (FAMD), Multiple Factor
Analysis (MFA) and Hierarchical Multiple Factor Analysis (HMFA) functions.
}
\examples{
\donttest{
# Principal component analysis
# ++++++++++++++++++++++++++
data(decathlon2)
decathlon2.active <- decathlon2[1:23, 1:10]
res.pca <- prcomp(decathlon2.active,  scale = TRUE)

# variable cos2 on axis 1
fviz_cos2(res.pca, choice="var", axes = 1, top = 10 )

# Change color
fviz_cos2(res.pca, choice="var", axes = 1,
         fill = "lightgray", color = "black") 
         
# Variable cos2 on axes 1 + 2
fviz_cos2(res.pca, choice="var", axes = 1:2)

# cos2 of individuals on axis 1
fviz_cos2(res.pca, choice="ind", axes = 1)

\dontrun{
# Correspondence Analysis
# ++++++++++++++++++++++++++
library("FactoMineR")
data("housetasks")
res.ca <- CA(housetasks, graph = FALSE)

# Visualize row cos2 on axes 1
fviz_cos2(res.ca, choice ="row", axes = 1)
# Visualize column cos2 on axes 1
fviz_cos2(res.ca, choice ="col", axes = 1)

# Multiple Correspondence Analysis
# +++++++++++++++++++++++++++++++++
library(FactoMineR)
data(poison)
res.mca <- MCA(poison, quanti.sup = 1:2, 
              quali.sup = 3:4, graph=FALSE)
              
# Visualize individual cos2 on axes 1
fviz_cos2(res.mca, choice ="ind", axes = 1, top = 20)
# Visualize variable category cos2 on axes 1
fviz_cos2(res.mca, choice ="var", axes = 1)

# Multiple Factor Analysis
# ++++++++++++++++++++++++
library(FactoMineR)
data(poison)
res.mfa <- MFA(poison, group=c(2,2,5,6), type=c("s","n","n","n"),
               name.group=c("desc","desc2","symptom","eat"),
               num.group.sup=1:2, graph=FALSE)
# Visualize individual cos2 on axes 1
# Select the top 20
fviz_cos2(res.mfa, choice ="ind", axes = 1, top = 20)
# Visualize categorical variable category cos2 on axes 1
fviz_cos2(res.mfa, choice ="quali.var", axes = 1)
}
               
 }
}
\references{
\url{https://www.sthda.com/english/}
}
\author{
Alboukadel Kassambara \email{alboukadel.kassambara@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_hmfa.R
\name{get_hmfa}
\alias{get_hmfa}
\alias{get_hmfa_ind}
\alias{get_hmfa_var}
\alias{get_hmfa_partial}
\title{Extract the results for individuals/variables/group/partial axes - HMFA}
\usage{
get_hmfa(
  res.hmfa,
  element = c("ind", "quanti.var", "quali.var", "group", "partial.node")
)

get_hmfa_ind(res.hmfa)

get_hmfa_var(res.hmfa, element = c("quanti.var", "quali.var", "group"))

get_hmfa_partial(res.hmfa)
}
\arguments{
\item{res.hmfa}{an object of class HMFA [FactoMineR].}

\item{element}{the element to subset from the output. Possible values are
"ind", "quanti.var", "quali.var", "group" or "partial.node".}
}
\value{
a list of matrices containing the results for the active 
  individuals, variables, groups and partial nodes, including : 
  \item{coord}{coordinates} \item{cos2}{cos2} 
  \item{contrib}{contributions}
}
\description{
Extract all the results (coordinates, squared cosine and
contributions) for the active individuals/quantitative variables/qualitative
variable categories/groups/partial axes from Hierarchical Multiple Factor
Analysis (HMFA) outputs.\cr\cr \itemize{ \item get_hmfa(): Extract the
results for variables and individuals \item get_hmfa_ind(): Extract the
results for individuals only \item get_mfa_var(): Extract the results for
variables (quantitatives, qualitatives and groups) 
\item get_hmfa_partial(): Extract the results for partial.node. }
}
\examples{
# Multiple Factor Analysis
# ++++++++++++++++++++++++
# Install and load FactoMineR to compute MFA
# install.packages("FactoMineR")
library("FactoMineR")
data(wine)
hierar <- list(c(2,5,3,10,9,2), c(4,2))
res.hmfa <- HMFA(wine, H = hierar, type=c("n",rep("s",5)), graph = FALSE)
 
 # Extract the results for qualitative variable categories
 var <- get_hmfa_var(res.hmfa, "quali.var")
 print(var)
 head(var$coord) # coordinates of qualitative variables
 head(var$cos2) # cos2 of qualitative variables
 head(var$contrib) # contributions of qualitative variables
 
 # Extract the results for individuals
 ind <- get_hmfa_ind(res.hmfa)
 print(ind)
 head(ind$coord) # coordinates of individuals
 head(ind$cos2) # cos2 of individuals
 head(ind$contrib) # contributions of individuals
 
 # You can also use the function get_hmfa()
 get_hmfa(res.hmfa, "ind") # Results for individuals
 get_hmfa(res.hmfa, "quali.var") # Results for qualitative variable categories
 
}
\author{
Alboukadel Kassambara \email{alboukadel.kassambara@gmail.com}

Fabian Mundt \email{f.mundt@inventionate.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fviz_dend.R
\name{fviz_dend}
\alias{fviz_dend}
\title{Enhanced Visualization of Dendrogram}
\usage{
fviz_dend(
  x,
  k = NULL,
  h = NULL,
  k_colors = NULL,
  palette = NULL,
  show_labels = TRUE,
  color_labels_by_k = TRUE,
  match_coord_colors = FALSE,
  label_cols = NULL,
  labels_font = "plain",
  labels_track_height = NULL,
  repel = FALSE,
  lwd = 0.7,
  type = c("rectangle", "circular", "phylogenic"),
  phylo_layout = "layout.auto",
  rect = FALSE,
  rect_border = "gray",
  rect_lty = 2,
  rect_fill = FALSE,
  lower_rect,
  horiz = FALSE,
  cex = 0.8,
  main = "Cluster Dendrogram",
  xlab = "",
  ylab = "Height",
  sub = NULL,
  ggtheme = theme_classic(),
  ...
)
}
\arguments{
\item{x}{an object of class dendrogram, hclust, agnes, diana, hcut, 
hkmeans or HCPC (FactoMineR).}

\item{k}{the number of groups for cutting the tree.}

\item{h}{a numeric value. Cut the dendrogram by cutting at height h. (k 
overrides h)}

\item{k_colors, palette}{a vector containing colors to be used for the groups.
It should contains k number of colors. Allowed values include also "grey" 
for grey color palettes; brewer palettes e.g. "RdBu", "Blues", ...;  and 
scientific journal palettes from ggsci R package, e.g.: "npg", "aaas", 
"lancet", "jco", "ucscgb", "uchicago", "simpsons" and "rickandmorty".}

\item{show_labels}{a logical value. If TRUE, leaf labels are shown. Default 
value is TRUE.}

\item{color_labels_by_k}{logical value. If TRUE, labels are colored
automatically by group when k != NULL.}

\item{match_coord_colors}{logical value. Default is FALSE, where dendrogram
colors follow the left-to-right leaf order. If TRUE, cluster colors are
remapped to cluster-label order so they match \code{\link{fviz_cluster}()}
and \code{\link{fviz_silhouette}()} for the same clustering.}

\item{label_cols}{a vector containing the colors for labels.}

\item{labels_font}{font face for the leaf labels of "rectangle"/"circular"
dendrograms. One of "plain" (default), "bold", "italic" or "bold.italic".
Default "plain" leaves labels unchanged.}

\item{labels_track_height}{a positive numeric value for adjusting the room for the 
labels. Used only when type = "rectangle".}

\item{repel}{logical value. Use repel = TRUE to avoid label overplotting when
type = "phylogenic".}

\item{lwd}{a numeric value specifying dendrogram branch and rectangle line
width.}

\item{type}{type of plot. Allowed values are one of "rectangle", "triangle", 
"circular", "phylogenic".}

\item{phylo_layout}{the layout to be used for phylogenic trees. Default value
is "layout.auto", which is kept as a compatibility alias for
\code{"layout_nicely"}. Allowed values include:
\code{\link[igraph]{layout.auto}}, \code{\link[igraph]{layout_nicely}},
\code{\link[igraph]{layout_with_drl}}, \code{\link[igraph]{layout_as_tree}},
\code{\link[igraph]{layout.gem}}, \code{\link[igraph]{layout_with_gem}},
\code{\link[igraph]{layout.mds}}, \code{\link[igraph]{layout_with_mds}} and
\code{\link[igraph]{layout_with_lgl}}.}

\item{rect}{logical value specifying whether to add a rectangle around 
groups. Used only when k != NULL.}

\item{rect_border, rect_lty}{border color and line type for rectangles.}

\item{rect_fill}{a logical value. If TRUE, fill the rectangle.}

\item{lower_rect}{a value of how low should the lower part of the rectangle 
around clusters. Ignored when rect = FALSE.}

\item{horiz}{a logical value. If TRUE, an horizontal dendrogram is drawn.}

\item{cex}{size of labels}

\item{main, xlab, ylab}{main and axis titles}

\item{sub}{Plot subtitle. Default is NULL (no subtitle). Set to a character
string to display a subtitle below the title, e.g.
\code{sub = paste0("Method: ", "ward.D2")}.}

\item{ggtheme}{function, ggplot2 theme name. Default value is 
theme_classic(). Allowed values include ggplot2 official themes: 
theme_gray(), theme_bw(), theme_minimal(), theme_classic(), theme_void(), 
....}

\item{...}{other arguments to be passed to the function plot.dendrogram()}
}
\value{
an object of class fviz_dend which is a ggplot with the attributes 
  "dendrogram" accessible using attr(x, "dendrogram"), where x is the result 
  of fviz_dend().
}
\description{
Draws easily beautiful dendrograms using either R base plot or 
  ggplot2. Provides also an option for drawing a circular dendrogram and 
  phylogenic trees.
}
\examples{
\donttest{
# Load and scale the data
data(USArrests)
df <- scale(USArrests)

# Hierarchical clustering
res.hc <- hclust(dist(df))

# Default plot
fviz_dend(res.hc)

# Increase branch and rectangle line widths
fviz_dend(res.hc, lwd = 2)

# Cut the tree
fviz_dend(res.hc, cex = 0.5, k = 4, color_labels_by_k = TRUE)

# Don't color labels, add rectangles
fviz_dend(res.hc, cex = 0.5, k = 4, 
 color_labels_by_k = FALSE, rect = TRUE)
 
# Change the color of tree using black color for all groups
# Change rectangle border colors
fviz_dend(res.hc, rect = TRUE, k_colors ="black",
rect_border = 2:5, rect_lty = 1)

# Customized color for groups
fviz_dend(res.hc, k = 4, 
 k_colors = c("#1B9E77", "#D95F02", "#7570B3", "#E7298A"))
 
 
 # Color labels using k-means clusters
 km.clust <- kmeans(df, 4)$cluster
 fviz_dend(res.hc, k = 4, 
   k_colors = c("blue", "green3", "red", "black"),
   label_cols =  km.clust[res.hc$order], cex = 0.6)

 # Phylogenic tree layouts support both compatibility aliases and
 # current igraph layout names
 if (requireNamespace("igraph", quietly = TRUE)) {
   fviz_dend(res.hc, type = "phylogenic", phylo_layout = "layout_nicely",
             show_labels = FALSE)
 }

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{factominer_category_map}
\alias{factominer_category_map}
\title{Map FactoMineR category labels to legacy naming patterns}
\usage{
factominer_category_map(X, element = c("quali.var", "quali.sup", "var"))
}
\arguments{
\item{X}{a FactoMineR object (MCA, MFA, FAMD, HMFA).}

\item{element}{element to map. Use "var" for MCA categories or "quali.var"
for MFA/FAMD/HMFA qualitative categories. "quali.sup" maps supplementary
qualitative categories when available.}
}
\value{
A data.frame with current labels, variable names, levels, and legacy
  naming patterns.
}
\description{
Map FactoMineR category labels to legacy naming patterns
}
\examples{
\donttest{
if (requireNamespace("FactoMineR", quietly = TRUE)) {
  data(poison)
  res.mca <- FactoMineR::MCA(poison, quanti.sup = 1:2, quali.sup = 3:4, graph = FALSE)
  head(factominer_category_map(res.mca, element = "var"))
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eclust.R
\name{eclust}
\alias{eclust}
\title{Visual enhancement of clustering analysis}
\usage{
eclust(
  x,
  FUNcluster = c("kmeans", "pam", "clara", "fanny", "hclust", "agnes", "diana",
    "hkmeans"),
  k = NULL,
  k.max = 10,
  stand = FALSE,
  graph = TRUE,
  hc_metric = "euclidean",
  hc_method = "ward.D2",
  gap_maxSE = list(method = "firstSEmax", SE.factor = 1),
  nboot = 100,
  verbose = interactive(),
  seed = 123,
  ...
)
}
\arguments{
\item{x}{numeric vector, data matrix or data frame. For hierarchical
clustering (\code{FUNcluster} = "hclust", "agnes" or "diana"), a
precomputed dissimilarity matrix (an object of class \code{"dist"}) may be
supplied directly; in that case \code{hc_metric} is ignored and \code{k}
must be specified. This allows custom distances such as Bray-Curtis
(e.g. \code{vegan::vegdist(df, "bray")}).}

\item{FUNcluster}{a clustering function including "kmeans", "pam", "clara",
"fanny", "hkmeans", "hclust", "agnes" and "diana". Abbreviation is allowed.}

\item{k}{the number of clusters to be generated. If NULL, the gap statistic
is used to estimate the appropriate number of clusters. For hierarchical
clustering, this automatic selection may return \code{k = 1}. In the case
of kmeans, \code{k} can be either the number of clusters, or a set of
initial (distinct) cluster centers.}

\item{k.max}{the maximum number of clusters to consider, must be at least 
two.}

\item{stand}{logical value; default is FALSE. If TRUE, then the data will be
standardized using the function \code{scale()}. Measurements are
standardized for each variable (column), by subtracting the variable's
mean value and dividing by the variable's standard deviation. If scaling
produces \code{NA} values, \code{eclust()} stops with a package-level
error.}

\item{graph}{logical value. If TRUE, cluster plot is displayed.}

\item{hc_metric}{character string specifying the metric to be used for 
calculating dissimilarities between observations. Allowed values are those 
accepted by the function dist() [including "euclidean", "manhattan", 
"maximum", "canberra", "binary", "minkowski"] and correlation based 
distance measures ["pearson", "spearman" or "kendall"]. Used only when
FUNcluster is a hierarchical clustering function such as one of "hclust",
"agnes" or "diana". Ignored when \code{x} is already a \code{"dist"} object.}

\item{hc_method}{the agglomeration method to be used (?hclust): "ward.D", 
"ward.D2", "single", "complete", "average", ...}

\item{gap_maxSE}{a list containing the parameters (method and SE.factor) for 
determining the location of the maximum of the gap statistic (Read the 
documentation ?cluster::maxSE).}

\item{nboot}{integer, number of Monte Carlo ("bootstrap") samples. Used only 
for determining the number of clusters using gap statistic.}

\item{verbose}{logical value. If TRUE, the result of progress is printed.}

\item{seed}{integer used for seeding the random number generator.}

\item{...}{other arguments to be passed to FUNcluster.}
}
\value{
Returns an object of class "eclust" containing the result of the 
  standard function used (e.g., kmeans, pam, hclust, agnes, diana, etc.).
  
  It also includes: \itemize{ \item cluster: the cluster assignment of
  observations after cutting the tree \item nbclust: the number of clusters
  \item silinfo: the silhouette information of observations, when available
  for solutions with at least two clusters, including $widths (silhouette
  width values of each observation), $clus.avg.widths (average silhouette
  width of each cluster) and $avg.width (average width of all clusters)
  \item size: the size of clusters \item data: a matrix containing the
  original or the standardized data (if stand = TRUE) } The "eclust" class
  has method for fviz_silhouette(), fviz_dend(), fviz_cluster().
}
\description{
Provides a convenient workflow for clustering analyses and
  ggplot2-based data visualization. When \code{k = NULL}, the gap statistic
  selects the number of clusters. Hierarchical backends may validly return
  \code{k = 1}; in that case \code{eclust()} returns a one-cluster result
  without silhouette information. Read more:
 \href{https://www.datanovia.com/en/blog/cluster-analysis-in-r-simplified-and-enhanced/}{Visual enhancement of clustering analysis}.
}
\examples{
# Load and scale data
data("USArrests")
df <- scale(USArrests)

# Enhanced k-means clustering
# nboot >= 500 is recommended
res.km <- eclust(df, "kmeans", nboot = 2)
# Silhouette plot
fviz_silhouette(res.km)
# Optimal number of clusters using gap statistics
res.km$nbclust
# Print result
 res.km
 
\dontrun{
# Enhanced hierarchical clustering
res.hc <- eclust(df, "hclust", nboot = 2) # compute hclust
fviz_dend(res.hc) # dendrogram
if (res.hc$nbclust > 1) fviz_silhouette(res.hc) # silhouette plot
}
 
}
\seealso{
\code{\link{fviz_silhouette}}, \code{\link{fviz_dend}}, 
  \code{\link{fviz_cluster}}
}
\author{
Alboukadel Kassambara \email{alboukadel.kassambara@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/decathlon2.R
\docType{data}
\name{decathlon2}
\alias{decathlon2}
\title{Athletes' performance in decathlon}
\format{
A data frame with 27 observations on the following 13 variables.
\describe{
\item{\code{X100m}}{a numeric vector}
\item{\code{Long.jump}}{a numeric vector}
\item{\code{Shot.put}}{a numeric vector}
\item{\code{High.jump}}{a numeric vector}
\item{\code{X400m}}{a numeric vector}
\item{\code{X110m.hurdle}}{a numeric vector}
\item{\code{Discus}}{a numeric vector}
\item{\code{Pole.vault}}{a numeric vector}
\item{\code{Javeline}}{a numeric vector}
\item{\code{X1500m}}{a numeric vector}
\item{\code{Rank}}{a numeric vector corresponding to the rank}
\item{\code{Points}}{a numeric vector specifying the point obtained}
\item{\code{Competition}}{a factor with levels \code{Decastar} \code{OlympicG}}
}
}
\source{
This data is a subset of decathlon data in FactoMineR package.
}
\usage{
data("decathlon2")
}
\description{
Athletes' performance during two sporting meetings
}
\examples{
\donttest{
data(decathlon2)
decathlon.active <- decathlon2[1:23, 1:10]
res.pca <- prcomp(decathlon.active, scale = TRUE)
fviz_pca_biplot(res.pca)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multishapes.R
\docType{data}
\name{multishapes}
\alias{multishapes}
\title{A dataset containing clusters of multiple shapes}
\format{
A data frame with 1100 observations on the following 3 variables. 
 \describe{
\item{\code{x}}{a numeric vector containing the x coordinates of observations}
\item{\code{y}}{a numeric vector containing the y coordinates of observations}
\item{\code{shape}}{a numeric vector corresponding to the cluster number of each observation.}
}
}
\usage{
data("multishapes")
}
\description{
Data containing clusters of any shapes. Useful for comparing
density-based clustering (DBSCAN) and standard partitioning methods such as
k-means clustering.
}
\details{
The dataset contains 5 clusters and some outliers/noises.
}
\examples{
\donttest{
data(multishapes)
plot(multishapes[,1], multishapes[, 2],
    col = multishapes[, 3], pch = 19, cex = 0.8)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_mfa.R
\name{get_mfa}
\alias{get_mfa}
\alias{get_mfa_ind}
\alias{get_mfa_var}
\alias{get_mfa_partial_axes}
\title{Extract the results for individuals/variables/group/partial axes - MFA}
\usage{
get_mfa(
  res.mfa,
  element = c("ind", "quanti.var", "quali.var", "quali.sup", "group", "partial.axes")
)

get_mfa_ind(res.mfa)

get_mfa_var(
  res.mfa,
  element = c("quanti.var", "quali.var", "quali.sup", "group")
)

get_mfa_partial_axes(res.mfa)
}
\arguments{
\item{res.mfa}{an object of class MFA [FactoMineR].}

\item{element}{the element to subset from the output. Possible values are
"ind", "quanti.var", "quali.var", "quali.sup", "group" or
"partial.axes".}
}
\value{
a list of matrices containing the results for the active 
individuals/quantitative variable categories/qualitative variable categories/groups/partial axes including : 
\item{coord}{coordinates for the individuals/quantitative variable categories/qualitative variable categories/groups/partial axes}
\item{cos2}{cos2 for the individuals/quantitative variable categories/qualitative variable categories/groups/partial axes}
\item{contrib}{contributions of the individuals/quantitative variable categories/qualitative variable categories/groups/partial axes}
\item{inertia}{inertia of the individuals/quantitative variable categories/qualitative variable categories/groups/partial axes}
}
\description{
Extract all the results (coordinates, squared cosine and contributions) 
for the active individuals/quantitative variables/qualitative variable categories/groups/partial axes from Multiple Factor Analysis (MFA) outputs.\cr\cr
\itemize{
\item get_mfa(): Extract the results for variables and individuals
\item get_mfa_ind(): Extract the results for individuals only
\item get_mfa_var(): Extract the results for variables (quantitatives, qualitatives and groups)
\item get_mfa_partial_axes(): Extract the results for partial axes only
}
}
\examples{
# Multiple Factor Analysis
# ++++++++++++++++++++++++
# Install and load FactoMineR to compute MFA
# install.packages("FactoMineR")
library("FactoMineR")
data(poison)
res.mfa <- MFA(poison, group=c(2,2,5,6), type=c("s","n","n","n"),
name.group=c("desc","desc2","symptom","eat"), num.group.sup=1:2,
graph = FALSE)
 
 # Extract the results for qualitative variable categories
 var <- get_mfa_var(res.mfa, "quali.var")
 print(var)
 head(var$coord) # coordinates of qualitative variables
 head(var$cos2) # cos2 of qualitative variables
 head(var$contrib) # contributions of qualitative variables
 
 # Extract the results for individuals
 ind <- get_mfa_ind(res.mfa)
 print(ind)
 head(ind$coord) # coordinates of individuals
 head(ind$cos2) # cos2 of individuals
 head(ind$contrib) # contributions of individuals
 
 # You can also use the function get_mfa()
 get_mfa(res.mfa, "ind") # Results for individuals
 get_mfa(res.mfa, "quali.var") # Results for qualitative variable categories
 get_mfa(res.mfa, "quali.sup") # Results for supplementary qualitative variable categories
 
}
\author{
Alboukadel Kassambara \email{alboukadel.kassambara@gmail.com}

Fabian Mundt \email{f.mundt@inventionate.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/facto_summarize.R
\name{facto_summarize}
\alias{facto_summarize}
\title{Subset and summarize the output of factor analyses}
\usage{
facto_summarize(
  X,
  element,
  node.level = 1,
  group.names,
  result = c("coord", "cos2", "contrib"),
  axes = 1:2,
  select = NULL
)
}
\arguments{
\item{X}{an object of class PCA, CA, MCA, FAMD, MFA and HMFA [FactoMineR]; prcomp
and princomp [stats]; dudi, pca, coa and acm [ade4]; ca [ca package]; expoOutput [ExPosition].}

\item{element}{the element to subset from the output. Possible values are
"row" or "col" for CA; "var", "ind", "mca.cor" or "quanti.sup" for MCA;
"var" or "ind" for PCA; and 'quanti.var', 'quali.var', 'quali.sup',
'group' or 'ind' for FAMD, MFA and HMFA.}

\item{node.level}{a single number indicating the HMFA node level.}

\item{group.names}{a vector containing the name of the groups (by default, 
NULL and the group are named group.1, group.2 and so on).}

\item{result}{the result to be extracted for the element. Possible values are
the combination of c("cos2", "contrib", "coord")}

\item{axes}{a numeric vector specifying the axes of interest. Values must be
positive integer indices within the available dimensions. Default values
are 1:2 for axes 1 and 2.}

\item{select}{a selection of variables. Allowed values are NULL or a list 
containing the arguments name, cos2 or contrib. Default is list(name = 
NULL, cos2 = NULL, contrib = NULL): \itemize{ \item name: is a character 
vector containing variable names to be selected \item cos2: if cos2 is in 
[0, 1], ex: 0.6, then variables with a cos2 > 0.6 are selected. if cos2 > 
1, ex: 5, then the top 5 variables with the highest cos2 are selected \item
contrib: if contrib > 1, ex: 5,  then the top 5 variables with the highest 
cos2 are selected. }}
}
\value{
A data frame containing the (total) coord, cos2 and the contribution 
  for the axes.
}
\description{
Subset and summarize the results of Principal Component
  Analysis (PCA), Correspondence Analysis (CA), Multiple Correspondence
  Analysis (MCA), Factor Analysis of Mixed Data (FAMD), Multiple Factor
  Analysis (MFA) and Hierarchical Multiple Factor Analysis (HMFA) functions
  from several packages. Axis indices are validated before extraction, and
  MCA quantitative supplementary summaries inherit the package-level error
  raised when that result is unavailable.
}
\details{
If length(axes) > 1, then the columns contrib and cos2 correspond to
  the total contributions and total cos2 of the axes. In this case, the 
  column coord is calculated as x^2 + y^2 + ...+; x, y, ... are the 
  coordinates of the points on the specified axes.
}
\examples{
# Principal component analysis
# +++++++++++++++++++++++++++++
data(decathlon2)
decathlon2.active <- decathlon2[1:23, 1:10]
res.pca <- prcomp(decathlon2.active,  scale = TRUE)

# Summarize variables on axes 1:2
facto_summarize(res.pca, "var", axes = 1:2)[,-1]
# Select the top 5 contributing variables
facto_summarize(res.pca, "var", axes = 1:2,
           select = list(contrib = 5))[,-1]
# Select variables with cos2 >= 0.6
facto_summarize(res.pca, "var", axes = 1:2,
           select = list(cos2 = 0.6))[,-1]
# Select by names
facto_summarize(res.pca, "var", axes = 1:2,
     select = list(name = c("X100m", "Discus", "Javeline")))[,-1]
           
# Summarize individuals on axes 1:2
facto_summarize(res.pca, "ind", axes = 1:2)[,-1]

# Correspondence Analysis
# ++++++++++++++++++++++++++
# Install and load FactoMineR to compute CA
# install.packages("FactoMineR")
library("FactoMineR")
data("housetasks")
res.ca <- CA(housetasks, graph = FALSE)
# Summarize row variables on axes 1:2
facto_summarize(res.ca, "row", axes = 1:2)[,-1]
# Summarize column variables on axes 1:2
facto_summarize(res.ca, "col", axes = 1:2)[,-1]

# Multiple Correspondence Analysis
# +++++++++++++++++++++++++++++++++
library(FactoMineR)
data(poison)
res.mca <- MCA(poison, quanti.sup = 1:2, 
              quali.sup = 3:4, graph=FALSE)
# Summarize variables on axes 1:2
res <- facto_summarize(res.mca, "var", axes = 1:2)
head(res)
# Summarize individuals on axes 1:2
res <- facto_summarize(res.mca, "ind", axes = 1:2)
head(res)
# Summarize quantitative supplementary variables on axes 1:2
res <- facto_summarize(res.mca, "quanti.sup", axes = 1:2)
head(res)

# Multiple factor Analysis
# +++++++++++++++++++++++++++++++++
library(FactoMineR)
data(poison)
res.mfa <- MFA(poison, group=c(2,2,5,6), type=c("s","n","n","n"),
               name.group=c("desc","desc2","symptom","eat"),
               num.group.sup=1:2, graph=FALSE)
# Summarize categorical variables on axes 1:2
res <- facto_summarize(res.mfa, "quali.var", axes = 1:2)
head(res)
# Summarize individuals on axes 1:2
res <- facto_summarize(res.mfa, "ind", axes = 1:2)
head(res)
}
\references{
\url{https://www.sthda.com/english/}
}
\author{
Alboukadel Kassambara \email{alboukadel.kassambara@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fviz_add.R
\name{fviz_add}
\alias{fviz_add}
\title{Add supplementary data to a plot}
\usage{
fviz_add(
  ggp,
  df,
  axes = c(1, 2),
  geom = c("point", "arrow"),
  color = "blue",
  addlabel = TRUE,
  labelsize = 4,
  pointsize = 2,
  shape = 19,
  linetype = "dashed",
  repel = FALSE,
  font.family = "",
  ...
)
}
\arguments{
\item{ggp}{a ggplot2 plot.}

\item{df}{a data frame containing the x and y coordinates}

\item{axes}{a numeric vector of length 2 specifying the components to be plotted.}

\item{geom}{a character specifying the geometry to be used for the graph
Allowed values are "point" or "arrow" or "text"}

\item{color}{the color to be used}

\item{addlabel}{a logical value. If TRUE, labels are added}

\item{labelsize}{the size of labels. Default value is 4}

\item{pointsize}{the size of points}

\item{shape}{point shape when geom ="point"}

\item{linetype}{the linetype to be used when geom ="arrow"}

\item{repel}{a boolean, whether to use ggrepel to avoid overplotting text
labels or not. The old \code{jitter} argument is kept for backward
compatibility and is converted to \code{repel = TRUE} with a deprecation warning.}

\item{font.family}{character vector specifying font family.}

\item{...}{Additional arguments, not used}
}
\value{
a ggplot2 plot
}
\description{
Add supplementary data to a plot
}
\examples{
\donttest{
# Principal component analysis
data(decathlon2)
decathlon2.active <- decathlon2[1:23, 1:10]
res.pca <- prcomp(decathlon2.active,  scale = TRUE)

# Visualize variables
p <- fviz_pca_var(res.pca)
print(p)

# Add supplementary variables
coord <- data.frame(PC1 = c(-0.7, 0.9), PC2 = c(0.25, -0.07))
rownames(coord) <- c("Rank", "Points")
print(coord)
fviz_add(p, coord, color ="blue", geom="arrow") 
 }
 
}
\references{
\url{https://www.sthda.com/english/}
}
\author{
Alboukadel Kassambara \email{alboukadel.kassambara@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as_factoextra.R
\name{as_factoextra_pca}
\alias{as_factoextra_pca}
\title{Build a factoextra-ready object from pre-computed coordinates}
\usage{
as_factoextra_pca(
  ind.coord,
  var.coord = NULL,
  eig = NULL,
  ind.cos2 = NULL,
  ind.contrib = NULL,
  var.cos2 = NULL,
  var.contrib = NULL,
  var.cor = NULL,
  scale.unit = FALSE
)
}
\arguments{
\item{ind.coord}{individual (observation) coordinates: a numeric matrix or
data frame with one column per dimension (the "scores"). Required.}

\item{var.coord}{optional variable coordinates / loadings: a numeric matrix or
data frame with one column per dimension. Supplying it enables
\code{fviz_pca_var()} and \code{fviz_pca_biplot()}.}

\item{eig}{optional numeric vector of eigenvalues (length \eqn{\ge} number of
dimensions). Used by the scree plot and to label the axes with the percentage
of explained variance. When \code{NULL} (default) it is set to the variance
of each coordinate column (the natural definition of a PCA eigenvalue).}

\item{ind.cos2, ind.contrib, var.cos2, var.contrib, var.cor}{optional pre-computed
quality (\code{cos2}), contribution and (variable) correlation matrices, with
the same dimensions as the corresponding coordinates. When omitted they are
derived from the coordinates (see \strong{Details}).}

\item{scale.unit}{logical. If \code{TRUE}, the variable coordinates are treated
as correlations and the correlation circle is drawn by \code{fviz_pca_var()}.
Default is \code{FALSE}.}
}
\value{
An object of class \code{c("factoextra_pca", "list")} holding the
  standardized \code{ind} (and \code{var}) results and eigenvalues, ready for
  the \code{fviz_pca_*()} functions.
}
\description{
\code{as_factoextra_pca()} wraps pre-computed individual (and, optionally,
variable) coordinates into an object that the \code{\link{fviz_pca}} family
(\code{fviz_pca_ind()}, \code{fviz_pca_var()}, \code{fviz_pca_biplot()}),
\code{\link{fviz_eig}}, \code{\link{fviz_contrib}} and \code{\link{fviz_cos2}}
can plot directly.

It lets you apply factoextra's visualizations to the output of \emph{any}
dimension-reduction method - for example \code{stats::cmdscale()},
\code{ape::pcoa()}, UMAP/t-SNE embeddings, \code{vegan::rda()}/\code{cca()},
or a custom analysis - without having to write a dedicated backend. You bring
the coordinates; factoextra draws the biplot, scree plot, contributions and
cos2.
}
\details{
When \code{cos2}/\code{contrib} are not supplied they are computed from the
coordinates:
\itemize{
  \item \code{contrib = 100 * coord^2 / colSums(coord^2)} - the exact
    contribution of each element to each dimension.
  \item \code{cos2 = coord^2 / rowSums(coord^2)} - the quality of
    representation \emph{within the supplied dimensions}. This equals the true
    cos2 only when all components are provided; with a truncated set of
    dimensions it is the quality restricted to that sub-space. Pass
    \code{ind.cos2}/\code{var.cos2} explicitly if you have the exact values.
}
}
\examples{
# 1. Bring your own coordinates: classical MDS (cmdscale) -> factoextra
d <- dist(scale(mtcars))
mds <- cmdscale(d, k = 3)
obj <- as_factoextra_pca(ind.coord = mds)
fviz_pca_ind(obj, repel = TRUE)
fviz_eig(obj)

# 2. Round-trip a prcomp result through the constructor (biplot)
pca <- prcomp(iris[, -5], scale. = TRUE)
obj2 <- as_factoextra_pca(
  ind.coord = pca$x,
  var.coord = pca$rotation,
  eig       = pca$sdev^2
)
fviz_pca_biplot(obj2, label = "var", col.ind = "steelblue")

}
\author{
Alboukadel Kassambara \email{alboukadel.kassambara@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/poison.R
\docType{data}
\name{poison}
\alias{poison}
\title{Poison}
\format{
A data frame with 55 rows and 15 columns.
}
\source{
This data is from FactoMineR package.
}
\usage{
data("poison")
}
\description{
This data is a result from a survey carried out on children of
 primary school who suffered from food poisoning. They were asked about their
 symptoms and about what they ate.
}
\examples{
\donttest{
library(FactoMineR)
data(poison)
res.mca <- MCA(poison, quanti.sup = 1:2, quali.sup = c(3,4), 
   graph = FALSE)
fviz_mca_biplot(res.mca, repel = TRUE)+
theme_minimal()

}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fviz_contrib.R
\name{fviz_contrib}
\alias{fviz_contrib}
\alias{fviz_pca_contrib}
\title{Visualize the contributions of row/column elements}
\usage{
fviz_contrib(
  X,
  choice = c("row", "col", "var", "ind", "quanti.var", "quali.var", "group",
    "partial.axes"),
  axes = 1,
  fill = "steelblue",
  color = "steelblue",
  sort.val = c("desc", "asc", "none"),
  top = Inf,
  xtickslab.rt = 45,
  ggtheme = theme_minimal(),
  ...
)

fviz_pca_contrib(
  X,
  choice = c("var", "ind"),
  axes = 1,
  fill = "steelblue",
  color = "steelblue",
  sortcontrib = c("desc", "asc", "none"),
  top = Inf,
  ...
)
}
\arguments{
\item{X}{an object of class PCA, CA, MCA, FAMD, MFA and HMFA [FactoMineR];
prcomp and princomp [stats]; dudi, pca, coa and acm [ade4]; ca [ca
package].}

\item{choice}{allowed values are "row" and "col" for CA;  "var" and "ind" for
PCA or MCA; "var", "ind", "quanti.var", "quali.var" and "group" for FAMD, MFA and HMFA.}

\item{axes}{a numeric vector specifying the dimension(s) of interest.}

\item{fill}{a fill color for the bar plot.}

\item{color}{an outline color for the bar plot.}

\item{sort.val}{a string specifying whether the value should be sorted. 
Allowed values are "none" (no sorting), "asc" (for ascending) or "desc" (for descending).}

\item{top}{a numeric value specifying the number of top elements to be shown.}

\item{xtickslab.rt}{rotation angle for x axis tick labels. Default is 45 degrees.}

\item{ggtheme}{function, ggplot2 theme name. Default value is theme_pubr().
Allowed values include ggplot2 official themes: theme_gray(), theme_bw(),
theme_minimal(), theme_classic(), theme_void(), ....}

\item{...}{other arguments to be passed to the function \link[ggpubr]{ggpar}.}

\item{sortcontrib}{see the argument sort.val}
}
\value{
a ggplot2 plot
}
\description{
This function can be used to visualize the contribution of rows/columns
from the results of Principal Component Analysis (PCA),
Correspondence Analysis (CA), Multiple Correspondence Analysis (MCA), Factor Analysis of Mixed Data (FAMD),
and Multiple Factor Analysis (MFA) functions.
}
\details{
The function fviz_contrib() creates a barplot of row/column contributions.
A reference dashed line is also shown on the barplot. This reference line
corresponds to the expected value if the contribution were uniform.\cr\cr
For a given dimension, any row/column with a contribution above the reference line could be
considered as important in contributing to the dimension.
}
\section{Functions}{
\itemize{
\item \code{fviz_pca_contrib()}: deprecated function. Use fviz_contrib()

}}
\examples{
\donttest{
# Principal component analysis
# ++++++++++++++++++++++++++
data(decathlon2)
decathlon2.active <- decathlon2[1:23, 1:10]
res.pca <- prcomp(decathlon2.active,  scale = TRUE)

# variable contributions on axis 1
fviz_contrib(res.pca, choice="var", axes = 1, top = 10 )

# Change theme and color
fviz_contrib(res.pca, choice="var", axes = 1,
         fill = "lightgray", color = "black") +
         theme_minimal() +
         theme(axis.text.x = element_text(angle=45))

# Variable contributions on axis 2
fviz_contrib(res.pca, choice="var", axes = 2)
# Variable contributions on axes 1 + 2
fviz_contrib(res.pca, choice="var", axes = 1:2)

# Contributions of individuals on axis 1
fviz_contrib(res.pca, choice="ind", axes = 1)

\dontrun{
# Correspondence Analysis
# ++++++++++++++++++++++++++
# Install and load FactoMineR to compute CA
# install.packages("FactoMineR")
library("FactoMineR")
data("housetasks")
res.ca <- CA(housetasks, graph = FALSE)

# Visualize row contributions on axes 1
fviz_contrib(res.ca, choice ="row", axes = 1)
# Visualize column contributions on axes 1
fviz_contrib(res.ca, choice ="col", axes = 1)

# Multiple Correspondence Analysis
# +++++++++++++++++++++++++++++++++
library(FactoMineR)
data(poison)
res.mca <- MCA(poison, quanti.sup = 1:2,
              quali.sup = 3:4, graph=FALSE)

# Visualize individual contributions on axes 1
fviz_contrib(res.mca, choice ="ind", axes = 1)
# Visualize variable category contributions on axes 1
fviz_contrib(res.mca, choice ="var", axes = 1)

# Multiple Factor Analysis
# ++++++++++++++++++++++++
library(FactoMineR)
data(poison)
res.mfa <- MFA(poison, group=c(2,2,5,6), type=c("s","n","n","n"),
               name.group=c("desc","desc2","symptom","eat"),
               num.group.sup=1:2, graph=FALSE)

# Visualize individual contributions on axes 1
fviz_contrib(res.mfa, choice ="ind", axes = 1, top = 20)
# Visualize categorical variable category contributions on axes 1
fviz_contrib(res.mfa, choice ="quali.var", axes = 1)
}

 }
}
\references{
\url{https://www.sthda.com/english/}
}
\author{
Alboukadel Kassambara \email{alboukadel.kassambara@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fviz_pca.R
\name{fviz_pca}
\alias{fviz_pca}
\alias{fviz_pca_ind}
\alias{fviz_pca_var}
\alias{fviz_pca_biplot}
\title{Visualize Principal Component Analysis}
\usage{
fviz_pca(X, ...)

fviz_pca_ind(
  X,
  axes = c(1, 2),
  geom = c("point", "text"),
  geom.ind = geom,
  repel = FALSE,
  habillage = "none",
  palette = NULL,
  addEllipses = FALSE,
  col.ind = "black",
  fill.ind = "white",
  col.ind.sup = "blue",
  alpha.ind = 1,
  shape.ind = NULL,
  select.ind = list(name = NULL, cos2 = NULL, contrib = NULL),
  ...
)

fviz_pca_var(
  X,
  axes = c(1, 2),
  geom = c("arrow", "text"),
  geom.var = geom,
  repel = FALSE,
  col.var = "black",
  fill.var = "white",
  alpha.var = 1,
  col.quanti.sup = "blue",
  col.circle = "grey70",
  select.var = list(name = NULL, cos2 = NULL, contrib = NULL),
  ...
)

fviz_pca_biplot(
  X,
  axes = c(1, 2),
  geom = c("point", "text"),
  geom.ind = geom,
  geom.var = c("arrow", "text"),
  col.ind = "black",
  fill.ind = "white",
  col.var = "steelblue",
  fill.var = "white",
  gradient.cols = NULL,
  label = "all",
  invisible = "none",
  repel = FALSE,
  habillage = "none",
  palette = NULL,
  addEllipses = FALSE,
  shape.ind = NULL,
  title = "PCA - Biplot",
  biplot.type = c("auto", "form", "covariance"),
  ...
)
}
\arguments{
\item{X}{an object of class PCA [FactoMineR]; prcomp and princomp [stats]; 
dudi and pca [ade4]; expOutput/epPCA [ExPosition].}

\item{...}{Additional arguments. \itemize{ \item in fviz_pca_ind() and 
fviz_pca_var(): Additional arguments are passed to the functions fviz() and
ggpubr::ggpar(). \item in fviz_pca_biplot() and fviz_pca(): Additional 
arguments are passed to fviz_pca_ind() and fviz_pca_var().}}

\item{axes}{a numeric vector of length 2 specifying the dimensions to be 
plotted.}

\item{geom}{a text specifying the geometry to be used for the graph. Allowed 
values are the combination of \code{c("point", "arrow", "text")}. Use 
\code{"point"} (to show only points); \code{"text"} to show only labels; 
\code{c("point", "text")} or \code{c("arrow", "text")} to show arrows and 
texts. Using \code{c("arrow", "text")} is sensible only for the graph of 
variables.}

\item{geom.ind, geom.var}{as \code{geom} but for individuals and variables, 
respectively. Default is geom.ind = c("point", "text), geom.var = 
c("arrow", "text").}

\item{repel}{a boolean, whether to use ggrepel to avoid overplotting text
labels or not. The old \code{jitter} argument is kept for backward
compatibility and is converted to \code{repel = TRUE} with a deprecation warning.}

\item{habillage}{an optional factor variable for coloring the observations by
groups. Default value is "none". If X is a PCA object from FactoMineR 
package, habillage can also specify the supplementary qualitative variable 
(by its index or name) to be used for coloring individuals by groups (see 
?PCA in FactoMineR).}

\item{palette}{the color palette to be used for coloring or filling by
groups. Allowed values include "grey" for grey color palettes; brewer
palettes e.g. "RdBu", "Blues", ...; or custom color palette e.g. c("blue",
"red"); and scientific journal palettes from ggsci R package, e.g.: "npg",
"aaas", "lancet", "jco", "ucscgb", "uchicago", "simpsons" and
"rickandmorty". Can be also a numeric vector of length(groups); in this
case a basic color palette is created using the function
\link[grDevices]{palette}.}

\item{addEllipses}{logical value. If TRUE, draws ellipses around the 
individuals when habillage != "none".}

\item{col.ind, col.var}{color for individuals and variables, respectively. Can
be a continuous variable or a factor variable. Possible values include also
: "cos2", "contrib", "coord", "x" or "y". In this case, the colors for 
individuals/variables are automatically controlled by their qualities of 
representation ("cos2"), contributions ("contrib"), coordinates (x^2+y^2, 
"coord"), x values ("x") or y values ("y"). To use automatic coloring (by 
cos2, contrib, ....), make sure that habillage ="none".}

\item{fill.ind, fill.var}{same as col.ind and col.var but for the fill color.}

\item{col.ind.sup}{color for supplementary individuals}

\item{alpha.ind, alpha.var}{controls the transparency of individual and 
variable colors, respectively. The value can variate from 0 (total 
transparency) to 1 (no transparency). Default value is 1. Possible values 
include also : "cos2", "contrib", "coord", "x" or "y". In this case, the 
transparency for the individual/variable colors are automatically 
controlled by their qualities ("cos2"), contributions ("contrib"), 
coordinates (x^2+y^2, "coord"), x values("x") or y values("y"). To use 
this, make sure that habillage ="none".}

\item{shape.ind}{optional factor variable to map the point \strong{shape} of
individuals, independently of their color. This allows colouring
individuals by one grouping variable (\code{col.ind}/\code{habillage}) and
shaping them by another (e.g. \code{fviz_pca_ind(res, col.ind = group1,
shape.ind = group2)}). Default \code{NULL} keeps the previous behaviour
(shape follows the colour grouping). Use \code{+ labs(shape = , colour = )}
to rename the legends.}

\item{select.ind, select.var}{a selection of individuals/variables to be
drawn. Allowed values are NULL or a list containing the arguments name,
cos2 or contrib: \itemize{ \item name: is a character vector containing
individuals/variables to be drawn \item cos2: if cos2 is in [0, 1], ex:
0.6, then individuals/variables with a cos2 > 0.6 are drawn. if cos2 > 1,
ex: 5, then the top 5 individuals/variables with the highest cos2 are
drawn. \item contrib: if contrib > 1, ex: 5,  then the top 5
individuals/variables with the highest contrib are drawn }}

\item{col.quanti.sup}{a color for the quantitative supplementary variables.}

\item{col.circle}{a color for the correlation circle. Used only when X is a 
PCA output.}

\item{gradient.cols}{vector of colors to use for n-colour gradient. Allowed
values include brewer and ggsci color palettes.}

\item{label}{a text specifying the elements to be labelled. Default value is 
"all". Allowed values are "none" or the combination of c("ind", "ind.sup", 
"quali", "var", "quanti.sup"). "ind" can be used to label only active 
individuals. "ind.sup" is for supplementary individuals. "quali" is for 
supplementary qualitative variables. "var" is for active variables. 
"quanti.sup" is for quantitative supplementary variables.}

\item{invisible}{a text specifying the elements to be hidden on the plot. 
Default value is "none". Allowed values are the combination of c("ind", 
"ind.sup", "quali", "var", "quanti.sup").}

\item{title}{the title of the graph}

\item{biplot.type}{type of biplot scaling for fviz_pca_biplot(). Options are:
\itemize{
  \item "auto" (default): Uses range-based rescaling for visualization
  \item "form": Form-oriented scaling (Gabriel-style). Prioritizes
    readability of individual relationships.
  \item "covariance": Covariance-oriented scaling (Gabriel-style).
    Prioritizes readability of variable relationships.
}
Note: "form" and "covariance" scaling requires prcomp or princomp objects.}
}
\value{
a ggplot
}
\description{
Principal component analysis (PCA) reduces the dimensionality of
  multivariate data, to two or three that can be visualized graphically with 
  minimal loss of information. fviz_pca() provides ggplot2-based elegant 
  visualization of PCA outputs from: i) prcomp and princomp [in built-in R 
  stats], ii) PCA [in FactoMineR], iii) dudi.pca [in ade4] and epPCA 
  [ExPosition]. Read more: 
  \href{https://www.sthda.com/english/articles/31-principal-component-methods-in-r-practical-guide/112-pca-principal-component-analysis-essentials/}{Principal
   Component Analysis}
  
  \itemize{ \item fviz_pca_ind(): Graph of individuals \item fviz_pca_var():
  Graph of variables \item fviz_pca_biplot(): Biplot of individuals and
  variables \item fviz_pca(): An alias of fviz_pca_biplot() }
  
  Note that, \code{fviz_pca_xxx()} functions are wrapper around the core
  function \code{\link{fviz}()}, which is also a wrapper around the
  function \code{\link[ggpubr]{ggscatter}()} [in ggpubr]. Therefore, further arguments, to be
  passed to the function \code{\link{fviz}()} and \code{\link[ggpubr]{ggscatter}()}, can be specified in
  \code{\link{fviz_pca_ind}()} and \code{\link{fviz_pca_var}()}.
}
\examples{
\donttest{
# Principal component analysis
# ++++++++++++++++++++++++++++++
data(iris)
res.pca <- prcomp(iris[, -5],  scale = TRUE)

# Graph of individuals
# +++++++++++++++++++++

# Default plot
# Use repel = TRUE to avoid overplotting (slow if many points)
fviz_pca_ind(res.pca, col.ind = "#00AFBB",
   repel = TRUE)

 
# 1. Control automatically the color of individuals 
   # using the "cos2" or the contributions "contrib"
   # cos2 = the quality of the individuals on the factor map
# 2. To keep only point or text use geom = "point" or geom = "text".
# 3. Change themes using ggtheme: http://www.sthda.com/english/wiki/ggplot2-themes

fviz_pca_ind(res.pca, col.ind="cos2", geom = "point",
   gradient.cols = c("white", "#2E9FDF", "#FC4E07" ))

# Color individuals by groups, add concentration ellipses
# Change group colors using RColorBrewer color palettes
# Read more: http://www.sthda.com/english/wiki/ggplot2-colors
# Remove labels: label = "none".
fviz_pca_ind(res.pca, label="none", habillage=iris$Species,
     addEllipses=TRUE, ellipse.level=0.95, palette = "Dark2")
             
     
# Change group colors manually
# Read more: http://www.sthda.com/english/wiki/ggplot2-colors
fviz_pca_ind(res.pca, label="none", habillage=iris$Species,
     addEllipses=TRUE, ellipse.level=0.95,
     palette = c("#999999", "#E69F00", "#56B4E9"))
      
# Select and visualize some individuals (ind) with select.ind argument.
 # - ind with cos2 >= 0.96: select.ind = list(cos2 = 0.96)
 # - Top 20 ind according to the cos2: select.ind = list(cos2 = 20)
 # - Top 20 contributing individuals: select.ind = list(contrib = 20)
 # - Select ind by names: select.ind = list(name = c("23", "42", "119") )
 
 # Example: Select the top 40 according to the cos2
fviz_pca_ind(res.pca, select.ind = list(cos2 = 40))

 
# Graph of variables
# ++++++++++++++++++++++++++++
  
# Default plot
fviz_pca_var(res.pca, col.var = "steelblue")
 
# Control variable colors using their contributions
fviz_pca_var(res.pca, col.var = "contrib", 
   gradient.cols = c("white", "blue", "red"),
   ggtheme = theme_minimal())
 
    
# Biplot of individuals and variables
# ++++++++++++++++++++++++++
# Keep only the labels for variables
# Change the color by groups, add ellipses
fviz_pca_biplot(res.pca, label = "var", habillage=iris$Species,
               addEllipses=TRUE, ellipse.level=0.95,
               ggtheme = theme_minimal())

# Biplot scaling modes:
# Form biplot - focus on individual distances
fviz_pca_biplot(res.pca, biplot.type = "form",
               label = "var", habillage = iris$Species)
# Covariance biplot - focus on variable correlations
fviz_pca_biplot(res.pca, biplot.type = "covariance",
               label = "var", habillage = iris$Species)

 }

}
\seealso{
\code{\link{fviz_ca}}, \code{\link{fviz_mca}}
}
\author{
Alboukadel Kassambara \email{alboukadel.kassambara@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fviz_mfa.R
\name{fviz_mfa}
\alias{fviz_mfa}
\alias{fviz_mfa_ind}
\alias{fviz_mfa_quali_biplot}
\alias{fviz_mfa_var}
\alias{fviz_mfa_axes}
\title{Visualize Multiple Factor Analysis}
\usage{
fviz_mfa_ind(
  X,
  axes = c(1, 2),
  geom = c("point", "text"),
  repel = FALSE,
  habillage = "none",
  palette = NULL,
  addEllipses = FALSE,
  col.ind = "blue",
  col.ind.sup = "darkblue",
  alpha.ind = 1,
  shape.ind = 19,
  col.quali.var.sup = "black",
  select.ind = list(name = NULL, cos2 = NULL, contrib = NULL),
  partial = NULL,
  col.partial = "group",
  ...
)

fviz_mfa_quali_biplot(
  X,
  axes = c(1, 2),
  geom = c("point", "text"),
  repel = FALSE,
  title = "Biplot of individuals and qualitative variables - MFA",
  ...
)

fviz_mfa_var(
  X,
  choice = c("quanti.var", "group", "quali.var", "quali.sup"),
  axes = c(1, 2),
  geom = c("point", "text"),
  repel = FALSE,
  habillage = "none",
  col.var = "red",
  alpha.var = 1,
  shape.var = 17,
  col.var.sup = "darkgreen",
  palette = NULL,
  select.var = list(name = NULL, cos2 = NULL, contrib = NULL),
  ...
)

fviz_mfa_axes(
  X,
  axes = c(1, 2),
  geom = c("arrow", "text"),
  col.axes = NULL,
  alpha.axes = 1,
  col.circle = "grey70",
  select.axes = list(name = NULL, contrib = NULL),
  repel = FALSE,
  ...
)

fviz_mfa(X, partial = "all", ...)
}
\arguments{
\item{X}{an object of class MFA [FactoMineR].}

\item{axes}{a numeric vector of length 2 specifying the dimensions to be 
plotted.}

\item{geom}{a text specifying the geometry to be used for the graph. Allowed 
values are the combination of \code{c("point", "arrow", "text")}. Use 
\code{"point"} (to show only points); \code{"text"} to show only labels; 
\code{c("point", "text")} or \code{c("arrow", "text")} to show arrows and 
texts. Using \code{c("arrow", "text")} is sensible only for the graph of 
variables.}

\item{repel}{a boolean, whether to use ggrepel to avoid overplotting text
labels or not. The old \code{jitter} argument is kept for backward
compatibility and is converted to \code{repel = TRUE} with a deprecation warning.}

\item{habillage}{an optional factor variable for coloring the observations by 
groups. Default value is "none". If X is an MFA object from FactoMineR 
package, habillage can also specify the index of the factor variable in the 
data.}

\item{palette}{the color palette to be used for coloring or filling by
groups. Allowed values include "grey" for grey color palettes; brewer
palettes e.g. "RdBu", "Blues", ...; or custom color palette e.g. c("blue",
"red"); and scientific journal palettes from ggsci R package, e.g.: "npg",
"aaas", "lancet", "jco", "ucscgb", "uchicago", "simpsons" and
"rickandmorty". Can be also a numeric vector of length(groups); in this
case a basic color palette is created using the function
\link[grDevices]{palette}.}

\item{addEllipses}{logical value. If TRUE, draws ellipses around the 
individuals when habillage != "none".}

\item{col.ind, col.var, col.axes}{color for individuals, variables and col.axes 
respectively. Can be a continuous variable or a factor variable. Possible
values include also : "cos2", "contrib", "coord", "x" or "y". In this case,
the colors for individuals/variables are automatically controlled by their
qualities ("cos2"), contributions ("contrib"), coordinates (x^2 + y^2 ,
"coord"), x values("x") or y values("y"). To use automatic coloring (by
cos2, contrib, ....), make sure that habillage ="none".}

\item{col.ind.sup}{color for supplementary individuals}

\item{alpha.ind, alpha.var, alpha.axes}{controls the transparency of individual,
variable, group and axes colors, respectively. The value can variate from 0 
(total transparency) to 1 (no transparency). Default value is 1. Possible 
values include also : "cos2", "contrib", "coord", "x" or "y". In this case, 
the transparency for individual/variable colors are automatically controlled
by their qualities ("cos2"), contributions ("contrib"), coordinates (x^2 + 
y^2 , "coord"), x values("x") or y values("y"). To use this, make sure that 
habillage ="none".}

\item{shape.ind, shape.var}{point shapes of individuals, variables, groups and 
axes}

\item{col.quali.var.sup}{color for supplementary qualitative variables. 
Default is "black".}

\item{select.ind, select.var, select.axes}{a selection of individuals/partial 
individuals/ variables/groups/axes to be drawn. Allowed values are NULL or a
list containing the arguments name, cos2 or contrib: \itemize{ \item name is
a character vector containing individuals/variables to be drawn \item cos2 
if cos2 is in [0, 1], ex: 0.6, then individuals/variables with a cos2 > 0.6 
are drawn. if cos2 > 1, ex: 5, then the top 5 individuals/variables with the
highest cos2 are drawn. \item contrib if contrib > 1, ex: 5,  then the top 5
individuals/variables with the highest cos2 are drawn }}

\item{partial}{list of the individuals for which the partial points should be
drawn. (by default, partial = NULL and no partial points are drawn). Use
partial = "all" to visualize partial points for all individuals.}

\item{col.partial}{color for partial individuals. By default, points are 
colored according to the groups.}

\item{...}{Arguments to be passed to the function fviz()}

\item{title}{the title of the graph}

\item{choice}{the graph to plot. Allowed values include one of c("quanti.var",
"quali.var", "quali.sup", "group") for plotting quantitative variables,
qualitative variables, supplementary qualitative variables and group of
variables, respectively.}

\item{col.var.sup}{color for supplementary variables.}

\item{col.circle}{a color for the correlation circle. Used only when X is a 
PCA output.}
}
\value{
a ggplot2 plot
}
\description{
Multiple factor analysis (MFA) is used to analyze a data set in 
 which individuals are described by several sets of variables (quantitative 
 and/or qualitative) structured into groups. fviz_mfa() provides
 ggplot2-based elegant visualization of MFA outputs from the R function: MFA
 [FactoMineR].\cr\cr \itemize{ \item fviz_mfa_ind(): Graph of individuals
 \item fviz_mfa_var(): Graph of variables \item fviz_mfa_axes(): Graph of
 partial axes \item fviz_mfa(): An alias of fviz_mfa_ind(res.mfa, partial =
 "all") \item fviz_mfa_quali_biplot(): Biplot of individuals and qualitative
 variables }
}
\examples{
# Compute Multiple Factor Analysis
library("FactoMineR")
data(wine)
res.mfa <- MFA(wine, group=c(2,5,3,10,9,2), type=c("n",rep("s",5)),
               ncp=5, name.group=c("orig","olf","vis","olfag","gust","ens"),
               num.group.sup=c(1,6), graph=FALSE)
               
# Eigenvalues/variances of dimensions
fviz_screeplot(res.mfa)
# Group of variables
fviz_mfa_var(res.mfa, "group")
# Quantitative variables
fviz_mfa_var(res.mfa, "quanti.var", palette = "jco", 
  col.var.sup = "violet", repel = TRUE)
# Graph of individuals colored by cos2
fviz_mfa_ind(res.mfa, col.ind = "cos2", 
  gradient.cols = c("#00AFBB", "#E7B800", "#FC4E07"),
  repel = TRUE)
# Partial individuals
fviz_mfa_ind(res.mfa, partial = rownames(wine)[1:3], col.partial = "black") 
# Partial axes
fviz_mfa_axes(res.mfa)


# Graph of categorical variable categories
# ++++++++++++++++++++++++++++++++++++++++
data(poison)
res.mfa <- MFA(poison, group=c(2,2,5,6), type=c("s","n","n","n"),
               name.group=c("desc","desc2","symptom","eat"),
               num.group.sup=1:2, graph=FALSE)

# Plot of qualitative variables
fviz_mfa_var(res.mfa, "quali.var")
# Plot of supplementary qualitative variables
fviz_mfa_var(res.mfa, "quali.sup")
 
 

# Biplot of categorical variable categories and individuals
# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 # Use repel = TRUE to avoid overplotting
grp <- as.factor(poison[, "Vomiting"])
fviz_mfa_quali_biplot(res.mfa, repel = FALSE, col.var = "#E7B800",
   habillage = grp, addEllipses = TRUE, ellipse.level = 0.95)

}
\references{
\url{https://www.sthda.com/english/}
}
\author{
Fabian Mundt \email{f.mundt@inventionate.de}

Alboukadel Kassambara \email{alboukadel.kassambara@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fviz_mca.R
\name{fviz_mca}
\alias{fviz_mca}
\alias{fviz_mca_ind}
\alias{fviz_mca_var}
\alias{fviz_mca_biplot}
\title{Visualize Multiple Correspondence Analysis}
\usage{
fviz_mca_ind(
  X,
  axes = c(1, 2),
  geom = c("point", "text"),
  geom.ind = geom,
  repel = FALSE,
  habillage = "none",
  palette = NULL,
  addEllipses = FALSE,
  col.ind = "blue",
  col.ind.sup = "darkblue",
  alpha.ind = 1,
  shape.ind = 19,
  map = "symmetric",
  select.ind = list(name = NULL, cos2 = NULL, contrib = NULL),
  ...
)

fviz_mca_var(
  X,
  choice = c("var.cat", "mca.cor", "var", "quanti.sup"),
  axes = c(1, 2),
  geom = c("point", "text"),
  geom.var = geom,
  repel = FALSE,
  col.var = "red",
  alpha.var = 1,
  shape.var = 17,
  col.quanti.sup = "blue",
  col.quali.sup = "darkgreen",
  map = "symmetric",
  select.var = list(name = NULL, cos2 = NULL, contrib = NULL),
  ...
)

fviz_mca_biplot(
  X,
  axes = c(1, 2),
  geom = c("point", "text"),
  geom.ind = geom,
  geom.var = geom,
  repel = FALSE,
  label = "all",
  invisible = "none",
  habillage = "none",
  addEllipses = FALSE,
  palette = NULL,
  arrows = c(FALSE, FALSE),
  map = "symmetric",
  title = "MCA - Biplot",
  ...
)

fviz_mca(X, ...)
}
\arguments{
\item{X}{an object of class MCA [FactoMineR], acm [ade4] and expOutput/epMCA
[ExPosition].}

\item{axes}{a numeric vector of length 2 specifying the dimensions to be 
plotted.}

\item{geom}{a text specifying the geometry to be used for the graph. Allowed 
values are the combination of \code{c("point", "arrow", "text")}. Use 
\code{"point"} (to show only points); \code{"text"} to show only labels; 
\code{c("point", "text")} or \code{c("arrow", "text")} to show arrows and 
texts. Using \code{c("arrow", "text")} is sensible only for the graph of 
variables.}

\item{geom.ind, geom.var}{as \code{geom} but for individuals and variables,
respectively. Default is geom.ind = c("point", "text), geom.var =
c("point", "text").}

\item{repel}{a boolean, whether to use ggrepel to avoid overplotting text
labels or not. The old \code{jitter} argument is kept for backward
compatibility and is converted to \code{repel = TRUE} with a deprecation warning.}

\item{habillage}{an optional factor variable for coloring the observations by 
groups. Default value is "none". If X is an MCA object from FactoMineR 
package, habillage can also specify the index of the factor variable in the 
data.}

\item{palette}{the color palette to be used for coloring or filling by
groups. Allowed values include "grey" for grey color palettes; brewer
palettes e.g. "RdBu", "Blues", ...; or custom color palette e.g. c("blue",
"red"); and scientific journal palettes from ggsci R package, e.g.: "npg",
"aaas", "lancet", "jco", "ucscgb", "uchicago", "simpsons" and
"rickandmorty". Can be also a numeric vector of length(groups); in this
case a basic color palette is created using the function
\link[grDevices]{palette}.}

\item{addEllipses}{logical value. If TRUE, draws ellipses around the 
individuals when habillage != "none".}

\item{col.ind, col.var}{color for individuals and variables, respectively. Can 
be a continuous variable or a factor variable. Possible values include also
: "cos2", "contrib", "coord", "x" or "y". In this case, the colors for
individuals/variables are automatically controlled by their qualities
("cos2"), contributions ("contrib"), coordinates (x^2 + y^2 , "coord"), x
values("x") or y values("y"). To use automatic coloring (by cos2, contrib,
....), make sure that habillage ="none".}

\item{col.ind.sup}{color for supplementary individuals}

\item{alpha.ind, alpha.var}{controls the transparency of individual and 
variable colors, respectively. The value can variate from 0 (total 
transparency) to 1 (no transparency). Default value is 1. Possible values 
include also : "cos2", "contrib", "coord", "x" or "y". In this case, the 
transparency for individual/variable colors are automatically controlled by 
their qualities ("cos2"), contributions ("contrib"), coordinates (x^2 + y^2 
, "coord"), x values("x") or y values("y"). To use this, make sure that 
habillage ="none".}

\item{shape.ind, shape.var}{point shapes of individuals and variables.}

\item{map}{character string specifying the map type. Allowed options include: 
"symmetric", "rowprincipal", "colprincipal", "symbiplot", "rowgab", 
"colgab", "rowgreen" and "colgreen". See details}

\item{select.ind, select.var}{a selection of individuals/variables to be drawn.
Allowed values are NULL or a list containing the arguments name, cos2 or 
contrib: \itemize{ \item name is a character vector containing 
individuals/variables to be drawn \item cos2 if cos2 is in [0, 1], ex: 0.6, 
then individuals/variables with a cos2 > 0.6 are drawn. if cos2 > 1, ex: 5, 
then the top 5 individuals/variables with the highest cos2 are drawn. \item 
contrib if contrib > 1, ex: 5,  then the top 5 individuals/variables with 
the highest contrib are drawn }}

\item{...}{Additional arguments. \itemize{ \item in fviz_mca_ind(), 
fviz_mca_var() and fviz_mca_cor(): Additional arguments are passed to the 
functions fviz() and ggpubr::ggpar(). \item in fviz_mca_biplot() and 
fviz_mca(): Additional arguments are passed to fviz_mca_ind() and 
fviz_mca_var().}}

\item{choice}{the graph to plot. Allowed values include: i) "var" and 
"mca.cor" for plotting the correlation between variables and principal 
dimensions; ii) "var.cat" for variable categories and iii) "quanti.sup" for
the supplementary quantitative variables.}

\item{col.quanti.sup, col.quali.sup}{a color for the quantitative/qualitative 
supplementary variables.}

\item{label}{a text specifying the elements to be labelled. Default value is 
"all". Allowed values are "none" or the combination of c("ind", 
"ind.sup","var", "quali.sup",  "quanti.sup"). "ind" can be used to label 
only active individuals. "ind.sup" is for supplementary individuals. "var" 
is for active variable categories. "quali.sup" is for supplementary 
qualitative variable categories. "quanti.sup" is for quantitative 
supplementary variables.}

\item{invisible}{a text specifying the elements to be hidden on the plot. 
Default value is "none". Allowed values are the combination of c("ind", 
"ind.sup","var", "quali.sup",  "quanti.sup").}

\item{arrows}{Vector of two logicals specifying if the plot should contain 
points (FALSE, default) or arrows (TRUE). First value sets the rows and the 
second value sets the columns.}

\item{title}{the title of the graph}
}
\value{
a ggplot
}
\description{
Multiple Correspondence Analysis (MCA) is an extension of simple 
 CA to analyse a data table containing more than two categorical variables. 
 fviz_mca() provides ggplot2-based elegant visualization of MCA outputs from 
 the R functions: MCA [in FactoMineR], acm [in ade4], and expOutput/epMCA [in
 ExPosition]. Read more: 
 \href{https://www.sthda.com/english/articles/31-principal-component-methods-in-r-practical-guide/114-mca-multiple-correspondence-analysis-in-r-essentials/}{Multiple
  Correspondence Analysis Essentials.}
 
 \itemize{ \item fviz_mca_ind(): Graph of individuals \item fviz_mca_var():
 Graph of variables \item fviz_mca_biplot(): Biplot of individuals and
 variables \item fviz_mca(): An alias of fviz_mca_biplot() }
}
\details{
The default plot of MCA is a "symmetric" plot in which both rows and 
 columns are in principal coordinates. In this situation, it's not possible 
 to interpret the distance between row points and column points. To overcome 
 this problem, the simplest way is to make an asymmetric plot. The argument 
 "map" can be used to change the plot type. For more explanation, read the 
 details section of fviz_ca documentation.
}
\examples{
# Multiple Correspondence Analysis
# ++++++++++++++++++++++++++++++
# Install and load FactoMineR to compute MCA
# install.packages("FactoMineR")
library("FactoMineR")
data(poison)
poison.active <- poison[1:55, 5:15]
head(poison.active)
res.mca <- MCA(poison.active, graph=FALSE)

# Graph of individuals
# +++++++++++++++++++++

# Default Plot
# Color of individuals: col.ind = "steelblue"
fviz_mca_ind(res.mca, col.ind = "steelblue")

# 1. Control automatically the color of individuals 
   # using the "cos2" or the contributions "contrib"
   # cos2 = the quality of the individuals on the factor map
# 2. To keep only point or text use geom = "point" or geom = "text".
# 3. Change themes: http://www.sthda.com/english/wiki/ggplot2-themes

fviz_mca_ind(res.mca, col.ind = "cos2", repel = TRUE)

\dontrun{     
# You can also control the transparency 
# of the color by the cos2
fviz_mca_ind(res.mca, alpha.ind="cos2") 
}
     
# Color individuals by groups, add concentration ellipses
# Remove labels: label = "none".
grp <- as.factor(poison.active[, "Vomiting"])
p <- fviz_mca_ind(res.mca, label="none", habillage=grp,
       addEllipses=TRUE, ellipse.level=0.95)
print(p)
      
    
# Change group colors using RColorBrewer color palettes
# Read more: http://www.sthda.com/english/wiki/ggplot2-colors
p + scale_color_brewer(palette="Dark2") +
    scale_fill_brewer(palette="Dark2") 
     
# Change group colors manually
# Read more: http://www.sthda.com/english/wiki/ggplot2-colors
p + scale_color_manual(values=c("#999999", "#E69F00"))+
 scale_fill_manual(values=c("#999999", "#E69F00"))
             
             
# Select and visualize some individuals (ind) with select.ind argument.
 # - ind with cos2 >= 0.4: select.ind = list(cos2 = 0.4)
 # - Top 20 ind according to the cos2: select.ind = list(cos2 = 20)
 # - Top 20 contributing individuals: select.ind = list(contrib = 20)
 # - Select ind by names: select.ind = list(name = c("44", "38", "53",  "39") )
 
# Example: Select the top 40 according to the cos2
fviz_mca_ind(res.mca, select.ind = list(cos2 = 20))

 
# Graph of variable categories
# ++++++++++++++++++++++++++++
# Default plot: use repel = TRUE to avoid overplotting
fviz_mca_var(res.mca, col.var = "#FC4E07")

# Control variable colors using their contributions
# use repel = TRUE to avoid overplotting
fviz_mca_var(res.mca, col.var = "contrib",
  gradient.cols = c("#00AFBB", "#E7B800", "#FC4E07"))
        
    
# Biplot
# ++++++++++++++++++++++++++
grp <- as.factor(poison.active[, "Vomiting"])
fviz_mca_biplot(res.mca, repel = TRUE, col.var = "#E7B800",
 habillage = grp, addEllipses = TRUE, ellipse.level = 0.95)
 
 \dontrun{
# Keep only the labels for variable categories: 
fviz_mca_biplot(res.mca, label ="var")

# Keep only labels for individuals
fviz_mca_biplot(res.mca, label ="ind")

# Hide variable categories
fviz_mca_biplot(res.mca, invisible ="var")

# Hide individuals
fviz_mca_biplot(res.mca, invisible ="ind")

# Control automatically the color of individuals using the cos2
fviz_mca_biplot(res.mca, label ="var", col.ind="cos2")
       
# Change the color by groups, add ellipses
fviz_mca_biplot(res.mca, label="var", col.var ="blue",
   habillage=grp, addEllipses=TRUE, ellipse.level=0.95)
               
# Select the top 30 contributing individuals
# And the top 10 variables
fviz_mca_biplot(res.mca,  
               select.ind = list(contrib = 30),
               select.var = list(contrib = 10)) 
}

}
\seealso{
\code{\link{get_mca}}, \code{\link{fviz_pca}}, \code{\link{fviz_ca}},
 \code{\link{fviz_mfa}}, \code{\link{fviz_hmfa}}
}
\author{
Alboukadel Kassambara \email{alboukadel.kassambara@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz.R
\name{clean_lock_files}
\alias{clean_lock_files}
\title{Clean up stale package lock files}
\usage{
clean_lock_files(package = "factoextra", lib = .libPaths()[1], ask = TRUE)
}
\arguments{
\item{package}{Character string specifying which package lock to remove.
Default is "factoextra". Use "all" to remove all lock files.}

\item{lib}{Library path to check. Default is the first library in .libPaths().}

\item{ask}{Logical. If TRUE (default), asks for confirmation before removing.}
}
\value{
Invisibly returns TRUE if files were removed, FALSE otherwise.
}
\description{
Removes stale lock directories that can prevent package installation.

Lock files are created during package installation and should be automatically
removed when installation completes. If installation is interrupted (e.g., by
closing R or a crash), these lock files may remain and block future installations.
}
\examples{
\dontrun{
# Remove factoextra lock file
clean_lock_files()

# Remove all lock files
clean_lock_files("all")

# Remove without confirmation
clean_lock_files(ask = FALSE)
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fviz_cluster.R
\name{fviz_cluster}
\alias{fviz_cluster}
\title{Visualize Clustering Results}
\usage{
fviz_cluster(
  object,
  data = NULL,
  choose.vars = NULL,
  stand = TRUE,
  axes = c(1, 2),
  geom = c("point", "text"),
  repel = FALSE,
  show.clust.cent = TRUE,
  ellipse = TRUE,
  ellipse.type = "convex",
  ellipse.level = 0.95,
  ellipse.alpha = 0.2,
  shape = NULL,
  pointsize = 1.5,
  labelsize = 12,
  main = "Cluster plot",
  xlab = NULL,
  ylab = NULL,
  outlier.color = "black",
  outlier.shape = 19,
  outlier.pointsize = pointsize,
  outlier.labelsize = labelsize,
  ggtheme = theme_grey(),
  ...
)
}
\arguments{
\item{object}{an object of class "partition" created by the functions pam(),
clara() or fanny() in cluster package; "kmeans" [in stats package]; "dbscan"
[in fpc package]; "Mclust" [in mclust]; "hkmeans", "eclust" [in factoextra].
Possible value are also any list object with data and cluster components
(e.g.: object = list(data = mydata, cluster = myclust)).}

\item{data}{the data that has been used for clustering. Required only when
object is a class of kmeans or dbscan.}

\item{choose.vars}{a character vector containing variables to be considered
for plotting.}

\item{stand}{logical value; if TRUE, data is standardized before principal
component analysis. If scaling produces \code{NA} values,
\code{fviz_cluster()} stops with a package-level error.}

\item{axes}{a numeric vector of length 2 specifying the dimensions to be
plotted.}

\item{geom}{a text specifying the geometry to be used for the graph. Allowed
values are the combination of c("point", "text"). Use "point" (to show only
points);  "text" to show only labels; c("point", "text") to show both types.}

\item{repel}{a boolean, whether to use ggrepel to avoid overplotting text
labels or not. The old \code{jitter} argument is kept for backward
compatibility and is converted to \code{repel = TRUE} with a deprecation warning.}

\item{show.clust.cent}{logical; if TRUE, shows cluster centers}

\item{ellipse}{logical value; if TRUE, draws outline around points of each
cluster}

\item{ellipse.type}{Character specifying frame type. Possible values are
'convex', 'confidence' or types supported by
\code{\link[ggplot2]{stat_ellipse}} including one of c("t", "norm",
"euclid").}

\item{ellipse.level}{the size of the concentration ellipse in normal
probability. Passed for \code{ggplot2::stat_ellipse} 's level. Ignored in
'convex'. Default value is 0.95.}

\item{ellipse.alpha}{Alpha for frame specifying the transparency level of fill
color. Use alpha = 0 for no fill color.}

\item{shape}{the shape of points.}

\item{pointsize}{the size of points}

\item{labelsize}{font size for the labels}

\item{main}{plot main title.}

\item{xlab, ylab}{character vector specifying x and y axis labels,
respectively. Use xlab = FALSE and ylab = FALSE to hide xlab and ylab,
respectively.}

\item{outlier.pointsize, outlier.color, outlier.shape, outlier.labelsize}{arguments for customizing outliers, which can be detected only in DBSCAN
clustering.}

\item{ggtheme}{function, ggplot2 theme name. Default value is theme_pubr().
Allowed values include ggplot2 official themes: theme_gray(), theme_bw(),
theme_minimal(), theme_classic(), theme_void(), ....}

\item{...}{other arguments to be passed to the functions
\code{\link[ggpubr]{ggscatter}} and \code{\link[ggpubr]{ggpar}}.}
}
\value{
a ggplot2 object.
}
\description{
Provides ggplot2-based elegant visualization of partitioning
 methods including kmeans [stats package]; pam, clara and fanny [cluster
 package]; dbscan [fpc package]; Mclust [mclust package]; HCPC [FactoMineR];
 hkmeans [factoextra]. Observations are represented by points in the plot,
 using principal components if ncol(data) > 2. An ellipse is drawn around
 each cluster. When \code{stand = TRUE}, the plotting data must remain
 finite after scaling.
}
\examples{
set.seed(123)

# Data preparation
# +++++++++++++++
data("iris")
head(iris)
# Remove species column (5) and scale the data
iris.scaled <- scale(iris[, -5])

# K-means clustering
# +++++++++++++++++++++
km.res <- kmeans(iris.scaled, 3, nstart = 10)

# Visualize kmeans clustering
# use repel = TRUE to avoid overplotting
fviz_cluster(km.res, iris[, -5], ellipse.type = "norm")


# Change the color palette and theme
fviz_cluster(km.res, iris[, -5],
   palette = "Set2", ggtheme = theme_minimal())

 \dontrun{
# Show points only
fviz_cluster(km.res, iris[, -5], geom = "point")
# Show text only
fviz_cluster(km.res, iris[, -5], geom = "text")

# PAM clustering
# ++++++++++++++++++++
requireNamespace("cluster", quietly = TRUE)
pam.res <- pam(iris.scaled, 3)
 # Visualize pam clustering
fviz_cluster(pam.res, geom = "point", ellipse.type = "norm")

# Hierarchical clustering
# ++++++++++++++++++++++++
# Use hcut() which compute hclust and cut the tree
hc.cut <- hcut(iris.scaled, k = 3, hc_method = "complete")
# Visualize dendrogram
fviz_dend(hc.cut, show_labels = FALSE, rect = TRUE)
# Visualize cluster
fviz_cluster(hc.cut, ellipse.type = "convex")

}



}
\seealso{
\code{\link{fviz_silhouette}}, \code{\link{hcut}},
 \code{\link{hkmeans}},  \code{\link{eclust}}, \code{\link{fviz_dend}}
}
\author{
Alboukadel Kassambara \email{alboukadel.kassambara@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hkmeans.R
\name{hkmeans}
\alias{hkmeans}
\alias{print.hkmeans}
\alias{hkmeans_tree}
\title{Hierarchical k-means clustering}
\usage{
hkmeans(
  x,
  k,
  hc.metric = "euclidean",
  hc.method = "ward.D2",
  iter.max = 10,
  km.algorithm = "Hartigan-Wong"
)

\method{print}{hkmeans}(x, ...)

hkmeans_tree(hkmeans, rect.col = NULL, ...)
}
\arguments{
\item{x}{a numeric matrix, data frame or vector}

\item{k}{a single integer specifying the number of clusters to be generated.
Must be at least 2 and smaller than \code{nrow(x)}.}

\item{hc.metric}{the distance measure to be used. Possible values are "euclidean", "maximum", "manhattan", 
"canberra", "binary" or "minkowski" (see ?dist).}

\item{hc.method}{the agglomeration method to be used. Possible values include "ward.D", "ward.D2", "single", 
"complete", "average", "mcquitty", "median"or "centroid" (see ?hclust).}

\item{iter.max}{the maximum number of iterations allowed for k-means.}

\item{km.algorithm}{the algorithm to be used for kmeans (see ?kmeans).}

\item{...}{others arguments to be passed to the function plot.hclust(); (see ? plot.hclust)}

\item{hkmeans}{an object of class hkmeans (returned by the function hkmeans())}

\item{rect.col}{Vector with border colors for the rectangles around clusters in dendrogram}
}
\value{
hkmeans returns an object of class "hkmeans" containing the following components:
\itemize{
\item The elements returned by the standard function kmeans() (see ?kmeans)
\item data: the data used for the analysis
\item hclust: an object of class "hclust" generated by the function hclust()
}
}
\description{
The final k-means clustering solution is very sensitive to the initial random selection 
of cluster centers. This function provides a solution using an hybrid approach by combining 
the hierarchical clustering and the k-means methods. The procedure is explained in "Details" section. Read more:
  \href{https://www.datanovia.com/en/lessons/hierarchical-k-means-clustering-optimize-clusters/}{Hybrid hierarchical k-means clustering for optimizing clustering outputs}.
\itemize{
\item hkmeans(): compute hierarchical k-means clustering
\item print.hkmeans(): prints the result of hkmeans
\item hkmeans_tree(): plots the initial dendrogram
}
}
\details{
The procedure is as follow:   
     
1. Compute hierarchical clustering
   
2. Cut the tree in k-clusters
   
3. compute the center (i.e the mean) of each cluster
     
4. Do k-means by using the set of cluster centers (defined in step 3) as the initial cluster centers. Optimize the clustering.  
    
This means that the final optimized partitioning obtained at step 4 might be different from the initial partitioning obtained at step 2. 
Consider mainly the result displayed by \code{fviz_cluster()}.
}
\examples{
\donttest{
# Load data
data(USArrests)
# Scale the data
df <- scale(USArrests)

# Compute hierarchical k-means clustering
res.hk <-hkmeans(df, 4)

# Elements returned by hkmeans()
names(res.hk)

# Print the results
res.hk

# Visualize the tree
hkmeans_tree(res.hk, cex = 0.6)
# or use this
fviz_dend(res.hk, cex = 0.6)


# Visualize the hkmeans final clusters
fviz_cluster(res.hk, ellipse.type = "norm", ellipse.level = 0.68)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hcut.R
\name{hcut}
\alias{hcut}
\title{Computes Hierarchical Clustering and Cut the Tree}
\usage{
hcut(
  x,
  k = 2,
  isdiss = inherits(x, "dist"),
  hc_func = c("hclust", "agnes", "diana"),
  hc_method = "ward.D2",
  hc_metric = "euclidean",
  stand = FALSE,
  graph = FALSE,
  ...
)
}
\arguments{
\item{x}{a numeric matrix, numeric data frame or a dissimilarity matrix.}

\item{k}{a single integer specifying the number of clusters to be generated.
Must be at least 2 and smaller than the number of observations.}

\item{isdiss}{logical value specifying whether \code{x} is already a
dissimilarity matrix. If TRUE, \code{x} must inherit from class
\code{"dist"} and contain only finite values.}

\item{hc_func}{the hierarchical clustering function to be used. Default value is "hclust". Possible values 
is one of "hclust", "agnes", "diana". Abbreviation is allowed.}

\item{hc_method}{the agglomeration method to be used (?hclust) for hclust() and agnes(): 
"ward.D", "ward.D2", "single", "complete", "average", ...}

\item{hc_metric}{character string specifying the metric to be used for calculating 
dissimilarities between observations. Allowed values are those accepted by the function dist() [including "euclidean", "manhattan", "maximum",
 "canberra", "binary", "minkowski"] and correlation based distance measures ["pearson", "spearman" or "kendall"].}

\item{stand}{logical value; default is FALSE. If TRUE, then the data will be standardized using the function scale(). 
Measurements are standardized for each variable (column), by subtracting the
variable's mean value and dividing by the variable's standard deviation. If
scaling produces \code{NA} values, \code{hcut()} stops with a package-level
error.}

\item{graph}{logical value. If TRUE, the dendrogram is displayed.}

\item{...}{not used.}
}
\value{
an object of class "hcut" containing the result 
of the standard function used (read the documentation of hclust, agnes, diana). 
      
It includes also:
\itemize{
\item cluster: the cluster assignment of observations after cutting the tree
\item nbclust: the number of clusters
\item silinfo: the silhouette information of observations (available when
  \code{k > 1})
\item size: the size of clusters
\item data: a matrix containing the original or  the standardized data (if stand = TRUE)
}
}
\description{
Computes hierarchical clustering (hclust, agnes, diana) and cuts the tree
into \code{k} clusters. It also accepts correlation-based distance measures
such as "pearson", "spearman" and "kendall". Direct calls require
\code{k >= 2}; helper-level one-cluster handling is implemented in callers
such as \code{eclust()} and \code{fviz_nbclust()}.
}
\examples{
\donttest{
data(USArrests)

# Compute hierarchical clustering and cut into 4 clusters
res <- hcut(USArrests, k = 4, stand = TRUE)

# Cluster assignments of observations
res$cluster
# Size of clusters
res$size

# Visualize the dendrogram
fviz_dend(res, rect = TRUE)

# Visualize the silhouette
fviz_silhouette(res)

# Visualize clusters as scatter plots
fviz_cluster(res)
}

}
\seealso{
\code{\link{fviz_dend}}, \code{\link{hkmeans}}, \code{\link{eclust}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_clust_tendency.R
\name{get_clust_tendency}
\alias{get_clust_tendency}
\title{Assessing Clustering Tendency}
\usage{
get_clust_tendency(
  data,
  n,
  graph = TRUE,
  gradient = list(low = "red", mid = "white", high = "blue"),
  seed = NULL
)
}
\arguments{
\item{data}{a numeric data frame or matrix. Columns are variables and rows 
are samples. Computation are done on rows (samples) by default. If you want
to calculate Hopkins statistic on variables, transpose the data before.}

\item{n}{a positive integer specifying the number of points selected from
sample space and from the observed data. Must be smaller than the number
of complete observations.}

\item{graph}{logical value; if TRUE the ordered dissimilarity image (ODI) is 
shown.}

\item{gradient}{a list containing three elements specifying the colors for 
low, mid and high values in the ordered dissimilarity image. The element 
"mid" can take the value of NULL.}

\item{seed}{an integer seed for reproducibility, or NULL to use the current
RNG stream. When non-NULL, the function restores the caller RNG state on
exit.}
}
\value{
A list containing the elements:
  
  - hopkins_stat for Hopkins statistic value
  
  - plot for ordered dissimilarity image. This is generated using the 
  function \code{\link{fviz_dist}}(dist.obj).
}
\description{
Before applying cluster methods, the first step is to assess 
  whether the data is clusterable, a process defined as the \strong{assessing
  of clustering tendency}. get_clust_tendency() assesses clustering tendency 
  using Hopkins' statistic and a visual approach. An ordered dissimilarity 
  image (ODI) is shown. Objects belonging to the same cluster are displayed 
  in consecutive order using hierarchical clustering. For more details and 
  interpretation, see 
  \href{https://www.datanovia.com/en/lessons/assessing-clustering-tendency/}{STHDA
   website: Assessing clustering tendency}.
}
\details{
\strong{Hopkins statistic}: If the value of Hopkins statistic is close to
1 (far above 0.5), then we can conclude that the dataset is significantly
clusterable. The statistic is calculated using the correct formula from
Cross and Jain (1982) with exponent d=D where D is the dimensionality
(number of columns) of the data. Under the null hypothesis of spatial
randomness, the Hopkins statistic follows a Beta(n, n) distribution.

\strong{Note on interpretation}: This function returns the Hopkins statistic H
where values close to 1 indicate clusterable data. Some other packages (e.g.,
\code{performance::check_clusterstructure}) return 1-H, where values close to
0 indicate clusterability. Always check the documentation of the specific
implementation you are using.

\strong{Breaking change}: factoextra uses the corrected Hopkins statistic
formula (Wright 2022). Results differ from legacy factoextra and a one-time
warning is emitted. Set \code{options(factoextra.warn_hopkins = FALSE)} to
silence the warning.

For large datasets, nearest-neighbor distances are computed with a low-memory
fallback when the full pairwise matrix would exceed
\code{getOption("factoextra.hopkins.max_matrix_cells", 2e7)} cells.

\strong{VAT (Visual Assessment of cluster Tendency)}: The VAT detects the
clustering tendency in a visual form by counting the number of square shaped
dark (or colored) blocks along the diagonal in a VAT image.
}
\examples{
data(iris)

# Silence the one-time compatibility warning in examples
old_hopkins_warn <- getOption("factoextra.warn_hopkins")
options(factoextra.warn_hopkins = FALSE)

# Clustering tendency
gradient_col = list(low = "steelblue", high = "white")
get_clust_tendency(iris[,-5], n = 50, gradient = gradient_col)
   
# Random uniformly distributed dataset
# (without any inherent clusters)
set.seed(123)
random_df <- apply(iris[, -5], 2, 
                   function(x){runif(length(x), min(x), max(x))}
                   )
get_clust_tendency(random_df, n = 50, gradient = gradient_col)
options(factoextra.warn_hopkins = old_hopkins_warn)

}
\seealso{
\code{\link{fviz_dist}}
}
\author{
Alboukadel Kassambara \email{alboukadel.kassambara@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fviz_hmfa.R
\name{fviz_hmfa}
\alias{fviz_hmfa}
\alias{fviz_hmfa_ind}
\alias{fviz_hmfa_var}
\alias{fviz_hmfa_quali_biplot}
\title{Visualize Hierarchical Multiple Factor Analysis}
\usage{
fviz_hmfa_ind(
  X,
  axes = c(1, 2),
  geom = c("point", "text"),
  repel = FALSE,
  habillage = "none",
  addEllipses = FALSE,
  shape.ind = 19,
  col.ind = "blue",
  col.ind.sup = "darkblue",
  alpha.ind = 1,
  select.ind = list(name = NULL, cos2 = NULL, contrib = NULL),
  partial = NULL,
  col.partial = "group",
  group.names = NULL,
  node.level = 1,
  ...
)

fviz_hmfa_var(
  X,
  choice = c("quanti.var", "quali.var", "group"),
  axes = c(1, 2),
  geom = c("point", "text"),
  repel = FALSE,
  col.var = "red",
  alpha.var = 1,
  shape.var = 17,
  col.var.sup = "darkgreen",
  select.var = list(name = NULL, cos2 = NULL, contrib = NULL),
  ...
)

fviz_hmfa_quali_biplot(
  X,
  axes = c(1, 2),
  geom = c("point", "text"),
  repel = FALSE,
  habillage = "none",
  title = "Biplot of individuals and qualitative variables - HMFA",
  ...
)

fviz_hmfa(X, ...)
}
\arguments{
\item{X}{an object of class HMFA [FactoMineR].}

\item{axes}{a numeric vector of length 2 specifying the dimensions to be 
plotted.}

\item{geom}{a text specifying the geometry to be used for the graph. Allowed 
values are the combination of \code{c("point", "arrow", "text")}. Use 
\code{"point"} (to show only points); \code{"text"} to show only labels; 
\code{c("point", "text")} or \code{c("arrow", "text")} to show arrows and 
texts. Using \code{c("arrow", "text")} is sensible only for the graph of 
variables.}

\item{repel}{a boolean, whether to use ggrepel to avoid overplotting text
labels or not. The old \code{jitter} argument is kept for backward
compatibility and is converted to \code{repel = TRUE} with a deprecation warning.}

\item{habillage}{an optional factor variable for coloring the observations by 
groups. Default value is "none". If X is an HMFA object from FactoMineR 
package, habillage can also specify the index of the factor variable in the 
data.}

\item{addEllipses}{logical value. If TRUE, draws ellipses around the 
individuals when habillage != "none".}

\item{shape.ind, shape.var}{point shapes of individuals and variables, 
respectively.}

\item{col.ind, col.var}{color for individuals, partial individuals and 
variables, respectively. Can be a continuous variable or a factor variable.
Possible values include also : "cos2", "contrib", "coord", "x" or "y". In
this case, the colors for individuals/variables are automatically controlled
by their qualities ("cos2"), contributions ("contrib"), coordinates (x^2 +
y^2 , "coord"), x values("x") or y values("y"). To use automatic coloring
(by cos2, contrib, ....), make sure that habillage ="none".}

\item{col.ind.sup}{color for supplementary individuals}

\item{alpha.ind, alpha.var}{controls the transparency of individual, partial 
individual and variable, respectively. The value can variate from 0 (total 
transparency) to 1 (no transparency). Default value is 1. Possible values 
include also : "cos2", "contrib", "coord", "x" or "y". In this case, the 
transparency for individual/variable colors are automatically controlled by 
their qualities ("cos2"), contributions ("contrib"), coordinates (x^2 + y^2 
, "coord"), x values("x") or y values("y"). To use this, make sure that 
habillage ="none".}

\item{select.ind, select.var}{a selection of individuals and variables to be 
drawn. Allowed values are NULL or a list containing the arguments name, cos2
or contrib: \itemize{ \item name is a character vector containing 
individuals/variables to be drawn \item cos2 if cos2 is in [0, 1], ex: 0.6, 
then individuals/variables with a cos2 > 0.6 are drawn. if cos2 > 1, ex: 5, 
then the top 5 individuals/variables with the highest cos2 are drawn. \item 
contrib if contrib > 1, ex: 5,  then the top 5 individuals/variables with 
the highest cos2 are drawn }}

\item{partial}{list of the individuals for which the partial points should be
drawn. (by default, partial = NULL and no partial points are drawn). Use
partial = "all" to visualize partial points for all individuals.}

\item{col.partial}{color for partial individuals. By default, points are 
colored according to the groups.}

\item{group.names}{a vector containing the name of the groups (by default, 
NULL and the group are named group.1, group.2 and so on).}

\item{node.level}{a single number indicating the HMFA node level to plot.}

\item{...}{Arguments to be passed to the function fviz() and ggpubr::ggpar()}

\item{choice}{the graph to plot. Allowed values include one of c("quanti.var",
"quali.var", "group") for plotting quantitative variables, qualitative 
variables and group of variables, respectively.}

\item{col.var.sup}{color for supplementary variables.}

\item{title}{the title of the graph}
}
\value{
a ggplot
}
\description{
Hierarchical Multiple Factor Analysis (HMFA) is, an extension of
 MFA, used in a situation where the data are organized into a hierarchical
 structure.  fviz_hmfa() provides ggplot2-based elegant visualization of HMFA
 outputs from the R function: HMFA [FactoMineR].\cr\cr \itemize{
 \item fviz_hmfa_ind(): Graph of individuals \item fviz_hmfa_var(): Graph of
 variables \item fviz_hmfa_quali_biplot(): Biplot of individuals and
 qualitative variables \item fviz_hmfa(): An alias of fviz_hmfa_ind() }
}
\examples{
# Hierarchical Multiple Factor Analysis
# ++++++++++++++++++++++++
# Install and load FactoMineR to compute MFA
# install.packages("FactoMineR")
library("FactoMineR")
data(wine)
hierar <- list(c(2,5,3,10,9,2), c(4,2))
res.hmfa <- HMFA(wine, H = hierar, type=c("n",rep("s",5)), graph = FALSE)

# Graph of individuals
# ++++++++++++++++++++
# Color of individuals: col.ind = "#2E9FDF"
# Use repel = TRUE to avoid overplotting (slow if many points)
fviz_hmfa_ind(res.hmfa, repel = TRUE, col.ind = "#2E9FDF")

# Color individuals by groups, add concentration ellipses
# Remove labels: label = "none".
# Change color palette to "jco". See ?ggpubr::ggpar
grp <- as.factor(wine[,1])
p <- fviz_hmfa_ind(res.hmfa, label="none", habillage=grp,
       addEllipses=TRUE, palette = "jco")
print(p)
 

# Graph of variables
# ++++++++++++++++++++++++++++++++++++++++
# Quantitative variables
fviz_hmfa_var(res.hmfa, "quanti.var")
# Graph of categorical variable categories
fviz_hmfa_var(res.hmfa, "quali.var")
# Groups of variables (correlation square)
fviz_hmfa_var(res.hmfa, "group")


# Biplot of categorical variable categories and individuals
# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
fviz_hmfa_quali_biplot(res.hmfa)
    
# Graph of partial individuals (starplot)
# +++++++++++++++++++++++++++++++++++++++
fviz_hmfa_ind(res.hmfa, partial = "all", col.partial = "black")
 

}
\references{
\url{https://www.sthda.com/english/}
}
\author{
Fabian Mundt \email{f.mundt@inventionate.de}

Alboukadel Kassambara \email{alboukadel.kassambara@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_ca.R
\name{get_ca}
\alias{get_ca}
\alias{get_ca_col}
\alias{get_ca_row}
\title{Extract the results for rows/columns - CA}
\usage{
get_ca(res.ca, element = c("row", "col"))

get_ca_col(res.ca)

get_ca_row(res.ca)
}
\arguments{
\item{res.ca}{an object of class CA [FactoMineR], ca [ca], coa [ade4];
correspondence [MASS].}

\item{element}{the element to subset from the output. Possible values are "row" or "col".}
}
\value{
a list of matrices containing the results for the active rows/columns including : 
\item{coord}{coordinates for the rows/columns}
\item{cos2}{cos2 for the rows/columns}
\item{contrib}{contributions of the rows/columns}
\item{inertia}{inertia of the rows/columns}
}
\description{
Extract all the results (coordinates, squared cosine, contributions and inertia) 
for the active row/column variables from Correspondence Analysis (CA) outputs.\cr\cr
\itemize{
\item get_ca(): Extract the results for rows and columns
\item get_ca_row(): Extract the results for rows only
\item get_ca_col(): Extract the results for columns only
}
}
\examples{
\donttest{
# Install and load FactoMineR to compute CA
# install.packages("FactoMineR")
 library("FactoMineR")
 data("housetasks")
 res.ca <- CA(housetasks, graph = FALSE)
 
# Result for column variables
 col <- get_ca_col(res.ca)
 col # print
 head(col$coord) # column coordinates
 head(col$cos2) # column cos2
 head(col$contrib) # column contributions
 
# Result for row variables
 row <- get_ca_row(res.ca)
 row # print
 head(row$coord) # row coordinates
 head(row$cos2) # row cos2
 head(row$contrib) # row contributions
 
 # You can also use the function get_ca()
 get_ca(res.ca, "row") # Results for rows
 get_ca(res.ca, "col") # Results for columns
 }
}
\references{
\url{https://www.sthda.com/english/}
}
\author{
Alboukadel Kassambara \email{alboukadel.kassambara@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.factoextra.R
\name{print.factoextra}
\alias{print.factoextra}
\title{Print method for an object of class factoextra}
\usage{
\method{print}{factoextra}(x, ...)
}
\arguments{
\item{x}{an object of class factoextra}

\item{...}{further arguments to be passed to print method}
}
\description{
Print method for an object of class factoextra
}
\examples{
 data(iris)
 res.pca <- prcomp(iris[, -5],  scale = TRUE)
 ind <- get_pca_ind(res.pca, data = iris[, -5])
 print(ind)
 
}
\author{
Alboukadel Kassambara \email{alboukadel.kassambara@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fviz_silhouette.R
\name{fviz_silhouette}
\alias{fviz_silhouette}
\title{Visualize Silhouette Information from Clustering}
\usage{
fviz_silhouette(sil.obj, label = FALSE, print.summary = TRUE, ...)
}
\arguments{
\item{sil.obj}{an object of class silhouette: pam, clara, fanny [in cluster
package]; eclust and hcut [in factoextra]. For \code{eclust} and
\code{hcut}, silhouette information must be available, which requires at
least two clusters.}

\item{label}{logical value. If true, x axis tick labels are shown}

\item{print.summary}{logical value. If true a summary of cluster silhouettes 
are printed in fviz_silhouette().}

\item{...}{other arguments to be passed to the function ggpubr::ggpar().}
}
\value{
a ggplot2 object.
}
\description{
Silhouette (Si) analysis is a cluster validation approach that
 measures how well an observation is clustered and it estimates the average
 distance between clusters. fviz_silhouette() provides ggplot2-based elegant
 visualization of silhouette information from i) the result of
 \code{\link[cluster]{silhouette}}(), \code{\link[cluster]{pam}}(),
 \code{\link[cluster]{clara}}() and \code{\link[cluster]{fanny}}() [in
 cluster package]; ii) \code{\link{eclust}}() and \code{\link{hcut}}() [in
 factoextra]. Results without silhouette information, such as one-cluster
 \code{eclust}/\code{hcut} objects, are rejected with a package-level error.
 
 Read more: 
 \href{https://www.datanovia.com/en/lessons/cluster-validation-statistics-must-know-methods/}{Clustering
  Validation Statistics}.
}
\details{
- Observations with a large silhouhette Si (almost 1) are very well 
 clustered.
 
 - A small Si (around 0) means that the observation lies between two
 clusters.
 
 - Observations with a negative Si are probably placed in the wrong cluster.

 - Silhouette plots require at least two clusters and available silhouette
 widths.
}
\examples{
set.seed(123)

# Data preparation
# +++++++++++++++
data("iris")
head(iris)
# Remove species column (5) and scale the data
iris.scaled <- scale(iris[, -5])

# K-means clustering
# +++++++++++++++++++++
km.res <- kmeans(iris.scaled, 3, nstart = 2)

# Visualize kmeans clustering
fviz_cluster(km.res, iris[, -5], ellipse.type = "norm")+
theme_minimal()

# Visualize silhouette information
requireNamespace("cluster", quietly = TRUE)
sil <- cluster::silhouette(km.res$cluster, dist(iris.scaled))
fviz_silhouette(sil)

# Identify observation with negative silhouette
neg_sil_index <- which(sil[, "sil_width"] < 0)
sil[neg_sil_index, , drop = FALSE]
\dontrun{
# PAM clustering
# ++++++++++++++++++++
requireNamespace("cluster", quietly = TRUE)
pam.res <- cluster::pam(iris.scaled, 3)
# Visualize pam clustering
fviz_cluster(pam.res, ellipse.type = "norm")+
theme_minimal()
# Visualize silhouette information
fviz_silhouette(pam.res)

# Hierarchical clustering
# ++++++++++++++++++++++++
# Use hcut() which compute hclust and cut the tree
hc.cut <- hcut(iris.scaled, k = 3, hc_method = "complete")
# Visualize dendrogram
fviz_dend(hc.cut, show_labels = FALSE, rect = TRUE)
# Visualize silhouette information
if (hc.cut$nbclust > 1) fviz_silhouette(hc.cut)
}
}
\seealso{
\code{\link{fviz_cluster}}, \code{\link{hcut}}, 
 \code{\link{hkmeans}},  \code{\link{eclust}}, \code{\link{fviz_dend}}
}
\author{
Alboukadel Kassambara \email{alboukadel.kassambara@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fviz_ellipses.R
\name{fviz_ellipses}
\alias{fviz_ellipses}
\title{Draw confidence ellipses around the categories}
\usage{
fviz_ellipses(
  X,
  habillage,
  axes = c(1, 2),
  addEllipses = TRUE,
  ellipse.type = "confidence",
  palette = NULL,
  pointsize = 1,
  geom = c("point", "text"),
  ggtheme = theme_bw(),
  ...
)
}
\arguments{
\item{X}{an object of class MCA, PCA or MFA.}

\item{habillage}{a numeric vector of indexes of variables or a
character vector of names of variables. Can be also a data frame containing grouping variables.}

\item{axes}{a numeric vector specifying the axes of interest. Values must be
positive integer indices within the available dimensions. Default values
are 1:2 for axes 1 and 2.}

\item{addEllipses}{logical value. If TRUE, draws ellipses around the 
individuals when habillage != "none".}

\item{ellipse.type}{Character specifying frame type. Possible values are 
\code{"convex"}, \code{"confidence"} or types supported by 
\code{\link[ggplot2]{stat_ellipse}()} including one of \code{c("t", "norm", 
"euclid")} for plotting concentration ellipses.

\itemize{ \item \code{"convex"}: plot convex hull of a set of points. \item 
\code{"confidence"}: plot confidence ellipses around group mean points as
\code{\link[FactoMineR]{coord.ellipse}()}[in FactoMineR]. \item \code{"t"}:
assumes a multivariate t-distribution. \item \code{"norm"}: assumes a
multivariate normal distribution. \item \code{"euclid"}: draws a circle with
the radius equal to level, representing the euclidean distance from the
center. This ellipse probably won't appear circular unless
\code{\link[ggplot2]{coord_fixed}()} is applied.}}

\item{palette}{the color palette to be used for coloring or filling by
groups. Allowed values include "grey" for grey color palettes; brewer
palettes e.g. "RdBu", "Blues", ...; or custom color palette e.g. c("blue",
"red"); and scientific journal palettes from ggsci R package, e.g.: "npg",
"aaas", "lancet", "jco", "ucscgb", "uchicago", "simpsons" and
"rickandmorty". Can be also a numeric vector of length(groups); in this
case a basic color palette is created using the function
\link[grDevices]{palette}.}

\item{pointsize}{the size of points}

\item{geom}{a text specifying the geometry to be used for the graph.  Allowed
values are the combination of c("point", "text"). Use "point" (to show only
points); "text" to show only labels; c("point", "text") to show both types.}

\item{ggtheme}{function, ggplot2 theme name. Default value is theme_pubr().
Allowed values include ggplot2 official themes: theme_gray(), theme_bw(),
theme_minimal(), theme_classic(), theme_void(), ....}

\item{...}{Arguments to be passed to the functions ggpubr::ggscatter()  &
ggpubr::ggpar().}
}
\value{
a ggplot
}
\description{
Draw confidence ellipses around the categories
}
\examples{
\donttest{

# Multiple Correspondence Analysis
# +++++++++++++++++++++++++++++++++
library(FactoMineR)
data(poison)
res.mca <- MCA(poison, quanti.sup = 1:2, 
              quali.sup = 3:4, graph=FALSE)
              
fviz_ellipses(res.mca, 3:4, geom = "point",
palette = "jco") 

 }
}
\author{
Alboukadel Kassambara \email{alboukadel.kassambara@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fviz_ca.R
\name{fviz_ca}
\alias{fviz_ca}
\alias{fviz_ca_row}
\alias{fviz_ca_col}
\alias{fviz_ca_biplot}
\title{Visualize Correspondence Analysis}
\usage{
fviz_ca_row(
  X,
  axes = c(1, 2),
  geom = c("point", "text"),
  geom.row = geom,
  shape.row = 19,
  col.row = "blue",
  alpha.row = 1,
  col.row.sup = "darkblue",
  select.row = list(name = NULL, cos2 = NULL, contrib = NULL),
  map = "symmetric",
  repel = FALSE,
  ...
)

fviz_ca_col(
  X,
  axes = c(1, 2),
  shape.col = 17,
  geom = c("point", "text"),
  geom.col = geom,
  col.col = "red",
  col.col.sup = "darkred",
  alpha.col = 1,
  select.col = list(name = NULL, cos2 = NULL, contrib = NULL),
  map = "symmetric",
  repel = FALSE,
  ...
)

fviz_ca_biplot(
  X,
  axes = c(1, 2),
  geom = c("point", "text"),
  geom.row = geom,
  geom.col = geom,
  label = "all",
  invisible = "none",
  arrows = c(FALSE, FALSE),
  repel = FALSE,
  title = "CA - Biplot",
  ...
)

fviz_ca(X, ...)
}
\arguments{
\item{X}{an object of class CA [FactoMineR], ca [ca], coa [ade4]; 
correspondence [MASS] and expOutput/epCA [ExPosition].}

\item{axes}{a numeric vector of length 2 specifying the dimensions to be 
plotted.}

\item{geom}{a character specifying the geometry to be used for the graph. 
Allowed values are the combination of c("point", "arrow", "text"). Use 
"point" (to show only points); "text" to show only labels; c("point", 
"text") or c("arrow", "text") to show both types.}

\item{geom.row, geom.col}{as \code{geom} but for row and column elements,
respectively. Default is geom.row = c("point", "text), geom.col =
c("point", "text").}

\item{shape.row, shape.col}{the point shapes to be used for row/column 
variables. Default values are 19 for rows and 17 for columns.}

\item{map}{character string specifying the map type. Allowed options include: 
"symmetric", "rowprincipal", "colprincipal", "symbiplot", "rowgab", 
"colgab", "rowgreen" and "colgreen". See details}

\item{repel}{a boolean, whether to use ggrepel to avoid overplotting text
labels or not. The old \code{jitter} argument is kept for backward
compatibility and is converted to \code{repel = TRUE} with a deprecation warning.}

\item{...}{Additional arguments. \itemize{ \item in fviz_ca_row() and 
fviz_ca_col(): Additional arguments are passed to the functions fviz() and
ggpubr::ggpar(). \item in fviz_ca_biplot() and fviz_ca(): Additional 
arguments are passed to fviz_ca_row() and fviz_ca_col().}}

\item{col.col, col.row}{color for column/row points. The default values are 
"red" and "blue", respectively. Can be a continuous variable or a factor
variable. Allowed values include also : "cos2", "contrib", "coord", "x" or
"y". In this case, the colors for row/column variables are automatically
controlled by their qualities ("cos2"), contributions ("contrib"),
coordinates (x^2 + y^2, "coord"), x values("x") or y values("y")}

\item{col.col.sup, col.row.sup}{colors for the supplementary column and row 
points, respectively.}

\item{alpha.col, alpha.row}{controls the transparency of colors. The value can 
variate from 0 (total transparency) to 1 (no transparency). Default value is
1. Allowed values include also : "cos2", "contrib", "coord", "x" or "y" as 
for the arguments col.col and col.row.}

\item{select.col, select.row}{a selection of columns/rows to be drawn. Allowed 
values are NULL or a list containing the arguments name, cos2 or contrib: 
\itemize{ \item name is a character vector containing column/row names to be
drawn \item cos2 if cos2 is in [0, 1], ex: 0.6, then columns/rows with a 
cos2 > 0.6 are drawn. if cos2 > 1, ex: 5, then the top 5 columns/rows with 
the highest cos2 are drawn. \item contrib if contrib > 1, ex: 5,  then the 
top 5 columns/rows with the highest contrib are drawn }}

\item{label}{a character vector specifying the elements to be labelled. 
Default value is "all". Allowed values are "none" or the combination of 
c("row", "row.sup", "col", "col.sup"). Use "col" to label only active column
variables; "col.sup" to label only supplementary columns; etc}

\item{invisible}{a character value specifying the elements to be hidden on the
plot. Default value is "none". Allowed values are the combination of 
c("row", "row.sup","col", "col.sup").}

\item{arrows}{Vector of two logicals specifying if the plot should contain 
points (FALSE, default) or arrows (TRUE). First value sets the rows and the 
second value sets the columns.}

\item{title}{the title of the graph}
}
\value{
a ggplot
}
\description{
Correspondence analysis (CA) is an extension of Principal 
 Component Analysis (PCA) suited to analyze frequencies formed by two 
 categorical variables. fviz_ca() provides ggplot2-based elegant 
 visualization of CA outputs from the R functions: CA [in FactoMineR], ca [in
 ca], coa [in ade4], correspondence [in MASS] and expOutput/epCA [in
 ExPosition]. Read more: 
 \href{https://www.sthda.com/english/articles/31-principal-component-methods-in-r-practical-guide/113-ca-correspondence-analysis-in-r-essentials/}{Correspondence
  Analysis}
 
 \itemize{ \item fviz_ca_row(): Graph of row variables \item fviz_ca_col():
 Graph of column variables \item fviz_ca_biplot(): Biplot of row and column
 variables \item fviz_ca(): An alias of fviz_ca_biplot() }
}
\details{
The default plot of (M)CA is a "symmetric" plot in which both rows
 and columns are in principal coordinates. In this situation, it's not
 possible to interpret the distance between row points and column points. To
 overcome this problem, the simplest way is to make an asymmetric plot. This
 means that, the column profiles must be presented in row space or
 vice-versa. The allowed options for the argument map are: \itemize{ \item
 "rowprincipal" or "colprincipal": asymmetric plots with either rows in
 principal coordinates and columns in standard coordinates, or vice versa.
 These plots preserve row metric or column metric respectively. \item
 "symbiplot": Both rows and columns are scaled to have variances equal to the
 singular values (square roots of eigenvalues), which gives a symmetric
 biplot but does not preserve row or column metrics. \item "rowgab" or
 "colgab": Asymmetric maps, proposed by Gabriel & Odoroff (1990), with rows
 (respectively, columns) in principal coordinates and columns (respectively,
 rows) in standard coordinates multiplied by the mass of the corresponding
 point. \item "rowgreen" or "colgreen": The so-called contribution biplots
 showing visually the most contributing points (Greenacre 2006b). These are
 similar to "rowgab" and "colgab" except that the points in standard
 coordinates are multiplied by the square root of the corresponding masses,
 giving reconstructions of the standardized residuals. }
}
\examples{
# Correspondence Analysis
# ++++++++++++++++++++++++++++++
# Install and load FactoMineR to compute CA
# install.packages("FactoMineR")

library("FactoMineR")
data(housetasks)
head(housetasks)
res.ca <- CA(housetasks, graph=FALSE)

# Biplot of rows and columns
# ++++++++++++++++++++++++++
# Symetric Biplot of rows and columns
fviz_ca_biplot(res.ca)

# Asymetric biplot, use arrows for columns
fviz_ca_biplot(res.ca, map ="rowprincipal",
 arrow = c(FALSE, TRUE))
 
# Keep only the labels for row points
fviz_ca_biplot(res.ca, label ="row")

# Keep only labels for column points
fviz_ca_biplot(res.ca, label ="col")

       
# Select the top 7 contributing rows
# And the top 3 columns
fviz_ca_biplot(res.ca,  
               select.row = list(contrib = 7),
               select.col = list(contrib = 3))

# Graph of row variables
# +++++++++++++++++++++
   
# Control automatically the color of row points
   # using the "cos2" or the contributions "contrib"
   # cos2 = the quality of the rows on the factor map
   # Change gradient color
   # Use repel = TRUE to avoid overplotting (slow if many points)
fviz_ca_row(res.ca, col.row = "cos2",
   gradient.cols = c("#00AFBB", "#E7B800", "#FC4E07"),
   repel = TRUE)

# You can also control the transparency 
# of the color by the "cos2" or "contrib"
fviz_ca_row(res.ca, alpha.row="contrib") 
      
# Select and visualize some rows with select.row argument.
 # - Rows with cos2 >= 0.5: select.row = list(cos2 = 0.5)
 # - Top 7 rows according to the cos2: select.row = list(cos2 = 7)
 # - Top 7 contributing rows: select.row = list(contrib = 7)
 # - Select rows by names: select.row = list(name = c("Breakfeast", "Repairs", "Holidays"))
 
 # Example: Select the top 7 contributing rows
fviz_ca_row(res.ca, select.row = list(contrib = 7))

 
# Graph of column points
# ++++++++++++++++++++++++++++

 
# Control colors using their contributions
fviz_ca_col(res.ca, col.col = "contrib",
   gradient.cols = c("#00AFBB", "#E7B800", "#FC4E07"))
       
# Select columns with select.col argument
   # You can select by contrib, cos2 and name 
   # as previously described for ind
# Select the top 3 contributing columns
fviz_ca_col(res.ca, select.col = list(contrib = 3))
    
    
 
}
\references{
\url{https://www.sthda.com/english/}
}
\seealso{
\code{\link{get_ca}}, \code{\link{fviz_pca}}, \code{\link{fviz_mca}}
}
\author{
Alboukadel Kassambara \email{alboukadel.kassambara@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{map_factominer_legacy_names}
\alias{map_factominer_legacy_names}
\title{Map legacy FactoMineR category names to current labels}
\usage{
map_factominer_legacy_names(
  X,
  names,
  element = c("quali.var", "quali.sup", "var"),
  quiet = FALSE
)
}
\arguments{
\item{X}{a FactoMineR object (MCA, MFA, FAMD, HMFA).}

\item{names}{character vector of category labels.}

\item{element}{element to map. Use "var" for MCA categories or "quali.var"
for MFA/FAMD/HMFA qualitative categories. "quali.sup" maps supplementary
qualitative categories when available.}

\item{quiet}{if TRUE, suppress warnings.}
}
\value{
Character vector of mapped labels.
}
\description{
Map legacy FactoMineR category names to current labels
}
\examples{
\donttest{
if (requireNamespace("FactoMineR", quietly = TRUE)) {
  data(poison)
  res.mca <- FactoMineR::MCA(poison, quanti.sup = 1:2, quali.sup = 3:4, graph = FALSE)
  map <- factominer_category_map(res.mca, element = "var")
  map_factominer_legacy_names(res.mca, map$legacy_underscore[1:3], element = "var", quiet = TRUE)
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fviz_nbclust.R
\name{fviz_nbclust}
\alias{fviz_nbclust}
\alias{fviz_gap_stat}
\title{Determining and Visualizing the Optimal Number of Clusters}
\usage{
fviz_nbclust(
  x,
  FUNcluster = NULL,
  method = c("silhouette", "wss", "gap_stat"),
  diss = NULL,
  k.max = 10,
  nboot = 100,
  verbose = interactive(),
  barfill = "steelblue",
  barcolor = "steelblue",
  linecolor = "steelblue",
  print.summary = TRUE,
  ...
)

fviz_gap_stat(
  gap_stat,
  linecolor = "steelblue",
  maxSE = list(method = "firstSEmax", SE.factor = 1)
)
}
\arguments{
\item{x}{numeric matrix or data frame. In the function fviz_nbclust(), x can
be the results of the function NbClust(). For \code{method = "silhouette"}
or \code{"wss"}, x may also be a precomputed dissimilarity (an object of
class \code{"dist"}); it is then passed directly to \code{FUNcluster},
which must be a dissimilarity-capable method (e.g. \code{cluster::pam} or
\code{factoextra::hcut}). \code{method = "gap_stat"} still needs the raw data.}

\item{FUNcluster}{a partitioning function which accepts as first argument a
(data) matrix like \code{x}, second argument, say \code{k >= 2}, the
number of clusters desired, and returns a list with a component named
\code{cluster} which contains the grouping of observations. Allowed values
include: \code{kmeans}, \code{cluster::pam}, \code{cluster::clara},
\code{cluster::fanny}, \code{hcut}, etc. In \code{method = "wss"} mode,
\code{fviz_nbclust()} computes the \code{k = 1} baseline internally instead
of calling \code{FUNcluster(x, 1, ...)}. This argument is not required
when \code{x} is an output of the function \code{NbClust::NbClust()}.}

\item{method}{the method to be used for estimating the optimal number of 
clusters. Possible values are "silhouette" (for average silhouette width), 
"wss" (for total within sum of square) and "gap_stat" (for gap statistics).}

\item{diss}{dist object as produced by dist(), i.e.: diss = dist(x, method = 
"euclidean"). Used to compute the average silhouette width of clusters, the
within sum of square and hierarchical clustering. If NULL, dist(x) is 
computed with the default method = "euclidean"}

\item{k.max}{the maximum number of clusters to consider, must be at least two.}

\item{nboot}{integer, number of Monte Carlo ("bootstrap") samples. Used only for determining the number of clusters 
using gap statistic.}

\item{verbose}{logical value. If TRUE, the result of progress is printed.}

\item{barfill, barcolor}{fill color and outline color for bars}

\item{linecolor}{color for lines}

\item{print.summary}{logical value. If true, the optimal number of clusters 
are printed in fviz_nbclust().}

\item{...}{optionally further arguments:
arguments for FUNcluster() in "wss"/"silhouette" modes; arguments for
\code{\link[cluster]{clusGap}}() in "gap_stat" mode. A \code{maxSE} list
can also be supplied in "gap_stat" mode and is forwarded to
\code{fviz_gap_stat()}.}

\item{gap_stat}{an object of class "clusGap" returned by the function
clusGap() [in cluster package]}

\item{maxSE}{a list containing the parameters (method and SE.factor) for
determining the location of the maximum of the gap statistic (Read the
documentation ?cluster::maxSE). Allowed values for maxSE$method include:
\itemize{ \item "globalmax": simply corresponds to the global maximum,
i.e., is which.max(gap) \item "firstmax": gives the location of the first
local maximum \item "Tibs2001SEmax": uses the criterion, Tibshirani et al
(2001) proposed: "the smallest k such that gap(k) >= gap(k+1) - s(k+1)".
It's also possible to use "the smallest k such that gap(k) >= gap(k+1) -
SE.factor*s(k+1)" where SE.factor is a numeric value which can be 1
(default), 2, 3, etc. \item "firstSEmax": location of the first f() value
which is not larger than the first local maximum minus SE.factor * SE.f,
i.e, within an "f S.E." range of that maximum. \item
see ?cluster::maxSE for more options }}
}
\value{
\itemize{ \item fviz_nbclust, fviz_gap_stat: return a ggplot2 }
}
\description{
Partitioning methods, such as k-means clustering require the 
  users to specify the number of clusters to be generated. \itemize{
  \item fviz_nbclust(): Determines and visualizes the optimal number of
  clusters using different methods: \strong{within cluster sums of squares},
  \strong{average silhouette} and \strong{gap statistics}. Silhouette values
  are evaluated only for \code{k = 2, \ldots, k.max} because average
  silhouette width is undefined for a one-cluster partition. The dashed
  guide line marks the best displayed silhouette width among those
  \code{k >= 2} candidates.
  \item fviz_gap_stat(): Visualizes the gap statistic generated by the
  function \code{\link[cluster]{clusGap}}() [in cluster package]. The optimal
  number of clusters is specified using the "firstmax" method
  (?cluster::clustGap). }

  For \code{method = "wss"}, \code{factoextra} computes the \code{k = 1}
  baseline internally so helper functions such as \code{hcut()} and
  \code{hkmeans()} can keep rejecting direct \code{k = 1} inputs.

  Read more:
  \href{https://www.datanovia.com/en/lessons/determining-the-optimal-number-of-clusters-3-must-know-methods/}{Determining
   the optimal number of clusters}
}
\section{Method selection for gap statistic}{

  The default method "firstSEmax" (developed by Martin Maechler, 2012) is
  recommended as a robust alternative to "Tibs2001SEmax". The original
  Tibshirani method can be overly conservative and often returns k=1 when
  standard deviations are large relative to gap differences. The "firstSEmax"
  method finds the smallest k within one standard error of the first local
  maximum, providing more stable results in practice.
}

\examples{
set.seed(123)

# Data preparation
# +++++++++++++++
data("iris")
head(iris)
# Remove species column (5) and scale the data
iris.scaled <- scale(iris[, -5])


# Optimal number of clusters in the data
# ++++++++++++++++++++++++++++++++++++++
# Examples are provided only for kmeans, but
# you can also use cluster::pam (for pam) or
#  hcut (for hierarchical clustering)
 
### Elbow method (look at the knee)
# Elbow method for kmeans
fviz_nbclust(iris.scaled, kmeans, method = "wss") +
geom_vline(xintercept = 3, linetype = 2)

# WSS with hierarchical clustering keeps the internal k = 1 baseline
fviz_nbclust(iris.scaled, hcut, method = "wss", hc_method = "complete")

# Average silhouette for kmeans
fviz_nbclust(iris.scaled, kmeans, method = "silhouette")

### Gap statistic
library(cluster)
set.seed(123)
# Compute gap statistic for kmeans
# we used B = 10 for demo. Recommended value is ~500
gap_stat <- clusGap(iris.scaled, FUN = kmeans, nstart = 25,
 K.max = 10, B = 10)
 print(gap_stat, method = "firstmax")
fviz_gap_stat(gap_stat)
 
# Gap statistic for hierarchical clustering
gap_stat <- clusGap(iris.scaled, FUN = hcut, K.max = 10, B = 10)
fviz_gap_stat(gap_stat)

 
}
\seealso{
\code{\link{fviz_cluster}}, \code{\link{eclust}}
}
\author{
Alboukadel Kassambara \email{alboukadel.kassambara@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/housetasks.R
\docType{data}
\name{housetasks}
\alias{housetasks}
\title{House tasks contingency table}
\format{
A data frame with 13 observations (house tasks) on the following 4 columns.
\describe{
 \item{\code{Wife}}{a numeric vector}
 \item{\code{Alternating}}{a numeric vector}
 \item{\code{Husband}}{a numeric vector}
 \item{\code{Jointly}}{a numeric vector}
}
}
\source{
This data is from FactoMineR package.
}
\usage{
data("housetasks")
}
\description{
A data frame containing the frequency of execution of 13 house
 tasks in the couple. This table is also available in ade4 package.
}
\examples{
library(FactoMineR)
data(housetasks)
res.ca <- CA(housetasks, graph=FALSE)
fviz_ca_biplot(res.ca, repel = TRUE)+
theme_minimal()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dist.R
\name{dist}
\alias{dist}
\alias{get_dist}
\alias{fviz_dist}
\title{Enhanced Distance Matrix Computation and Visualization}
\usage{
get_dist(x, method = "euclidean", stand = FALSE, ...)

fviz_dist(
  dist.obj,
  order = TRUE,
  show_labels = TRUE,
  lab_size = NULL,
  gradient = list(low = "red", mid = "white", high = "blue")
)
}
\arguments{
\item{x}{a numeric matrix or a data frame.}

\item{method}{the distance measure to be used. This must be one of 
"euclidean", "maximum", "manhattan", "canberra", "binary", "minkowski", 
"pearson", "spearman" or "kendall".}

\item{stand}{logical value; default is FALSE. If TRUE, then the data will be
standardized using the function \code{scale()}. Measurements are
standardized for each variable (column), by subtracting the variable's
mean value and dividing by the variable's standard deviation. If scaling
produces \code{NA} values, \code{get_dist()} stops with a package-level
error.}

\item{...}{other arguments to be passed to the function dist() when using get_dist().}

\item{dist.obj}{an object of class "dist" as generated by the function
\code{dist()} or \code{get_dist()}, containing only finite dissimilarity
values.}

\item{order}{logical value. if TRUE the ordered dissimilarity image (ODI) is shown.}

\item{show_labels}{logical value. If TRUE, the labels are displayed.}

\item{lab_size}{the size of labels.}

\item{gradient}{a list containing three elements specifying the colors for low, mid and high values in 
the ordered dissimilarity image. The element "mid" can take the value of NULL.}
}
\value{
\itemize{ \item get_dist(): returns an object of class "dist". \item 
  fviz_dist(): returns a ggplot2 }
}
\description{
Clustering methods classify data samples into groups of similar 
  objects. This process requires some methods for measuring the distance or 
  the (dis)similarity between the observations. Read more: 
  \href{https://www.datanovia.com/en/lessons/clustering-distance-measures/}{STHDA
   website - clarifying distance measures.}. \itemize{ \item get_dist():
  Computes a distance matrix between the rows of a data matrix. Compared to
  the standard \code{\link[stats]{dist}}() function, it supports
  correlation-based distance measures including "pearson", "kendall" and
  "spearman" methods. \item fviz_dist(): Visualizes a distance matrix }

  When \code{stand = TRUE}, scaling that produces non-finite values is
  rejected. \code{fviz_dist()} also validates that supplied distance objects
  contain only finite values before plotting.
}
\examples{
data(USArrests)
res.dist <- get_dist(USArrests, stand = TRUE, method = "pearson")

fviz_dist(res.dist, 
   gradient = list(low = "#00AFBB", mid = "white", high = "#FC4E07"))
}
\seealso{
\code{\link[stats]{dist}}
}
\author{
Alboukadel Kassambara \email{alboukadel.kassambara@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fviz_mclust.R
\name{fviz_mclust}
\alias{fviz_mclust}
\alias{fviz_mclust_bic}
\title{Plot Model-Based Clustering Results using ggplot2}
\usage{
fviz_mclust(
  object,
  what = c("classification", "uncertainty", "BIC"),
  ellipse.type = "norm",
  ellipse.level = 0.4,
  ggtheme = theme_classic(),
  ...
)

fviz_mclust_bic(
  object,
  model.names = NULL,
  shape = 19,
  color = "model",
  palette = NULL,
  legend = NULL,
  main = "Model selection",
  xlab = "Number of components",
  ylab = "BIC",
  ...
)
}
\arguments{
\item{object}{an object of class Mclust}

\item{what}{choose from one of the following three options: "classification" (default), "uncertainty" and "BIC".}

\item{ellipse.type}{Character specifying frame type. Possible values are
'convex', 'confidence' or types supported by
\code{\link[ggplot2]{stat_ellipse}} including one of c("t", "norm",
"euclid").}

\item{ellipse.level}{the size of the concentration ellipse in normal
probability. Passed for \code{ggplot2::stat_ellipse} 's level. Ignored in
'convex'. Default value is 0.95.}

\item{ggtheme}{function, ggplot2 theme name. Default value is theme_pubr().
Allowed values include ggplot2 official themes: theme_gray(), theme_bw(),
theme_minimal(), theme_classic(), theme_void(), ....}

\item{...}{other arguments to be passed to the functions \link{fviz_cluster} and \link[ggpubr]{ggpar}.}

\item{model.names}{one or more model names corresponding to models fit in object. The default
is to plot the BIC for all of the models fit.}

\item{shape}{point shape. To change point shape by model names use shape = "model".}

\item{color}{point and line color.}

\item{palette}{the color palette to be used for coloring or filling by
groups. Allowed values include "grey" for grey color palettes; brewer
palettes e.g. "RdBu", "Blues", ...; or custom color palette e.g. c("blue",
"red"); and scientific journal palettes from ggsci R package, e.g.: "npg",
"aaas", "lancet", "jco", "ucscgb", "uchicago", "simpsons" and
"rickandmorty". Can be also a numeric vector of length(groups); in this
case a basic color palette is created using the function
\link[grDevices]{palette}.}

\item{legend}{character specifying legend position. Allowed values are one of
c("top", "bottom", "left", "right", "none"). To remove the legend use
legend = "none". Legend position can be also specified using a numeric
vector c(x, y); see details section.}

\item{main}{plot main title.}

\item{xlab}{character vector specifying x axis labels. Use xlab = FALSE to
hide xlab.}

\item{ylab}{character vector specifying y axis labels. Use ylab = FALSE to
hide ylab.}
}
\value{
A ggplot2 object.
}
\description{
Plots the classification, the uncertainty and the BIC values returned by the Mclust() function.
}
\section{Functions}{
\itemize{
\item \code{fviz_mclust()}: Plots classification and uncertainty.

\item \code{fviz_mclust_bic()}: Plots the BIC values.

}}
\examples{

if(requireNamespace("mclust", quietly = TRUE)){

# Compute model-based-clustering
library("mclust")
data("diabetes")
mc <- Mclust(diabetes[, -1])

# Visualize BIC values
fviz_mclust_bic(mc)

# Visualize classification
fviz_mclust(mc, "classification", geom = "point")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R
\name{deprecated}
\alias{deprecated}
\alias{get_mfa_quanti_var}
\alias{get_mfa_quali_var}
\alias{get_mfa_group}
\alias{fviz_mfa_ind_starplot}
\alias{fviz_mfa_group}
\alias{fviz_mfa_quanti_var}
\alias{fviz_mfa_quali_var}
\alias{get_hmfa_quanti_var}
\alias{get_hmfa_quali_var}
\alias{get_hmfa_group}
\alias{fviz_hmfa_quanti_var}
\alias{fviz_hmfa_quali_var}
\alias{fviz_hmfa_ind_starplot}
\alias{fviz_hmfa_group}
\title{Deprecated Functions}
\usage{
get_mfa_quanti_var(res.mfa)

get_mfa_quali_var(res.mfa)

get_mfa_group(res.mfa)

fviz_mfa_ind_starplot(X, ...)

fviz_mfa_group(X, ...)

fviz_mfa_quanti_var(X, ...)

fviz_mfa_quali_var(X, ...)

get_hmfa_quanti_var(res.hmfa)

get_hmfa_quali_var(res.hmfa)

get_hmfa_group(res.hmfa)

fviz_hmfa_quanti_var(X, ...)

fviz_hmfa_quali_var(X, ...)

fviz_hmfa_ind_starplot(X, ...)

fviz_hmfa_group(X, ...)
}
\arguments{
\item{res.mfa}{an object of class MFA [FactoMineR].}

\item{X}{an object of class MFA or HMFA [FactoMineR].}

\item{...}{Other arguments.}

\item{res.hmfa}{an object of class HMFA [FactoMineR].}
}
\description{
Deprecated functions. Will be removed in a future version.

\itemize{
\item get_mfa_quanti_var(). Deprecated. Use get_mfa_var(res.mfa, "quanti.var") instead.
\item get_mfa_quali_var().  Deprecated. Use get_mfa_var(res.mfa, "quali.var") instead.
\item get_mfa_group(). Deprecated. Use get_mfa_var(res.mfa, "group") instead.

\item fviz_mfa_ind_starplot(): Star graph of individuals (draws partial points).
Deprecated. Use fviz_mfa_ind(res.mfa, partial = "all") instead.
\item fviz_mfa_quanti_var(): Graph of quantitative variables. Deprecated. Use fviz_mfa(X, "quanti.var") instead.
\item fviz_mfa_quali_var(): Graph of qualitative variables. Deprecated. Use fviz_mfa(X, "quali.var") instead.

 \item
get_hmfa_quanti_var(). Deprecated. Use get_hmfa_var(res.hmfa, "quanti.var")
instead. \item get_hmfa_quali_var().  Deprecated. Use get_hmfa_var(res.hmfa,
"quali.var") instead. \item get_hmfa_group(). Deprecated. Use
get_hmfa_var(res.hmfa, "group") instead.

\item fviz_hmfa_ind_starplot(): Graph of partial individuals. Deprecated. Use fviz_hmfa_ind(X, partial = "all") instead.
\item fviz_hmfa_quanti_var(): Graph of quantitative variables. Deprecated. Use fviz_hmfa_var(X, "quanti.var") instead.
\item fviz_hmfa_quali_var(): Graph of qualitative variables. Deprecated. Use fviz_hmfa_var(X, "quali.var") instead.
\item fviz_hmfa_group(): Graph of the groups representation. Deprecated. Use fviz_hmfa_var(X, "group") instead.

}
}
\author{
Alboukadel Kassambara \email{alboukadel.kassambara@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_mca.R
\name{get_mca}
\alias{get_mca}
\alias{get_mca_var}
\alias{get_mca_ind}
\title{Extract the results for individuals/variables - MCA}
\usage{
get_mca(res.mca, element = c("var", "ind", "mca.cor", "quanti.sup"))

get_mca_var(res.mca, element = c("var", "mca.cor", "quanti.sup"))

get_mca_ind(res.mca)
}
\arguments{
\item{res.mca}{an object of class MCA [FactoMineR], acm [ade4], expoOutput/epMCA [ExPosition].}

\item{element}{the element to subset from the output. Possible values are
"var" for variables, "ind" for individuals, "mca.cor" for correlation
between variables and principal dimensions, and "quanti.sup" for
quantitative supplementary variables in FactoMineR MCA results.}
}
\value{
a list of matrices containing the results for the active 
  individuals/variable categories including : \item{coord}{coordinates for 
  the individuals/variable categories} \item{cos2}{cos2 for the 
  individuals/variable categories} \item{contrib}{contributions of the 
  individuals/variable categories} \item{inertia}{inertia of the 
  individuals/variable categories}
}
\description{
Extract all the results (coordinates, squared cosine and
  contributions) for the active individuals/variable categories from
  Multiple Correspondence Analysis (MCA) outputs.\cr\cr \itemize{ \item
  get_mca(): Extract the results for variables and individuals \item
  get_mca_ind(): Extract the results for individuals only \item
  get_mca_var(): Extract the results for variables only }

  For FactoMineR MCA results, \code{get_mca()} and \code{get_mca_var()} also
  support \code{element = "quanti.sup"} for quantitative supplementary
  variables and report a clean package-level error when that result is
  absent.
}
\examples{
\donttest{
# Multiple Correspondence Analysis
# ++++++++++++++++++++++++++++++
# Install and load FactoMineR to compute MCA
# install.packages("FactoMineR")
library("FactoMineR")
data(poison)
res.mca <- MCA(poison, quanti.sup = 1:2, graph = FALSE)
 
 # Extract the results for variable categories
 var <- get_mca_var(res.mca)
 print(var)
 head(var$coord) # coordinates of variables
 head(var$cos2) # cos2 of variables
 head(var$contrib) # contributions of variables
 
 # Extract the results for individuals
 ind <- get_mca_ind(res.mca)
 print(ind)
 head(ind$coord) # coordinates of individuals
 head(ind$cos2) # cos2 of individuals
 head(ind$contrib) # contributions of individuals
 
 # You can also use the function get_mca()
 get_mca(res.mca, "ind") # Results for individuals
 get_mca(res.mca, "var") # Results for variable categories
 quanti.sup <- get_mca(res.mca, "quanti.sup")
 head(quanti.sup$coord) # coordinates of quantitative supplementary variables
 }
 
}
\references{
\url{https://www.sthda.com/english/}
}
\author{
Alboukadel Kassambara \email{alboukadel.kassambara@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fviz_famd.R
\name{fviz_famd}
\alias{fviz_famd}
\alias{fviz_famd_ind}
\alias{fviz_famd_var}
\title{Visualize Factor Analysis of Mixed Data}
\usage{
fviz_famd_ind(
  X,
  axes = c(1, 2),
  geom = c("point", "text"),
  repel = FALSE,
  habillage = "none",
  palette = NULL,
  addEllipses = FALSE,
  col.ind = "blue",
  col.ind.sup = "darkblue",
  alpha.ind = 1,
  shape.ind = 19,
  col.quali.var = "black",
  select.ind = list(name = NULL, cos2 = NULL, contrib = NULL),
  gradient.cols = NULL,
  ...
)

fviz_famd_var(
  X,
  choice = c("var", "quanti.var", "quali.var", "quali.sup"),
  axes = c(1, 2),
  geom = c("point", "text"),
  repel = FALSE,
  col.var = "red",
  alpha.var = 1,
  shape.var = 17,
  col.var.sup = "darkgreen",
  select.var = list(name = NULL, cos2 = NULL, contrib = NULL),
  ...
)

fviz_famd(X, ...)
}
\arguments{
\item{X}{an object of class FAMD [FactoMineR].}

\item{axes}{a numeric vector of length 2 specifying the dimensions to be 
plotted.}

\item{geom}{a text specifying the geometry to be used for the graph. Allowed 
values are the combination of \code{c("point", "arrow", "text")}. Use 
\code{"point"} (to show only points); \code{"text"} to show only labels; 
\code{c("point", "text")} or \code{c("arrow", "text")} to show arrows and 
texts. Using \code{c("arrow", "text")} is sensible only for the graph of 
variables.}

\item{repel}{a boolean, whether to use ggrepel to avoid overplotting text
labels or not. The old \code{jitter} argument is kept for backward
compatibility and is converted to \code{repel = TRUE} with a deprecation warning.}

\item{habillage}{an optional factor variable for coloring the observations by
groups. Default value is "none". If X is an MFA object from FactoMineR 
package, habillage can also specify the index of the factor variable in the
data.}

\item{palette}{the color palette to be used for coloring or filling by
groups. Allowed values include "grey" for grey color palettes; brewer
palettes e.g. "RdBu", "Blues", ...; or custom color palette e.g. c("blue",
"red"); and scientific journal palettes from ggsci R package, e.g.: "npg",
"aaas", "lancet", "jco", "ucscgb", "uchicago", "simpsons" and
"rickandmorty". Can be also a numeric vector of length(groups); in this
case a basic color palette is created using the function
\link[grDevices]{palette}.}

\item{addEllipses}{logical value. If TRUE, draws ellipses around the 
individuals when habillage != "none".}

\item{col.ind, col.var}{color for individuals and variables, respectively. Can
be a continuous variable or a factor variable. Possible values include also
: "cos2", "contrib", "coord", "x" or "y". In this case, the colors for
individuals/variables are automatically controlled by their qualities
("cos2"), contributions ("contrib"), coordinates (x^2 + y^2 , "coord"), x
values("x") or y values("y"). To use automatic coloring (by cos2, contrib,
....), make sure that habillage ="none".}

\item{col.ind.sup}{color for supplementary individuals}

\item{alpha.ind, alpha.var}{controls the transparency of individuals and 
variables, respectively. The value can variate from 0 (total transparency) 
to 1 (no transparency). Default value is 1. Possible values include also : 
"cos2", "contrib", "coord", "x" or "y". In this case, the transparency for 
individual/variable colors are automatically controlled by their qualities 
("cos2"), contributions ("contrib"), coordinates (x^2 + y^2 , "coord"), x 
values("x") or y values("y"). To use this, make sure that habillage 
="none".}

\item{shape.ind, shape.var}{point shapes of individuals, variables, groups and
axes}

\item{col.quali.var}{color for qualitative variables in fviz_famd_ind(). 
Default is "black".}

\item{select.ind, select.var}{a selection of individuals and variables to be 
drawn. Allowed values are NULL or a list containing the arguments name, 
cos2 or contrib: \itemize{ \item name is a character vector containing 
individuals/variables to be drawn \item cos2 if cos2 is in [0, 1], ex: 0.6,
then individuals/variables with a cos2 > 0.6 are drawn. if cos2 > 1, ex: 5,
then the top 5 individuals/variables with the highest cos2 are drawn. \item
contrib if contrib > 1, ex: 5,  then the top 5 individuals/variables with 
the highest cos2 are drawn }}

\item{gradient.cols}{vector of colors to use for n-colour gradient. Allowed
values include brewer and ggsci color palettes.}

\item{...}{Arguments to be passed to the function fviz()}

\item{choice}{The graph to plot in fviz_famd_var(). Allowed values include
one of c("var", "quanti.var", "quali.var", "quali.sup").}

\item{col.var.sup}{color for supplementary variables.}
}
\value{
a ggplot
}
\description{
Factor analysis of mixed data (FAMD) is, a particular case of 
  MFA, used to analyze a data set containing both quantitative and
  qualitative variables. fviz_famd() provides ggplot2-based elegant
  visualization of FAMD outputs from the R function: FAMD [FactoMineR].\cr\cr
  \itemize{ \item fviz_famd_ind(): Graph of individuals
  \item fviz_famd_var(): Graph of variables \item fviz_famd(): An alias of
  fviz_famd_ind(res.famd) }
}
\examples{
# Compute FAMD
 library("FactoMineR")
 data(wine)
 res.famd <- FAMD(wine[,c(1,2, 16, 22, 29, 28, 30,31)], graph = FALSE)
 res.famd.sup <- FAMD(wine[,c(1,2, 16, 22, 29, 28, 30,31)],
                      sup.var = 2, graph = FALSE)
               
# Eigenvalues/variances of dimensions
fviz_screeplot(res.famd)
# Graph of variables
fviz_famd_var(res.famd)
# Quantitative variables
fviz_famd_var(res.famd, "quanti.var", repel = TRUE, col.var = "black")
# Qualitative variables
fviz_famd_var(res.famd, "quali.var", col.var = "black")
# Supplementary qualitative variable categories
fviz_famd_var(res.famd.sup, "quali.sup", col.var = "darkgreen")
# Graph of individuals colored by cos2
fviz_famd_ind(res.famd, col.ind = "cos2", 
  gradient.cols = c("#00AFBB", "#E7B800", "#FC4E07"),
  repel = TRUE)


}
\author{
Alboukadel Kassambara \email{alboukadel.kassambara@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_famd.R
\name{get_famd}
\alias{get_famd}
\alias{get_famd_ind}
\alias{get_famd_var}
\title{Extract the results for individuals and variables - FAMD}
\usage{
get_famd(
  res.famd,
  element = c("ind", "var", "quanti.var", "quali.var", "quali.sup")
)

get_famd_ind(res.famd)

get_famd_var(
  res.famd,
  element = c("var", "quanti.var", "quali.var", "quali.sup")
)
}
\arguments{
\item{res.famd}{an object of class FAMD [FactoMineR].}

\item{element}{the element to subset from the output. Possible values are
"ind", "var", "quanti.var", "quali.var" or "quali.sup".}
}
\value{
a list of matrices containing the results for the active 
individuals and variables, including : 
\item{coord}{coordinates of individuals/variables.}
\item{cos2}{cos2 values representing the quality of representation on the factor map.}
\item{contrib}{contributions of individuals / variables to the principal components.}
}
\description{
Extract all the results (coordinates, squared cosine and contributions) 
for the active individuals and variables from Factor Analysis of Mixed Data (FAMD) outputs.\cr\cr
\itemize{
\item get_famd(): Extract the results for variables and individuals
\item get_famd_ind(): Extract the results for individuals only
\item get_famd_var(): Extract the results for quantitative and qualitative variables only
}
}
\examples{

 # Compute FAMD
 library("FactoMineR")
 data(wine)
 res.famd <- FAMD(wine[,c(1,2, 16, 22, 29, 28, 30,31)], graph = FALSE)
 res.famd.sup <- FAMD(wine[,c(1,2, 16, 22, 29, 28, 30,31)],
                      sup.var = 2, graph = FALSE)
 
 # Extract the results for qualitative variable categories
 quali.var <- get_famd_var(res.famd, "quali.var")
 print(quali.var)
 head(quali.var$coord) # coordinates of qualitative variables

 # Extract the results for supplementary qualitative variable categories
 quali.sup <- get_famd_var(res.famd.sup, "quali.sup")
 print(quali.sup)
 head(quali.sup$coord) # coordinates of supplementary qualitative variables
 
 # Extract the results for quantitative variables
 quanti.var <- get_famd_var(res.famd, "quanti.var")
 print(quanti.var)
 head(quanti.var$coord) # coordinates
 
 # Extract the results for individuals
 ind <- get_famd_ind(res.famd)
 print(ind)
 head(ind$coord) # coordinates of individuals
 
 
}
\author{
Alboukadel Kassambara \email{alboukadel.kassambara@gmail.com}
}
